/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */





#include <stdio.h>


#include <dbus/dbus.h>

#include <pthread.h>
#include <vconf.h>
#include <vconf-keys.h>
#include "mf-main.h"
#include "mf-conf.h"
#include "mf-util.h"
#include "mf-callback.h"
#include "mf-fm-svc-wrapper.h"

#include "mf-widget.h"
#include "mf-fs-util.h"
#include "mf-launch.h"
#include "mf-dlog.h"
#include "mf-inotify-handle.h"
#include "mf-copy.h"
#include "mf-delete.h"
#include "mf-move.h"
#include "mf-otg.h"
#include "mf-resource.h"
#include "mf-language-mgr.h"
#include "mf-s-beam.h"
#include "mf-share.h"


struct appdata *temp_data;
pthread_mutex_t pb_mutex = PTHREAD_MUTEX_INITIALIZER;
/* mutex for refresh */
pthread_mutex_t refresh_mutex = PTHREAD_MUTEX_INITIALIZER;
/* mutex for dbus message */
pthread_mutex_t refresh_lock;
pthread_cond_t refresh_finished;
unsigned int refresh_flag = 0;
typedef struct _mf_dir_event_t {
	int event;
	char *name;
} mf_dir_event_t;

void mf_callback_save_cb(void *data, Evas_Object * obj, void *event_info);
void mf_callback_cancel_cb(void *data, Evas_Object * obj, void *event_info);
static void __mf_callback_mmc_removed(void *data, MF_STORAGE storage);

/******************************
** Prototype    : mf_callback_warning_popup_cb
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_warning_popup_cb(void *data, Evas_Object * obj, void *event_info)
{
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	Evas_Object *btn = (Evas_Object *)obj;
	const char *label = elm_object_text_get(btn);

	if (g_strcmp0(label, GET_SYS_STR(MF_BUTTON_LABEL_OK)) == 0 ) {
		evas_object_del(ap->mf_MainWindow.pNormalPopup);
		ap->mf_MainWindow.pNormalPopup = NULL;
	}
}

/******************************
** Prototype    : mf_callback_app_rotate_cb
** Description  :
** Input        : enum appcore_rm mode
**                void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
#ifdef MYFILE_SPLIT_VIEW

static Eina_Bool __mf_callback_do_rotation_cb (void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = data;
	assert(ap);

	mf_widget_create(ap);
	mf_widget_set_main_layout_content(ap);

	if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
		if (ap->mf_Status.view_type != mf_view_root_category && ap->mf_Status.view_type != mf_view_root) {
			mf_split_view_refresh(ap);
		}
	} else {
		if (!(ap->mf_Status.more == MORE_EDIT ||
		    ap->mf_Status.more == MORE_CREATE_FOLDER ||
		    ap->mf_Status.more == MORE_THUMBNAIL_RENAME ||
		    (ap->mf_Status.more == MORE_RENAME && ap->mf_Status.preMore == MORE_EDIT) ||
		    ap->mf_Status.more == MORE_SEARCH || (ap->mf_Status.more == MORE_RENAME && ap->mf_Status.preMore == MORE_SEARCH))) {
			if (ap->mf_Status.more == MORE_DEFAULT) {
				mf_navi_bar_create_default_view(ap);
				mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);
			} else if (ap->mf_Status.more == MORE_INTERNAL_COPY_MOVE || ap->mf_Status.more == MORE_INTERNAL_COPY || ap->mf_Status.more == MORE_INTERNAL_MOVE
				   || ap->mf_Status.more == MORE_DATA_COPYING || ap->mf_Status.more == MORE_DATA_MOVING) {
				mf_navi_bar_create_path_select_view(ap);
				mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);
			}
		}
		if (ap->mf_Status.more == MORE_EDIT) {
			mf_navi_bar_title_set(ap);
		}

	}

	if (ap->mf_Status.more == MORE_RENAME || ap->mf_Status.more == MORE_SEARCH || ap->mf_Status.more == MORE_CREATE_FOLDER || (ap->mf_Status.more == MORE_RENAME && ap->mf_Status.preMore == MORE_SEARCH)) {
		if (ap->mf_MainWindow.pEntry) {
			if (ap->mf_Status.flagIME != EINA_FALSE) {
				if (ap->mf_MainWindow.pNormalPopup == NULL) {
					elm_object_focus_set(ap->mf_MainWindow.pEntry, EINA_TRUE);
				}
			}
		}
	}
	if (ap->mf_Status.more == MORE_SEARCH) {
		if (ap->mf_MainWindow.pSearchEntry) {
			if (ap->mf_Status.flagIME != EINA_FALSE) {
				if (ap->mf_MainWindow.pNormalPopup == NULL) {
					elm_object_focus_set(ap->mf_MainWindow.pSearchEntry, EINA_TRUE);
				}
			}
		}
	}

	mf_genlist_gl_edit_item_selected(ap);/*note: for plm issue:P121002-2027*/
	mf_navi_bar_title_set(ap);
	MF_TRACE_END;

	return ECORE_CALLBACK_CANCEL;
}

void mf_callback_app_rotate_cb(app_device_orientation_e  mode, void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = data;
	assert(ap);

	if (ap->mf_Status.pre_rotate_type == mode)
		return;

        const char *config = elm_config_profile_get();
        if (!strcmp(config, "desktop"))
                return;

	enum ug_event evt = UG_EVENT_NONE;

	switch (mode) {
	case APP_DEVICE_ORIENTATION_270:
		ap->mf_Status.rotation_angle = -90;
		ap->mf_Status.view_mode_end = FALSE;
		ap->mf_Status.pre_rotate_type = APP_DEVICE_ORIENTATION_270;
		evt = UG_EVENT_ROTATE_LANDSCAPE;
		break;

	case APP_DEVICE_ORIENTATION_90:
		ap->mf_Status.rotation_angle = 90;
		ap->mf_Status.view_mode_end = FALSE;
		ap->mf_Status.pre_rotate_type = APP_DEVICE_ORIENTATION_90;
		evt = UG_EVENT_ROTATE_LANDSCAPE_UPSIDEDOWN;
		break;

	case APP_DEVICE_ORIENTATION_180:
		ap->mf_Status.rotation_angle = 180;
		ap->mf_Status.view_mode_end = TRUE;
		ap->mf_Status.pre_rotate_type = APP_DEVICE_ORIENTATION_180;
		evt = UG_EVENT_ROTATE_PORTRAIT_UPSIDEDOWN;
		break;

	case APP_DEVICE_ORIENTATION_0 :
	default:
		ap->mf_Status.rotation_angle = 0;
		ap->mf_Status.view_mode_end = TRUE;
		ap->mf_Status.pre_rotate_type = APP_DEVICE_ORIENTATION_0;
		evt = UG_EVENT_ROTATE_PORTRAIT;
		break;
	}

	ug_send_event(evt);

	if (ap->mf_MainWindow.pContextPopup) {
		evas_object_del(ap->mf_MainWindow.pContextPopup);
		ap->mf_MainWindow.pContextPopup = NULL;
		ap->mf_Status.flagCtxPopShow = EINA_TRUE;
	} else {
		ap->mf_Status.flagCtxPopShow = EINA_FALSE;
	}

	if (mode == APP_DEVICE_ORIENTATION_270 || mode == APP_DEVICE_ORIENTATION_90) {
		ap->mf_Status.rotation_type = MF_ROTATE_LANDSCAPE;
	} else {
		ap->mf_Status.rotation_type = MF_ROTATE_PORTRAIT;
	}
	elm_win_rotation_with_resize_set(ap->mf_MainWindow.pWindow, ap->mf_Status.rotation_angle);
	__mf_callback_do_rotation_cb(ap);

	MF_TRACE_END;
}

#else

void mf_callback_app_rotate_cb(app_device_orientation_e  mode, void *data)
{
	struct appdata *ap = data;
	assert(ap);

	ap->mf_Status.rotation_type = mode;

	if (ap->mf_FileOperation.ugImageViewer) {
		enum ug_mode md = ug_get_mode(ap->mf_FileOperation.ugImageViewer);
		enum ug_event ev = UG_EVENT_ROTATE_LANDSCAPE;
		if (md == UG_MODE_FULLVIEW) {
			switch (mode) {
			case APP_DEVICE_ORIENTATION_0 :
				elm_win_rotation_with_resize_set(ap->mf_MainWindow.pWindow, 0);
				ev = UG_EVENT_ROTATE_PORTRAIT;
				break;
			case APP_DEVICE_ORIENTATION_180:
				elm_win_rotation_with_resize_set(ap->mf_MainWindow.pWindow, 180);
				ev = UG_EVENT_ROTATE_PORTRAIT_UPSIDEDOWN;
				break;
			case APP_DEVICE_ORIENTATION_270:
				elm_win_rotation_with_resize_set(ap->mf_MainWindow.pWindow, 270);
				ev = UG_EVENT_ROTATE_LANDSCAPE;
				break;
			case APP_DEVICE_ORIENTATION_90:
				elm_win_rotation_with_resize_set(ap->mf_MainWindow.pWindow, 90);
				ev = UG_EVENT_ROTATE_LANDSCAPE_UPSIDEDOWN;
				break;
			default:
				break;
			}
			/*send event to ug */
			ug_send_event(ev);
		}
	}
}
#endif

#ifdef MYFILE_SPLIT_VIEW
void mf_callback_storage_cb(void *data, const char *label)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	mf_retm_if(label == NULL, "label is NULL");

	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap->mf_Status.path == NULL, "ap->mf_Status.path is NULL");
	mf_retm_if(ap->mf_Status.path->str == NULL, "ap->mf_Status.path->str is NULL");
	Evas_Object *pNaviBar = NULL;
	myfileNaviBar *pNavi_s_inuse = NULL;
	myfileNaviBar *pNavi_s_toshow = NULL;
	myfileNaviBar *pNaviBarStruct = NULL;
	char *rootpath = NULL;
	int current_storage = MYFILE_PHONE;
	int current_more = ap->mf_Status.more;
	pNavi_s_inuse = mf_navi_bar_get_in_use(ap);

	current_storage = mf_fm_svc_wrapper_get_location(ap->mf_Status.path->str);

	if (g_strcmp0(pNavi_s_inuse->pNaviLabel, label) == 0) {
		MF_TRACE_END;
		return;
	} else {
		if (ap->mf_Status.more == MORE_SEARCH) {
			ap->mf_Status.more = MORE_DEFAULT;
		}
		pNavi_s_inuse->naviFlagInUse = FALSE;
		pNaviBarStruct = mf_navi_bar_get_navi_from_navilist(ap->mf_MainWindow.plistNaviBar, label);


		if (pNaviBarStruct == NULL) {
			/*2.0	get root path by storage label */
			rootpath = mf_fm_svc_wrapper_get_root_path_by_tab_label(label);
			CHAR_CHECK_NULL_GOTO(rootpath, ERROR_EXIT);

			/*2.1	create the navi for the tab */
			pNavi_s_toshow = malloc(sizeof(myfileNaviBar));
			CHAR_CHECK_NULL_GOTO(pNavi_s_toshow, ERROR_EXIT);
			memset(pNavi_s_toshow, 0, sizeof(myfileNaviBar));

			pNavi_s_toshow->naviFlagInUse = TRUE;
			/*2.2	set path as root path of the storage */
			pNavi_s_toshow->pCurrentPath = g_strdup(rootpath);
			pNavi_s_toshow->pNaviLabel = strdup(label);
			free(rootpath);
			/*2.3	insert phone navi into the navi_list */
			ap->mf_MainWindow.plistNaviBar = eina_list_append(ap->mf_MainWindow.plistNaviBar, pNavi_s_toshow);

			g_string_free(ap->mf_Status.path, TRUE);
			ap->mf_Status.path = NULL;
			ap->mf_Status.path = g_string_new(pNavi_s_toshow->pCurrentPath);
			if (ap->mf_Status.more == MORE_INTERNAL_COPY || ap->mf_Status.more == MORE_INTERNAL_MOVE
			    || ap->mf_Status.more == MORE_INTERNAL_COPY_MOVE) {
				SAFE_FREE_GSTRING(pNavi_s_toshow->pre_operation.path);
					pNavi_s_toshow->pre_operation.path = g_string_new(pNavi_s_toshow->pCurrentPath);
					pNavi_s_toshow->naviFlagEditJoin = TRUE;
			}
			if (ap->mf_Status.more == MORE_DEFAULT) {
				mf_navi_bar_create_default_view(ap);
				mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);
			} else if (ap->mf_Status.more == MORE_INTERNAL_COPY_MOVE || ap->mf_Status.more == MORE_INTERNAL_COPY || ap->mf_Status.more == MORE_INTERNAL_MOVE
				   || ap->mf_Status.more == MORE_DATA_COPYING || ap->mf_Status.more == MORE_DATA_MOVING) {
				mf_navi_bar_create_path_select_view(ap);
				mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);
			}

			ap->mf_Status.more = current_more;
			goto SUCCESS_EXIT;
		} else {
			/*2.1	get the navi of the tab */
			pNaviBar = ap->mf_MainWindow.pNaviBar;
			pNavi_s_toshow = mf_navi_bar_get_struct_by_label(ap, label);
			pNavi_s_toshow->naviFlagInUse = TRUE;

			/*2.2	set related status value. */
			g_string_free(ap->mf_Status.path, TRUE);
			ap->mf_Status.path = NULL;

			ap->mf_Status.path = g_string_new(pNavi_s_toshow->pCurrentPath);
			/*2.3	update the content to catch update */
			/*Todo: How to ensure insert only once */
			if (ap->mf_Status.more == MORE_DEFAULT) {
				mf_navi_bar_create_default_view(ap);
				mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);
			} else if (ap->mf_Status.more == MORE_INTERNAL_COPY_MOVE || ap->mf_Status.more == MORE_INTERNAL_COPY || ap->mf_Status.more == MORE_INTERNAL_MOVE
				   || ap->mf_Status.more == MORE_DATA_COPYING || ap->mf_Status.more == MORE_DATA_MOVING) {
				mf_navi_bar_create_path_select_view(ap);
				mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);
			}
			goto SUCCESS_EXIT;
		}

	}

ERROR_EXIT:
	SAFE_FREE_CHAR(pNavi_s_toshow);
	SAFE_FREE_CHAR(rootpath);
	MF_TRACE_END;
	return;

SUCCESS_EXIT:
	MF_TRACE_END;
	return;

}
#endif

/******************************
** Prototype    : tab_callback
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_tab_cb(void *data, const char * path)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	mf_retm_if(path == NULL, "path is NULL");
	struct appdata *ap = (struct appdata *)data;

	const char *label = NULL;
	Evas_Object *pNaviBar = NULL;
	myfileNaviBar *pNavi_s_toshow = NULL;
	myfileNaviBar *pNavi_s_inuse = NULL;
#if 0
	Elm_Object_Item *it = NULL;
	it = (Elm_Object_Item *)event_info;
	label = elm_object_item_text_get(it);
#endif
	int optStorage = 0;
	optStorage = mf_fm_svc_wrapper_get_location(path);
	switch (optStorage) {
	case MYFILE_PHONE:
		label = GET_SYS_STR(MF_LABEL_PHONE);
		break;
	case MYFILE_MMC:
		label = GET_SYS_STR(MF_LABEL_MMC);
		break;
	case MYFILE_OTG:
		label = GET_STR(MF_LABEL_OTG);
		break;
	default:
		break;
	}


	if (ap->mf_Status.more == MORE_RENAME)
		mf_callback_destory_rename(ap);

	if (label != NULL) {
		pNavi_s_inuse = mf_navi_bar_get_in_use(ap);
		if (pNavi_s_inuse == NULL || pNavi_s_inuse->pNaviLabel == NULL) {
			MF_TRACE_END;
			return;
		}

#if 0
		if (ap->mf_MainWindow.pNaviItem) {
			mf_navi_bottom_item_del(&ap->mf_MainWindow.pNaviItem);
		}
#endif

		if (ap->mf_Status.more == MORE_EDIT)
			ap->mf_Status.more = MORE_DEFAULT;
#ifdef	MYFILE_SPLIT_VIEW
		if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
			mf_callback_storage_cb(ap, label);
			MF_TRACE_END;
			return;
		}
#endif
		/*0. set the previous navi to be out of use */
		pNavi_s_inuse->naviFlagInUse = FALSE;
		/*1. get navi bar pointer by the label */
		pNaviBar = ap->mf_MainWindow.pNaviBar;
		pNavi_s_toshow = mf_navi_bar_get_navi_from_navilist(ap->mf_MainWindow.plistNaviBar, label);
		/*2. create navi bar if not created, otherwise get it by pNaviBar */
		if (pNavi_s_toshow == NULL) {
			/*2.0   get root path by storage label */
			char *rootpath = mf_fm_svc_wrapper_get_root_path_by_tab_label(label);
			if (rootpath == NULL) {
				MF_TRACE_END;
				return;
			}
			/*2.1   create the navi for the tab */
			pNavi_s_toshow = malloc(sizeof(myfileNaviBar));
			if (pNavi_s_toshow == NULL) {
				free(rootpath);
				rootpath = NULL;
				mf_debug("pNavi_s_toshow is NULL");
				MF_TRACE_END;
				return;
			}
			memset(pNavi_s_toshow, 0, sizeof(myfileNaviBar));

			pNavi_s_toshow->naviFlagInUse = TRUE;
			/*2.2   set path as root path of the storage */
			pNavi_s_toshow->pCurrentPath = g_strdup(rootpath);
			pNavi_s_toshow->pNaviLabel = strdup(label);
			free(rootpath);
			/*2.3   insert phone navi into the navi_list */
			ap->mf_MainWindow.plistNaviBar = eina_list_append(ap->mf_MainWindow.plistNaviBar, pNavi_s_toshow);

			if (ap->mf_Status.path != NULL) {
				g_string_free(ap->mf_Status.path, TRUE);
				ap->mf_Status.path = NULL;
			}
			ap->mf_Status.path = g_string_new(pNavi_s_toshow->pCurrentPath);
			if (ap->mf_Status.more == MORE_INTERNAL_COPY || ap->mf_Status.more == MORE_INTERNAL_MOVE
			    || ap->mf_Status.more == MORE_INTERNAL_COPY_MOVE) {
				pNavi_s_toshow->pre_operation.path = g_string_new(pNavi_s_toshow->pCurrentPath);
				mf_navi_bar_create_path_select_view(ap);
				mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);
				pNavi_s_toshow->naviFlagEditJoin = TRUE;
			} else {
				mf_navi_bar_create_default_view(ap);
				mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);
			}
			MF_TRACE_END;


		} else {
			/*2.1   get the navi of the tab */
			pNavi_s_toshow = mf_navi_bar_get_struct_by_label(ap, label);
			pNavi_s_toshow->naviFlagInUse = TRUE;
			char *rootpath = mf_fm_svc_wrapper_get_root_path_by_tab_label(label);

			/*2.2   set related status value. */
			SAFE_FREE_GSTRING(ap->mf_Status.path);
			SAFE_FREE_CHAR(pNavi_s_toshow->pCurrentPath);
			ap->mf_Status.path = g_string_new(rootpath);
			pNavi_s_toshow->pCurrentPath = rootpath;
			/*2.3   update the content to catch update */
			/*Todo: How to ensure insert only once */
			if ((ap->mf_Status.more == MORE_INTERNAL_COPY || ap->mf_Status.more == MORE_INTERNAL_MOVE
			     || ap->mf_Status.more == MORE_INTERNAL_COPY_MOVE)) {
				mf_navi_bar_create_path_select_view(ap);
				pNavi_s_toshow->naviFlagEditJoin = TRUE;
			} else {
				mf_navi_bar_create_default_view(ap);
			}
			mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);
			MF_TRACE_END;
		}

	}

}

/******************************
** Prototype    : click_callback
** Description  :
** Input        : struct appdata* data
**                mfAction key
**                GString* path
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_upper_folder_cb(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	if (ap->mf_Status.flagNaviPush) {
		mf_debug("the transition of the naviframe is not finished");
		return;
	}

	GString *parent_path = NULL;
	int error_code = 0;
	int mmc_card = 0;
	error_code = mf_util_is_mmc_on(&mmc_card);

	parent_path = mf_fm_svc_wrapper_get_file_parent_path(ap->mf_Status.path);

	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "get navi structure failed");
	if (ap->mf_Status.path != NULL) {
		g_string_free(ap->mf_Status.path, TRUE);
		ap->mf_Status.path = NULL;
	}
	ap->mf_Status.path = parent_path;

	if (pNavi_s->pCurrentPath != NULL) {
		free(pNavi_s->pCurrentPath);
		pNavi_s->pCurrentPath = NULL;
	}
	pNavi_s->pCurrentPath = g_strdup(parent_path->str);

	if (ap->mf_Status.more == MORE_DEFAULT) {
		mf_navi_bar_create_default_view(ap);
	}
	mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);
}

void mf_callback_upper_button_pressed_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	Evas_Object *upper_button_ic = (Evas_Object *)data;
	elm_image_file_set(upper_button_ic, MF_TITLE_ICON_UPPER_PRESS, NULL);
}

void mf_callback_upper_button_unpressed_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	Evas_Object *upper_button_ic = (Evas_Object *)data;
	elm_image_file_set(upper_button_ic, MF_TITLE_ICON_UPPER, NULL);
}

void mf_callback_upper_click_cb(void *data, Evas_Object * obj, void *event_info)
{
	assert(data);
	struct appdata *ap = data;


	if (ap->mf_Status.flagNaviPush) {
		mf_debug("the transition of the naviframe is not finished");
		return;
	}

	if (ap->mf_Status.more == MORE_CREATE_FOLDER) {
		return;
	}

	if (ap->mf_Status.view_type == mf_view_root) {
		return;
	}
	mf_callback_remove_dir_watch();

	if (ap->mf_Status.more == MORE_RENAME) {
		mf_callback_destory_rename(ap);
	}

	if (ap->mf_MainWindow.pNaviItem) {
		mf_navi_bottom_item_del(&ap->mf_MainWindow.pNaviItem);
	}

	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "get navi structure failed");
	GString *parent_path = NULL;

	if (mf_fm_svc_wrapper_is_root_path(ap->mf_Status.path->str)) {
		ap->mf_Status.view_type = mf_view_root;
#ifdef MYFILE_SPLIT_VIEW
		if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
			mf_navi_bar_set_panes_navi_to_main(ap);
		}
#endif

	} else {
		parent_path = mf_fm_svc_wrapper_get_file_parent_path(ap->mf_Status.path);

		g_string_free(ap->mf_Status.path, TRUE);
		ap->mf_Status.path = NULL;
		ap->mf_Status.path = parent_path;
#ifdef MYFILE_USB_OTG
		/* if click items under otg root path, it means some disk take into use*/
		int storage = mf_fm_svc_wrapper_is_root_path(ap->mf_Status.path->str);
		if (storage == MYFILE_OTG) {	/* a usb disk take into use */
			char *dev_in_use = mf_otg_get_node_in_use(ap);
			if (dev_in_use != NULL)
				mf_otg_set_node_using_flag(ap, dev_in_use, FALSE);
		} else {
		}
#endif
		if (pNavi_s->pCurrentPath != NULL) {
			free(pNavi_s->pCurrentPath);
			pNavi_s->pCurrentPath = NULL;
		}
		pNavi_s->pCurrentPath = g_strdup(parent_path->str);

	}


	mf_widget_refresh_view(ap);


}

#ifdef MYFILE_SPLIT_VIEW
void mf_callback_left_click_cb(struct appdata *data, GString * path, Elm_Object_Item *item)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	mf_retm_if(path == NULL, "path is NULL");
	mf_retm_if(path->str == NULL, "path->str is NULL");

	struct appdata *ap = (struct appdata *)data;

	if (ap->mf_Status.path == NULL || ap->mf_Status.path->str == NULL) {
		return;
	}
	if (ap->mf_Status.flagNaviPush) {
		mf_debug("the transition of the naviframe is not finished");
		return;
	}

	Evas_Object *pGenlist = NULL;

	int locate = mf_fm_svc_wrapper_get_location(ap->mf_Status.path->str);
	int click_locate = MYFILE_PHONE;

	pGenlist = ap->mf_MainWindow.sSplitData.pSplitLeftGenlist;

	click_locate = mf_fm_svc_wrapper_get_location(path->str);

	if (locate == click_locate) {

		int error_code = 0;
		int mmc_card = 0;
		bool result = FALSE;

		myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
		mf_retm_if(pNavi_s == NULL, "get navi structure failed");
		error_code = mf_util_is_mmc_on(&mmc_card);
		if (ap->mf_Status.more == MORE_DEFAULT || ap->mf_Status.more == MORE_INTERNAL_COPY || ap->mf_Status.more == MORE_INTERNAL_MOVE
		    || ap->mf_Status.more == MORE_INTERNAL_COPY_MOVE || ap->mf_Status.more == MORE_SEARCH) {
			SAFE_FREE_GSTRING(ap->mf_Status.path);
			ap->mf_Status.path = g_string_new(path->str);
			SAFE_FREE_CHAR(pNavi_s->pCurrentPath);
			pNavi_s->pCurrentPath = g_strdup(path->str);
#ifdef MYFILE_USB_OTG
			/* if click items under otg root path, it means some disk take into use*/
			error_code = 0;
			error_code = mf_file_attr_is_disk_link(path->str, &result);
			if (error_code == 0 && result == TRUE) {	/* a usb disk take into use */
				/*
				**we don't need to set the previous device flag to be false
				**because click cannot jump from one device to another
				*/
				mf_otg_set_node_using_flag(ap, path->str,TRUE);
			}
#endif
			/*set new path */
			if (ap->mf_Status.more == MORE_DEFAULT) {
				mf_navi_bar_create_default_view(ap);
				mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);
			} else if (ap->mf_Status.more == MORE_INTERNAL_COPY_MOVE || ap->mf_Status.more == MORE_INTERNAL_COPY || ap->mf_Status.more == MORE_INTERNAL_MOVE
				   || ap->mf_Status.more == MORE_DATA_COPYING || ap->mf_Status.more == MORE_DATA_MOVING) {
				mf_navi_bar_create_path_select_view(ap);
				mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);
			}
		}

	} else {

		if (g_strcmp0(path->str, PHONE_FOLDER) == 0) {
			mf_callback_storage_cb(ap, GET_SYS_STR(MF_LABEL_PHONE));
		} else if (g_strcmp0(path->str, MEMORY_FOLDER) == 0) {
			mf_callback_storage_cb(ap, GET_SYS_STR(MF_LABEL_MMC));
		} else if (g_strcmp0(path->str, OTG_FOLDER) == 0) {
			mf_callback_storage_cb(ap, GET_STR(MF_LABEL_OTG));
		}
	}

	MF_TRACE_END;
}
#endif
void mf_callback_click_cb(struct appdata *data, mfAction key, GString * path)
{
	mf_debug("Start");
	assert(data);
	mf_debug("key:%d path:%s\n", key, path->str);

	struct appdata *ap = data;
	if (ap->mf_Status.path == NULL || ap->mf_Status.path->str == NULL) {
		return;
	}

	if (ap->mf_Status.flagNaviPush) {
		mf_debug("the transition of the naviframe is not finished");
		return;
	}
	if (ap->mf_Status.view_type == mf_view_root) {
		ap->mf_Status.view_type = mf_view_normal;
#ifdef MYFILE_SPLIT_VIEW
		if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
			mf_navi_bar_reset_panes_main(ap);
		}
#endif
		mf_callback_tab_cb(ap, path->str);
		return;
	}
	GString *new_path = NULL;

	int error_code = 0;
	int mmc_card = 0;
	bool result = FALSE;

	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "get navi structure failed");

	error_code = mf_util_is_mmc_on(&mmc_card);
	if (ap->mf_Status.more == MORE_DEFAULT || ap->mf_Status.more == MORE_INTERNAL_COPY || ap->mf_Status.more == MORE_INTERNAL_MOVE
	    || ap->mf_Status.more == MORE_INTERNAL_COPY_MOVE || ap->mf_Status.more == MORE_SEARCH) {
		if (key == MFACTION_CLICK) {
			if (mf_fm_svc_wrapper_is_dir(path)) {
				if (ap->mf_Status.more == MORE_SEARCH) {
					if (ap->mf_FileOperation.search_result_list) {
						mf_util_free_eina_list_with_data(&(ap->mf_FileOperation.search_result_list), MYFILE_TYPE_CHAR);
					}
					elm_object_focus_set(ap->mf_MainWindow.pSearchEntry, EINA_FALSE);
					ap->mf_Status.more = MORE_DEFAULT;
#ifdef MYFILE_SPLIT_VIEW
					if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
						mf_navi_bar_reset_panes_main(ap);
					}
#endif

				}
				new_path = g_string_new(path->str);

#ifdef MYFILE_USB_OTG
				/* if click items under otg root path, it means some disk take into use*/
				error_code = 0;
				error_code = mf_file_attr_is_disk_link(new_path->str, &result);
				if (error_code == 0 && result == TRUE) {	/* a usb disk take into use */
					/*
					**we don't need to set the previous device flag to be false
					**because click cannot jump from one device to another
					*/
					mf_otg_set_node_using_flag(ap, new_path->str,TRUE);
				}
#endif
				/*set new path */
				if (ap->mf_Status.path != NULL) {
					g_string_free(ap->mf_Status.path, TRUE);
					ap->mf_Status.path = NULL;
				}
				ap->mf_Status.path = new_path;

				if (pNavi_s->pCurrentPath != NULL) {
					free(pNavi_s->pCurrentPath);
					pNavi_s->pCurrentPath = NULL;
				}
				pNavi_s->pCurrentPath = g_strdup(new_path->str);
				mf_widget_refresh_view(ap);
			} else {
				struct timeval mytime;
				gettimeofday(&mytime, NULL);
				mf_debug("[myfiles click time] %ld sec %ld usec \n", mytime.tv_sec, mytime.tv_usec);

				int ret = 0;
				ret = mf_launch_service(ap, path->str);
				mf_debug("ret is %d\n", ret);
			}
		}

		else if (key == MFACTION_FLICK) {
			mf_debug("flick\n");
		}
	}
}

/******************************
** Prototype    : __mf_callback_imk_cancel_cb
** Description  :
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static int __mf_callback_imk_cancel_cb(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	mf_callback_cancel_cb(ap, NULL, NULL);
	return 0;
}

/******************************
** Prototype    : __mf_callback_illegal_char_popup_cb
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static void __mf_callback_illegal_char_popup_cb(void *data, Evas_Object * obj, void *event_info)
{
	struct appdata *ap = (struct appdata *)data;
	assert(ap);
	Evas_Object *btn = (Evas_Object *)obj;
	const char *label = elm_object_text_get(btn);

	if (g_strcmp0(label, GET_SYS_STR(MF_BUTTON_LABEL_YES)) == 0 ) {
		evas_object_del(ap->mf_MainWindow.pNormalPopup);
		ap->mf_MainWindow.pNormalPopup = NULL;

		Evas_Object *entry = ap->mf_MainWindow.pEntry;
		if (entry != NULL)
			elm_object_focus_set(entry, EINA_TRUE);
	} else if (g_strcmp0(label, GET_SYS_STR(MF_BUTTON_LABEL_NO)) == 0 ) {
		evas_object_del(ap->mf_MainWindow.pNormalPopup);
		ap->mf_MainWindow.pNormalPopup = NULL;
		ecore_idler_add((Ecore_Task_Cb) __mf_callback_imk_cancel_cb, ap);
	}

}

/******************************
** Prototype    : __mf_ime_mkdir_cb
** Description  :
** Input        : void *data
**                char* fileName
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static int __mf_callback_ime_mkdir_cb(void *data, char *fileName)
{
	mf_debug();
	struct appdata *ap = (struct appdata *)data;
	assert(ap);
	assert(fileName);

	int ret = 0;
	const char *message = NULL;
	const char *left_label = NULL;
	const char *right_label = NULL;

	left_label = GET_SYS_STR(MF_BUTTON_LABEL_YES);
	right_label = GET_SYS_STR(MF_BUTTON_LABEL_NO);
	if (strlen(fileName)) {

		if (ap->mf_Status.path == NULL || ap->mf_Status.path->str == NULL) {
			mf_debug("ap->mf_Status.path is NULL\n");
			return MYFILE_ERR_ALLOCATE_MEMORY_FAIL;
		}
		GString *fullpathdir = g_string_new(ap->mf_Status.path->str);

		if (fullpathdir == NULL) {
			mf_util_operation_alloc_failed(ap);
			return MYFILE_ERR_ALLOCATE_MEMORY_FAIL;
		}
		/*check the space */
		gchar *test_space = g_strstrip(fileName);
		if (test_space == NULL) {
			g_string_free(fullpathdir, TRUE);
			fullpathdir = NULL;
			mf_util_operation_alloc_failed(ap);
			return MYFILE_ERR_ALLOCATE_MEMORY_FAIL;
		}
		mf_debug("====================================fileName is [%s] test_space is [%s]", fileName, test_space);

		GString *fullpathname = g_string_new(test_space);

		if (fullpathname == NULL) {
			g_string_free(fullpathdir, TRUE);
			fullpathdir = NULL;
			mf_util_operation_alloc_failed(ap);
			return MYFILE_ERR_ALLOCATE_MEMORY_FAIL;
		}

		if (strlen(test_space) == 0) {
			message = GET_STR(MF_POPUP_MSG_NAME_INVALID);	/*TODO */
			ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT_TWO_BTN, NULL, message,
									       left_label, right_label, NULL,
									       __mf_callback_illegal_char_popup_cb, ap);

			g_string_free(fullpathdir, TRUE);
			fullpathdir = NULL;
			g_string_free(fullpathname, TRUE);
			fullpathname = NULL;
			return MYFILE_ERR_DIR_CREATE_FAIL;
		}

		/*check if input name is valid */
		if (mf_file_attr_is_valid_name(fullpathname->str) == MYFILE_ERR_INVALID_FILE_NAME) {

			message = GET_STR(MF_MSG_ILLEGAL_CHAR);
			ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT_TWO_BTN, NULL, message,
									       left_label, right_label, NULL,
									       __mf_callback_illegal_char_popup_cb, ap);
			mf_debug("Name contains illegal character!!!");
			g_string_free(fullpathdir, TRUE);
			fullpathdir = NULL;
			g_string_free(fullpathname, TRUE);
			fullpathname = NULL;

			return MYFILE_ERR_DIR_CREATE_FAIL;
		}

		GString *fullpath = NULL;
		char *temp_fullpath = g_strconcat(fullpathdir->str, "/", fullpathname->str, NULL);
		if (temp_fullpath) {
			fullpath = g_string_new(temp_fullpath);

			free(temp_fullpath);
			temp_fullpath = NULL;
			if (fullpath == NULL) {
				g_string_free(fullpathdir, TRUE);
				fullpathdir = NULL;
				g_string_free(fullpathname, TRUE);
				fullpathname = NULL;
				mf_util_operation_alloc_failed(ap);
				return MYFILE_ERR_ALLOCATE_MEMORY_FAIL;
			}
		} else {
			g_string_free(fullpathdir, TRUE);
			fullpathdir = NULL;
			g_string_free(fullpathname, TRUE);
			fullpathname = NULL;
			return MYFILE_ERR_ALLOCATE_MEMORY_FAIL;
		}

		mf_debug("length is %d\n", strlen(fileName));
		mf_debug("fullpath is %s\n", fullpath->str);
		/*check whether DIR name is override(DIR name has no extention) */
		/*check whether path length is override */
		if ((strlen(fullpathdir->str) + strlen(fullpathname->str)) > MYFILE_FILE_PATH_LEN_MAX) {

			message = GET_STR(MF_MSG_PATH_REACH_MAX_LEN);
			ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT_TWO_BTN, NULL, message,
									       left_label, right_label, NULL,
									       __mf_callback_illegal_char_popup_cb, ap);
			mf_debug("Full path length should be  less than 4096 Retry!!!");
			g_string_free(fullpathdir, TRUE);
			fullpathdir = NULL;
			g_string_free(fullpathname, TRUE);
			fullpathname = NULL;
			ret = MYFILE_ERR_DIR_CREATE_FAIL;
		}
		/*check if duplicated name */
		else if (mf_file_attr_is_duplicated_name(ap->mf_Status.path->str, test_space) == MYFILE_ERR_DUPLICATED_NAME) {
			message = GET_STR(MF_MSG_DUP_NAME);
			ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT_TWO_BTN, NULL, message,
									       left_label, right_label, NULL,
									       __mf_callback_illegal_char_popup_cb, ap);

			mf_debug("Duplicated Name, Retry?");
			g_string_free(fullpathdir, TRUE);
			fullpathdir = NULL;
			g_string_free(fullpathname, TRUE);
			fullpathname = NULL;
			ret = MYFILE_ERR_DIR_CREATE_FAIL;
		}
		/*check if DIR name is all spaces */
		else {
			ret = mf_fm_svc_wrapper_create_service(ap, fullpath);
			/*check whether operate on read only area */
			if (errno == EROFS) {
				message = GET_STR(MF_MSG_OPER_READ_ONLY);
				ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT, NULL, message, NULL, NULL, NULL, NULL, NULL);
				ret = MYFILE_ERR_DIR_CREATE_FAIL;
			} else if (ret) {
				message = GET_STR(MF_MSG_CREATE_DIR_FAILED);
				ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT, NULL, message, NULL, NULL, NULL, NULL, NULL);
				mf_debug("create folder error	%d	\n", ret);
			}

			g_string_free(fullpathdir, TRUE);
			fullpathdir = NULL;
			g_string_free(fullpathname, TRUE);
			fullpathname = NULL;
			g_string_free(fullpath, TRUE);
			fullpath = NULL;

		}
	} else {
		message = GET_STR(MF_MSG_EMPTY_FOLDER_NAME);

		ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT_TWO_BTN, NULL, message,
								       left_label, right_label, NULL, __mf_callback_illegal_char_popup_cb, ap);
		ret = MYFILE_ERR_DIR_CREATE_FAIL;
	}
	return ret;
}

/******************************
** Prototype    : mf_callback_cancel_cb
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_cancel_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");
	mf_retm_if(ap->mf_Status.path == NULL, "ap->mf_Status.path is NULL");
	mf_retm_if(ap->mf_Status.path->str == NULL, "ap->mf_Status.path->str is NULL");

	if (ap->mf_Status.flagNaviPush) {
		mf_debug("the transition of the naviframe is not finished");
		return;
	}

	Evas_Object *newContent;

	Eina_List *l = NULL;
	void *item = NULL;
	myfileNaviBar *pNavi_s = NULL;
	myfileNaviBar *pNavi_toshow = NULL;
	int flagTabbar = 0;
	switch (ap->mf_Status.more) {
	case MORE_INTERNAL_COPY:
	case MORE_INTERNAL_MOVE:
	case MORE_INTERNAL_COPY_MOVE:
		mf_debug();
		if (ap->mf_FileOperation.source != NULL) {
			g_string_free(ap->mf_FileOperation.source, TRUE);
			ap->mf_FileOperation.source = NULL;
		}
		/*1.    state set */
		/*1.1   status set to default */
		ap->mf_Status.preMore = ap->mf_Status.more;
		ap->mf_Status.more = MORE_EDIT;
		flagTabbar = mf_fm_svc_wrapper_is_root_path(ap->mf_Status.path->str);
		/*1.2   free the path to set it to the one just when edit clicked later */
		g_string_free(ap->mf_Status.path, TRUE);
		ap->mf_Status.path = NULL;
		/*2.    navigation bar state recovery */
		EINA_LIST_FOREACH(ap->mf_MainWindow.plistNaviBar, l, item) {
			pNavi_s = (myfileNaviBar *) item;
			/*2.2   pop all the temprory views */
			/*2.3   recover the path just before when edit click */
			pNavi_s->pCurrentPath = strdup(pNavi_s->pre_operation.path->str);
			/*2.5   set navi related status */
			if (pNavi_s->naviFlagEditStart == TRUE) {
				/*2.5-1 set path in mf_status */
				ap->mf_Status.path = g_string_new(pNavi_s->pCurrentPath);
				/*2.5-2 set InUse flag */
				pNavi_s->naviFlagInUse = TRUE;
				/*2.5-3 set pNavi_toshow for refreshing */
				pNavi_toshow = pNavi_s;
			} else {
				pNavi_s->naviFlagInUse = FALSE;
			}
		}

		if (pNavi_toshow == NULL) {
			mf_debug("pNavi_toshow is NULL");
			return;
		}


		ap->mf_Status.more = MORE_DEFAULT;
		if (ap->mf_Status.view_type == mf_view_root) {
			ap->mf_Status.view_type = ap->mf_Status.preViewType;
#ifdef MYFILE_SPLIT_VIEW
			if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
				if (ap->mf_Status.preViewType != mf_view_root_category) {
					mf_navi_bar_reset_panes_main(ap);
				}
			}
#endif
		} else {
			ap->mf_Status.view_type = ap->mf_Status.preViewType;
#ifdef MYFILE_SPLIT_VIEW
			if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
				if (ap->mf_Status.preViewType == mf_view_root_category) {
					mf_navi_bar_set_panes_navi_to_main(ap);
				}
			}
#endif
		}

		mf_navi_bar_create_default_view(ap);
		mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);

		ap->mf_Status.more = MORE_EDIT;
		mf_widget_refresh_view(ap);

		ap->mf_Status.preMore = MORE_DEFAULT;
		return;
	case MORE_RENAME:
		mf_callback_destory_rename(ap);
		if (ap->mf_Status.preMore == MORE_SEARCH) {
			goto SEARCH_CANCEL;
		}
		break;
		/* when cancle rename, just destory the rename relative, and then the mode will change to Edit
		   then do what cancle edit do, so here not need "break" */
	case MORE_EDIT:
		/*1     pop edit view */
		if (ap->mf_MainWindow.pLabelLayout) {
			mf_widget_unset_part_content(ap->mf_MainWindow.pNaviLayout, "sel_contents");
			if (ap->mf_MainWindow.pLabelLayout)
				evas_object_del(ap->mf_MainWindow.pLabelLayout);

			ap->mf_MainWindow.pLabelLayout = NULL;
		}
		ap->mf_Status.flagFolderSelect = FALSE;
		ap->mf_Status.flagFileSelect = FALSE;
		pNavi_toshow = mf_navi_bar_get_in_use(ap);
		mf_retm_if(pNavi_toshow == NULL, "get navi in use failed");
		ap->mf_Status.more = MORE_DEFAULT;
			/*3.    refresh the content of the view */

		mf_navi_bar_recover_info_box(ap, pNavi_toshow);
		Evas_Object *btn = NULL;
		btn = elm_object_item_part_content_unset(ap->mf_MainWindow.pNaviItem, TITLE_LEFT_BTN);
		SAFE_FREE_OBJ(btn);

		mf_genlist_update_default_view_item_style(ap);
		newContent = mf_navi_bar_content_create(ap);
		mfNaviBarSetContent(ap, ap->mf_MainWindow.pNaviLayout, newContent);

		//mf_ctrl_bar_reset(ap);
		mf_navi_bar_reset_ctrl_button(ap);
		mf_navi_add_back_button(ap);
		/*4.    set tab enable */
		mf_navi_bar_title_set(ap);
		break;
	case MORE_SEARCH:
	     SEARCH_CANCEL:
		if (ap->mf_Status.more == MORE_SEARCH)
			ap->mf_MainWindow.pSearchViewSweepItem = NULL;
#ifdef MYFILE_SPLIT_VIEW
		if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
			/**** 1. unset naviframe from mainlayout **/
			if (ap->mf_Status.view_type != mf_view_root_category && ap->mf_Status.view_type != mf_view_root) {
				mf_navi_bar_reset_panes_main(ap);
			}
		}
#endif

		if (ap->mf_FileOperation.search_IME_hide_timer != NULL) {
			ecore_timer_del(ap->mf_FileOperation.search_IME_hide_timer);
			ap->mf_FileOperation.search_IME_hide_timer = NULL;
		}

		if (ap->mf_FileOperation.search_result_list) {
			mf_util_free_eina_list_with_data(&(ap->mf_FileOperation.search_result_list), MYFILE_TYPE_CHAR);
		}

		if (ap->mf_Status.search_handler > 0) {
			mf_search_stop(ap->mf_Status.search_handler);
			mf_search_finalize(&ap->mf_Status.search_handler);
		}
		ap->mf_Status.more = MORE_DEFAULT;
		elm_object_focus_set(ap->mf_MainWindow.pEntry, EINA_FALSE);
		evas_object_del(ap->mf_MainWindow.pEntry);
		ap->mf_MainWindow.pSearchEntry = NULL;

		mf_widget_refresh_view(ap);
		ap->mf_Status.preMore = MORE_DEFAULT;

		break;
	case MORE_CREATE_FOLDER:
#ifdef MYFILE_SPLIT_VIEW
		if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
			if (ap->mf_Status.view_type != mf_view_root_category && ap->mf_Status.view_type != mf_view_root) {
				mf_navi_bar_reset_panes_main(ap);
			}
		}
#endif
		ap->mf_Status.more = ap->mf_Status.preMore;
		elm_object_focus_set(ap->mf_MainWindow.pEntry, EINA_FALSE);
		evas_object_del(ap->mf_MainWindow.pEntry);
		ap->mf_MainWindow.pEntry = NULL;
		mf_widget_imf_hide_callback_del(ap->mf_MainWindow.pConformant);

		mf_widget_refresh_view(ap);


		break;
	case MORE_THUMBNAIL_RENAME:
#ifdef MYFILE_SPLIT_VIEW
		if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
			if (ap->mf_Status.view_type != mf_view_root_category && ap->mf_Status.view_type != mf_view_root) {
				mf_navi_bar_reset_panes_main(ap);
			}
		}
#endif
		ap->mf_Status.more = MORE_DEFAULT;//ap->mf_Status.preMore;
		ap->mf_Status.preMore = MORE_THUMBNAIL_RENAME;
		elm_object_focus_set(ap->mf_MainWindow.pEntry, EINA_FALSE);
		evas_object_del(ap->mf_MainWindow.pEntry);
		ap->mf_MainWindow.pEntry = NULL;
		mf_widget_imf_hide_callback_del(ap->mf_MainWindow.pConformant);

		mf_widget_refresh_view(ap);
		ap->mf_Status.preMore = MORE_DEFAULT;
		break;
	default:
		ap->mf_Status.more = MORE_DEFAULT;
		/*4.    set tab enable */
		break;
	}

	if (ap->mf_MainWindow.pLabel) {
		evas_object_del(ap->mf_MainWindow.pLabel);
		ap->mf_MainWindow.pLabel = NULL;
	}
}

/******************************
** Prototype    : __mf_popup_show_vk_cb
** Description  : Samsung
** Input        : void *data
**                Evas_Object * obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_popup_show_vk_cb(void *data, Evas_Object *obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	Evas_Object *pEntry = NULL;

	evas_object_del(ap->mf_MainWindow.pNormalPopup);

	ap->mf_MainWindow.pNormalPopup = NULL;

	pEntry = ap->mf_MainWindow.pEntry;
	elm_entry_cursor_end_set(ap->mf_MainWindow.pEntry);

	elm_object_focus_set(ap->mf_MainWindow.pEntry, EINA_TRUE);
	MF_TRACE_END;

}

static void __mf_callback_refresh_rename(void *data, GString *pre_name, GString *new_name)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	mf_retm_if(pre_name == NULL, "pre_name is NULL");
	mf_retm_if(pre_name->str == NULL, "pre_name->str is NULL");
	mf_retm_if(new_name == NULL, "new_name is NULL");
	mf_retm_if(new_name->str == NULL, "new_name->str is NULL");
	struct appdata *ap = (struct appdata *)data;

	mfItemData_s *item_data = NULL;

	item_data = elm_object_item_data_get(ap->mf_FileOperation.rename_item);
	if (item_data == NULL) {
		return;
	}

	if (g_string_equal(item_data->m_ItemName, pre_name)) {
		g_string_free(item_data->m_ItemName, TRUE);
		item_data->m_ItemName = g_string_new(new_name->str);
		elm_object_item_data_set(ap->mf_FileOperation.rename_item, item_data);
		elm_genlist_item_update(ap->mf_FileOperation.rename_item);
		if (ap->mf_Status.view_type == mf_view_root_category) {
			mf_util_update_item_from_list_by_name(&ap->mf_FileOperation.category_list, pre_name->str, new_name->str);
		}
	}
	mf_callback_destory_rename(ap);
}

void mf_callback_destory_rename(void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is null");

	if (ap->mf_Status.more != MORE_RENAME) {
		return;
	}

	myfileNaviBar *pNavi_toshow = NULL;
	pNavi_toshow = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_toshow == NULL, "get navi in use failed");

	ap->mf_Status.more = ap->mf_Status.preMore;
	elm_object_focus_set(ap->mf_MainWindow.pEntry, EINA_FALSE);
	if (ap->mf_FileOperation.file_name_suffix != NULL) {
		SAFE_FREE_CHAR(ap->mf_FileOperation.file_name_suffix);
	}
	if (ap->mf_FileOperation.to_rename) {
		SAFE_FREE_GSTRING(ap->mf_FileOperation.to_rename);
	}
	if (ap->mf_FileOperation.rename_item) {
		elm_genlist_item_flip_set(ap->mf_FileOperation.rename_item, EINA_FALSE);
		elm_genlist_item_select_mode_set(ap->mf_FileOperation.rename_item,  ELM_OBJECT_SELECT_MODE_DEFAULT);
		elm_genlist_item_update(ap->mf_FileOperation.rename_item);
		ap->mf_FileOperation.rename_item = NULL;
	}

	if (ap->mf_MainWindow.pEntry) {
		ap->mf_MainWindow.pEntry = NULL;
	}

	if (ap->mf_Status.preMore == MORE_DEFAULT) {
		mf_navi_bar_set_upper_item_disable(ap, FALSE);
		mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_DEFAULT_ALL, FALSE);
		//mf_ctrl_bar_item_set_disable(ap->mf_MainWindow.pNaviCtrlBar, CTRL_DISABLE_DEFAULT_ALL, FALSE);
	} else {
		mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_EDIT_ALL, FALSE);
		//mf_ctrl_bar_item_set_disable(ap->mf_MainWindow.pNaviCtrlBar, CTRL_DISABLE_EDIT_ALL, FALSE);

		if (ap->mf_Status.iCtrlDisableItem > 0) {
			mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, ap->mf_Status.iCtrlDisableItem, TRUE);
			//mf_ctrl_bar_item_set_disable(ap->mf_MainWindow.pNaviCtrlBar, ap->mf_Status.iCtrlDisableItem, TRUE);
		}
	}
	MF_TRACE_END;
}


/******************************
** Prototype    : mf_callback_rename_save_cb
** Description  :
** Input        : void *ad
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_rename_save_cb(void *ad, Evas_Object *obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)ad;
	mf_retm_if(ap == NULL, "ap is NULL");

	char *name = NULL;
	Evas_Object *pEntry = NULL;
	const char *entry_data = NULL;
	int ret = MYFILE_ERR_NONE;
	const char *message = NULL;
	GString *from = NULL;
	GString *to = NULL;
	char *pFullPath = NULL;
	char *pName = NULL;
	if (ap->mf_Status.more != MORE_RENAME && ap->mf_Status.more != MORE_THUMBNAIL_RENAME) {
		MF_TRACE_END;
		return;
	}
	if (ap->mf_Status.flagNaviPush) {
		mf_debug("the transition of the naviframe is not finished");
		MF_TRACE_END;
		return;
	}

	pEntry = ap->mf_MainWindow.pEntry;
	if (pEntry == NULL && ap->mf_Status.more == MORE_RENAME) {
		mf_callback_destory_rename(ap);
		MF_TRACE_END;
		return;
	}
	entry_data = elm_entry_entry_get(pEntry);
	if (entry_data) {
		name = elm_entry_markup_to_utf8(entry_data);
		if (name == NULL) {
			message = GET_STR(MF_RENAME_MSG_GET_NAME_FAILED);
			goto INVAILD_NAME_EXIT;
		}
		if (strlen(name) == 0 && ap->mf_Status.more == MORE_THUMBNAIL_RENAME)
		{
			SAFE_FREE_CHAR(name);
			message = GET_STR(MF_MSG_GET_NAME_FAILED);
			ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT_TWO_BTN, NULL, message,
									       GET_SYS_STR(MF_BUTTON_LABEL_YES), GET_SYS_STR(MF_BUTTON_LABEL_NO), NULL,
									       __mf_callback_illegal_char_popup_cb, ap);
			return;
		}
	} else {
		message = GET_STR(MF_RENAME_MSG_GET_NAME_FAILED);
		goto INVAILD_NAME_EXIT;
	}

	if (strlen(name)) {

		if (ap->mf_FileOperation.file_name_suffix != NULL && strlen(ap->mf_FileOperation.file_name_suffix) > 0) {
			pName = g_strconcat(name, ".", ap->mf_FileOperation.file_name_suffix, NULL);
			CHAR_CHECK_NULL_GOTO(pName, ALLOC_FAILED_EXIT);
		} else {
			pName = g_strdup(name);
			CHAR_CHECK_NULL_GOTO(pName, ALLOC_FAILED_EXIT);
		}

		GString *parent_path = mf_fm_svc_wrapper_get_file_parent_path(ap->mf_FileOperation.to_rename);
		GSTRING_CHECK_NULL_GOTO(parent_path, ALLOC_FAILED_EXIT);
		pFullPath = g_strconcat(parent_path->str, "/", pName, NULL);
		CHAR_CHECK_NULL_GOTO(pFullPath, ALLOC_FAILED_EXIT);
		SAFE_FREE_GSTRING(parent_path);

		switch (mf_util_is_valid_name_check(pName)) {

		case MF_INTERNAL_FILE_NAME_EMPTY:
		case MF_INTERNAL_FILE_NAME_IGNORE:
			message = GET_STR(MF_MSG_SET_NAME_DOT);
			goto INVAILD_NAME_EXIT;
		case MF_INTERNAL_FILE_NAME_CHUG:
			message = GET_STR(MF_MSG_SET_NAME_ALL_SPACE);
			goto INVAILD_NAME_EXIT;
		case MF_INTERNAL_FILE_NAME_MAX_LENGTH:
			message = GET_STR(MF_MSG_NAME_REACH_MAX_LEN);
			goto INVAILD_NAME_EXIT;
		case MF_INTERNAL_FILE_NAME_INVALID_CHAR:
			message = GET_STR(MF_MSG_ILLEGAL_CHAR);
			goto INVAILD_NAME_EXIT;
		case MF_INTERNAL_FILE_NAME_NULL:
			message = GET_STR(MF_MSG_NO_NAME_WARNING);
			goto INVAILD_NAME_EXIT;
		default:
			break;
		}


		if (strlen(pFullPath) > MYFILE_FILE_PATH_LEN_MAX) {
			message = GET_STR(MF_MSG_PATH_REACH_MAX_LEN);
			goto INVAILD_NAME_EXIT;
		}

		from = g_string_new(ap->mf_FileOperation.to_rename->str);
		GSTRING_CHECK_NULL_GOTO(from, ALLOC_FAILED_EXIT);
		to = g_string_new(pFullPath);
		GSTRING_CHECK_NULL_GOTO(from, ALLOC_FAILED_EXIT);

		if (!g_string_equal(from, to)) {
			ret = mf_fm_svc_wrapper_rename_service(ap, from, to);
			if (ret != MYFILE_ERR_NONE) {
				if (errno == EROFS) {
					message = GET_STR(MF_MSG_OPER_READ_ONLY);
					goto INVAILD_NAME_EXIT;
				} else if (ret == MYFILE_ERR_INVALID_DIR_NAME || ret == MYFILE_ERR_INVALID_FILE_NAME) {
					message = GET_STR(MF_MSG_ILLEGAL_CHAR);
					goto INVAILD_NAME_EXIT;
				} else if (ret == MYFILE_ERR_DUPLICATED_NAME) {
					message = GET_STR(MF_RENAME_MSG_DUP_NAME);
					goto INVAILD_NAME_EXIT;
				} else {
					message = GET_STR(MF_MSG_UNKNOW_REASON_RENAME_FAILED);
					goto INVAILD_NAME_EXIT;
				}
			} else {
				if (ap->mf_Status.more == MORE_RENAME)
					__mf_callback_refresh_rename(ap, from, to);
				else if (ap->mf_Status.more == MORE_THUMBNAIL_RENAME){
#ifdef MYFILE_SPLIT_VIEW
					if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
						mf_navi_bar_reset_panes_main(ap);
					}
#endif
					ap->mf_Status.more = MORE_DEFAULT;
					ap->mf_Status.preMore = MORE_THUMBNAIL_RENAME;
					elm_object_focus_set(ap->mf_MainWindow.pEntry, EINA_FALSE);
					evas_object_del(ap->mf_MainWindow.pEntry);
					ap->mf_MainWindow.pEntry = NULL;
					mf_widget_imf_hide_callback_del(ap->mf_MainWindow.pConformant);
					if (ap->mf_Status.view_type == mf_view_root_category) {
						mf_util_update_item_from_list_by_name(&ap->mf_FileOperation.category_list, from->str, to->str);
					}

					mf_widget_refresh_view(ap);
					ap->mf_Status.preMore = MORE_DEFAULT;
				}
				goto NORMAL_EXIT;
			}
		} else {

			if (ap->mf_Status.more == MORE_RENAME)
				mf_callback_destory_rename(ap);
			else if (ap->mf_Status.more == MORE_THUMBNAIL_RENAME){
				mf_callback_cancel_cb(ap, NULL, NULL);
			}
			goto NORMAL_EXIT;
		}
	}else {
		mf_callback_destory_rename(ap);
		goto NORMAL_EXIT;
	}
NORMAL_EXIT:
	SAFE_FREE_CHAR(name);
	SAFE_FREE_CHAR(pName);
	SAFE_FREE_CHAR(pFullPath);
	SAFE_FREE_GSTRING(from);
	SAFE_FREE_GSTRING(to);
	MF_TRACE_END;
	return;

INVAILD_NAME_EXIT:
	SAFE_FREE_CHAR(name);
	SAFE_FREE_CHAR(pName);
	SAFE_FREE_CHAR(pFullPath);
	SAFE_FREE_GSTRING(from);
	SAFE_FREE_GSTRING(to);

	if (ap->mf_Status.more == MORE_THUMBNAIL_RENAME)
	{
		ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT_TWO_BTN, NULL, message,
								       GET_SYS_STR(MF_BUTTON_LABEL_YES), GET_SYS_STR(MF_BUTTON_LABEL_NO), NULL,
								       __mf_callback_illegal_char_popup_cb, ap);
		return;
	} else {
		mf_callback_destory_rename(ap);
		ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT, NULL, message, NULL, NULL, NULL, mf_popup_show_vk_cb, ap);
	}

	MF_TRACE_END;
	return;

ALLOC_FAILED_EXIT:
	SAFE_FREE_CHAR(name);
	SAFE_FREE_CHAR(pName);
	SAFE_FREE_CHAR(pFullPath);
	SAFE_FREE_GSTRING(from);
	mf_util_operation_alloc_failed(ap);
	MF_TRACE_END;
	return;

}


/******************************
** Prototype    : mf_callback_save_cb
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_save_cb(void *data, Evas_Object *obj, void *event_info)
{
	mf_debug();
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	if (ap->mf_Status.flagNaviPush) {
		mf_debug("the transition of the naviframe is not finished");
		return;
	}

	int ret = 0;
	int noContentFlag = 0;
	const char *message = NULL;
	const char *left_label = NULL;
	const char *right_label = NULL;

	if (ap->mf_Status.more == MORE_RENAME || ap->mf_Status.more == MORE_THUMBNAIL_RENAME) {
		mf_callback_rename_save_cb(ap, NULL, NULL);
	} else if (ap->mf_Status.more == MORE_CREATE_FOLDER) {
		const char *entry_data = NULL;
		char *name = NULL;

		/*hide IMF*/
		if (ap->mf_MainWindow.pEntry != NULL) {
			Ecore_IMF_Context *imf_context = elm_entry_imf_context_get(ap->mf_MainWindow.pEntry);
			if (imf_context != NULL)
				ecore_imf_context_hide(imf_context);
		}

		entry_data = elm_entry_entry_get(ap->mf_MainWindow.pEntry);

		if (entry_data) {
			name = elm_entry_markup_to_utf8(entry_data);
		} else {
			message = GET_STR(MF_MSG_GET_NAME_FAILED);
			left_label = GET_SYS_STR(MF_BUTTON_LABEL_YES);
			right_label = GET_SYS_STR(MF_BUTTON_LABEL_NO);
			ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT_TWO_BTN, NULL, message,
									       left_label, right_label, NULL,
									       __mf_callback_illegal_char_popup_cb, ap);
			return;
		}

		if (name == NULL)
			return;

		ret = __mf_callback_ime_mkdir_cb(ap, name);
		if (ret == 0) {
#ifdef MYFILE_SPLIT_VIEW
			if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
				if (ap->mf_Status.view_type != mf_view_root_category && ap->mf_Status.view_type != mf_view_root) {
					mf_navi_bar_reset_panes_main(ap);
				}
			}
#endif
			ap->mf_Status.more = ap->mf_Status.preMore;
			elm_object_focus_set(ap->mf_MainWindow.pEntry, EINA_FALSE);
			evas_object_del(ap->mf_MainWindow.pEntry);
			ap->mf_MainWindow.pEntry = NULL;
			mf_widget_imf_hide_callback_del(ap->mf_MainWindow.pConformant);

			mf_widget_refresh_view(ap);

			if (ap->mf_Status.more == MORE_DEFAULT && noContentFlag) {
				mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_NOCONTENT_VIEW, false);
				//mf_ctrl_bar_item_set_disable(ap->mf_MainWindow.pNaviCtrlBar, CTRL_DISABLE_NOCONTENT_VIEW, false);
			}
		}

		if (name != NULL) {
			free(name);
			name = NULL;
		}
	}
}


void mf_callback_popup_deleted_cb(void *data, Evas_Object * obj, void *event_info)
{
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	if (ap->mf_MainWindow.pNormalPopup != NULL) {
		evas_object_del(ap->mf_MainWindow.pNormalPopup);
	}
	ap->mf_MainWindow.pNormalPopup = NULL;

}

/******************************
** Prototype    : mf_callback_launch_send_cb
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_launch_send_cb(void *data, Evas_Object *obj, void *event_info)
{
	mf_debug();
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	if (ap->mf_FileRecordList.selected_files && mf_util_get_eina_list_len(ap->mf_FileRecordList.selected_files) > 0) {
		switch (ap->mf_Status.more) {
		case MORE_SENDVIABLUETOOTH:
		case MORE_SENDVIAEMAIL:
		case MORE_SENDVIAMESSAGE:
			mf_share_launch_multi_file(data);
			break;
		default:
			break;
		}
	} else {
		const char *message = GET_SYS_STR(MF_LABEL_NOTHING_SELECTED);
		ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT, NULL, message, NULL, NULL, NULL, NULL, NULL);
	}
	ap->mf_Status.more = MORE_EDIT;
}

/******************************
** Prototype    : __mf_callback_idle_launch_send_cb
** Description  :
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static int __mf_callback_idle_launch_send_cb(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	mf_callback_launch_send_cb(ap, NULL, NULL);
	return 0;
}


/******************************
** Prototype    : mf_callback_send_view_cb
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_send_response_cb(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	if (ap->mf_MainWindow.pContextPopup != NULL) {
		evas_object_del(ap->mf_MainWindow.pContextPopup);
	}
	ap->mf_MainWindow.pContextPopup = NULL;

	ecore_idler_add((Ecore_Task_Cb) __mf_callback_idle_launch_send_cb, ap);
}


/******************************
** Prototype    : __mf_callback_refresh_list_by
** Description  :
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static int __mf_callback_refresh_list_by(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	assert(ap);
	Evas_Object *playout = ap->mf_MainWindow.pNaviLayout;
	mf_retvm_if(playout == NULL, 0, "get conformant failed");
	Evas_Object *newContent = NULL;
#ifdef MYFILE_SPLIT_VIEW
	if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
		mf_split_view_left_genlist_refresh(ap);
	}
#endif

	if (ap->mf_Status.flagNoContent != EINA_TRUE) {
		newContent = mf_navi_bar_content_create(ap);
		mfNaviBarSetContent(ap, playout, newContent);
	}
	ap->mf_Status.more = MORE_DEFAULT;
	return 0;
}

/******************************
** Prototype    : __mf_callback_list_by_response_cb
** Description  : Samsung
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_list_by_response_cb(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	if (ap->mf_MainWindow.pNormalPopup != NULL) {
		evas_object_del(ap->mf_MainWindow.pNormalPopup);
	}
	ap->mf_MainWindow.pNormalPopup = NULL;

	ecore_idler_add((Ecore_Task_Cb) __mf_callback_refresh_list_by, ap);

}

/******************************
** Prototype    : mf_callback_list_by_view_cb
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_list_by_view_cb(void *data, Evas_Object * obj, void *event_info)
{
	mf_debug();
	struct appdata *ap = (struct appdata *)data;
	assert(ap);
	if (ap->mf_Status.flagNaviPush) {
		mf_debug("the transition of the naviframe is not finished");
		return;
	}
	SAFE_FREE_OBJ(ap->mf_MainWindow.pContextPopup);
	ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TITLE_LIST_BTN, GET_STR(LABEL_SORT_BY),
							       NULL, NULL, NULL, NULL, mf_callback_popup_deleted_cb, ap);
}

/******************************
** Prototype    : mf_callback_edit_cb
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
#ifdef MYFILE_SPLIT_VIEW
static void __mf_callback_edit_init(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	struct appdata *ap = (struct appdata*)data;
	//ap->mf_MainWindow.pEditView.pBox = NULL;
	ap->mf_MainWindow.pEditView.pGenlist = NULL;
	ap->mf_MainWindow.pEditView.pGengrid = NULL;
	//ap->mf_MainWindow.pEditView.pSelectAllCheckBox = NULL;
	//ap->mf_MainWindow.pEditView.pSelectAllLayout = NULL;
	//ap->mf_MainWindow.pEditView.pPathInfo = NULL;
	//ap->mf_MainWindow.pEditView.pInfoBox = NULL;
	ap->mf_MainWindow.pEditView.bSelectAllChecked = FALSE;;
	ap->mf_MainWindow.pEditView.iTotalCount = 0;
	ap->mf_MainWindow.pEditView.iCheckedCount = 0;
	MF_TRACE_END;
}

#endif

static void __mf_callback_set_select_all_total_count(void *data)
{

	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	struct appdata *ap = (struct appdata*)data;
	int folder_len = 0;
	int file_len = 0;

	folder_len = eina_list_count(ap->mf_FileOperation.folder_list);
	file_len = eina_list_count(ap->mf_FileOperation.file_list);
	ap->mf_MainWindow.pEditView.iTotalCount = (folder_len + file_len);
}

static void __mf_callback_save_edit_view_status(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	struct appdata *ap = (struct appdata*)data;
	myfileNaviBar *pNavi_s = NULL;
	Eina_List *l = NULL;
	void *item = NULL;
	/*backup the edit point for every navigation bar to get recovery */
	EINA_LIST_FOREACH(ap->mf_MainWindow.plistNaviBar, l, item) {
		pNavi_s = (myfileNaviBar *) item;
		if(pNavi_s == NULL)
			continue;

		if (pNavi_s->pre_operation.path != NULL)
			g_string_free(pNavi_s->pre_operation.path, TRUE);
			pNavi_s->pre_operation.path = g_string_new(pNavi_s->pCurrentPath);

		if (pNavi_s->naviFlagInUse == TRUE) {
			pNavi_s->naviFlagEditStart = TRUE;
			pNavi_s->naviFlagEditJoin = TRUE;
		} else {
			pNavi_s->naviFlagEditStart = FALSE;
		}
	}
	ap->mf_Status.preViewType = ap->mf_Status.view_type;

	MF_TRACE_END;
}


void mf_callback_edit_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	assert(ap);
	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "get navigation bar in use failed");

	ap->mf_Status.more = MORE_EDIT;
	mf_callback_remove_dir_watch();
#ifdef MYFILE_SPLIT_VIEW
	__mf_callback_edit_init(ap);
#endif
	/*backup the edit point for every navigation bar to get recovery */
	__mf_callback_save_edit_view_status(ap);
#ifdef MYFILE_USB_OTG
	/* if start from OTG, set node editstart flag */
	if (mf_fm_svc_wrapper_get_location(ap->mf_Status.path->str)== MYFILE_OTG) {
		char *dev_in_use = mf_otg_get_node_in_use(ap);
		mf_otg_set_node_editstart_flag(ap, dev_in_use, TRUE);
	}
#endif

	mf_navi_bar_create_edit_view(ap);

	mf_navi_bar_edit_list_update(ap);

	__mf_callback_set_select_all_total_count(ap);
	ap->mf_MainWindow.pEditView.bSelectAllChecked = EINA_FALSE;
	//elm_check_state_pointer_set(ap->mf_MainWindow.pEditView.pSelectAllCheckBox, &ap->mf_MainWindow.pEditView.bSelectAllChecked);

	/*disable all the tab item if tab exists */
	MF_TRACE_END;
}


/******************************
** Prototype    : mf_callback_icu_update_cb
** Description  :
** Input        : void *data
** Output       : int
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2011/06/30
**    Author       : Samsung
**    Modification : Created function
**/
void mf_callback_icu_update_cb(void *data)
{
	ug_send_event(UG_EVENT_REGION_CHANGE);

	mf_debug("region changed");
	assert(data);
	struct appdata *ap = (struct appdata *)data;

	if (ap->mf_Status.flagIcuInit == FALSE)
		return;
	/* finalize the previous icu session */
	mf_util_icu_finalize(ap);
	/* start a new icu session*/
	mf_util_icu_init(ap);
	/* check if we should refresh the list */
	int iSortTypeValue = 0;
	mf_util_get_vconf_value(VCONF_TYPE_SORT_TYPE, &iSortTypeValue);
	if (iSortTypeValue != MYFILE_SORT_BY_DATE_R2O && iSortTypeValue != MYFILE_SORT_BY_DATE_O2R) {
		return ;
	}

	if (ap->mf_Status.more == MORE_INTERNAL_COPY_MOVE || ap->mf_Status.more == MORE_INTERNAL_COPY || ap->mf_Status.more == MORE_INTERNAL_MOVE)
		return ;

	myfileNaviBar *pNaviInUse = mf_navi_bar_get_in_use(ap);
	if (pNaviInUse == NULL || ap->mf_MainWindow.pNaviLayout == NULL)
		return ;

	/*get current genlist */
	Evas_Object *currgenlist = ap->mf_MainWindow.pNaviGenlist;
	if (currgenlist == NULL)
		return ;

	evas_object_del(currgenlist);
	/*create new genlist */
	Evas_Object *newgenlist = mf_navi_bar_content_create(ap);
	mfNaviBarSetContent(ap, ap->mf_MainWindow.pNaviLayout, newgenlist);

	return ;
}


/******************************
** Prototype    : mf_callback_new_folder_create_cb
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_new_folder_create_cb(void *data, Evas_Object * obj, void *event_info)
{
	mf_debug();
	struct appdata *ap = (struct appdata *)data;
	assert(ap);


	if (ap->mf_Status.flagNaviPush) {
		mf_debug("the transition of the naviframe is not finished");
		return;
	}
	SAFE_FREE_OBJ(ap->mf_MainWindow.pContextPopup);

	if (ap->mf_Status.more == MORE_CREATE_FOLDER)
		return;
	int ret = 0;
	if (ap->mf_Status.folder_count >= MAX_FOLDER_COUNT) {
		const char *message = GET_STR(MF_MSG_FOLDER_NUM_MAX);
		ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT, NULL, message, NULL, NULL, NULL, NULL, NULL);
		return;
	}
	ap->mf_Status.preMore = ap->mf_Status.more;
	ap->mf_Status.more = MORE_CREATE_FOLDER;

	ret = mf_util_check_disk_space(ap);
	if (ret == MYFILE_ERR_NO_FREE_SPACE) {
		ap->mf_Status.more = MORE_DEFAULT;
		return;
	}

	mf_navi_bar_create_new_folder_view(ap);
	mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);

}


void mf_callback_rename_create_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	mfItemData_s *params = (mfItemData_s *) data;
	struct appdata *ap = (struct appdata *)params->ap;

	SAFE_FREE_OBJ(ap->mf_MainWindow.pContextPopup);

	if (ap->mf_Status.more == MORE_THUMBNAIL_RENAME)
		return;
	mf_widget_unset_part_content(ap->mf_MainWindow.pNaviLayout, "sel_contents");

	SAFE_FREE_OBJ(ap->mf_MainWindow.pLabelLayout);
	int ret = 0;

	ap->mf_Status.preMore = ap->mf_Status.more;
	ap->mf_Status.more = MORE_THUMBNAIL_RENAME;

	ret = mf_util_check_disk_space(ap);

	mf_navi_bar_create_rename_view(params);
	mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);
	MF_TRACE_END;

}

void mf_callback_max_len_reached_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");
	mf_popup_indicator_popup(ap, mf_util_get_text(MF_LABEL_MAX_CHARACTER_REACHED));
	MF_TRACE_END;
}




/******************************
** Prototype    : mf_callback_init_operation_cancel
** Description  : Samsung
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_init_operation_cancel(void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	if (ap->mf_FileOperation.pCancel) {
		mf_cancel_free(ap->mf_FileOperation.pCancel);
		ap->mf_FileOperation.pCancel = NULL;
	}

	ap->mf_FileOperation.pCancel = mf_cancel_new();
	MF_TRACE_END;
}

/******************************
** Prototype    : __mf_callback_idle_refresh
** Description  : Samsung
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static int __mf_callback_idle_refresh(void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, 0, "ap is NULL");
	mf_util_refresh_screen(ap);

	MF_TRACE_END;
	return ECORE_CALLBACK_CANCEL;
}


/******************************
** Prototype    : __mf_callback_progress_bar_state_cb
** Description  : Samsung
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static void __mf_callback_progress_bar_state_cb(void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	mf_fo_msg *msg = ap->mf_FileOperation.pMessage;
	char lable[10] = { '\0', };
	double value = 0.0;
	char count_label[10] = { '\0' };
	int count = 0;
	int total_count = 0;
	Evas_Object *layout = ap->mf_MainWindow.pProgressLayout;
	mf_retm_if(layout == NULL, "layout is NULL");

	value = (double)msg->current_size / msg->total_size;
	elm_progressbar_value_set(ap->mf_FileOperation.progress_bar, value);

	snprintf(lable, sizeof(lable), "%d%s", (int)(value * 100), "%");

	edje_object_part_text_set(elm_layout_edje_get(layout), "elm.text.left", lable);


	total_count = ap->mf_FileOperation.iTotalCount;
	count = msg->current_index;

	if (count <= total_count) {
		snprintf(count_label, sizeof(count_label), "%d/%d", count, total_count);
	}
	edje_object_part_text_set(elm_layout_edje_get(layout), "elm.text.right", count_label);

	MF_TRACE_END;
}



/******************************
** Prototype    : mf_callback_progress_bar_cancel_cb
** Description  : Samsung
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_progress_bar_cancel_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	mf_cancel_do_cancel(ap->mf_FileOperation.pCancel);
	mf_msg_request_handled_send();

	MF_TRACE_END;
}


/******************************
** Prototype    : mfCopyCB
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_copy_move_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");
	mf_retm_if(event_info == NULL, "event_info is NULL");

	if (ap->mf_Status.flagNaviPush) {
		mf_debug("the transition of the naviframe is not finished");
		return;
	}


	Elm_Object_Item *item = (Elm_Object_Item *) event_info;
	const char *label = elm_object_item_text_get(item);

	printf("label is [%s]\n", label);
	SAFE_FREE_OBJ(ap->mf_MainWindow.pContextPopup);
	if (label) {
		ap->mf_Status.flagFolderSelect = FALSE;
		ap->mf_Status.flagFileSelect = FALSE;

		if (ap->mf_FileRecordList.value_saver != NULL) {
			mf_util_free_eina_list_with_data(&ap->mf_FileRecordList.value_saver, MYFILE_TYPE_GSTRING);
			ap->mf_FileRecordList.value_saver = NULL;
		}
		ap->mf_FileRecordList.value_saver = mf_util_get_all_selected(ap);

		if (mf_util_get_eina_list_len(ap->mf_FileRecordList.value_saver) < 1) {
			ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT_BTN, NULL, GET_SYS_STR(MF_LABEL_NOTHING_SELECTED),
									       GET_SYS_STR(MF_BUTTON_LABEL_OK), NULL, NULL, mf_callback_warning_popup_cb, ap);
			return;
		}

		if (ap->mf_FileOperation.source != NULL) {
			g_string_free(ap->mf_FileOperation.source, TRUE);
			ap->mf_FileOperation.source = NULL;
		}
		ap->mf_FileOperation.source = g_string_new(ap->mf_Status.path->str);
		if (g_strcmp0(label, GET_SYS_STR(LABEL_MOVE)) == 0) {
			ap->mf_Status.more = MORE_INTERNAL_MOVE;
		} else if (g_strcmp0(label, GET_SYS_STR(LABEL_COPY)) == 0) {
			ap->mf_Status.more = MORE_INTERNAL_COPY;
		} else {
			return;
		}

		if (ap->mf_MainWindow.pLabelLayout) {
			mf_widget_unset_part_content(ap->mf_MainWindow.pNaviLayout, "sel_contents");
			if (ap->mf_MainWindow.pLabelLayout)
				evas_object_del(ap->mf_MainWindow.pLabelLayout);
			ap->mf_MainWindow.pLabelLayout = NULL;
		}
		/*pop select view*/
		/*set Navigation Bar for Destination selection */
		ap->mf_Status.view_type = mf_view_root;
#ifdef MYFILE_SPLIT_VIEW
		if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
			mf_split_view_unset_right_content(ap);
			mf_widget_unset_main_layout_content(ap->mf_MainWindow.pMainLayout);
			mf_split_view_hide(ap);
			evas_object_show(ap->mf_MainWindow.pMainLayout);
			elm_object_part_content_set(ap->mf_MainWindow.pMainLayout, "elm.swallow.content", ap->mf_MainWindow.pNaviBar);
		}
#endif
		mf_widget_refresh_view(ap);
	}
	MF_TRACE_END;
}


/******************************
** Prototype    : mf_callback_paste_here_cb
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_paste_here_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	if (ap->mf_Status.flagNaviPush) {
		mf_debug("the transition of the naviframe is not finished");
		return;
	}

	int source_len = 0;
	int dest_len = 0;
	GList *pSourceList = NULL;
	int count = 0;
	gchar *pDestPath = NULL;

	mf_retm_if(ap->mf_FileOperation.source == NULL, "ap->mf_FileOperation.source is NULL");
	mf_retm_if(ap->mf_FileOperation.source->str == NULL, "ap->mf_FileOperation.source is NULL");

	mf_retm_if(ap->mf_Status.path == NULL, "ap->mf_Status.path is NULL");
	mf_retm_if(ap->mf_Status.path->str == NULL, "ap->mf_Status.path->str is NULL");

	ap->mf_Status.more = MORE_DATA_COPYING;
	ap->mf_FileOperation.destination = ap->mf_Status.path;

	if (ap->mf_FileOperation.source && ap->mf_FileOperation.source->str) {
		source_len = strlen(ap->mf_FileOperation.source->str);
	}

	if (ap->mf_FileOperation.destination && ap->mf_FileOperation.destination->str) {
		dest_len = strlen(ap->mf_FileOperation.destination->str);
	}

	if (source_len == 0 || dest_len == 0) {
		MF_TRACE_END;
		return;
	}
	/*glib api change */

	if (!mf_util_check_forbidden_operation(ap)) {
		return;
	}
	if (source_len == dest_len && strcmp(ap->mf_FileOperation.destination->str, ap->mf_FileOperation.source->str) == 0) {
		ap->mf_FileOperation.refresh_type = TRUE;
	} else {
		ap->mf_FileOperation.refresh_type = FALSE;
	}

	mf_util_merge_eina_list_to_glist(ap->mf_FileRecordList.value_saver, &pSourceList);

	if (pSourceList) {
		count = g_list_length(pSourceList);
	} else {
		MF_TRACE_END;
		return;
	}

	ap->mf_FileOperation.iTotalCount = count;
	pDestPath = g_strdup(ap->mf_Status.path->str);
	if (pDestPath == NULL) {
		g_list_free(pSourceList);
		pSourceList = NULL;
		MF_TRACE_END;
		return;
	}

	mf_callback_init_operation_cancel(ap);
	ap->mf_FileOperation.iRequestType = MF_REQ_NONE;

	ap->mf_FileOperation.pSourceList = pSourceList;

	if (mf_copy_copy_items(pSourceList, pDestPath, ap->mf_FileOperation.pCancel, TRUE, ap) == 0) {
		ap->mf_FileOperation.iOperationSuccessFlag = TRUE;
		mf_callback_remove_dir_watch();
		ap->mf_MainWindow.pProgressPopup = mf_popup_create_pb_popup(ap, GET_SYS_STR(MF_MSG_COPYING), count, mf_callback_progress_bar_cancel_cb, ap);
	} else {
		mf_util_exception_func(ap);
	}

	g_free(pDestPath);
	pDestPath = NULL;

	MF_TRACE_END;
}


/******************************
** Prototype    : mf_move_here_cb
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_move_here_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");
	if (ap->mf_Status.flagNaviPush) {
		mf_debug("the transition of the naviframe is not finished");
		return;
	}

	int source_len = 0;
	int dest_len = 0;
	GList *pSourceList = NULL;
	int count = 0;
	gchar *pDestPath = NULL;

	mf_retm_if(ap->mf_FileOperation.source == NULL, "ap->mf_FileOperation.source is NULL");
	mf_retm_if(ap->mf_FileOperation.source->str == NULL, "ap->mf_FileOperation.source is NULL");

	mf_retm_if(ap->mf_Status.path == NULL, "ap->mf_Status.path is NULL");
	mf_retm_if(ap->mf_Status.path->str == NULL, "ap->mf_Status.path->str is NULL");
	ap->mf_Status.more = MORE_DATA_MOVING;

	ap->mf_FileOperation.destination = ap->mf_Status.path;

	if (ap->mf_FileOperation.source && ap->mf_FileOperation.source->str) {
		source_len = strlen(ap->mf_FileOperation.source->str);
	}

	if (ap->mf_FileOperation.destination && ap->mf_FileOperation.destination->str) {
		dest_len = strlen(ap->mf_FileOperation.destination->str);
	}

	if (source_len == 0 || dest_len == 0) {
		MF_TRACE_END;
		return;
	}
	/*glib api change */
	if (!mf_util_check_forbidden_operation(ap)) {
		return;
	} else {
		ap->mf_FileOperation.refresh_type = FALSE;
	}

	mf_util_merge_eina_list_to_glist(ap->mf_FileRecordList.value_saver, &pSourceList);

	if (pSourceList) {
		count = g_list_length(pSourceList);
	} else {
		MF_TRACE_END;
		return;
	}

	ap->mf_FileOperation.iTotalCount = count;
	pDestPath = g_strdup(ap->mf_Status.path->str);
	if (pDestPath == NULL) {
		g_list_free(pSourceList);
		pSourceList = NULL;
		MF_TRACE_END;
		return;
	}

	mf_callback_init_operation_cancel(ap);
	ap->mf_FileOperation.iRequestType = MF_REQ_NONE;

	ap->mf_FileOperation.pSourceList = pSourceList;
	if (mf_move_move_items(pSourceList, pDestPath, ap->mf_FileOperation.pCancel, true, ap) == 0) {
		ap->mf_FileOperation.iOperationSuccessFlag = TRUE;
		mf_callback_remove_dir_watch();
		ap->mf_MainWindow.pProgressPopup = mf_popup_create_pb_popup(ap, GET_SYS_STR(MF_MSG_MOVING), count, mf_callback_progress_bar_cancel_cb, ap);
	} else {
		mf_util_exception_func(ap);
	}
	g_free(pDestPath);
	pDestPath = NULL;
}

/******************************
** Prototype    : __mf_callback_confirm_delete
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static void __mf_callback_confirm_delete(void *data, Evas_Object * obj, void *event_info)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	Evas_Object *btn = (Evas_Object *)obj;
	const char *label = elm_object_text_get(btn);

	if (g_strcmp0(label, GET_SYS_STR(LABEL_CANCEL)) == 0 ) {
		evas_object_del(ap->mf_MainWindow.pNormalPopup);
		ap->mf_MainWindow.pNormalPopup = NULL;
		return;
	} else if (g_strcmp0(label, GET_SYS_STR(LABEL_DELETE)) == 0) {
		if (ap->mf_MainWindow.pNormalPopup) {
			evas_object_del(ap->mf_MainWindow.pNormalPopup);
			ap->mf_MainWindow.pNormalPopup = NULL;
		}

		if (ap->mf_MainWindow.pLabelLayout) {
			mf_widget_unset_part_content(ap->mf_MainWindow.pNaviLayout, "sel_contents");
			if (ap->mf_MainWindow.pLabelLayout)
				evas_object_del(ap->mf_MainWindow.pLabelLayout);
			ap->mf_MainWindow.pLabelLayout = NULL;
		}

		ap->mf_FileOperation.refresh_type = TRUE;

		/*setting item check */
		Eina_List *selected_list = mf_util_get_all_selected(ap);

		if (ap->mf_FileRecordList.value_saver != NULL) {
			mf_util_free_eina_list_with_data(&ap->mf_FileRecordList.value_saver, MYFILE_TYPE_GSTRING);
			ap->mf_FileRecordList.value_saver = NULL;
		}
		ap->mf_FileRecordList.value_saver = selected_list;

		if (mf_util_get_eina_list_len(ap->mf_FileRecordList.value_saver) < 1) {
			ap->mf_MainWindow.pNormalPopup =
				mf_popup_create_popup(ap, POPMODE_TEXT_BTN, NULL, GET_SYS_STR(MF_LABEL_NOTHING_SELECTED),
						      GET_SYS_STR(MF_BUTTON_LABEL_OK), NULL, NULL, mf_callback_warning_popup_cb, ap);
		} else {
			GList *pSourceList = NULL;
			int count = 0;

			mf_util_merge_eina_list_to_glist(ap->mf_FileRecordList.value_saver, &pSourceList);
			if (pSourceList) {
				count = g_list_length(pSourceList);
			} else {
				MF_TRACE_END;
				return;
			}

			ap->mf_FileOperation.iTotalCount = count;
			mf_callback_init_operation_cancel(ap);
			ap->mf_Status.more = MORE_DELETE;
			ap->mf_FileOperation.iRequestType = MF_REQ_NONE;

			ap->mf_FileOperation.pSourceList = pSourceList;
			mf_callback_remove_dir_watch();

			if (mf_delete_items(pSourceList, ap->mf_FileOperation.pCancel, TRUE, ap) == 0) {
				ap->mf_FileOperation.iOperationSuccessFlag = TRUE;
				mf_callback_remove_dir_watch();
				ap->mf_MainWindow.pProgressPopup =
					mf_popup_create_pb_popup(ap, GET_SYS_STR(MF_MSG_DELETING), count, mf_callback_progress_bar_cancel_cb, ap);
			} else {
				mf_util_exception_func(ap);
			}
		}
	}
}

/******************************
** Prototype    : mf_callback_delete_cb
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_delete_cb(void *data, Evas_Object * obj, void *event_info)
{
	struct appdata *ap = (struct appdata *)data;
	assert(ap);
	if (ap->mf_Status.flagNaviPush) {
		mf_debug("the transition of the naviframe is not finished");
		return;
	}
	mf_debug("delete clicked\n");
	ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TITLE_TEXT_TWO_BTN,
							       NULL,
							       GET_SYS_STR(MF_BUTTON_LABEL_DEL),
							       GET_SYS_STR(LABEL_DELETE), GET_SYS_STR(LABEL_CANCEL), NULL, __mf_callback_confirm_delete, ap);
}


/******************************
** Prototype    : mf_callback_thread_pipe_cb
** Description  :
** Input        : void *data
**                void *buffer
**                unsigned int nbyte
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_operation_request_rename_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	mf_fo_request *request = ap->mf_FileOperation.pRequest;

	if (ap->mf_MainWindow.pProgressPopup) {
		evas_object_show(ap->mf_MainWindow.pProgressPopup);
	}

	ap->mf_FileOperation.iRequestType = MF_REQ_RENAME;
	mf_request_set_result(request, MF_REQ_RENAME);
	MF_TRACE_END;

}

void mf_callback_operation_request_replace_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	mf_fo_request *request = ap->mf_FileOperation.pRequest;

	if (ap->mf_MainWindow.pProgressPopup) {
		evas_object_show(ap->mf_MainWindow.pProgressPopup);
	}

	ap->mf_FileOperation.iRequestType = MF_REQ_MERGE;
	if (ap->mf_FileOperation.refresh_type == TRUE) {
		mf_request_set_result(request, MF_REQ_SKIP);
	} else {
		mf_request_set_result(request, MF_REQ_MERGE);
	}
	MF_TRACE_END;

}

void mf_callback_operation_request_cancel_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	mf_fo_request *request = ap->mf_FileOperation.pRequest;

	if (ap->mf_MainWindow.pProgressPopup) {
		evas_object_show(ap->mf_MainWindow.pProgressPopup);
	}

	mf_request_set_result(request, MF_REQ_CANCEL);
	MF_TRACE_END;

}


void mf_callback_thread_pipe_cb(void *data, void *buffer, unsigned int nbyte)
{
	MF_TRACE_BEGIN;
	mf_debug(":::::::::: Main thread id = %d ::::::::::", (int)pthread_self());
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	mf_fo_msg *pPipeMsg = (mf_fo_msg *) buffer;
	char *message = NULL;
	int error_type = 0;

	if (!MYFILE_MAGIC_CHECK(pPipeMsg, MYFILE_MAGIC_PIPE_DATA)) {
		mf_error(":::::::::::::p_pipe_data ERROR:::::::::::::");
		return;
	}
	ap->mf_FileOperation.pMessage = pPipeMsg;
	ap->mf_FileOperation.pRequest = pPipeMsg->request;
	ap->mf_FileOperation.sync_pipe = pPipeMsg->pipe;
	mf_debug("::::::::::: pPipeMsg->msg_type is [%d]", pPipeMsg->msg_type);

	mf_util_set_pm_lock(ap, EINA_TRUE);
	switch (pPipeMsg->msg_type) {
	case MF_MSG_NONE:
	case MF_MSG_SKIP:
	case MF_MSG_SYNC:
		mf_msg_request_handled_send();
		break;
	case MF_MSG_CANCELLED:
		if (ap->mf_MainWindow.pProgressPopup) {
			evas_object_del(ap->mf_MainWindow.pProgressPopup);
			ap->mf_MainWindow.pProgressPopup = NULL;
		}
		ap->mf_MainWindow.pFinishPopup = mf_popup_create_popup(ap, POPMODE_TEXT_NOT_DISABLED, NULL,
								       "Canceling...", NULL, NULL, NULL, NULL, NULL);
		mf_msg_request_handled_send();
		ap->mf_FileOperation.iOperationSuccessFlag = FALSE;
		break;
	case MF_MSG_DOING:
		if (ap->mf_Status.view_type == mf_view_root_category && (ap->mf_Status.more == MORE_DELETE || ap->mf_Status.more == MORE_IDLE_DELETE)) {
			mf_util_remove_item_from_list_by_name(&ap->mf_FileOperation.category_list, pPipeMsg->current_real);
		}
		__mf_callback_progress_bar_state_cb(ap);
		mf_msg_request_handled_send();
		break;
	case MF_MSG_REQUEST:
		if (ap->mf_FileOperation.iRequestType == MF_REQ_RENAME) {
			mf_request_set_result(pPipeMsg->request, MF_REQ_RENAME);

		} else if (ap->mf_FileOperation.iRequestType == MF_REQ_MERGE) {
			if (ap->mf_FileOperation.refresh_type == TRUE) {
				mf_request_set_result(pPipeMsg->request, MF_REQ_SKIP);
			} else {
				mf_request_set_result(pPipeMsg->request, MF_REQ_MERGE);
			}
		} else {
			ap->mf_FileOperation.iRequestType = MF_REQ_RENAME;
			mf_request_set_result(pPipeMsg->request, MF_REQ_RENAME);
		}
		break;
	case MF_MSG_ERROR:
		error_type = FO_ERROR_CHECK(pPipeMsg->error_code);
		switch (error_type) {
		case MF_FO_ERR_PERMISSION:
			message = g_strconcat("Operation failed<br>", GET_STR(MF_MSG_PERMISSION_ERR), NULL);
			break;
		case MF_FO_ERR_ARGUMENT:
			message = g_strconcat("Operation failed<br>", GET_STR(MF_MSG_ARG_ERR), NULL);
			break;
		case MF_FO_ERR_FAULT:
			message = g_strconcat("Operation failed<br>", GET_STR(MF_MSG_FAULT_ERR), NULL);
			break;
		case MF_FO_ERR_TYPE:
			message = g_strconcat("Operation failed<br>", GET_STR(MF_MSG_FILE_TYPE_ERR), NULL);
			break;
		case MF_FO_ERR_MAX_OPEN:
			message = g_strconcat("Operation failed<br>", GET_STR(MF_MSG_MAX_OPEN_ERR), NULL);
			break;
		case MF_FO_ERR_SPACE:
			message = g_strconcat("Operation failed<br>", GET_STR(MF_MSG_NO_SPACE), NULL);
			break;
		case MF_FO_ERR_RO:
			message = g_strconcat("Operation failed<br>", GET_STR(MF_MSG_RO_ERR), NULL);
			break;
		case MF_FO_ERR_LOOP:
			message = g_strconcat("Operation failed<br>", GET_STR(MF_MSG_LOOP_ERR), NULL);
			break;
		case MF_FO_ERR_MEM:
			message = g_strconcat("Operation failed<br>", GET_STR(MF_MSG_MEM_ERR), NULL);
			break;
		case MF_FO_ERR_NOT_EXIST:
			message = g_strconcat("Operation failed<br>", GET_SYS_STR(MF_LABEL_FILE_NOT_EXIST), NULL);
			break;
		case MF_FO_ERR_LONG_NAME:
			message = g_strconcat("Operation failed<br>", GET_STR(MF_MSG_LONG_NAME_ERR), NULL);
			break;
		case MF_FO_ERR_BIG_SIZE:
			message = g_strconcat("Operation failed<br>", GET_STR(MF_MSG_BIG_SIZE_ERR), NULL);
			break;
		case MF_FO_ERR_UNKNOWN:
			message = g_strconcat("Operation failed<br>", GET_STR(MF_MSG_UNKNOWN_ERR), NULL);
			break;
		case MF_FO_ERR_IO:
			message = g_strconcat("Operation failed<br>", GET_STR(MF_MSG_IO_ERR), NULL);
			break;
		default:
			message = g_strconcat("Operation failed<br>", GET_STR(MF_MSG_DEFAULT_ERR), NULL);
			break;
		}

		if (ap->mf_MainWindow.pProgressPopup) {
			evas_object_del(ap->mf_MainWindow.pProgressPopup);
			ap->mf_MainWindow.pProgressPopup = NULL;
		}

		if (ap->mf_MainWindow.pFinishPopup) {
			evas_object_del(ap->mf_MainWindow.pFinishPopup);
			ap->mf_MainWindow.pFinishPopup = NULL;
		}

		SAFE_FREE_CHAR(ap->mf_FileOperation.pOperationMsg);
		if (error_type ==  MF_FO_ERR_SPACE) {
			ap->mf_FileOperation.pOperationMsg = message;
		} else {
			if (ap->mf_Status.more == MORE_DATA_COPYING) {
				ap->mf_FileOperation.pOperationMsg = g_strdup(GET_STR(MF_MSG_COPY_FAILED));
			} else if (ap->mf_Status.more == MORE_DATA_MOVING) {
				ap->mf_FileOperation.pOperationMsg = g_strdup(GET_SYS_STR(MF_MSG_MOVE_FAILED));
			} else if (ap->mf_Status.more == MORE_DELETE || ap->mf_Status.more == MORE_IDLE_DELETE) {
				ap->mf_FileOperation.pOperationMsg = g_strdup(GET_SYS_STR(MF_LABEL_DELETE_FAILED));
			} else {
				ap->mf_FileOperation.pOperationMsg = message;
			}
		}


		ap->mf_FileOperation.pGenlistItem = NULL;
		g_free(message);
		ap->mf_FileOperation.iOperationSuccessFlag = FALSE;
		mf_msg_request_handled_send();
		break;
	case MF_MSG_END:

		if (ap->mf_FileOperation.source) {
			g_string_free(ap->mf_FileOperation.source, TRUE);
			ap->mf_FileOperation.source = NULL;
		}
		if (ap->mf_FileOperation.pSourceList) {
			g_list_free(ap->mf_FileOperation.pSourceList);
			ap->mf_FileOperation.pSourceList = NULL;
		}
		if (ap->mf_MainWindow.pProgressPopup) {
			evas_object_del(ap->mf_MainWindow.pProgressPopup);
			ap->mf_MainWindow.pProgressPopup = NULL;
		}
		ecore_idler_add((Ecore_Task_Cb)__mf_callback_idle_refresh, ap);
		if (pPipeMsg->pipe) {
			ecore_pipe_del(pPipeMsg->pipe);
			pPipeMsg->pipe = NULL;
		}
		mf_msg_request_handled_send();

		mf_util_set_pm_lock(ap, EINA_FALSE);
		if (ap->mf_FileRecordList.value_saver != NULL) {
			mf_util_free_eina_list_with_data(&ap->mf_FileRecordList.value_saver, MYFILE_TYPE_GSTRING);
			ap->mf_FileRecordList.value_saver = NULL;
		}
		break;
	default:
		mf_msg_request_handled_send();
		break;
	}

}


void mf_callback_exception_popup_cb(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	if (ap->mf_MainWindow.pNormalPopup) {
		evas_object_del(ap->mf_MainWindow.pNormalPopup);
		ap->mf_MainWindow.pNormalPopup = NULL;
	}

	mf_callback_cancel_cb(ap, NULL, NULL);
}

/******		Memory card connection/removal handler		******/
static void __mf_callback_mmc_connected(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "appdata is NULL");
	mf_retm_if(ap->mf_Status.path == NULL || ap->mf_Status.path->str == NULL, "mf_Status.path is NULL");

	Evas_Object *entry = NULL;
	ap->mf_Status.iStorageState |= MYFILE_MMC;

	if (ap->mf_Status.more == MORE_CREATE_FOLDER || ap->mf_Status.more == MORE_SEARCH || (ap->mf_Status.more == MORE_RENAME && ap->mf_Status.preMore == MORE_SEARCH))
		return;

	if (ap->mf_Status.view_type != mf_view_root) {
		return;
	}

	mf_util_action_storage_insert(ap, GET_SYS_STR(MF_LABEL_MMC));
	if (ap->mf_Status.more == MORE_RENAME) {
		entry = ap->mf_MainWindow.pEntry;
		if (entry != NULL) {
			elm_object_focus_set(entry, EINA_TRUE);
		}
	}
	mf_navi_bar_title_set(ap);

}

static void __mf_callback_mmc_remove_view_operation(void *data, int optStorage)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "appdata is NULL");

	Evas_Object *parent = NULL;
	const char *labelStorage = NULL;
	if (ap->mf_Status.flagViewType == MF_VIEW_STYLE_LIST || ap->mf_Status.flagViewType == MF_VIEW_SYTLE_LIST_DETAIL) {
		parent = ap->mf_MainWindow.pNaviGenlist;
	} else {
		parent = ap->mf_MainWindow.pNaviGengrid;
	}
	switch (optStorage) {
	case MYFILE_PHONE:
		labelStorage = PHONE_FOLDER;
		break;
	case MYFILE_MMC:
		labelStorage = MEMORY_FOLDER;
		break;
	case MYFILE_OTG:
		labelStorage = OTG_FOLDER;
		break;
	default:
		break;
	}
	mf_widget_item_remove(parent, labelStorage, ap->mf_Status.flagViewType);
#ifdef MYFILE_SPLIT_VIEW
	if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
		if (optStorage == MYFILE_MMC) {
			elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
			ap->mf_MainWindow.sSplitData.mmc_item = NULL;
		}
	}
#endif

}

static void __mf_callback_mmc_remove_category_view_items(void *data, int optStorage)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "appdata is NULL");

	Evas_Object *parent = NULL;
	if (ap->mf_Status.flagViewType == MF_VIEW_STYLE_LIST || ap->mf_Status.flagViewType == MF_VIEW_SYTLE_LIST_DETAIL) {
		parent = ap->mf_MainWindow.pNaviGenlist;
	} else {
		parent = ap->mf_MainWindow.pNaviGengrid;
	}
	mf_widget_items_remove(parent, optStorage, ap->mf_Status.flagViewType);
	mf_util_remove_item_from_list_by_location(&ap->mf_FileOperation.category_list, optStorage);
}

static void __mf_callback_mmc_removed(void *data, MF_STORAGE storage)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "appdata is NULL");
	mf_retm_if(ap->mf_Status.path == NULL || ap->mf_Status.path->str == NULL, "mf_Status.path is NULL");

	Evas_Object *entry = NULL;
	char *labelStorage = NULL;
	MF_STORAGE optStorage = MYFILE_NONE;
	myfileNaviBar *pNaviInUseStruct = NULL;
	myfileNaviBar *pNaviToShowStruct = NULL;
	Evas_Object *pNaviBarStorageView = NULL;
	bool bEditStartFlag = FALSE;

	optStorage = storage;
	switch (optStorage) {
	case MYFILE_PHONE:
		labelStorage = GET_SYS_STR(MF_LABEL_PHONE);
		break;
	case MYFILE_MMC:
		labelStorage = GET_SYS_STR(MF_LABEL_MMC);
		break;
	default:
		break;
	}
	/*1.2.1 check if mmc is editstar navi */
	pNaviBarStorageView = ap->mf_MainWindow.pNaviBar;

	mf_debug("before remove is [%d]", ap->mf_Status.iStorageState);
	ap->mf_Status.iStorageState = (ap->mf_Status.iStorageState ^ optStorage);
	mf_debug("after remove is [%d]", ap->mf_Status.iStorageState);

	/*3 factors; */
	/*
	 **     1.      status                          ap->mf_Status.more
	 **     2.      is storage in Use       ap->mf_Status.path
	 **     3.      is source path in storage in inter-storage operation    flagEditStartView
	 */

	if (ap->mf_Status.view_type == mf_view_root) {
		if(mf_fm_svc_wrapper_get_location(ap->mf_Status.path->str) != optStorage) {
			switch (ap->mf_Status.more) {
			case MORE_DEFAULT:
				mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
				__mf_callback_mmc_remove_view_operation(ap, optStorage);
				break;
			case MORE_INTERNAL_COPY_MOVE:
			case MORE_INTERNAL_COPY:
			case MORE_INTERNAL_MOVE:
				bEditStartFlag = mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
				if (bEditStartFlag == TRUE) {
					pNaviToShowStruct = mf_navi_bar_recover_list(ap);
					mf_navi_bar_refresh_recovered_view(ap, pNaviToShowStruct);
				} else {
					__mf_callback_mmc_remove_view_operation(ap, optStorage);
				}
				break;
			default:
				break;
			}
		}else {
			switch (ap->mf_Status.more) {
			case MORE_DEFAULT:
				ap->mf_Status.more = MORE_DEFAULT;
				mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
				__mf_callback_mmc_remove_view_operation(ap, optStorage);
				break;
			case MORE_INTERNAL_COPY_MOVE:
			case MORE_INTERNAL_COPY:
			case MORE_INTERNAL_MOVE:
				bEditStartFlag = mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
				if (bEditStartFlag == TRUE) {
					mf_navi_bar_recover_list(ap);
				}

				pNaviInUseStruct = mf_navi_bar_get_in_use(ap);
#ifdef MYFILE_SPLIT_VIEW

				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif

				mf_navi_bar_refresh_recovered_view(ap, pNaviInUseStruct);
				break;
			default:
				break;
			}
		}
	} else if (ap->mf_Status.view_type == mf_view_root_category) {
		switch (ap->mf_Status.more) {
		case MORE_DEFAULT:
		case MORE_EDIT:
		case MORE_RENAME:
			mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
			__mf_callback_mmc_remove_category_view_items(ap, optStorage);
			break;
		default:
			break;
		}
	} else if(mf_fm_svc_wrapper_get_location(ap->mf_Status.path->str) != optStorage) {
		switch (ap->mf_Status.more) {
		case MORE_DEFAULT:
		case MORE_EDIT:
		case MORE_IDLE_DELETE:
		case MORE_DELETE:
		case MORE_RENAME:
			/*      not a inter-storage operation */
			mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
			if (ap->mf_Status.iStorageState == MYFILE_PHONE) {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif

			} else {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif
			}

			if (ap->mf_Status.more == MORE_RENAME) {
				entry = ap->mf_MainWindow.pEntry;
				if (entry != NULL) {
					elm_object_focus_set(entry, EINA_TRUE);
				}
			}
			break;
		case MORE_SEARCH:
		case MORE_THUMBNAIL_RENAME:
			mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
			break;
		case MORE_CREATE_FOLDER:
			if (ap->mf_Status.preMore == MORE_DEFAULT) {
				mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
				break;
			} else if (ap->mf_Status.preMore == MORE_INTERNAL_COPY_MOVE || ap->mf_Status.preMore == MORE_INTERNAL_COPY
			           || ap->mf_Status.preMore == MORE_INTERNAL_MOVE) {
				bEditStartFlag = mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
				if (bEditStartFlag == TRUE) {
					ap->mf_Status.more = MORE_DEFAULT;
					pNaviToShowStruct = mf_navi_bar_recover_list(ap);
					mf_navi_bar_refresh_recovered_view(ap, pNaviToShowStruct);
				}
			}
			if (bEditStartFlag == TRUE) {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (ap->mf_Status.view_type != mf_view_root_category && ap->mf_Status.view_type != mf_view_root) {
						mf_navi_bar_reset_panes_main(ap);
					}
				}
#endif
			}
			break;
		case MORE_INTERNAL_COPY:
		case MORE_INTERNAL_MOVE:
		case MORE_INTERNAL_COPY_MOVE:
			bEditStartFlag = mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
			if (bEditStartFlag == TRUE) {
				pNaviToShowStruct = mf_navi_bar_recover_list(ap);
				mf_navi_bar_refresh_recovered_view(ap, pNaviToShowStruct);
			}

			if (ap->mf_Status.iStorageState == MYFILE_PHONE) {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif

			} else {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif
			}
			break;

		case MORE_DATA_COPYING:
		case MORE_DATA_MOVING:
			bEditStartFlag = mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
			if (bEditStartFlag == TRUE) {
				ap->mf_Status.flagStorageRemoved = EINA_TRUE;

				ap->mf_MainWindow.pMmcRemovedPopup = mf_popup_create_popup(ap, POPMODE_TEXT_NOT_DISABLED, NULL,
											   "Mmc removed...", NULL, NULL, NULL, NULL, NULL);
			} else {
				ap->mf_Status.flagStorageRemoved = EINA_FALSE;
			}

			if (ap->mf_Status.iStorageState == MYFILE_PHONE) {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif

			} else {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif
			}

			break;
		default:
			break;
		}
	} else {
		switch (ap->mf_Status.more) {
		case MORE_DEFAULT:
			ap->mf_Status.more = MORE_DEFAULT;
			mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
			pNaviInUseStruct = mf_navi_bar_get_in_use(ap);
			if (ap->mf_Status.iStorageState == MYFILE_PHONE) {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif

			} else {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif
			}
			/*set status path && refresh the screen */
			mf_navi_bar_refresh_recovered_view(ap, pNaviInUseStruct);
			break;
		case MORE_SEARCH:
		case MORE_THUMBNAIL_RENAME:
			if (ap->mf_FileOperation.search_IME_hide_timer != NULL) {
				ecore_timer_del(ap->mf_FileOperation.search_IME_hide_timer);
				ap->mf_FileOperation.search_IME_hide_timer = NULL;
			}
			ap->mf_Status.more = MORE_DEFAULT;
			ap->mf_Status.preMore = MORE_DEFAULT;
			mf_navi_bar_remove_list_item_by_label(ap, labelStorage);

#ifdef MYFILE_SPLIT_VIEW
			if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
				if (ap->mf_Status.view_type != mf_view_root_category && ap->mf_Status.view_type != mf_view_root) {
					mf_navi_bar_reset_panes_main(ap);
				}
			}
#endif
			pNaviInUseStruct = mf_navi_bar_get_in_use(ap);
			mf_navi_bar_refresh_recovered_view(ap, pNaviInUseStruct);

			break;
		case MORE_EDIT:
			bEditStartFlag = mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
			if (bEditStartFlag == TRUE) {
				mf_navi_bar_recover_list(ap);
			}

			pNaviInUseStruct = mf_navi_bar_get_in_use(ap);
			if (ap->mf_Status.iStorageState == MYFILE_PHONE) {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif

			} else {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif
			}
			mf_navi_bar_refresh_recovered_view(ap, pNaviInUseStruct);
			break;
		case MORE_IDLE_DELETE:

			ap->mf_MainWindow.pMmcRemovedPopup = mf_popup_create_popup(ap, POPMODE_TEXT_NOT_DISABLED, NULL,
										   "Mmc removed...", NULL, NULL, NULL, NULL, NULL);
			/*ToDo: check whether get the error before this; */
			if (ap->mf_FileOperation.pCancel) {
				mf_cancel_do_cancel(ap->mf_FileOperation.pCancel);
			}

			mf_navi_bar_remove_list_item_by_label(ap, labelStorage);

			ap->mf_Status.flagStorageRemoved = EINA_TRUE;

			pNaviInUseStruct = mf_navi_bar_get_in_use(ap);
			if (ap->mf_Status.iStorageState == MYFILE_PHONE) {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif

			} else {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif
			}
			mf_navi_bar_refresh_recovered_view(ap, pNaviInUseStruct);
			break;
		case MORE_RENAME:
			if (ap->mf_Status.preMore == MORE_DEFAULT) {
				ap->mf_Status.more = MORE_DEFAULT;
				mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
			} else if (ap->mf_Status.preMore == MORE_EDIT) {
				mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
				ap->mf_Status.more = MORE_DEFAULT;
				mf_navi_bar_recover_list(ap);
			}

			pNaviInUseStruct = mf_navi_bar_get_in_use(ap);
			mf_navi_bar_refresh_recovered_view(ap, pNaviInUseStruct);

			if (ap->mf_Status.iStorageState == MYFILE_PHONE) {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif

			} else {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif
			}
			break;
		case MORE_CREATE_FOLDER:
			if (ap->mf_MainWindow.pEditField != NULL) {
				evas_object_hide(ap->mf_MainWindow.pEditField);
			}
			if (ap->mf_Status.preMore == MORE_DEFAULT) {
				ap->mf_Status.more = MORE_DEFAULT;
				mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
			} else if (ap->mf_Status.preMore == MORE_INTERNAL_COPY_MOVE || ap->mf_Status.preMore == MORE_INTERNAL_COPY
				   || ap->mf_Status.preMore == MORE_INTERNAL_MOVE) {
				bEditStartFlag = mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
				if (bEditStartFlag == TRUE) {
					ap->mf_Status.more = MORE_DEFAULT;
					pNaviToShowStruct = mf_navi_bar_recover_list(ap);
					mf_navi_bar_refresh_recovered_view(ap, pNaviToShowStruct);
				} else {
					ap->mf_Status.more = ap->mf_Status.preMore;
				}

			}

			pNaviInUseStruct = mf_navi_bar_get_in_use(ap);
			mf_navi_bar_refresh_recovered_view(ap, pNaviInUseStruct);
#ifdef MYFILE_SPLIT_VIEW
			if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
				if (ap->mf_Status.view_type != mf_view_root_category && ap->mf_Status.view_type != mf_view_root) {
					mf_navi_bar_reset_panes_main(ap);
				}
			}
#endif

			break;
		case MORE_INTERNAL_COPY:
		case MORE_INTERNAL_MOVE:
		case MORE_INTERNAL_COPY_MOVE:
			bEditStartFlag = mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
			if (bEditStartFlag == TRUE) {
				mf_navi_bar_recover_list(ap);
			}

			pNaviInUseStruct = mf_navi_bar_get_in_use(ap);

			if (ap->mf_Status.iStorageState == MYFILE_PHONE) {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif

			} else {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif
			}
			mf_navi_bar_refresh_recovered_view(ap, pNaviInUseStruct);
			break;

		case MORE_DATA_COPYING:
		case MORE_DATA_MOVING:
			ap->mf_MainWindow.pMmcRemovedPopup = mf_popup_create_popup(ap, POPMODE_TEXT_NOT_DISABLED, NULL,
										   "Mmc removed...", NULL, NULL, NULL, NULL, NULL);
			bEditStartFlag = mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
			if (bEditStartFlag == TRUE) {
				ap->mf_Status.flagStorageRemoved = EINA_TRUE;
			} else {
				ap->mf_Status.flagStorageRemoved = EINA_FALSE;
			}

			pNaviInUseStruct = mf_navi_bar_get_in_use(ap);
			/*reset the path for the navigation bar to show*/
			if (ap->mf_Status.path != NULL) {
				g_string_free(ap->mf_Status.path, TRUE);
				ap->mf_Status.path = NULL;
			}
			ap->mf_Status.path = g_string_new(pNaviInUseStruct->pCurrentPath);

			mf_navi_bar_refresh_recovered_view(ap, pNaviInUseStruct);
			/*check navigation bar count to decide the main layout content*/
			if (ap->mf_Status.iStorageState == MYFILE_PHONE) {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif

			} else {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif
			}
			break;
		case MORE_DELETE:
			ap->mf_MainWindow.pMmcRemovedPopup = mf_popup_create_popup(ap, POPMODE_TEXT_NOT_DISABLED, NULL,
										   "Mmc removed...", NULL, NULL, NULL, NULL, NULL);
			if (ap->mf_FileOperation.pCancel) {
				mf_cancel_do_cancel(ap->mf_FileOperation.pCancel);
			}
			mf_navi_bar_remove_list_item_by_label(ap, labelStorage);
			pNaviInUseStruct = mf_navi_bar_get_in_use(ap);

			if (ap->mf_Status.iStorageState == MYFILE_PHONE) {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif

			} else {
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (optStorage == MYFILE_MMC) {
						elm_object_item_del(ap->mf_MainWindow.sSplitData.mmc_item);
						ap->mf_MainWindow.sSplitData.mmc_item = NULL;
					}
				}
#endif
			}
			mf_navi_bar_refresh_recovered_view(ap, pNaviInUseStruct);
			break;
		default:
			break;
		}

	}
	if (ap->mf_MainWindow.pMmcRemovedPopup) {
		mf_debug();
		evas_object_del(ap->mf_MainWindow.pMmcRemovedPopup);
		ap->mf_MainWindow.pMmcRemovedPopup = NULL;
	}
	mf_navi_bar_title_set(ap);

}
static void __mf_callback_storage_changed_cb(keynode_t *key, void *data)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "appdata is NULL");
	mf_retm_if(ap->mf_Status.path == NULL || ap->mf_Status.path->str == NULL, "mf_Status.path is NULL");
	int optStorage = MYFILE_NONE;

	if (VCONF_TYPE_INT == vconf_keynode_get_type(key)
	    && VCONFKEY_SYSMAN_MMC_MOUNTED == vconf_keynode_get_int(key)) {
		if (!(ap->mf_Status.iStorageState & MYFILE_MMC)) {
			__mf_callback_mmc_connected(ap);
		}
		return;
	}

	if (VCONFKEY_SYSMAN_MMC_REMOVED == vconf_keynode_get_int(key)
	    || VCONFKEY_SYSMAN_MMC_INSERTED_NOT_MOUNTED == vconf_keynode_get_int(key)) {
		mf_debug("mmc removed");
		optStorage = MYFILE_MMC;
		ap->mf_Status.flagStorageRemoved = EINA_TRUE;
	}

	if (optStorage == MYFILE_NONE) {
		mf_debug("get removed storage failed\n");
		return;
	}
	/*here we handle the remove action */
	if (ap->mf_Status.iStorageState & optStorage) {
		__mf_callback_mmc_removed(ap, optStorage);
	}
	return;
}


int mf_callback_set_mmc_state_cb(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	int mmc_state = 0;
	mf_retvm_if(ap == NULL, -1, "appdata is NULL");

	mf_util_is_mmc_on(&mmc_state);
	return vconf_notify_key_changed(VCONFKEY_SYSMAN_MMC_STATUS, __mf_callback_storage_changed_cb, ap);
}
/******		end of Memory card connection/removal handler	******/

/******		USB otg connection/removal handler		******/
#ifdef MYFILE_USB_OTG
static void __mf_callback_otg_removal_cb(keynode_t *key, void *data)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "appdata is NULL");
	mf_retm_if(ap->mf_Status.path == NULL || ap->mf_Status.path->str == NULL, "mf_Status.path is NULL");

	myfileNaviBar *pNaviStorageStruct = NULL;
	Evas_Object *pNaviBarStorageView = NULL;
	bool bAllRemovedFlag = FALSE;	/*all otg devices are removed*/
	/* first generated the removed node list */
	mf_otg_generate_removed_list(ap);
	int count_original = eina_list_count(ap->mf_FileOperation.otg_dev_list);
	int count_removed = eina_list_count(ap->mf_FileOperation.otg_dev_removed_list);
	if(count_original > count_removed)
		bAllRemovedFlag = FALSE;
	else
		bAllRemovedFlag = TRUE;
	/*1.2.1 check if otg is editstar navi */
	pNaviStorageStruct = mf_navi_bar_get_struct_by_label(ap, GET_STR(MF_LABEL_OTG));
	pNaviBarStorageView = ap->mf_MainWindow.pNaviBar;

	/*3 factors; */
	/*
	 **	1.	status				ap->mf_Status.more
	 **	2.	is storage in Use	ap->mf_Status.path
	 **	3.	is source path in storage in inter-storage operation	flagEditStartView
	 */

	if (bAllRemovedFlag == TRUE) {	/*1. otg storage all removed*/
					/*we should check if tab bar need to be removed */
		switch (ap->mf_Status.more) {
		case MORE_DEFAULT:
			mf_otg_dev_all_removed_update(ap, MF_OTG_OPT_NOR_POPUP_DEL);
			break;
		case MORE_SEARCH:
			if (pNaviStorageStruct && pNaviStorageStruct->naviFlagInUse == TRUE) {
				if (ap->mf_FileOperation.sync_pipe != NULL) {
					ecore_pipe_del(ap->mf_FileOperation.sync_pipe);
					ap->mf_FileOperation.sync_pipe = NULL;
				}

				if (ap->mf_FileOperation.search_IME_hide_timer != NULL) {
					ecore_timer_del(ap->mf_FileOperation.search_IME_hide_timer);
					ap->mf_FileOperation.search_IME_hide_timer = NULL;
				}

				if (ap->mf_Status.search_handler > 0) {
					mf_search_stop(ap->mf_Status.search_handler);
				}

				if (ap->mf_Status.search_handler > 0) {
					mf_search_finalize(&ap->mf_Status.search_handler);
				}

			}
			mf_otg_dev_all_removed_update(ap, MF_OTG_OPT_NOR_POPUP_DEL);
			break;
		case MORE_EDIT:
			mf_debug();
			if (pNaviStorageStruct && pNaviStorageStruct->naviFlagInUse == TRUE)
				mf_otg_dev_all_removed_update(ap, MF_OTG_OPT_TABBAR_ENABLE | MF_OTG_OPT_THEME_RESET | MF_OTG_OPT_NOR_POPUP_DEL);
			else
				mf_otg_dev_all_removed_update(ap, MF_OTG_OPT_THEME_RESET | MF_OTG_OPT_NOR_POPUP_DEL);
			break;
		case MORE_IDLE_DELETE:
		case MORE_DELETE:
			mf_otg_dev_all_removed_update(ap, MF_OTG_OPT_NONE);
			break;
		case MORE_RENAME:
		case MORE_CREATE_FOLDER:
			if (pNaviStorageStruct && (pNaviStorageStruct->naviFlagInUse == TRUE || pNaviStorageStruct->naviFlagEditStart == TRUE))
				mf_otg_dev_all_removed_update(ap, MF_OTG_OPT_TABBAR_ENABLE | MF_OTG_OPT_NOR_POPUP_DEL);
			else
				mf_otg_dev_all_removed_update(ap, MF_OTG_OPT_NONE);
			break;
		case MORE_INTERNAL_COPY:
		case MORE_INTERNAL_MOVE:
		case MORE_INTERNAL_COPY_MOVE:
			mf_otg_dev_all_removed_update(ap, MF_OTG_OPT_NONE);
			break;
		case MORE_DATA_COPYING:
		case MORE_DATA_MOVING:
			mf_otg_dev_all_removed_update(ap, MF_OTG_OPT_REQ_POPUP_DEL);
			break;
		default:
			break;
		}
	} else {	/*otg storages part removed*/
			/*we don't need to check tab bar status*/
		mf_debug("not all removed");
		switch (ap->mf_Status.more) {
		case MORE_DEFAULT:
			mf_otg_dev_removed_update(ap, MF_OTG_OPT_NOR_POPUP_DEL);
			break;
		case MORE_SEARCH:
			mf_otg_dev_removed_update(ap, MF_OTG_OPT_NOR_POPUP_DEL);
			break;
		case MORE_EDIT:
			if (pNaviStorageStruct && pNaviStorageStruct->naviFlagInUse == TRUE)
				mf_otg_dev_removed_update(ap, MF_OTG_OPT_TABBAR_ENABLE | MF_OTG_OPT_THEME_RESET | MF_OTG_OPT_NOR_POPUP_DEL);
			else
				mf_otg_dev_removed_update(ap, MF_OTG_OPT_THEME_RESET | MF_OTG_OPT_NOR_POPUP_DEL);
			break;
		case MORE_IDLE_DELETE:
		case MORE_DELETE:
			mf_otg_dev_removed_update(ap, MF_OTG_OPT_NONE);
			break;
		case MORE_RENAME:
		case MORE_CREATE_FOLDER:
			if (pNaviStorageStruct && (pNaviStorageStruct->naviFlagInUse == TRUE || pNaviStorageStruct->naviFlagEditStart == TRUE))
				mf_otg_dev_removed_update(ap, MF_OTG_OPT_TABBAR_ENABLE | MF_OTG_OPT_NOR_POPUP_DEL);
			else
				mf_otg_dev_removed_update(ap, MF_OTG_OPT_NONE);
			break;
		case MORE_INTERNAL_COPY:
		case MORE_INTERNAL_MOVE:
		case MORE_INTERNAL_COPY_MOVE:
			mf_otg_dev_removed_update(ap, MF_OTG_OPT_NONE);
			break;
		case MORE_DATA_COPYING:
		case MORE_DATA_MOVING:
			mf_otg_dev_removed_update(ap, MF_OTG_OPT_REQ_POPUP_DEL);
			break;
		default:
			break;
		}
	}

	mf_debug("before remove is [%d]", ap->mf_Status.iStorageState);
	if(bAllRemovedFlag == TRUE)
		ap->mf_Status.iStorageState = (ap->mf_Status.iStorageState ^ MYFILE_OTG);
	mf_debug("after remove is [%d]", ap->mf_Status.iStorageState);
}


static void __mf_callback_otg_connected_cb(keynode_t *key, void *data)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "appdata is NULL");
	mf_retm_if(ap->mf_Status.path == NULL || ap->mf_Status.path->str == NULL, "mf_Status.path is NULL");
	Evas_Object *entry = NULL;

	/* otg-dev-list update */
	mf_debug("before update: [%d]", eina_list_count(ap->mf_FileOperation.otg_dev_list));
	mf_otg_list_update(ap, true);
	mf_debug("after update: [%d]", eina_list_count(ap->mf_FileOperation.otg_dev_list));
	/* we have 2 cases:
	 * 1. otg tab already exists -- update the genlist
	 * 2. otg tab does not exist -- add tab
	*/
	if (ap->mf_Status.iStorageState & MYFILE_OTG) {
		mf_debug("path is %s", ap->mf_Status.path->str);
		if (g_strcmp0(ap->mf_Status.path->str, OTG_FOLDER) == 0) {
			mf_debug();
			mf_util_ex_disk_list_update(ap);
			mf_debug();
		}
	} else {
		mf_debug();
		ap->mf_Status.iStorageState |= MYFILE_OTG;
		mf_util_action_storage_insert(ap, GET_STR(MF_LABEL_OTG));
		mf_debug();
	}

	mf_debug();
	if (ap->mf_Status.more == MORE_RENAME) {
		entry = ap->mf_MainWindow.pEntry;
		if (entry != NULL) {
			elm_object_focus_set(entry, EINA_TRUE);
		}
	}
	return;
}

void mf_callback_set_otg_state_cb(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "appdata is NULL");
	mf_retm_if(ap->mf_Status.path == NULL || ap->mf_Status.path->str == NULL, "mf_Status.path is NULL");

	mf_debug("path is %s", ap->mf_Status.path->str);

	if (ap->mf_FileOperation.otg_dev_list != NULL) {
		mf_otg_list_clean(ap, MF_OTG_LIST_DEV);
	}

	mf_otg_init(ap);
	vconf_notify_key_changed(VCONFKEY_SYSMAN_ADDED_USB_STORAGE, __mf_callback_otg_connected_cb, ap);
	vconf_notify_key_changed(VCONFKEY_SYSMAN_REMOVED_USB_STORAGE, __mf_callback_otg_removal_cb, ap);
}
#endif
/******		end of USB otg connection/removal handler	******/


static GString *__mf_callback_dir_pipe_get_parent(GString *path)
{

	mf_retvm_if(path == NULL, NULL, "path is NULL");
	mf_retvm_if(path->str == NULL, NULL, "path->str is NULL");

	if (ecore_file_exists(path->str)) {
		return path;
	} else if (mf_fm_svc_wrapper_is_root_path(path->str)) {
		return path;
	} else {
		GString *parent = mf_fm_svc_wrapper_get_file_parent_path(path);
		SAFE_FREE_GSTRING(path);
		__mf_callback_dir_pipe_get_parent(parent);
	}
	return path;
}
static void __mf_callback_dir_pipe_cb(void *data, void *buffer, unsigned int nbyte)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "appdata is NULL");

	if (ap->mf_Status.more != MORE_DEFAULT)
			return;
	if (ap->mf_Status.view_type == mf_view_root || ap->mf_Status.view_type == mf_view_root_category) {
		return;
	}
	if (g_strcmp0(ap->mf_Status.monitor_path, ap->mf_Status.path->str)) {
		return;
	}
	if (buffer) {
		mf_dir_event_t *msg = (mf_dir_event_t *) buffer;
		mf_debug("event : %d, name : %s", msg->event, msg->name);

		int iIsNoContent = 0;
		Evas_Object *newContent = NULL;

		iIsNoContent = ap->mf_Status.flagNoContent;
		mfItemData_s *itemData = NULL;
		Elm_Object_Item *it = NULL;
		GString *parent = NULL;
		GString *current = NULL;
		Evas_Object *parent_obj = NULL;
		char *path = NULL;
		int count = 0;

		switch (msg->event) {
		case MF_INOTI_CREATE:
		case MF_INOTI_MOVE_IN:
			if(msg->name) {
				path = g_strconcat(ap->mf_Status.path->str, "/", msg->name, NULL);
				int file_type = 0;
				if (mf_file_attr_is_dir(path)) {
					file_type = FILE_TYPE_DIR;
				} else {
					file_type = FILE_TYPE_FILE;
				}
				fsNodeInfo *pNode = mf_util_generate_pnode(path, file_type);
				if (ap->mf_Status.flagViewType == MF_VIEW_STYLE_LIST || ap->mf_Status.flagViewType == MF_VIEW_SYTLE_LIST_DETAIL) {
					if (ap->mf_MainWindow.pNaviGenlist == NULL) {
						ap->mf_MainWindow.pNaviGenlist = mf_widget_create_genlist(ap->mf_MainWindow.pNaviBar);
						evas_object_smart_callback_add(ap->mf_MainWindow.pNaviGenlist, "drag,start,right", mf_genlist_gl_mode_sweep_set, ap);
						evas_object_smart_callback_add(ap->mf_MainWindow.pNaviGenlist, "drag,start,left", mf_genlist_gl_mode_sweep_unset, ap);
						evas_object_smart_callback_add(ap->mf_MainWindow.pNaviGenlist, "drag,start,up", mf_genlist_gl_mode_cancel, ap);
						evas_object_smart_callback_add(ap->mf_MainWindow.pNaviGenlist, "drag,start,down", mf_genlist_gl_mode_cancel, ap);
						evas_object_smart_callback_add(ap->mf_MainWindow.pNaviGenlist, "selected", mf_genlist_gl_selected, ap);

						evas_object_del(elm_object_part_content_unset(ap->mf_MainWindow.pNaviLayout, "content"));
						elm_object_part_content_set(ap->mf_MainWindow.pNaviLayout, "content", ap->mf_MainWindow.pNaviGenlist);
					}
					parent_obj = ap->mf_MainWindow.pNaviGenlist;
				} else {
					if (ap->mf_MainWindow.pNaviGengrid == NULL) {
						ap->mf_MainWindow.pNaviGengrid = mf_gengrid_create_grid(ap->mf_MainWindow.pNaviBar);
						evas_object_del(elm_object_part_content_unset(ap->mf_MainWindow.pNaviLayout, "content"));
						elm_object_part_content_set(ap->mf_MainWindow.pNaviLayout, "content", ap->mf_MainWindow.pNaviGengrid);
					}
					parent_obj = ap->mf_MainWindow.pNaviGengrid;
				}
				mf_widget_item_append(parent_obj, pNode, ap);

				if (pNode != NULL) {
					SAFE_FREE_CHAR(pNode->path);
					SAFE_FREE_CHAR(pNode->name);
					SAFE_FREE_CHAR(pNode->ext);
					free(pNode);
					pNode = NULL;
				}
			}
			/*1 TODO:  add new item to list */
			if (ap->mf_Status.more == MORE_DEFAULT && iIsNoContent) {
				mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_NOCONTENT_VIEW, false);
				//mf_ctrl_bar_item_set_disable(ap->mf_MainWindow.pNaviCtrlBar, CTRL_DISABLE_NOCONTENT_VIEW, false);
			}
			break;
		case MF_INOTI_DELETE:
		case MF_INOTI_MOVE_OUT:
			path = g_strconcat(ap->mf_Status.path->str, "/", msg->name, NULL);
			if (ap->mf_Status.flagViewType == MF_VIEW_STYLE_LIST || ap->mf_Status.flagViewType == MF_VIEW_SYTLE_LIST_DETAIL) {
				it = elm_genlist_first_item_get(ap->mf_MainWindow.pNaviGenlist);
				while (it) {
					itemData = elm_object_item_data_get(it);
					if (itemData->m_ItemName == NULL || itemData->m_ItemName->str == NULL) {
						mf_debug();
						continue;
					}
					if (g_strcmp0(path, itemData->m_ItemName->str) == 0) {
						elm_object_item_del(it);
						break;
					}

					it = elm_genlist_item_next_get(it);
				}
				count = elm_genlist_items_count(ap->mf_MainWindow.pNaviGenlist);
				ap->mf_Status.flagNoContent = !count;

			} else {
				it = elm_gengrid_first_item_get(ap->mf_MainWindow.pNaviGengrid);
				while (it) {
					itemData = elm_object_item_data_get(it);
					if (itemData->m_ItemName == NULL || itemData->m_ItemName->str == NULL) {
						continue;
					}
					if (g_strcmp0(path, itemData->m_ItemName->str) == 0) {
						elm_object_item_del(it);
						break;
					}

					it = elm_gengrid_item_next_get(it);
				}
				count = elm_gengrid_items_count(ap->mf_MainWindow.pNaviGengrid);
				ap->mf_Status.flagNoContent = !count;

			}
			/*1 TODO:  remove item from list */
			if (ap->mf_Status.more == MORE_DEFAULT && ap->mf_Status.flagNoContent) {
				mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_NOCONTENT_VIEW, true);
			}
			break;
		case MF_INOTI_MODIFY:
#ifdef MYFILE_SPLIT_VIEW
			//todo: optmize the refresh
			if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
				mf_widget_refresh_view(ap);
			}
			else
#endif
			{
				newContent = mf_navi_bar_content_create(ap);
				Evas_Object *pConformant = mfNaviBarConformGet(ap);
				mfNaviBarSetContent(ap, pConformant, newContent);
			}
			/*1 TODO:  update item in list */
			break;
		case MF_INOTI_DELETE_SELF:
		case MF_INOTI_MOVE_SELF:
		{
			current = g_string_new(ap->mf_Status.path->str);
			parent = __mf_callback_dir_pipe_get_parent(current);

			myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);

			g_string_free(ap->mf_Status.path, TRUE);
			ap->mf_Status.path = NULL;
			ap->mf_Status.path = parent;
#ifdef MYFILE_USB_OTG
			/* if click items under otg root path, it means some disk take into use*/
			int storage = mf_fm_svc_wrapper_is_root_path(ap->mf_Status.path->str);
			if (storage == MYFILE_OTG) {	/* a usb disk take into use */
				char *dev_in_use = mf_otg_get_node_in_use(ap);
				if (dev_in_use != NULL)
					mf_otg_set_node_using_flag(ap, dev_in_use, FALSE);
			}
#endif
			if (pNavi_s->pCurrentPath != NULL) {
				free(pNavi_s->pCurrentPath);
				pNavi_s->pCurrentPath = NULL;
			}
			pNavi_s->pCurrentPath = g_strdup(parent->str);



			mf_widget_refresh_view(ap);
		}
			break;
		default:
			mf_error("Unknown event");
			break;
		}


		if (msg->name)
			free(msg->name);
		SAFE_FREE_CHAR(path);
		mf_inotify_handle_request_handled_send();
	}
}

static void __mf_callback_dir_update_cb(mf_inotify_event event, char *name, void *data)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "appdata is NULL");

	mf_debug("event : %d, name : %s", event, name);

	mf_dir_event_t buffer;

	buffer.event = event;
	buffer.name = strdup(name);

	ecore_pipe_write(ap->mf_FileOperation.inotify_pipe, &buffer, sizeof(buffer));

	return;
}

int mf_callback_add_dir_watch(const char *path, void *data)
{
	mf_retvm_if(data == NULL, 0, "data is NULL");

	struct appdata *ap = (struct appdata *)data;
	SAFE_FREE_CHAR(ap->mf_Status.monitor_path);
	ap->mf_Status.monitor_path = g_strdup(path);
	return mf_inotify_handle_add_watch(path, __mf_callback_dir_update_cb, data);
}

int mf_callback_remove_dir_watch(void)
{
	return mf_inotify_handle_rm_watch();
}

int mf_callback_create_dir_monitor(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, -1, "appdata is NULL");

	if (ap->mf_FileOperation.inotify_pipe)
		ecore_pipe_del(ap->mf_FileOperation.inotify_pipe);

	ap->mf_FileOperation.inotify_pipe = ecore_pipe_add(__mf_callback_dir_pipe_cb, (const void *)ap);

	return mf_inotify_handle_init_inotify();
}

void mf_callback_destory_dir_monitor(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "appdata is NULL");

	if (ap->mf_FileOperation.inotify_pipe) {
		ecore_pipe_del(ap->mf_FileOperation.inotify_pipe);
		ap->mf_FileOperation.inotify_pipe = NULL;
	}

	mf_inotify_handle_finalize_inotify();

	return;
}

void mf_callback_backbutton_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata *)data;

	if (ap->mf_Status.more == MORE_DEFAULT) {
		if (ap->mf_Status.view_type == mf_view_root) {
			Evas_Object *win = ap->mf_MainWindow.pWindow;
			elm_win_lower(win);
		} else if (ap->mf_Status.view_type == mf_view_root_category) {
			ap->mf_Status.view_type = mf_view_root;
			mf_widget_refresh_view(ap);
		} else {
			mf_callback_upper_click_cb(ap, NULL, NULL);
		}
	} else {
		mf_callback_cancel_cb(ap, NULL, NULL);
	}
}

void mf_callback_new_folder_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is null");

	Eina_Bool entry_empty = elm_entry_is_empty(ap->mf_MainWindow.pEntry);

	if (entry_empty) {
		elm_object_signal_emit(ap->mf_MainWindow.pEditField, "elm,state,eraser,hide", "elm");
	} else {
		elm_object_signal_emit(ap->mf_MainWindow.pEditField, "elm,state,eraser,show", "elm");
	}
	MF_TRACE_END;

}
void mf_callback_imf_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is null");

	const char *entry_data = NULL;
	char *name = NULL;
	char new_str[MYFILE_FILE_NAME_LEN_MAX] = { '\0', };

	entry_data = elm_entry_entry_get(ap->mf_MainWindow.pEntry);
	mf_retm_if(entry_data == NULL, "entry_data is null");
	name = elm_entry_markup_to_utf8(entry_data);
	mf_retm_if(name == NULL, "name is null");

	Eina_Bool entry_empty = elm_entry_is_empty(ap->mf_MainWindow.pEntry);

	if (entry_empty) {
		elm_object_signal_emit(ap->mf_MainWindow.pEditField, "elm,state,eraser,hide", "elm");
	} else {
		elm_object_signal_emit(ap->mf_MainWindow.pEditField, "elm,state,eraser,show", "elm");
	}

	if (mf_file_attr_is_valid_name(name) != MYFILE_ERR_NONE) {
		strncpy(new_str, name, MYFILE_FILE_NAME_LEN_MAX - 1);
		if (strlen(name) > 0)
			new_str[strlen(name) - 1] = '\0';
		elm_entry_entry_set(ap->mf_MainWindow.pEntry, new_str);
		elm_entry_cursor_end_set(ap->mf_MainWindow.pEntry);
		elm_object_focus_set(ap->mf_MainWindow.pEntry, EINA_FALSE);

		if (ap->mf_MainWindow.pNormalPopup) {
			SAFE_FREE_OBJ(ap->mf_MainWindow.pNormalPopup);
		}
		ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT, NULL, GET_STR(MF_MSG_RENAME_ILLEGAL_CHAR), NULL, NULL,
								       NULL, mf_popup_show_vk_cb, ap);
	}
	SAFE_FREE_CHAR(name);
	MF_TRACE_END;
}


void mf_callback_rename_button_callback(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mfItemData_s *params = (mfItemData_s *)data;
	mf_retm_if(params == NULL, "input data is NULL");

	struct appdata *ap = (struct appdata *)params->ap;
	mf_retm_if(ap == NULL, "input ap is NULL");

	Elm_Object_Item *it = params->item;
	mf_retm_if(it == NULL, "input item is NULL");

	myfileNaviBar *pNavi_toshow = NULL;
	pNavi_toshow = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_toshow == NULL, "get navi in use failed");

	if (ap->mf_Status.more == MORE_RENAME) {
		mf_callback_rename_save_cb(ap, NULL, NULL);
	}
	mf_genlist_sweep_item_recover(ap->mf_MainWindow.pNaviGenlist);

	ap->mf_Status.preMore = ap->mf_Status.more;
	ap->mf_Status.more = MORE_RENAME;
	elm_genlist_item_flip_set(it, EINA_TRUE);
	elm_genlist_item_select_mode_set(it, ELM_OBJECT_SELECT_MODE_NONE);
	ap->mf_FileOperation.rename_item = it;

	if (ap->mf_Status.preMore == MORE_DEFAULT) {

		mf_navi_bar_set_upper_item_disable(ap, true);
		mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_DEFAULT_ALL, true);
		//mf_ctrl_bar_item_set_disable(ap->mf_MainWindow.pNaviCtrlBar, CTRL_DISABLE_DEFAULT_ALL, TRUE);
	} else {
		ap->mf_Status.iCtrlDisableItem = mf_navi_bar_button_get_disable(ap->mf_MainWindow.pNaviItem);
		mf_debug("ap->mf_Status.iCtrlDisableItem is [%d]", ap->mf_Status.iCtrlDisableItem);
		mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_EDIT_ALL, true);
		//mf_ctrl_bar_item_set_disable(ap->mf_MainWindow.pNaviCtrlBar, CTRL_DISABLE_EDIT_ALL, TRUE);
	}


	MF_TRACE_END;
}
void mf_callback_detail_ctx_cb(void *data, Evas_Object * obj, void *event_info)
{
	mf_retm_if(data == NULL, "data is NULL");
	mfItemData_s *itemData = (mfItemData_s *)data;
	struct appdata* ap = (struct appdata *)itemData->ap;
	mf_retm_if(ap == NULL, "ap is NULL");

	SAFE_FREE_OBJ(ap->mf_MainWindow.pContextPopup);
	mf_launch_load_ug(ap, itemData->m_ItemName->str, MF_LOAD_UG_DETAIL, NULL);
}
static void __mf_callback_delete_in_idle(void *data, Evas_Object * obj, void *event_info)
{
	struct appdata *ap = (struct appdata *)data;

	Evas_Object *btn = (Evas_Object *)obj;
	const char *label = elm_object_text_get(btn);

	if (g_strcmp0(label, GET_SYS_STR(LABEL_CANCEL)) == 0 ) {
		SAFE_FREE_OBJ(ap->mf_MainWindow.pNormalPopup);
		return;
	} else if (g_strcmp0(label, GET_SYS_STR(LABEL_DELETE)) == 0 ) {
		SAFE_FREE_OBJ(ap->mf_MainWindow.pNormalPopup);

		GList *pSourceList = NULL;
		int count = 0;

		mf_util_merge_eina_list_to_glist(ap->mf_FileRecordList.value_saver, &pSourceList);

		if (pSourceList) {
			count = g_list_length(pSourceList);
		} else {
			MF_TRACE_END;
			return;
		}
		mf_callback_init_operation_cancel(ap);
		ap->mf_FileOperation.iTotalCount = count;
		ap->mf_Status.preMore = ap->mf_Status.more;
		ap->mf_Status.more = MORE_IDLE_DELETE;
		ap->mf_FileOperation.pSourceList = pSourceList;
		ap->mf_FileOperation.iOperationSuccessFlag = TRUE;
		mf_delete_items(pSourceList, ap->mf_FileOperation.pCancel, TRUE, ap);
		SAFE_FREE_OBJ(ap->mf_MainWindow.pProgressPopup);
		ap->mf_MainWindow.pProgressPopup = mf_popup_create_pb_popup(ap, GET_SYS_STR(MF_MSG_DELETING), count, mf_callback_progress_bar_cancel_cb, ap);
	}
}

void mf_callback_delete_button_cb(void *data, Evas_Object * obj, void *event_info)
{

	mfItemData_s *params = (mfItemData_s *) data;
	struct appdata *ap = (struct appdata *)params->ap;

	SAFE_FREE_OBJ(ap->mf_MainWindow.pContextPopup);

	if (ap->mf_FileRecordList.value_saver != NULL) {
		mf_util_free_eina_list_with_data(&ap->mf_FileRecordList.value_saver, MYFILE_TYPE_GSTRING);
		ap->mf_FileRecordList.value_saver = NULL;
	}
	ap->mf_FileRecordList.value_saver = eina_list_append(ap->mf_FileRecordList.value_saver, g_string_new(params->m_ItemName->str));
	SAFE_FREE_OBJ(ap->mf_MainWindow.pNormalPopup);
	ap->mf_FileOperation.idle_delete_item = params->item;
	ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TITLE_TEXT_TWO_BTN,
							       NULL,
							       GET_SYS_STR(MF_BUTTON_LABEL_DEL),
							       GET_SYS_STR(LABEL_DELETE), GET_SYS_STR(LABEL_CANCEL),
							       NULL, __mf_callback_delete_in_idle, ap);
}

void mf_callback_view_as_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	struct appdata *ap = (struct appdata *)data;

	SAFE_FREE_OBJ(ap->mf_MainWindow.pContextPopup);
	ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_VIEW_AS_LIST, GET_STR(LABEL_VIEW_AS), NULL, NULL, NULL, NULL, mf_callback_popup_deleted_cb, ap);
}
void mf_callback_view_style_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	struct appdata *ap = (struct appdata *)data;

	SAFE_FREE_OBJ(ap->mf_MainWindow.pContextPopup);
	if (ap->mf_Status.flagViewType == MF_VIEW_STYLE_LIST) {
		ap->mf_Status.flagViewType = MF_VIEW_STYLE_THUMBNAIL;
	} else {
		ap->mf_Status.flagViewType = MF_VIEW_STYLE_LIST;
	}

	mf_util_set_view_style(ap->mf_Status.flagViewType);

	if (ap->mf_Status.flagNoContent != EINA_TRUE) {
		mf_widget_refresh_view(ap);
	}
	MF_TRACE_END;
}

void mf_callback_home_button_pressed_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	Evas_Object *home_button_ic = (Evas_Object *)data;
	elm_image_file_set(home_button_ic, MF_TITLE_ICON_HOME_PRESS, NULL);
}

void mf_callback_home_button_unpressed_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	Evas_Object *home_button_ic = (Evas_Object *)data;
	elm_image_file_set(home_button_ic, MF_TITLE_ICON_HOME, NULL);
}

void mf_callback_select_all_button_pressed_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	Evas_Object *select_all_button_ic = (Evas_Object *)data;
	elm_image_file_set(select_all_button_ic, MF_TITLE_ICON_SELECT_ALL_PRESS, NULL);
}

void mf_callback_select_all_button_unpressed_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	Evas_Object *select_all_button_ic = (Evas_Object *)data;
	elm_image_file_set(select_all_button_ic, MF_TITLE_ICON_SELECT_ALL, NULL);
}

void mf_callback_home_button_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	struct appdata *ap = (struct appdata *)data;
	int storage = MYFILE_PHONE;
	myfileNaviBar *pNavi_s = NULL;


	pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "pNavi_s is NULL");

	mf_callback_remove_dir_watch();

	if (ap->mf_Status.more == MORE_RENAME) {
		mf_callback_destory_rename(ap);
	}

	ap->mf_Status.view_type = mf_view_root;
#ifdef MYFILE_SPLIT_VIEW

	if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
		mf_navi_bar_set_panes_navi_to_main(ap);

	}
#endif
	storage = mf_fm_svc_wrapper_get_location(ap->mf_Status.path->str);
	switch (storage) {
		case MYFILE_PHONE:
			SAFE_FREE_GSTRING(ap->mf_Status.path);
			ap->mf_Status.path = g_string_new(PHONE_FOLDER);
			break;
		case MYFILE_MMC:
			SAFE_FREE_GSTRING(ap->mf_Status.path);
			ap->mf_Status.path = g_string_new(MEMORY_FOLDER);
			break;
		case MYFILE_OTG:
			SAFE_FREE_GSTRING(ap->mf_Status.path);
			ap->mf_Status.path = g_string_new(OTG_FOLDER);
			break;
		default:
			return;
	}
#ifdef MYFILE_USB_OTG
	/* if click items under otg root path, it means some disk take into use*/
	if (storage == MYFILE_OTG) {	/* a usb disk take into use */
		char *dev_in_use = mf_otg_get_node_in_use(ap);
		if (dev_in_use != NULL)
			mf_otg_set_node_using_flag(ap, dev_in_use, FALSE);
	}
#endif
	SAFE_FREE_CHAR(pNavi_s->pCurrentPath);
	pNavi_s->pCurrentPath = g_strdup(ap->mf_Status.path->str);

	if (ap->mf_MainWindow.pNaviItem) {
		mf_navi_bottom_item_del(&ap->mf_MainWindow.pNaviItem);
	}
	mf_widget_refresh_view(ap);
	MF_TRACE_END;
}

void mf_callback_more_button_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata *)data;
	ap->mf_Status.navi_item_selected = MF_NAVI_BTN_MORE;
	mf_context_popup_create_more(ap, obj);
	return;
}

void mf_callback_eraser_clicked_cb(void *data, Evas_Object *obj, const char *emission, const char *source)
{
	elm_entry_entry_set(data, "");
}

void mf_callback_list_by_button_callback(void *data, Evas_Object * obj, void *event_info)
{
	assert(data);
	assert(obj);
	struct appdata *ap = (struct appdata *)data;

	Evas_Object *btn = (Evas_Object *)obj;
	const char *label = elm_object_text_get(btn);
	int iListBySortType = 0;

	if (g_strcmp0(label, GET_STR(MF_LABEL_DESCENDING)) == 0 ) {
		switch(ap->mf_Status.iSelectedSortType) {
			case 0:
				iListBySortType = MYFILE_SORT_BY_NAME_Z2A;
				break;
			case 1:
				iListBySortType = MYFILE_SORT_BY_DATE_R2O;
				break;

			case 2:
				iListBySortType = MYFILE_SORT_BY_SIZE_L2S;
				break;
			case 3:
				iListBySortType = MYFILE_SORT_BY_TYPE_Z2A;
				break;
			default:
				iListBySortType = MYFILE_SORT_BY_DATE_R2O;
				break;
		}
	} else if(g_strcmp0(label, GET_STR(MF_LABEL_ASCENDING)) == 0 ) {
		switch(ap->mf_Status.iSelectedSortType) {
			case 0:
				iListBySortType = MYFILE_SORT_BY_NAME_A2Z;
				break;
			case 1:
				iListBySortType = MYFILE_SORT_BY_DATE_O2R;
				break;

			case 2:
				iListBySortType = MYFILE_SORT_BY_SIZE_S2L;
				break;
			case 3:
				iListBySortType = MYFILE_SORT_BY_TYPE_A2Z;
				break;
			default:
				iListBySortType = MYFILE_SORT_BY_DATE_O2R;
				break;
		}

	}
	mf_util_set_sort_type(iListBySortType);
	mf_callback_list_by_response_cb(ap);
}

void mf_callback_sbeam_cb(void *user_data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(user_data == NULL, "user_data is NULL");

	struct appdata *ap = (struct appdata *)user_data;

	if (ap->mf_FileRecordList.selected_files) {
		mf_util_free_eina_list_with_data(&ap->mf_FileRecordList.selected_files, MYFILE_TYPE_GSTRING);
	}
	ap->mf_FileRecordList.selected_files = mf_util_get_all_selected(ap);

	Eina_List *files = NULL;

	if (ap->mf_Status.more == MORE_EDIT) {
		files = mf_util_get_all_selected(ap);
	}

	mf_sbeam_share_files(ap->mf_MainWindow.pWindow, files);
	mf_util_free_eina_list_with_data(&files, MYFILE_TYPE_GSTRING);
	files = NULL;
}

void mf_callback_extension_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "user_data is NULL");

	struct appdata *ap = (struct appdata *)data;

	Evas_Object *playout = ap->mf_MainWindow.pNaviLayout;
	mf_retm_if(playout == NULL, "get conformant failed");
	Evas_Object *newContent = NULL;
	Elm_Object_Item *item = (Elm_Object_Item *) event_info;
	const char *label = elm_object_item_text_get(item);
	if (g_strcmp0(label, GET_STR(MF_LABEL_SHOW_EXTENSION)) == 0) {
		mf_util_set_extension_state(MF_EXTENSION_SHOW);
		ap->mf_Status.iExtensionState = MF_EXTENSION_SHOW;
	} else if (g_strcmp0(label, GET_STR(MF_LABEL_HIDE_EXTENSION)) == 0) {
		mf_util_set_extension_state(MF_EXTENSION_HIDE);
		ap->mf_Status.iExtensionState = MF_EXTENSION_HIDE;
	} else {
		SAFE_FREE_OBJ(ap->mf_MainWindow.pContextPopup);
		return;
	}
	SAFE_FREE_OBJ(ap->mf_MainWindow.pContextPopup);
	if (ap->mf_Status.flagNoContent != EINA_TRUE) {
		newContent = mf_navi_bar_content_create(ap);
		mfNaviBarSetContent(ap, playout, newContent);

	}
	ap->mf_Status.more = MORE_DEFAULT;
}

/******************************
** Prototype    : mf_callback_nofity_show_callback
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_nofity_show_callback(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");
	edje_object_signal_emit(_EDJ(ap->mf_MainWindow.pLabelLayout), "elm,state,show,default", "elm");
	return;
}

/******************************
** Prototype    : mf_callback_nofity_hide_callback
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_callback_nofity_hide_callback(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "invalid parameter ap is NULL");

	edje_object_signal_emit(_EDJ(ap->mf_MainWindow.pLabelLayout), "elm,state,hide,default", "elm");
	return;
}

void mf_callback_operation_timeout_cb(void *data, Evas_Object *obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	if (obj) {
		evas_object_del(obj);
		ap->mf_MainWindow.pLabelLayout = NULL;
		ap->mf_MainWindow.pOperationNotify = NULL;
	}
	MF_TRACE_END;
	return;
}

Eina_Bool mf_callback_popup_del_idler_cb(void *data)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(data == NULL, ECORE_CALLBACK_CANCEL, "data is NULL");
	struct appdata *ap = (struct appdata *)data;
	SAFE_FREE_OBJ(ap->mf_MainWindow.pNormalPopup);
	ap->mf_Status.popup_del_idler = NULL;

	return ECORE_CALLBACK_CANCEL;
}

#ifdef MYFILE_MOTION_FEATURE

void mf_callback_motion_double_tap(Evas_Object *genlist)
{
	MF_TRACE_BEGIN;
	mf_retm_if(genlist == NULL, "genlist is NULL");
	Elm_Object_Item *item = elm_genlist_first_item_get(genlist);
	elm_genlist_item_bring_in(item, ELM_GENLIST_ITEM_SCROLLTO_TOP);

}
#endif

void mf_callback_profile_changed_cb(void *data, Evas_Object * obj, void *event_info)
{
        MF_TRACE_BEGIN;
        mf_retm_if(data == NULL, "data is NULL");
        struct appdata *ap = (struct appdata *)data;
        const char *profile = elm_config_profile_get();

        if (!strcmp(profile, "desktop")) {
                elm_object_item_part_content_set(ap->mf_MainWindow.pNaviItem, "prev_btn", NULL);
                elm_win_indicator_mode_set(ap->mf_MainWindow.pWindow, ELM_WIN_INDICATOR_HIDE);
        }
        else {   // mobile
                Evas_Object *btn = elm_button_add(ap->mf_MainWindow.pNaviBar);
                elm_object_style_set(btn, "naviframe/end_btn/default");
                evas_object_smart_callback_add(btn, "clicked", mf_callback_backbutton_clicked_cb, ap);
                elm_object_item_part_content_set(ap->mf_MainWindow.pNaviItem, "prev_btn", btn);

                elm_win_indicator_mode_set(ap->mf_MainWindow.pWindow, ELM_WIN_INDICATOR_SHOW);
        }
}

void mf_callback_share_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata *)data;
	int share_mode = 0;
	int file_mode = 0;


	if (ap->mf_FileRecordList.selected_files) {
		mf_util_free_eina_list_with_data(&ap->mf_FileRecordList.selected_files, MYFILE_TYPE_GSTRING);
	}
	ap->mf_FileRecordList.selected_files = mf_util_get_all_selected(ap);

	file_mode = mf_share_mode_get(ap->mf_FileRecordList.selected_files);

	if ((file_mode & SHARE_FILE_MODE_OTHERS) || ((file_mode & SHARE_FILE_MODE_IMAGE) && (file_mode & SHARE_FILE_MODE_VIDEO))) {
		share_mode = SHARE_MODE_NORMAL;
	} else if (file_mode & SHARE_FILE_MODE_MULTI_IMAGE) {
		share_mode = SHARE_MODE_MULTI_IMAGE;
	} else if (file_mode & SHARE_FILE_MODE_MULTI_VIDEO) {
		share_mode = SHARE_MODE_MULTI_VIDEO;
	} else if (file_mode & SHARE_FILE_MODE_IMAGE) {
		share_mode = SHARE_MODE_IMAGE;
	} else if (file_mode & SHARE_FILE_MODE_VIDEO) {
		share_mode = SHARE_MODE_VIDEO;
	} else {
		share_mode = SHARE_MODE_NORMAL;
	}
	mf_context_popup_create_share(ap, share_mode, obj);
}

void mf_callback_thumb_created_cb(media_content_error_e error, const char *path, void *user_data)
{
	mf_retm_if(user_data == NULL, "user_data is NULL");
	mf_retm_if(path == NULL, "path is NULL");
	mfItemData_s *pListData = (mfItemData_s *)user_data;
	mf_retm_if(pListData->item == NULL, "pListData->item is NULL");

	if (ecore_file_exists(path)) {
		mf_debug("Update item with new thumbnail[%s]", path);
		SAFE_FREE_CHAR(pListData->thumb_path);
		pListData->thumb_path = g_strdup(path);
		if (pListData->ap->mf_Status.flagViewType == MF_VIEW_STYLE_THUMBNAIL) {
			elm_gengrid_item_update(pListData->item);
		} else {
			elm_genlist_item_update(pListData->item);
		}
	} else {
		mf_debug("Invalid thumb path!");
	}
}

bool mf_callback_create_thumbnail(void *data, media_thumbnail_completed_cb callback)
{
	mf_retvm_if(data == NULL, -1, "filter is NULL");
	mfItemData_s *pListData = (mfItemData_s *)data;

	int ret = -1;

	ret = media_info_create_thumbnail(pListData->media, callback,
					  pListData);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		mf_debug("Failed to create thumbnail!");
		return -1;
	}
	return 0;

	return false;
}
void mf_callback_entry_button_pressed_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	Evas_Object *entry_button_ic = (Evas_Object *)data;
	elm_image_file_set(entry_button_ic, MF_ICON_ENTRY_FOLDER_PRESS, NULL);
}

void mf_callback_entry_button_unpressed_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	Evas_Object *entry_button_ic = (Evas_Object *)data;
	elm_image_file_set(entry_button_ic, MF_ICON_ENTRY_FOLDER, NULL);
}

void mf_callback_entry_cb(void *data, Evas_Object * obj, void *event_info)
{
	mf_retm_if(data == NULL, "data is NULL");
	mfItemData_s *pItemData = (mfItemData_s *)data;

	struct appdata *ap = pItemData->ap;

	if (ap->mf_Status.path == NULL || ap->mf_Status.path->str == NULL) {
		return;
	}

	if (ap->mf_Status.flagNaviPush) {
		mf_debug("the transition of the naviframe is not finished");
		return;
	}

	GString *path = mf_fm_svc_wrapper_get_file_parent_path(pItemData->m_ItemName);
	mf_retm_if(path == NULL, "path is NULL");
	mf_retm_if(path->str == NULL, "path->str is NULL");

	int error_code = 0;
	int mmc_card = 0;
	bool result = FALSE;

	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "get navi structure failed");

	error_code = mf_util_is_mmc_on(&mmc_card);
	if (ap->mf_Status.more == MORE_DEFAULT
	    || ap->mf_Status.more == MORE_INTERNAL_COPY
	    || ap->mf_Status.more == MORE_INTERNAL_MOVE
    	    || ap->mf_Status.more == MORE_INTERNAL_COPY_MOVE) {
		if (mf_fm_svc_wrapper_is_dir(path)) {

#ifdef MYFILE_USB_OTG
			/* if click items under otg root path, it means some disk take into use*/
			error_code = 0;
			error_code = mf_file_attr_is_disk_link(path->str, &result);
			if (error_code == 0 && result == TRUE) {	/* a usb disk take into use */
				/*
				**we don't need to set the previous device flag to be false
				**because click cannot jump from one device to another
				*/
				mf_otg_set_node_using_flag(ap, path->str,TRUE);
			}
#endif
			/*set new path */
			if (ap->mf_Status.path != NULL) {
				g_string_free(ap->mf_Status.path, TRUE);
				ap->mf_Status.path = NULL;
			}
			ap->mf_Status.path = path;

			if (pNavi_s->pCurrentPath != NULL) {
				free(pNavi_s->pCurrentPath);
				pNavi_s->pCurrentPath = NULL;
			}
			pNavi_s->pCurrentPath = g_strdup(path->str);
			ap->mf_Status.view_type = mf_view_normal;
			mf_widget_refresh_view(ap);
		}
	}
}

void mf_callback_imf_preedit_change_cb(void *data, Evas_Object *obj, void *event_info)
{
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata *)data;

	Eina_Bool entry_empty = elm_entry_is_empty(ap->mf_MainWindow.pEntry);

	if (entry_empty) {
		elm_object_signal_emit(ap->mf_MainWindow.pEditField, "elm,state,eraser,hide", "elm");
	} else {
		elm_object_signal_emit(ap->mf_MainWindow.pEditField, "elm,state,eraser,show", "elm");
	}

	return;
#if 0
	SAFE_FREE(view_data->editing_text);
	const char *preedit_str = elm_entry_entry_get(obj);
	view_data->editing_text = g_strdup(preedit_str);
#endif
}
void mf_callback_imf_hide_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *entry = (Evas_Object *)data;
	elm_object_focus_set(entry, EINA_FALSE);
}

void mf_callback_naviframe_title_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	Elm_Object_Item *navi_it = event_info;
	if (!navi_it) return;


	Evas_Object *label = elm_object_item_part_content_get(navi_it, "elm.swallow.title");
	if (!label)
		return;
	elm_label_slide_go(label);
}




