/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Eina.h>
#include <Evas.h>
#include <Ethumb.h>
#include <assert.h>


#include <drm_client.h>
#include <drm_client_types.h>

#include "mf-main.h"
#include "mf-fs-util.h"
#include "mf-drm.h"

int mf_drm_is_converted_fl(char *path)
{
	drm_bool_type_e is_allowed = 0;
	drm_action_type_e action = DRM_IS_FORWARDING_ALLOWED;
	drm_action_allowed_data_s action_data;

	int ret = -1;
	memset(&action_data,0x0,sizeof(drm_action_allowed_data_s));

	SAFE_STRCPY(action_data.file_path, path);

	ret = drm_is_action_allowed(action,&action_data,&is_allowed);
	if(DRM_RETURN_SUCCESS == ret && DRM_TRUE == is_allowed){
		return true;
	}else{
		return false;
	}
}

int mf_drm_is_drm_file(const char *file_fullpath)
{
	if (file_fullpath == NULL)
		return MYFILE_ERR_DRM_PERMISSION_DENY;
	drm_bool_type_e drmFlag = DRM_FALSE;
	int res = 0;
	res = drm_is_drm_file(file_fullpath, &drmFlag);

	if (res == DRM_RETURN_SUCCESS && drmFlag == DRM_TRUE)
		return 0;
	else
		return MYFILE_ERR_DRM_PERMISSION_DENY;
}

drm_file_mime_type_e mf_drm_check_drm_mime_type(char *mime_type)
{
	gchar **result = NULL;
	if (mime_type != NULL) {
		result = g_strsplit(mime_type, "/", 0);
		if (result && *result) {
			if (g_strcmp0(*result, "audio") == 0) {
				g_strfreev(result);
				return MYFILE_DRM_RINGTONE_FILE;
			} else if (g_strcmp0(*result, "image") == 0) {
				g_strfreev(result);
				return MYFILE_DRM_IMAGE_FILE;
			} else {
				g_strfreev(result);
				return MYFILE_DRM_UNKNOW_FILE;
			}
		} else {
			g_strfreev(result);
			return MYFILE_DRM_UNKNOW_FILE;
		}
	} else {
		return MYFILE_DRM_UNKNOW_FILE;
	}
}

