/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */




#include "mf-main.h"
#include "mf-conf.h"
#include "mf-fs-util.h"
#include "mf-launch.h"
#include "mf-resource.h"
#include "mf-widget.h"
#include "mf-fm-svc-wrapper.h"

#ifndef SAFE_STRING_FREE
#define SAFE_STRING_FREE(x) do { if ((x) != NULL) { free(x); x = NULL; } } while (0)
#endif

#define EMAIL_UG_NAME			"email-composer-efl"
#define MESSAGE_UG_NAME			"msg-composer-efl"
#define BLUETOOTH_UG_NAME		"setting-bluetooth-efl"
#define DETAIL_UG_NAME			"myfile-detail-efl"
#define MYFILE_UG_NAME			"myfile-efl"



#define MF_SHARE_INTERNAL_LEN	12
#define MF_ARGV_SNS_XWINDOW_ID	"XWINDOW_ID"

static void mf_launch_send_via_appcontrol(void *data, char *path, MF_LOAD_UG_TYPE type);


static void __mf_launch_ug_layout_cb(ui_gadget_h ug, enum ug_mode mode, void *priv)
{
	MF_TRACE_BEGIN;
	mf_retm_if(priv == NULL, "priv is NULL");

	struct appdata *ap = (struct appdata *)priv;
	Evas_Object *base = NULL;

	base = ug_get_layout(ug);
	if (!base) {
		ug_destroy(ug);
		return;
	}

	switch (mode) {
	case UG_MODE_FULLVIEW:
		evas_object_size_hint_weight_set(base, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		//elm_win_resize_object_add(ap->mf_MainWindow.pWindow, base);
		evas_object_show(base);
		break;
	default:
		break;
	}
}

#if 0
static void __mf_launch_ug_result_cb(ui_gadget_h ug, bundle * result, void *priv)
{
	MF_TRACE_BEGIN;
	mf_retm_if(ug == NULL, "ug is NULL");
	mf_retm_if(priv == NULL, "priv is NULL");

	ug_destroy(ug);
	MF_TRACE_END;
}
#endif

static void __mf_launch_ug_myfile_result_cb(ui_gadget_h ug, service_h service, void *priv)
{
	MF_TRACE_BEGIN;
	mf_retm_if(priv == NULL, "priv is NULL");

	struct appdata *ap = (struct appdata *)priv;

	service_h service_r = ap->mf_Bundle.recv_service;
	service_h reply = NULL;
	char *ret = NULL;
	int service_ret = -1;

	if (service_create(&reply) != SERVICE_ERROR_NONE) {
		mf_debug("service_create failed!");
		return;
	}

	service_get_extra_data(service, "result", &ret);

	if (ret) {
		service_add_extra_data(reply, SERVICE_DATA_SELECTED, ret);
		service_ret = service_reply_to_launch_request(reply, service_r,
							      SERVICE_RESULT_SUCCEEDED);
	} else {
		service_ret = service_reply_to_launch_request(reply, service_r,
							      SERVICE_RESULT_CANCELED);
	}

	service_destroy(reply);
	if (service_ret != SERVICE_ERROR_NONE)
		mf_debug("service_reply_to_launch_request failed!");


	ap->mf_SharedGadget.ug = NULL;

	if (mf_navi_bar_is_navi_empty(ap, GET_SYS_STR(MF_LABEL_PHONE)) == TRUE) {
		elm_exit();
	} else {
		elm_win_lower(ap->mf_MainWindow.pWindow);
	}

	MF_TRACE_END;
}

Eina_Bool mf_launch_myfile_ug_exist(void *data)
{
	mf_retvm_if(data == NULL, EINA_FALSE, "data is NULL");

	struct appdata *ap = (struct appdata *)data;

	if (mf_fm_svc_wrapper_get_location(ap->mf_Status.path->str) == MYFILE_PHONE
	    && mf_navi_bar_is_navi_empty(ap, GET_SYS_STR(MF_LABEL_PHONE)) == TRUE) {
		return EINA_TRUE;
	} else {
		return EINA_FALSE;
	}
}

static void __mf_launch_ug_destory_cb(ui_gadget_h ug, void *priv)
{
	MF_TRACE_BEGIN;
	mf_retm_if(priv == NULL, "priv is NULL");

	ug_destroy(ug);
	struct appdata *ap = (struct appdata *)priv;
	ap->mf_SharedGadget.ug = NULL;
	ap->mf_Status.flagIME = EINA_TRUE;
	MF_TRACE_END;
}

static void __mf_launch_ug_myfile_ug_destory_cb(ui_gadget_h ug, void *priv)
{
	MF_TRACE_BEGIN;
	mf_retm_if(priv == NULL, "priv is NULL");

	ug_destroy(ug);
	struct appdata *ap = (struct appdata *)priv;
	ap->mf_SharedGadget.ug = NULL;

	if (mf_launch_myfile_ug_exist(ap)) {
		elm_exit();
	} else {
		elm_win_lower(ap->mf_MainWindow.pWindow);
	}
	MF_TRACE_END;
}


ui_gadget_h mf_launch_load_ug_myfile(void *data)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(data == NULL, NULL, "data is NULL");

	struct appdata *ap = (struct appdata *)data;
	ui_gadget_h ug = NULL;
	struct ug_cbs cbs = { 0, };
	if (ap->mf_SharedGadget.ug) {
		mf_debug("Already exits some Gallery UG called by me, destroy it first!");
		ug_destroy(ap->mf_SharedGadget.ug);
		ap->mf_SharedGadget.ug = NULL;
	}

	service_h service;
	int ret = 0;
	ret = service_create(&service);

	mf_retvm_if(ret != SERVICE_ERROR_NONE, NULL, "service create failed");

	cbs.layout_cb = __mf_launch_ug_layout_cb;
	cbs.result_cb = __mf_launch_ug_myfile_result_cb;
	cbs.destroy_cb = __mf_launch_ug_myfile_ug_destory_cb;
	cbs.priv = ap;

	UG_INIT_EFL(ap->mf_MainWindow.pWindow, UG_OPT_INDICATOR_ENABLE);

	service_add_extra_data(service, "path", ap->mf_Bundle.path);
	service_add_extra_data(service, "select_type", ap->mf_Bundle.select_type);
	service_add_extra_data(service, "file_type", ap->mf_Bundle.file_type);
	service_add_extra_data(service, "marked_mode", ap->mf_Bundle.marked_mode);
	service_add_extra_data(service, "drm_type", ap->mf_Bundle.drm_type);
	ug = ug_create(NULL, MYFILE_UG_NAME, UG_MODE_FULLVIEW, service, &cbs);

	if (ug == NULL) {
		mf_debug("Fail to Create UG");
		service_destroy(service);
		MF_TRACE_END;
		return NULL;
	} else {
		service_destroy(service);
		MF_TRACE_END;
		return ug;
	}
}

void mf_launch_sns_load(void *data, char *path, MF_LOAD_UG_TYPE type, char *file_count)
{
	mf_retm_if(data == NULL, "data is NULL");
	mf_retm_if(path == NULL, "path is NULL");

	struct appdata *ap = (struct appdata *)data;
	const char *pkg_name = NULL;
	service_h service = NULL;
	service_create(&service);
	if (service == NULL) {
		return;
	}
	switch (type) {
	default:
		break;
	}
	service_set_package(service, pkg_name);
	int ret = service_send_launch_request(service, NULL, NULL);

	if (ret != SERVICE_ERROR_NONE) {
		mf_debug("service_send_launch_request failed[%d]!", ret);
	}

	service_destroy(service);

	return;
}
ui_gadget_h mf_launch_load_ug(void *data, char *path, MF_LOAD_UG_TYPE type, char *file_count)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(data == NULL, NULL, "data is NULL");
	mf_retvm_if(path == NULL, NULL, "path is NULL");

	if (type == MF_LOAD_UG_EMAIL
	) {
		mf_launch_send_via_appcontrol(data, path, type);
		return NULL;
	}
	struct appdata *ap = (struct appdata *)data;
	ui_gadget_h ug = NULL;
	struct ug_cbs cbs = { 0, };

	service_h service;
	int ret = 0;
	ret = service_create(&service);

	mf_retvm_if(ret != SERVICE_ERROR_NONE, NULL, "service create failed");

	cbs.layout_cb = __mf_launch_ug_layout_cb;
	//cbs.result_cb = __mf_launch_ug_result_cb;
	cbs.destroy_cb = __mf_launch_ug_destory_cb;
	cbs.priv = ap;

	UG_INIT_EFL(ap->mf_MainWindow.pWindow, UG_OPT_INDICATOR_ENABLE);

	switch (type) {
	case MF_LOAD_UG_MESSAGE:
		service_add_extra_data(service, "ATTACHFILE", path);
		ug = ug_create(NULL, MESSAGE_UG_NAME, UG_MODE_FULLVIEW, service, &cbs);
		break;
	case MF_LOAD_UG_BLUETOOTH:
		service_add_extra_data(service, "launch-type", "send");
		service_add_extra_data(service, "filecount", file_count);
		service_add_extra_data(service, "files", path);
		ug = ug_create(NULL, BLUETOOTH_UG_NAME, UG_MODE_FULLVIEW, service, &cbs);
		break;
	case MF_LOAD_UG_DETAIL:
		service_add_extra_data(service, "Path", path);
		ug = ug_create(NULL, DETAIL_UG_NAME, UG_MODE_FULLVIEW, service, &cbs);
		break;

	default:
		break;
	}

	if (ug == NULL) {
		mf_debug("Fail to Create UG : %d", type);
		service_destroy(service);
		MF_TRACE_END;
		return NULL;
	} else {
		if (type == MF_LOAD_UG_DETAIL) {
			ap->mf_Status.flagIME = EINA_FALSE;
		}
		service_destroy(service);
		MF_TRACE_END;
		return ug;
	}
}

static void __mf_launch_service_reply_cb(service_h request, service_h reply, service_result_e result, void *user_data)
{
	MF_TRACE_BEGIN;
	switch(result) {
	case SERVICE_RESULT_SUCCEEDED:
		mf_debug("SERVICE_RESULT_SUCCEEDED");
		break;
	case SERVICE_RESULT_FAILED:
		mf_debug("SERVICE_RESULT_FAILED");
		break;
	case SERVICE_RESULT_CANCELED:
		mf_debug("SERVICE_RESULT_CANCELED");
		break;
	default:
		mf_debug("Unhandled value: %d!", result);
		break;
	}
	MF_TRACE_END;
}

static void mf_launch_send_via_appcontrol(void *data, char *path, MF_LOAD_UG_TYPE type)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	mf_retm_if(path == NULL, "path is NULL");


	struct appdata *ap = (struct appdata *)data;
	const char *pkg_name = NULL;
        service_h service = NULL;
        int ret = service_create(&service);
        if (ret != SERVICE_ERROR_NONE) {
                mf_debug("service_create()... [0x%x]", ret);
                goto END;
        }

        ret = service_set_operation(service, SERVICE_OPERATION_SEND);
        if (ret != SERVICE_ERROR_NONE) {
                mf_debug("service_set_operation()... [0x%x]", ret);
                goto END;
        }

        switch (type) {
        case MF_LOAD_UG_EMAIL:
                ret = service_add_extra_data(service, "RUN_TYPE", "5");
                if (ret != SERVICE_ERROR_NONE) {
                        mf_debug("service_add_extra_data()... [0x%x]", ret);
                        goto END;
                }
		pkg_name = EMAIL_UG_NAME;

		ret = service_set_app_id(service, pkg_name);
		if (ret != SERVICE_ERROR_NONE) {
			mf_debug("service_set_app_id()... [0x%x]", ret);
			goto END;
		}
		 /* set window */
		ret = service_set_window(service, elm_win_xwindow_get(ap->mf_MainWindow.pWindow));
		if (ret != SERVICE_ERROR_NONE) {
			mf_debug("service_set_window()... [0x%x]", ret);
			goto END;
		}
                break;
	default:
                mf_debug("Not supported type.. [%d]", type);
                goto END;
        }

        /* attatchment */
        ret = service_set_uri(service, path);
        if (ret != SERVICE_ERROR_NONE) {
                mf_debug("service_set_uri()... [0x%x]", ret);
                goto END;
        }

        /* appcontrol name */

	ret = service_send_launch_request(service, __mf_launch_service_reply_cb, NULL);
	if (ret != SERVICE_ERROR_NONE) {
		mf_debug("service_send_launch_request()... [0x%x]", ret);
		goto END;
	}


END:
	if (service) {
		service_destroy(service);
		service = NULL;
	}

}




/******************************
** Prototype    : mf_launch_service
** Description  :
** Input        : void *data
**                char *path
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void result_cb(service_h request, service_h reply, service_result_e result, void *user_data)
{
	mf_debug("result is [%d]", result);
}

int mf_launch_service(void *data, char *path)
{
	mf_retvm_if(data == NULL, LAUNCH_TYPE_FAIL, "data is NULL");
	mf_retvm_if(path == NULL, LAUNCH_TYPE_FAIL, "path is NULL");

	fsFileType category = FILE_TYPE_NONE;
	int ret = 0;
	/* defence code */
	if (ecore_file_file_get(path) == NULL) {
		return LAUNCH_TYPE_FAIL;
	}
	if (ecore_file_is_dir(path)) {
		return LAUNCH_TYPE_DIR;
	}


	service_h service;

	ret = service_create(&service);
        if (ret != SERVICE_ERROR_NONE) {
                mf_debug("service_create()... [0x%x]", ret);
                goto END;
        }
	mf_file_attr_get_file_category(path, &category);
	if (category == FILE_TYPE_HTML) {
		char *html_path = g_strconcat("file://", path, NULL);
		ret = service_set_uri(service, html_path);
		SAFE_FREE_CHAR(html_path);
	} else if (category == FILE_TYPE_IMAGE) {
		service_add_extra_data(service, "View By", "By Folder");
		ret = service_set_uri(service, path);
	} else {
		ret = service_set_uri(service, path);
	}

	if (ret != SERVICE_ERROR_NONE) {
		mf_debug("service_set_uri()... [0x%x]", ret);
		goto END;
	}
	ret = service_set_operation(service, SERVICE_OPERATION_VIEW);
	if (ret != SERVICE_ERROR_NONE) {
		mf_debug("service_set_operation()... [0x%x]", ret);
		goto END;
	}
	ret = service_send_launch_request(service, result_cb, NULL);
	if (ret == SERVICE_ERROR_APP_NOT_FOUND) {
		mf_popup_create_popup(data, POPMODE_TEXT, NULL, GET_SYS_STR(MF_LABEL_NO_APP), NULL, NULL, NULL, NULL, NULL);
	} else if (ret != SERVICE_ERROR_NONE) {
		mf_debug("service_send_launch_request()... [0x%x]", ret);
	} else {
		mf_util_set_recent_file(path);
		mf_debug("service_send_launch_request()... [0x%x]", ret);
	}

END:
	if (service) {
		service_destroy(service);
	}
	return LAUNCH_TYPE_FORK;

}

