/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include "mf-sensor.h"
#include "mf-dlog.h"
#include <sensors.h>

static sensor_h g_sensor;

int mf_sensor_init(void)
{
	int res = SENSOR_ERROR_NONE;
	bool doubletap = 0;

	res = sensor_is_supported(SENSOR_MOTION_DOUBLETAP, &doubletap);
	MF_CHECK_VAL(res == SENSOR_ERROR_NONE, res);

	if(!doubletap)
		mf_error("sensor not supported. doubletap: %d", doubletap);

	if(doubletap)
	{
		res = sensor_create(&g_sensor);
		if(res != SENSOR_ERROR_NONE)
			mf_error("Fail sensor_create");
	}
	else
		mf_error("motion not supported");

	return res;
}

int mf_sensor_finalize(void)
{
	int res = SENSOR_ERROR_NONE;
	MF_CHECK_VAL(g_sensor, -1);
	res = sensor_destroy(g_sensor);
	g_sensor = NULL;

	return res;
}

int mf_sensor_start(mf_motion_type type)
{
	MF_CHECK_VAL(g_sensor, -1);

	int res = SENSOR_ERROR_NONE;
	switch(type)
	{
	case MF_MOTION_FACEDOWN:
		res = sensor_start(g_sensor, SENSOR_MOTION_FACEDOWN);
		break;
	case MF_MOTION_DOUBLETAP:
		res = sensor_start(g_sensor, SENSOR_MOTION_DOUBLETAP);
		break;
	default:
		mf_error("Invalid motion type: 0x%x", type);
	}

	if(res != SENSOR_ERROR_NONE)
		mf_error("Fail sensor_start: type: %d ", type);

	mf_debug("sensor started");
	return res;
}

int mf_sensor_stop(mf_motion_type type)
{
	MF_CHECK_VAL(g_sensor, -1);

	int res = SENSOR_ERROR_NONE;
	switch(type)
	{
	case MF_MOTION_FACEDOWN:
		res = sensor_stop(g_sensor, SENSOR_MOTION_FACEDOWN);
		break;
	case MF_MOTION_DOUBLETAP:
		res = sensor_stop(g_sensor, SENSOR_MOTION_DOUBLETAP);
		break;
	default:
		mf_error("Invalid motion type: 0x%x", type);
	}

	if(res != SENSOR_ERROR_NONE)
		mf_error("Fail sensor_stop: type: %d ", type);

	mf_debug("sensor stopped");
	return res;
}

int mf_sensor_set_callback(mf_motion_type type, mf_sensor_cb callback, void *user_data)
{
	MF_CHECK_VAL(g_sensor, -1);

	int res = SENSOR_ERROR_NONE;
	switch(type)
	{
	case MF_MOTION_FACEDOWN:
		res = sensor_motion_facedown_set_cb(g_sensor, callback, user_data);
		break;
	case MF_MOTION_DOUBLETAP:
		res = sensor_motion_doubletap_set_cb(g_sensor, callback, user_data);
		break;
	default:
		mf_error("Invalid motion type: 0x%x", type);
		break;
	}
	if(res != SENSOR_ERROR_NONE)
		mf_error("Error: set sensor callback res = 0x%x");
	return res;
}

int mf_sensor_unset_callback(mf_motion_type type)
{
	MF_CHECK_VAL(g_sensor, -1);

	int res = SENSOR_ERROR_NONE;
	switch(type)
	{
	case MF_MOTION_FACEDOWN:
		res = sensor_motion_facedown_unset_cb(g_sensor);
		break;
	case MF_MOTION_DOUBLETAP:
		res = sensor_motion_doubletap_unset_cb(g_sensor);
		break;
	default:
		mf_error("Invalid motion type: 0x%x", type);
		break;
	}
	if(res != SENSOR_ERROR_NONE)
		mf_error("Error: set sensor callback res = 0x%x");
	return res;
}

