/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */







#include <vconf.h>
#include <pthread.h>
#include <glib.h>
#include <assert.h>
#include <power.h>
#include <runtime_info.h>



#include "mf-util.h"
#include "mf-widget.h"
#include "mf-fm-svc-wrapper.h"
#include "mf-fs-util.h"
#include "mf-copy.h"
#include "mf-resource.h"

#ifndef VCONFKEY_MYFILE_LIST_BY
#define VCONFKEY_MYFILE_LIST_BY		"db/private/org.tizen.myfile/list_by"
#endif

#ifndef VCONFKEY_MYFILE_VIEW_STYLE
#define VCONFKEY_MYFILE_VIEW_STYLE	"db/private/org.tizen.myfile/view_style"
#endif

#ifndef VCONFKEY_MYFILE_EXTENSION_STATE
#define VCONFKEY_MYFILE_EXTENSION_STATE		"db/private/org.tizen.myfile/extension_state"
#endif

#ifndef VCONFKEY_MYFILE_RECENT_FILE
#define VCONFKEY_MYFILE_RECENT_FILE		"db/private/org.tizen.myfile/recent_file"
#endif


#define MF_TIMER_INTERVAL_VIBRATION 0.5
#define MF_VIBRATION_DEVICE 0
#define MF_VIBRATION_DURATION 500
#define DEF_BUF_LEN            (512)


extern struct appdata *temp_data;
extern pthread_mutex_t pb_mutex;
extern pthread_mutex_t refresh_mutex;
struct navi_content_pop_s {
	myfileNaviBar *pNavi;
	struct appdata *ap;
};

/******************************
** Prototype    : __mf_util_is_mmc_supported
** Description  :
** Input        : int* supported
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static int __mf_util_is_mmc_supported(int *supported)
{
	int data = -1;
	int error_code = 0;

	if (supported == NULL) {
		mf_debug("supported == NULL");
		return MYFILE_ERR_SRC_ARG_INVALID;
	}

	*supported = 0;

	error_code = vconf_get_int(VCONFKEY_SYSMAN_MMC_STATUS, &data);
	if (error_code) {
		return MYFILE_ERR_GET_VCONF_FAIL;
	}

	if (data != VCONFKEY_SYSMAN_MMC_MOUNTED) {
		*supported = 0;
	} else {
		*supported = 1;
	}

	return MYFILE_ERR_NONE;
}

/******************************
** Prototype    : mf_util_is_mmc_on
** Description  :
** Input        : int *mmc_card
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
int mf_util_is_mmc_on(int *mmc_card)
{
	int error_code = 0;

	error_code = __mf_util_is_mmc_supported(mmc_card);
	/*
	   if(mmc_card == TRUE){
	   mf_debug("mmc is on phone\n");
	   get_phone_status(&ret);
	   } */
	mf_debug("**************mmc_card is [%d]", *mmc_card);
	return error_code;

}

#ifdef MYFILE_USB_OTG
static int __mf_util_is_otg_supported(int *supported)
{

	if (supported == NULL) {
		mf_debug("supported == NULL");
		return MYFILE_ERR_SRC_ARG_INVALID;
	}

	*supported = 0;

	if(!ecore_file_exists(OTG_FOLDER) || ecore_file_dir_is_empty(OTG_FOLDER)) {
		*supported = 0;
	} else {
		*supported = 1;
	}

	return MYFILE_ERR_NONE;
}

int mf_util_is_otg_on(int *otg)
{
	int error_code = 0;

	error_code = __mf_util_is_otg_supported(otg);
	mf_debug("**************otg is [%d]", *otg);
	return error_code;
}
#endif

void mf_util_action_storage_insert(void *data, char *pItemLabel)
{

	mf_debug();
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	fsNodeInfo *pNode = NULL;

	if (ap->mf_Status.iStorageState & MYFILE_MMC) {
		pNode = (fsNodeInfo *) malloc(sizeof(fsNodeInfo));
		memset(pNode, 0, sizeof(fsNodeInfo));
		/*set path */
		pNode->path = g_strdup(STORAGE_PARENT);
		pNode->name = g_strdup(MMC_NAME);
		pNode->type = FILE_TYPE_DIR;
		ap->mf_FileOperation.folder_list = eina_list_append(ap->mf_FileOperation.folder_list, pNode);
	}
	if (ap->mf_Status.iStorageState & MYFILE_OTG) {
		pNode = (fsNodeInfo *) malloc(sizeof(fsNodeInfo));
		memset(pNode, 0, sizeof(fsNodeInfo));
		/*set path */
		pNode->path = g_strdup(STORAGE_PARENT);
		pNode->name = g_strdup(OTG_NAME);
		pNode->type = FILE_TYPE_DIR;
		ap->mf_FileOperation.folder_list = eina_list_append(ap->mf_FileOperation.folder_list, pNode);
	}


	if (ap->mf_Status.flagViewType == MF_VIEW_STYLE_LIST || ap->mf_Status.flagViewType == MF_VIEW_SYTLE_LIST_DETAIL) {
		if (ap->mf_Status.iStorageState & MYFILE_MMC) {
			mf_widget_item_append(ap->mf_MainWindow.pNaviGenlist, pNode, ap);
		}

		if (ap->mf_Status.iStorageState & MYFILE_OTG) {
			mf_widget_item_append(ap->mf_MainWindow.pNaviGenlist, pNode, ap);
		}
	} else {
		if (ap->mf_Status.iStorageState & MYFILE_MMC) {
			mf_widget_item_append(ap->mf_MainWindow.pNaviGengrid, pNode, ap);
		}

		if (ap->mf_Status.iStorageState & MYFILE_OTG) {
			mf_widget_item_append(ap->mf_MainWindow.pNaviGengrid, pNode, ap);
		}

	}
#ifdef MYFILE_SPLIT_VIEW
	if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE)
	{
		if (g_strcmp0(pItemLabel, GET_SYS_STR(MF_LABEL_MMC)) == 0) {
			Elm_Object_Item *it = NULL;
			it = mf_genlist_split_root_item_append(ap, ap->mf_MainWindow.sSplitData.pSplitLeftGenlist, MEMORY_FOLDER);
			ap->mf_MainWindow.sSplitData.mmc_item = it;
		}

		if (g_strcmp0(pItemLabel, GET_STR(MF_LABEL_OTG)) == 0) {
			Elm_Object_Item *it = NULL;
			it = mf_genlist_split_root_item_append(ap, ap->mf_MainWindow.sSplitData.pSplitLeftGenlist, OTG_FOLDER);
			ap->mf_MainWindow.sSplitData.otg_item = it;
		}
	}
#endif
}

/******************************
** Prototype    : mf_util_operation_alloc_failed
** Description  :
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_util_operation_alloc_failed(void *data)
{
	struct appdata *ap = (struct appdata *)data;

	ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT, NULL, GET_STR(MF_MSG_MEMORY_NOT_ENOUGH),
							       NULL, NULL, NULL, (Evas_Smart_Cb) elm_exit, NULL);
}
/**ecore_job handler related**/
void mf_util_add_to_handler_list(void *data, Ecore_Job *handler, char *label)
{
	struct appdata *ap = (struct appdata *)data;
	assert(ap);
	mf_retm_if(handler == NULL, "handler is NULL");
	mf_retm_if(label == NULL, "label is NULL");

	myfileEcoreHandleNode *newNode = calloc(1, sizeof(myfileEcoreHandleNode));

	if (newNode == NULL) {
		mf_util_operation_alloc_failed(ap);
		return;
	}

	newNode->handler = handler;
	newNode->navi_label = g_strdup(label);

	if (newNode->navi_label == NULL) {
		SAFE_FREE_CHAR(newNode->navi_label);
		SAFE_FREE_CHAR(newNode);
		mf_util_operation_alloc_failed(ap);
		return;
	}

	ap->mf_FileOperation.job_pop_list = eina_list_append(ap->mf_FileOperation.job_pop_list, newNode);
}

void mf_util_delete_from_handler_list(void *data, char *label, bool flag_del)
{
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	Eina_List *l;
	void *node;

	EINA_LIST_FOREACH(ap->mf_FileOperation.job_pop_list, l, node) {
		myfileEcoreHandleNode *pNode = (myfileEcoreHandleNode *)node;
		if (pNode != NULL) {
			if (g_strcmp0(pNode->navi_label, label) == 0) {
				if (pNode->handler != NULL) {
					if (flag_del == TRUE)
						ecore_job_del(pNode->handler);
					pNode->handler = NULL;
				}
				ap->mf_FileOperation.job_pop_list = eina_list_remove(ap->mf_FileOperation.job_pop_list, pNode);
				SAFE_FREE_CHAR(pNode->navi_label);
				SAFE_FREE_CHAR(pNode);
				break;
			}
		}
	}
}

void mf_util_clear_handler_list(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	Eina_List *l;
	void *node;

	EINA_LIST_FOREACH(ap->mf_FileOperation.job_pop_list, l, node) {
		myfileEcoreHandleNode *pNode = (myfileEcoreHandleNode *)node;
		if(pNode != NULL) {
			if (pNode->handler != NULL) {
				ecore_job_del(pNode->handler);
				pNode->handler = NULL;
			}
			SAFE_FREE_CHAR(pNode->navi_label);
			SAFE_FREE_CHAR(pNode);
		}
	}
}
/**end of ecore_job handler related**/


/******************************
** Prototype    : mf_util_refresh_screen
** Description  :
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
int mf_util_refresh_screen(void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	Eina_List *l = NULL;
	void *item = NULL;
	myfileNaviBar *pNavi_s = NULL;
	myfileNaviBar *pNavi_toshow = NULL;

	if (ap->mf_Status.more == MORE_IDLE_DELETE) {
		if (ap->mf_Status.preMore == MORE_DEFAULT) {
			ap->mf_Status.more = MORE_DEFAULT;
			mf_widget_refresh_view(ap);
			if (ap->mf_FileOperation.iOperationSuccessFlag) {
				mf_popup_indicator_popup(ap, GET_SYS_STR(MF_MSG_DELETE_SUCCESS));
			} else {
				mf_popup_indicator_popup(ap, ap->mf_FileOperation.pOperationMsg);
			}

		} else {
			ap->mf_Status.more = ap->mf_Status.preMore;
			if (ap->mf_FileOperation.idle_delete_item) {
				mfItemData_s *params = elm_object_item_data_get(ap->mf_FileOperation.idle_delete_item);
				if (params && params->m_ItemName) {
					char *path = params->m_ItemName->str;
					if (path && !ecore_file_exists(path)) {
						elm_object_item_del(ap->mf_FileOperation.idle_delete_item);
					}
				}
			}
			ap->mf_FileOperation.idle_delete_item = NULL;
			if (ap->mf_FileOperation.iOperationSuccessFlag) {
				mf_popup_indicator_popup(ap, GET_SYS_STR(MF_MSG_DELETE_SUCCESS));
			} else {
				mf_popup_indicator_popup(ap, ap->mf_FileOperation.pOperationMsg);
			}
			return MYFILE_ERR_NONE;
		}
	} else {
		/*0     set state to be Default */
		int current_more = ap->mf_Status.more;
		ap->mf_Status.more = MORE_DEFAULT;
		/*1     recover all the navigation bar */
		EINA_LIST_FOREACH(ap->mf_MainWindow.plistNaviBar, l, item) {
			pNavi_s = (myfileNaviBar *) item;
			if (pNavi_s == NULL) {
				continue;
			}
			/*1.1	pop the temporary view in each navigation bar*/
			if (pNavi_s->naviFlagEditJoin == TRUE) {
				pNavi_s->naviFlagEditJoin = FALSE;
			}
			/*1.2	reset current path for each navigation bar*/
			SAFE_FREE_CHAR(pNavi_s->pCurrentPath);
			pNavi_s->pCurrentPath = g_strdup(pNavi_s->pre_operation.path->str);

			/*2.1   free the backup data structure */
			g_string_free(pNavi_s->pre_operation.path, TRUE);
			pNavi_s->pre_operation.path = NULL;
			memset(&(pNavi_s->pre_operation), 0, sizeof(oper_record));

			if (pNavi_s->naviFlagInUse == TRUE) {
				/*2.2 set pNavi_toshow for refreshing */
				pNavi_toshow = pNavi_s;
			}
		}
		if (pNavi_toshow == NULL) {
			mf_debug("pNavi_toshow is NULL");
			return MYFILE_ERR_GET_NAVI_FAILED;
		} else {
			/*3.    refresh the content of the view */
				if (pNavi_toshow->pCurrentPath != NULL) {
					free(pNavi_toshow->pCurrentPath);
					pNavi_toshow->pCurrentPath = NULL;
				}
				pNavi_toshow->pCurrentPath = g_strdup(ap->mf_Status.path->str);
				mf_widget_refresh_view(ap);
		}
		char *label = NULL;
		if (ap->mf_FileOperation.iOperationSuccessFlag) {
			switch(current_more) {
			case MORE_DATA_COPYING:
				label =  GET_SYS_STR(MF_MSG_COPY_SCCESS);
				break;
			case MORE_DATA_MOVING:
				label =  GET_SYS_STR(MF_MSG_MOVE_SUCCESS);
				break;
			case MORE_IDLE_DELETE:
			case MORE_DELETE:
				label =  GET_SYS_STR(MF_MSG_DELETE_SUCCESS);
				break;
			default:
				break;
			}
		} else {
			label =  ap->mf_FileOperation.pOperationMsg;
		}

		mf_popup_indicator_popup(ap, label);
		SAFE_FREE_CHAR(ap->mf_FileOperation.pOperationMsg);
	}
	if (ap->mf_Status.flagNoContent == 1) {
		if (ap->mf_MainWindow.pNaviCtrlBar) {
			mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_NOCONTENT_VIEW, TRUE);
			//mf_ctrl_bar_item_set_disable(ap->mf_MainWindow.pNaviCtrlBar, CTRL_DISABLE_NOCONTENT_VIEW, true);
		}
	}
	if (ap->mf_MainWindow.pFinishPopup) {
		evas_object_del(ap->mf_MainWindow.pFinishPopup);
		ap->mf_MainWindow.pFinishPopup = NULL;
	}
	if (ap->mf_MainWindow.pMmcRemovedPopup) {
		evas_object_del(ap->mf_MainWindow.pMmcRemovedPopup);
		ap->mf_MainWindow.pMmcRemovedPopup = NULL;
	}
	MF_TRACE_END;
	return MYFILE_ERR_NONE;
}



/******************************
** Prototype    : mf_util_check_forbidden_operation
** Description  :
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
bool mf_util_check_forbidden_operation(void *data)
{
	/*
	 **    forbidden operations includes:
	 **    1.  Recursion move/copy
	 **    2.  move with same file in the same folder
	 */

	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	Eina_List *l = NULL;
	GString *pNode = NULL;
	GString *from = NULL;
	GString *to = ap->mf_FileOperation.destination;
	Eina_List *pSourceList = ap->mf_FileRecordList.value_saver;
	const char *message = NULL;

	EINA_LIST_FOREACH(pSourceList, l, pNode) {
		if (pNode) {
			from = pNode;
			if (mf_fm_svc_wrapper_detect_recursion(from, to) == MYFILE_REPORT_RECURSION_DETECT) {
				if (ap->mf_Status.more == MORE_DATA_MOVING) {
					message = GET_STR(MF_MSG_MOVE_FAILED1);
					ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT, NULL, message,
											       NULL, NULL, NULL, NULL, NULL);

					ap->mf_Status.more = MORE_INTERNAL_MOVE;
				} else if (ap->mf_Status.more == MORE_DATA_COPYING) {
					message = GET_STR(MF_MSG_COPY_FAILED1);
					ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT, NULL, message,
											       NULL, NULL, NULL, NULL, NULL);

					ap->mf_Status.more = MORE_INTERNAL_COPY;
				}
				ap->mf_Status.more = MORE_INTERNAL_COPY_MOVE;
				return false;
			}
		}
	}

	from = ap->mf_FileOperation.source;
	if (ap->mf_Status.more == MORE_DATA_MOVING) {
		if (!strcmp(from->str, to->str)) {
			message = GET_STR(MF_MSG_MOVE_FAILED2);
			ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT, NULL, message, NULL, NULL, NULL, NULL, NULL);
			ap->mf_Status.more = MORE_INTERNAL_MOVE;
			return false;
		}
	}
	return true;
}

/******************************
** Prototype    : mf_util_check_disk_space
** Description  :
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
int mf_util_check_disk_space(void *data)
{
	mf_debug();
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	MF_STORAGE state = MYFILE_NONE;
	unsigned long free_space = 0;

	if (ap->mf_Status.more == MORE_CREATE_FOLDER) {
		state = mf_fm_svc_wrapper_get_location(ap->mf_Status.path->str);
		free_space = mf_fm_svc_wrapper_get_free_space(state);
		mf_debug("free size is %lu\n", free_space);
		/*
		 **     in vfat fs type, sector size is 16K.
		 **     it is to say that the limited size of the free space should be 16K
		 **     or it will report space used up.
		 **     check free_space == 0 can make sure at least 16K is free on the disk
		 **     while every dir takes 4K
		 */
		if (free_space == 0) {
			ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT_BTN, NULL,
									       GET_STR(MF_MSG_NOT_ENOUGH_SPACE), GET_SYS_STR(MF_BUTTON_LABEL_OK),
									       NULL, NULL, mf_callback_warning_popup_cb, ap);
			mf_debug("Not Enough free size\n");
			return MYFILE_ERR_NO_FREE_SPACE;
		}
	}

	return MYFILE_ERR_NONE;
}


/******************************
** Prototype    : mf_util_get_eina_list_len
** Description  :
** Input        : const Eina_List* list
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
int mf_util_get_eina_list_len(const Eina_List *list)
{
	return eina_list_count(list);
}


void mf_util_free_eina_list_with_data(Eina_List **list, MYFILE_CONTENT_TYPE type)
{
	if (list == NULL || *list == NULL) {
		return;
	}

	void *pNode = NULL;
	Eina_List *l = NULL;

	switch (type) {
	case MYFILE_TYPE_GSTRING:
		EINA_LIST_FOREACH(*list, l, pNode) {
			if (pNode != NULL) {
				g_string_free(pNode, TRUE);
				pNode = NULL;
			}
		}
		break;
	case MYFILE_TYPE_CHAR:
		EINA_LIST_FOREACH(*list, l, pNode) {
			if (pNode != NULL) {
				free(pNode);
				pNode = NULL;
			}
		}
		break;
	case MYFILE_TYPE_FSNODE:
		EINA_LIST_FOREACH(*list, l, pNode) {
			fsNodeInfo *Node = (fsNodeInfo *)pNode;
			if (Node != NULL) {
				SAFE_FREE_CHAR(Node->path);
				SAFE_FREE_CHAR(Node->name);
				SAFE_FREE_CHAR(Node->ext);

				free(Node);
				Node = NULL;
			}
		}
		break;
	default:
		break;
	}

	eina_list_free(*list);
	*list = NULL;
	return;
}

int mf_util_icu_init(void *data)
{
	MF_TRACE_BEGIN;
	assert(data);
	struct appdata *ap = (struct appdata *)data;
	char *skeleton;
	UChar customSkeleton[MYFILE_ICU_ARR_LENGTH] = { '\0' };
	int skeletonLength = 0;
	char *timezone_id = NULL;
	UChar utimezone_id[BUFF_SIZE] = {0,};

	UErrorCode status = U_ZERO_ERROR;
	UDateTimePatternGenerator *generator;
	UDateFormat *formatter;
	UChar bestPattern[MYFILE_ICU_ARR_LENGTH] = { 0, };
	int32_t bestPatternLength;
	bool timeformat = false;

	timezone_id = vconf_get_str(VCONFKEY_SETAPPL_TIMEZONE_ID);

	if (timezone_id == NULL) {
		return MYFILE_ERR_INVALID_ARG;
	}

	u_uastrcpy(utimezone_id, timezone_id);

	ucal_setDefaultTimeZone(utimezone_id , &status);

	int ret = runtime_info_get_value_bool(RUNTIME_INFO_KEY_24HOUR_CLOCK_FORMAT_ENABLED, &timeformat);

	if (ret != 0) {
		mf_debug();
		return MYFILE_ERR_SETTING_RESET_FAIL;
	}

	if (!timeformat) {
		skeleton = g_strdup(MYFILE_DATEFORMAT_12);
	} else {
		skeleton = g_strdup(MYFILE_DATEFORMAT_24);
	}

	skeletonLength = strlen(skeleton);
	if (u_uastrncpy(customSkeleton, skeleton, skeletonLength) == NULL) {
		return MYFILE_ERR_INVALID_ARG;
	}

	char *region = vconf_get_str(VCONFKEY_REGIONFORMAT);
	if (region == NULL) {
		return MYFILE_ERR_INVALID_ARG;
	}

	uloc_setDefault(getenv("LC_TIME"), &status);

	generator = udatpg_open(region, &status);
	if (generator == NULL) {
		return MYFILE_ERR_INVALID_ARG;
	}
	ap->mf_Status.generator = generator;

	bestPatternLength = udatpg_getBestPattern(generator, customSkeleton, u_strlen(customSkeleton), bestPattern, MYFILE_ICU_ARR_LENGTH, &status);
	if (bestPatternLength <= 0) {
		return MYFILE_ERR_INVALID_ARG;
	}

	formatter = udat_open(UDAT_IGNORE, UDAT_IGNORE, region, NULL, -1, bestPattern, -1, &status);
	if (formatter == 0) {
		return MYFILE_ERR_INVALID_ARG;
	}
	ap->mf_Status.formatter = formatter;
	ap->mf_Status.flagIcuInit = TRUE;

	MF_TRACE_END;
	return MYFILE_ERR_NONE;
	}

void mf_util_icu_finalize(void *data)
{
	MF_TRACE_BEGIN;
	assert(data);
	struct appdata *ap = (struct appdata *)data;

	if (ap->mf_Status.generator != NULL) {
		udatpg_close(ap->mf_Status.generator);
		ap->mf_Status.generator = NULL;
	}

	if (ap->mf_Status.formatter) {
		udat_close(ap->mf_Status.formatter);
		ap->mf_Status.formatter = NULL;
	}

	ap->mf_Status.flagIcuInit = FALSE;
	MF_TRACE_END;
}

char *mf_util_icu_translate(void *data, UDate date)
{
	MF_TRACE_BEGIN;
	assert(data);
	struct appdata *ap = (struct appdata *)data;

	if (ap->mf_Status.flagIcuInit == FALSE)
	{
		if(MYFILE_ERR_NONE !=mf_util_icu_init(ap))
		{
			mf_debug("invalid icu init check int");
			return NULL;
		}
	}

	UErrorCode status = U_ZERO_ERROR;
	UChar formatted[MYFILE_ICU_ARR_LENGTH] = { 0, };
	char formattedString[MYFILE_ICU_ARR_LENGTH] = { 0, };
	int32_t formattedLength;

	formattedLength = udat_format(ap->mf_Status.formatter, date, formatted, MYFILE_ICU_ARR_LENGTH, NULL, &status);
	if (formattedLength <= 0) {
		return NULL;
	}

	u_austrcpy(formattedString, formatted);
	if (strlen(formattedString) == 0) {
		return NULL;
	}

	MF_TRACE_END;
	return g_strdup(formattedString);
}

int mf_util_get_vconf_value(MYFILE_VCONF_TYPE type, int *value)
{
	int ret = MYFILE_ERR_NONE;
	switch (type) {
	case VCONF_TYPE_SORT_TYPE:
		{
			ret = vconf_get_int(VCONFKEY_MYFILE_LIST_BY, value);
			if (ret < 0) {
				mf_warnig("fail to get list_by value, set it default");
				vconf_set_int(VCONFKEY_MYFILE_LIST_BY, MYFILE_SORT_BY_DATE_R2O);
				*value = MYFILE_SORT_BY_DATE_R2O;
			} else {
				if (*value <= MYFILE_SORT_BY_NONE || *value >= MYFILE_SORT_BY_MAX) {
					mf_warnig("invaild list by type[%d], set it default", *value);
					vconf_set_int(VCONFKEY_MYFILE_LIST_BY, MYFILE_SORT_BY_DATE_R2O);
					*value = MYFILE_SORT_BY_DATE_R2O;
				}
			}
			return MYFILE_ERR_NONE;
		}
		break;
	case VCONF_TYPE_VIEW_STYLE:
		ret = vconf_get_int(VCONFKEY_MYFILE_VIEW_STYLE, value);
		if (ret < 0) {
			mf_warnig("fail to get list_by value, set it default");
			vconf_set_int(VCONFKEY_MYFILE_VIEW_STYLE, MF_VIEW_STYLE_LIST);
			*value = MF_VIEW_STYLE_LIST;
		} else {
			if (*value < MF_VIEW_STYLE_LIST || *value > MF_VIEW_STYLE_THUMBNAIL) {
				mf_warnig("invaild list by type[%d], set it default", *value);
				vconf_set_int(VCONFKEY_MYFILE_VIEW_STYLE, MF_VIEW_STYLE_LIST);
				*value = MF_VIEW_STYLE_LIST;
			}
		}
		break;
	case VCONF_TYPE_DATE_FORMAT:
		ret = vconf_get_int(VCONFKEY_SETAPPL_DATE_FORMAT_INT, value);
		break;
	case VCONF_TYPE_NFC_ENABLE:
		ret = vconf_get_int(VCONFKEY_NFC_FEATURE, value);
		break;
	case VCONF_TYPE_EXTENSION_STATE:
		ret = vconf_get_int(VCONFKEY_MYFILE_EXTENSION_STATE, value);
		if (ret < 0) {
			mf_warnig("fail to get list_by value, set it default");
			vconf_set_int(VCONFKEY_MYFILE_EXTENSION_STATE, MF_EXTENSION_SHOW);
			*value = MF_EXTENSION_SHOW;
		} else {
			if (*value < MF_VIEW_STYLE_LIST || *value > MF_VIEW_STYLE_THUMBNAIL) {
				mf_warnig("invaild list by type[%d], set it default", *value);
				vconf_set_int(VCONFKEY_MYFILE_EXTENSION_STATE, MF_EXTENSION_SHOW);
				*value = MF_EXTENSION_SHOW;
			}
		}
		break;
	default:
		break;
	}
	if (ret != MYFILE_ERR_NONE) {
		ret = MYFILE_ERR_VCONF_GET_FAILED;
	}
	return ret;
}

void mf_util_set_sort_type(int value)
{
	if (value <= MYFILE_SORT_BY_NONE || value >= MYFILE_SORT_BY_MAX) {
		mf_warnig("invaild list by type[%d], set it default", value);
		vconf_set_int(VCONFKEY_MYFILE_LIST_BY, MYFILE_SORT_BY_DATE_R2O);
	} else {
		mf_debug("value is [%d]", value);
		vconf_set_int(VCONFKEY_MYFILE_LIST_BY, value);
	}
	return;
}

void mf_util_set_extension_state(int value)
{
	if (value <= MF_EXTENSION_NONE || value >= MF_EXTENSION_MAX) {
		mf_warnig("invaild list by type[%d], set it default", value);
		vconf_set_int(VCONFKEY_MYFILE_EXTENSION_STATE, MF_EXTENSION_SHOW);
	} else {
		mf_debug("value is [%d]", value);
		vconf_set_int(VCONFKEY_MYFILE_EXTENSION_STATE, value);
	}
	return;
}

void mf_util_set_view_style(int value)
{
	if (value < MF_VIEW_STYLE_LIST || value > MF_VIEW_STYLE_THUMBNAIL) {
		mf_warnig("invaild list by type[%d], set it default", value);
		vconf_set_int(VCONFKEY_MYFILE_VIEW_STYLE, MF_VIEW_STYLE_LIST);
	} else {
		mf_debug("value is [%d]", value);
		vconf_set_int(VCONFKEY_MYFILE_VIEW_STYLE, value);
	}
	return;
}

void mf_util_set_recent_file(char *path)
{
	if (path) {
		mf_debug("path is [%s]", path);
		int ret = vconf_set_str(VCONFKEY_MYFILE_RECENT_FILE, path);
		mf_debug("ret is [%d]", ret);

	}
	return;
}

void mf_util_merge_eina_list_to_glist(const Eina_List *eSource, GList **gSource)
{
	mf_retm_if(eSource == NULL, "eSource is NULL");
	mf_retm_if(gSource == NULL, "gSource is NULL");


	const Eina_List *l = NULL;
	GString *pNode = NULL;

	EINA_LIST_FOREACH(eSource, l, pNode) {
		if (pNode) {
			*gSource = g_list_append(*gSource, pNode->str);
		}
	}

}

void mf_util_exception_func(void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	switch (ap->mf_Status.more) {
	case MORE_DATA_COPYING:
		ap->mf_Status.more = MORE_INTERNAL_COPY;
		ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT, NULL,
								       GET_STR(MF_MSG_COPY_FAILED2), NULL, NULL, NULL, (Evas_Smart_Cb)mf_callback_exception_popup_cb, ap);
		break;
	case MORE_DATA_MOVING:
		ap->mf_Status.more = MORE_INTERNAL_MOVE;
		ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT, NULL,
								       GET_STR(MF_MSG_MOVE_FAILED3), NULL, NULL, NULL, (Evas_Smart_Cb)mf_callback_exception_popup_cb, ap);
		break;
	case MORE_DELETE:
		ap->mf_Status.more = ap->mf_Status.preMore;
		ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT, NULL, GET_STR(MF_MSG_DEL_FAILED1), NULL, NULL, NULL, NULL, NULL);
		break;
	default:
		break;
	}
	MF_TRACE_END;
}

int mf_util_is_valid_name_check(const char *name)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(name == NULL, MF_INTERNAL_FILE_NAME_NULL, "name is NULL");
	gchar *temp_name = NULL;
	int ret = MYFILE_ERR_NONE;

	int length = strlen(name);
	if (length) {
		if (strncmp(name, ".", length) == 0 || strncmp(name, "..", length) == 0) {
			ret = MF_INTERNAL_FILE_NAME_IGNORE;
			goto EXIT;
		}

		temp_name = g_strconcat(name, NULL);
		if (strlen(g_strstrip(temp_name)) == 0) {
			ret = MF_INTERNAL_FILE_NAME_CHUG;
			goto EXIT;
		}

		if (mf_util_character_count_get(name) > MYFILE_FILE_NAME_LEN_MAX) {
			ret = MF_INTERNAL_FILE_NAME_MAX_LENGTH;
			goto EXIT;
		}

		if (mf_file_attr_is_valid_name(name) != MYFILE_ERR_NONE) {
			ret = MF_INTERNAL_FILE_NAME_INVALID_CHAR;
		}
	} else {
		ret = MF_INTERNAL_FILE_NAME_EMPTY;
	}

EXIT:
	if (temp_name) {
		g_free(temp_name);
		temp_name = NULL;
	}
	MF_TRACE_END;
	return ret;
}

#ifdef MYFILE_USB_OTG
void mf_util_ex_disk_list_update(void *data)
{
	assert(data);
	struct appdata *ap = (struct appdata *)data;
	mf_debug();
	int iIsNoContent = 0;
	Evas_Object *newContent = NULL;

	iIsNoContent = ap->mf_Status.flagNoContent;
	newContent = mf_navi_bar_content_create(ap);
	Evas_Object *pConformant = mfNaviBarConformGet(ap);
	mfNaviBarSetContent(ap, pConformant, newContent);
	mf_debug();
}
#endif


char *mf_util_upper_folder_name_get(void *data, GString *fullpath)
{
	mf_retvm_if(data == NULL, NULL, "data is NULL");
	mf_retvm_if(fullpath == NULL, NULL, "fullpath is NULL");
	mf_retvm_if(fullpath->str == NULL, NULL, "fullpath is NULL");
	struct appdata *ap = (struct appdata *)data;

	GString *parent_path = NULL;
	GString *gName = NULL;
	char *upper_name = NULL;

	parent_path = mf_fm_svc_wrapper_get_file_parent_path(fullpath);

	mf_retvm_if(parent_path == NULL, NULL, "fullpath is NULL");
	mf_retvm_if(parent_path->str == NULL, NULL, "fullpath is NULL");


	if (!g_strcmp0(parent_path->str, PHONE_FOLDER)) {
		upper_name = g_strdup(GET_SYS_STR(MF_LABEL_PHONE));
	} else if (!g_strcmp0(parent_path->str, MEMORY_FOLDER)) {
		upper_name = g_strdup(GET_SYS_STR(MF_LABEL_MMC));
#ifdef MYFILE_USB_OTG
	} else if (!g_strcmp0(parent_path->str, OTG_FOLDER)){
		upper_name = g_strdup(GET_STR(MF_LABEL_OTG));
#endif
	} else {
		gName = mf_fm_svc_wrapper_get_file_name(parent_path);
		GSTRING_CHECK_NULL_GOTO(gName, FAILED_EXIT);
		upper_name = g_strdup(gName->str);
	}

	CHAR_CHECK_NULL_GOTO(upper_name, FAILED_EXIT);
	SAFE_FREE_GSTRING(parent_path);
	SAFE_FREE_GSTRING(gName);
	return upper_name;

FAILED_EXIT:
	SAFE_FREE_GSTRING(parent_path);
	SAFE_FREE_GSTRING(gName);
	mf_util_operation_alloc_failed(ap);
	return NULL;

}

void mf_util_set_pm_lock(void *data, Eina_Bool isLock)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata *)data;

	int ret = 0;
	if (ap->mf_Status.flagLCDLock != isLock) {
		if (ap->mf_Status.flagLCDLock == EINA_TRUE) {
			ap->mf_Status.flagLCDLock = EINA_FALSE;
		} else {
			ap->mf_Status.flagLCDLock = EINA_TRUE;
		}

		if (ap->mf_Status.flagLCDLock) {
			mf_debug("lock the LCD_OFF");
			ret = power_lock_state(POWER_STATE_SCREEN_OFF, 0);
		} else {
			mf_debug("unlock the LCD_OFF");
			ret = power_unlock_state(POWER_STATE_SCREEN_OFF);
		}
		if (ret != 0)
			mf_debug("fail to lock(unlock)");

	}
	MF_TRACE_END;
}

long mf_util_character_count_get(const char *original)
{
	mf_retvm_if(original == NULL, 0, "input string is NULL");
	long count = 0;
	char *utf8_form = g_locale_to_utf8(original, -1, NULL, NULL, NULL);
	if (utf8_form == NULL)
		return count;
	else {
		  count = g_utf8_strlen(utf8_form, -1);
		  free(utf8_form);
		  mf_debug("utf8 count is %ld", count);
		  return count;
	}
}

MF_THEME_TYPE mf_util_get_theme(void)
{
	MF_THEME_TYPE theme = THEME_INVALID;
	char *name;

	char *theme_name = vconf_get_str(VCONFKEY_SETAPPL_WIDGET_THEME_STR);
	mf_debug("theme is [%s]", theme_name);
	mf_retvm_if(theme_name == NULL, THEME_ERROR, "Failed to get vconf value");	/* file system exceptional handle */

	name = strrchr(theme_name, '/');

	if (!name)
		name = theme_name;
	else
		++name;

	mf_debug("name is [%s]", name);
	//SETTING_TRACE("name:%s", name);
	/* special theme process.. */
	if (0 == g_strcmp0("tizen-black.edj", name) || 0 == g_strcmp0("tizen-black-hd.edj", name)) {
		theme = THEME_NBEAT_BLACK;
	} else if (0 == g_strcmp0("tizen.edj", name) || 0 == g_strcmp0("tizen-hd.edj", name)) {
		theme = THEME_NBEAT;
	} else {
		theme = THEME_ERROR;
	}
	free(theme_name);
	theme_name = NULL;
	mf_debug("theme is [%d]", theme);
	return theme;
}

Eina_List *mf_util_get_all_selected(void *data)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(data == NULL, NULL, "data is NULL");
	struct appdata *ap = (struct appdata*)data;

	Eina_List *select_list = NULL;

	if (ap->mf_Status.flagViewType == MF_VIEW_STYLE_LIST || ap->mf_Status.flagViewType == MF_VIEW_SYTLE_LIST_DETAIL) {
		mf_genlist_get_list_selected_files(ap->mf_MainWindow.pEditView.pGenlist, &select_list);
	} else {
		mf_gengrid_get_grid_selected_files(ap->mf_MainWindow.pEditView.pGengrid, &select_list);
	}
	MF_TRACE_END;
	return select_list;

}

Eina_List *mf_util_get_all_selected_items(void *data)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(data == NULL, NULL, "data is NULL");
	struct appdata *ap = (struct appdata*)data;

	Eina_List *select_list = NULL;

	if (ap->mf_Status.flagViewType == MF_VIEW_STYLE_LIST || ap->mf_Status.flagViewType == MF_VIEW_SYTLE_LIST_DETAIL) {
		mf_genlist_get_list_selected_items(ap->mf_MainWindow.pEditView.pGenlist, &select_list);
	} else {
		mf_gengrid_get_grid_selected_items(ap->mf_MainWindow.pEditView.pGengrid, &select_list);
	}
	MF_TRACE_END;
	return select_list;

}


gboolean mf_util_is_file_selected(Eina_List **source, GString *path)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(*source == NULL, FALSE, "source is NULL");
	mf_retvm_if(path == NULL, FALSE, "path is NULL");

	bool flag = FALSE;
	Eina_List *l = NULL;
	void *data = NULL;

	EINA_LIST_FOREACH(*source, l, data) {
		GString *source_path = (GString *)data;
		if (source_path && source_path->str) {
			mf_debug("source_path is [%s] path is [%s]", source_path->str, path->str);
			if (g_strcmp0(source_path->str, path->str) == 0) {
				flag = TRUE;
				mf_debug("flag is [%d]source_path is [%s] path is [%s]",flag, source_path->str, path->str);
				*source = eina_list_remove(*source, source_path);
				g_string_free(source_path, TRUE);
				source_path = NULL;
				int ret = eina_list_count(*source);
				if (ret == 0) {
					*source = NULL;
				}
				break;
			} else {
				continue;
			}
		}
	}
	MF_TRACE_END;
	return flag;
}

int mf_util_generate_list_data(const char *path, Eina_List ** default_dir_list, Eina_List ** user_dir_list, Eina_List ** file_list)
{
	MF_TRACE_BEGIN;
	assert(path);
	assert(default_dir_list);
	assert(user_dir_list);
	assert(file_list);
	int error_code = 0;
	Eina_List *dir_list = NULL;
	error_code = mf_fm_svc_wrapper_get_file_list(path, &dir_list, file_list);

	if (error_code != MYFILE_ERR_NONE) {
		return error_code;
	}

	/*      classify the dir list to default and user defined       */
	error_code = mf_fm_svc_wrapper_classify_dir_list(dir_list, default_dir_list, user_dir_list);
	if (error_code != MYFILE_ERR_NONE) {
		return error_code;
	}
	eina_list_free(dir_list);
	MF_TRACE_END;
	return error_code;
}


int mf_util_generate_file_list(void *data)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(data == NULL, MYFILE_ERR_INVALID_ARG, "data is null");
	struct appdata *ap = (struct appdata *)data;

	Eina_List *file_list = NULL;
	Eina_List *default_dir_list = NULL;
	Eina_List *user_dir_list = NULL;
	int error_code = 0;

	mf_retvm_if(ap->mf_Status.path == NULL, MYFILE_ERR_INVALID_ARG, "ap->mf_Status.path is NULL");
	mf_retvm_if(ap->mf_Status.path->str == NULL, MYFILE_ERR_INVALID_ARG, "ap->mf_Status.path->str is NULL");

	error_code = mf_util_generate_list_data(ap->mf_Status.path->str, &default_dir_list, &user_dir_list, &file_list);
	if (error_code != MYFILE_ERR_NONE) {
		/*Todo: we need to free all the Eina_List*/
		return error_code;
	}


	if (ap->mf_FileOperation.folder_list) {
		mf_util_free_eina_list_with_data(&(ap->mf_FileOperation.folder_list), MYFILE_TYPE_FSNODE);
	}
	if (ap->mf_FileOperation.file_list) {
		mf_util_free_eina_list_with_data(&(ap->mf_FileOperation.file_list), MYFILE_TYPE_FSNODE);
	}

	ap->mf_FileOperation.file_list = file_list;
	ap->mf_FileOperation.default_dir_list = default_dir_list;
	ap->mf_FileOperation.user_dir_list = user_dir_list;

	return error_code;
}

int mf_util_generate_root_view_file_list(Eina_List **list, int storage_state)
{
	MF_TRACE_BEGIN;

	fsNodeInfo *pNode = NULL;
	pNode = (fsNodeInfo *) malloc(sizeof(fsNodeInfo));
	memset(pNode, 0, sizeof(fsNodeInfo));
	/*set path */
	pNode->path = g_strdup(PHONE_PARENT);
	pNode->name = g_strdup(PHONE_NAME);
	mf_file_attr_get_file_stat(PHONE_FOLDER, &pNode);
	pNode->type = FILE_TYPE_DIR;
	*list = eina_list_append(*list, pNode);

	if (storage_state & MYFILE_MMC) {
		pNode = (fsNodeInfo *) malloc(sizeof(fsNodeInfo));
		memset(pNode, 0, sizeof(fsNodeInfo));
		/*set path */
		pNode->path = g_strdup(STORAGE_PARENT);
		pNode->name = g_strdup(MMC_NAME);
		mf_file_attr_get_file_stat(MEMORY_FOLDER, &pNode);
		pNode->type = FILE_TYPE_DIR;
		*list = eina_list_append(*list, pNode);
	}

	if (storage_state & MYFILE_OTG) {
		pNode = (fsNodeInfo *) malloc(sizeof(fsNodeInfo));
		memset(pNode, 0, sizeof(fsNodeInfo));
		/*set path */
		pNode->path = g_strdup(STORAGE_PARENT);
		pNode->name = g_strdup(OTG_NAME);
		mf_file_attr_get_file_stat(OTG_FOLDER, &pNode);
		pNode->type = FILE_TYPE_DIR;
		*list = eina_list_append(*list, pNode);
	}

	return 0;
}


void mf_util_sort_the_file_list(void *data)
{

	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is null");
	struct appdata *ap = (struct appdata *)data;

	int iSortTypeValue = 0;

	mf_util_get_vconf_value(VCONF_TYPE_SORT_TYPE, &iSortTypeValue);

	if (ap->mf_Status.view_type == mf_view_root_category) {
		mf_fs_oper_sort_list(&ap->mf_FileOperation.category_list, iSortTypeValue);
	}
	mf_fs_oper_sort_list(&ap->mf_FileOperation.file_list, iSortTypeValue);

	ap->mf_FileOperation.folder_list = eina_list_merge(ap->mf_FileOperation.user_dir_list, ap->mf_FileOperation.default_dir_list);
	ap->mf_FileOperation.user_dir_list = NULL;
	ap->mf_FileOperation.default_dir_list = NULL;
	/*need to sort folder items only By Name and Date*/
	if (iSortTypeValue == MYFILE_SORT_BY_NAME_A2Z || iSortTypeValue == MYFILE_SORT_BY_NAME_Z2A || iSortTypeValue == MYFILE_SORT_BY_DATE_R2O
	    || iSortTypeValue == MYFILE_SORT_BY_DATE_O2R) {
	    	ap->mf_Status.iFolderSortType = iSortTypeValue;
		mf_fs_oper_sort_list(&ap->mf_FileOperation.folder_list, iSortTypeValue);
	} else {
		mf_fs_oper_sort_list(&ap->mf_FileOperation.folder_list, ap->mf_Status.iFolderSortType);
	}
	MF_TRACE_END;
}


void mf_util_rotate_state_set(void *data, app_device_orientation_e  rotate_mode)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is null");
	struct appdata *ap = (struct appdata *)data;
#ifdef MYFILE_SPLIT_VIEW

	switch (rotate_mode) {
	case APP_DEVICE_ORIENTATION_270:
		  ap->mf_Status.rotation_angle = -90;
		  ap->mf_Status.view_mode_end = FALSE;
		  ap->mf_Status.pre_rotate_type = APP_DEVICE_ORIENTATION_270;
		  break;

	case APP_DEVICE_ORIENTATION_90:
		  ap->mf_Status.rotation_angle = 90;
		  ap->mf_Status.view_mode_end = FALSE;
		  ap->mf_Status.pre_rotate_type = APP_DEVICE_ORIENTATION_90;
		  break;

	case APP_DEVICE_ORIENTATION_180:
		  ap->mf_Status.rotation_angle = 0; //180;
		  ap->mf_Status.view_mode_end = TRUE;
		  ap->mf_Status.pre_rotate_type = APP_DEVICE_ORIENTATION_180;
		  break;

	case APP_DEVICE_ORIENTATION_0 :
	default:
		  ap->mf_Status.rotation_angle = 0;
		  ap->mf_Status.view_mode_end = TRUE;
		  ap->mf_Status.pre_rotate_type = APP_DEVICE_ORIENTATION_0;
		  break;
	}
	if (rotate_mode == APP_DEVICE_ORIENTATION_270 || rotate_mode == APP_DEVICE_ORIENTATION_90) {
		ap->mf_Status.rotation_type = MF_ROTATE_LANDSCAPE;
	} else {
		ap->mf_Status.rotation_type = MF_ROTATE_PORTRAIT;
	}
	mf_debug("rotate_mode is [%d] rotation_type is [%d]", rotate_mode, ap->mf_Status.rotation_type);

	elm_win_rotation_with_resize_set(ap->mf_MainWindow.pWindow, ap->mf_Status.rotation_angle);

#endif
	MF_TRACE_END;
}

const char *mf_util_search_markup_keyword(const char *string, char *searchword, bool *result)
{
	//SEARCH_FUNC_START;
	MF_TRACE_BEGIN;
	char pstr[DEF_BUF_LEN + 1] = {0,};
	static char return_string[DEF_BUF_LEN + 1] = { 0, };
	int word_len = 0;
	int search_len = 0;
	int i = 0;
	bool found = false;
	gchar* markup_text_start = NULL;
	gchar* markup_text_end= NULL;
	gchar* markup_text= NULL;

	mf_retvm_if(string == NULL, NULL, "string is NULL");
	mf_retvm_if(searchword == NULL, NULL, "searchword is NULL");
	mf_retvm_if(result == NULL, NULL, "result is NULL");

	if(g_utf8_validate(string,-1,NULL)) {

		strncpy(pstr, string, DEF_BUF_LEN);

		word_len = strlen(pstr);
		search_len = strlen(searchword);

		for (i = 0; i < word_len; i++) {
			if (!strncasecmp(searchword, &pstr[i], search_len)) {
				found = true;
				break;
			}
		}

		*result = found;
		memset(return_string, 0x00, DEF_BUF_LEN+1);

		if (found) {
			if (i == 0) {
				markup_text = g_markup_escape_text(&pstr[0], search_len);
				markup_text_end = g_markup_escape_text(&pstr[search_len], word_len-search_len);
				mf_retvm_if(markup_text == NULL, NULL, "markup_text is NULL");
				mf_retvm_if(markup_text_end == NULL, NULL, "markup_text_end is NULL");

				snprintf(return_string,
							DEF_BUF_LEN,
							"<color=#ff0000ff>%s</color>%s",
							markup_text,
							(char*)markup_text_end);
				SAFE_FREE_CHAR(markup_text);
				SAFE_FREE_CHAR(markup_text_end);
			} else {
				markup_text_start = g_markup_escape_text(&pstr[0], i);
				markup_text = g_markup_escape_text(&pstr[i], search_len);

				markup_text_end =  g_markup_escape_text(&pstr[i+search_len], word_len-(i+search_len));
				mf_retvm_if(markup_text_start == NULL, NULL, "markup_text_start is NULL");
				mf_retvm_if(markup_text == NULL, NULL, "markup_text is NULL");
				mf_retvm_if(markup_text_end == NULL, NULL, "markup_text_end is NULL");
				snprintf(return_string,
							DEF_BUF_LEN,
							"%s<color=#ff0000ff>%s</color>%s",
							(char*)markup_text_start,
							markup_text,
							(char*)markup_text_end);
				SAFE_FREE_CHAR(markup_text);
				SAFE_FREE_CHAR(markup_text_start);
				SAFE_FREE_CHAR(markup_text_end);
			}
		} else {
			snprintf(return_string, DEF_BUF_LEN, "%s", pstr);
		}
	}
	MF_TRACE_END;

	return return_string;

}

char *mf_util_get_text(const char *ID)
{
	MF_CHECK_NULL(ID);
	char *str;

	if(strstr(ID, "IDS_COM"))
		str = dgettext("sys_string", ID);
	else
		str = gettext(ID);

	return str;
}

fsNodeInfo *mf_util_generate_pnode(const char *path, int file_type)
{
	mf_retvm_if(path == NULL, NULL, "path is NULL");
	fsNodeInfo *pNode = NULL;
	int error = MYFILE_ERR_NONE;

	pNode = (fsNodeInfo *) malloc(sizeof(fsNodeInfo));
	if (pNode == NULL) {
		return NULL;
	}
	memset(pNode, 0, sizeof(fsNodeInfo));
	char *parent = NULL;
	error = mf_file_attr_get_parent_path(path, &parent);
	if (error == MYFILE_ERR_NONE) {
		pNode->path = parent;
		pNode->name = g_strdup(ecore_file_file_get(path));
		mf_file_attr_get_file_stat(path, &pNode);
		if (file_type == FILE_TYPE_DIR) {
			pNode->type = file_type;
		} else {
			mf_file_attr_get_file_category(path, &(pNode->type));
		}
	} else {
		SAFE_FREE_CHAR(pNode);
	}
	return pNode;
}
void mf_util_generate_list(Eina_List **list, const char *path, int file_type)
{
	mf_retm_if(list == NULL, "list is NULL");
	mf_retm_if(path == NULL, "path is NULL");

	fsNodeInfo *pNode = mf_util_generate_pnode(path, file_type);

	if (pNode)
		*list = eina_list_append(*list, pNode);
}


void mf_util_remove_item_from_list_by_location(Eina_List **list, int location)
{
	mf_retm_if(list == NULL, "list is NULL");

	Eina_List *l = NULL;
	fsNodeInfo *node = NULL;

	EINA_LIST_FOREACH(*list, l, node) {
		if ((fsNodeInfo *)node != NULL && ((fsNodeInfo *)node)->path != NULL) {
			if (mf_fm_svc_wrapper_get_location(node->path) == location){
				SAFE_FREE_CHAR(node->path);
				SAFE_FREE_CHAR(node->name);
				SAFE_FREE_CHAR(node->ext);
				SAFE_FREE_CHAR(node);
				*list = eina_list_remove_list(*list, l);
			}
		}
	}
}

void mf_util_update_item_from_list_by_name(Eina_List **list, const char *path, char *new_name)
{
	mf_retm_if(list == NULL, "list is NULL");

	Eina_List *l = NULL;
	fsNodeInfo *node = NULL;

	EINA_LIST_FOREACH(*list, l, node) {
		if ((fsNodeInfo *)node != NULL && ((fsNodeInfo *)node)->name != NULL && ((fsNodeInfo *)node)->path != NULL) {
			char *real_name = g_strconcat(((fsNodeInfo *)node)->path, "/", ((fsNodeInfo *)node)->name, NULL);
			mf_debug("real is [%s] path is [%s]", real_name, path);
			if (g_strcmp0(real_name, path) == 0){
				SAFE_FREE_CHAR(node->name);
				node->name = g_strdup(ecore_file_file_get(new_name));
			}
			SAFE_FREE_CHAR(real_name);
		}
	}
}

void mf_util_remove_item_from_list_by_name(Eina_List **list, const char *path)
{
	mf_retm_if(list == NULL, "list is NULL");

	Eina_List *l = NULL;
	fsNodeInfo *node = NULL;

	EINA_LIST_FOREACH(*list, l, node) {
		if ((fsNodeInfo *)node != NULL && ((fsNodeInfo *)node)->name != NULL && ((fsNodeInfo *)node)->path != NULL) {
			char *real_name = g_strconcat(((fsNodeInfo *)node)->path, "/", ((fsNodeInfo *)node)->name, NULL);
			mf_debug("real is [%s] path is [%s]", real_name, path);
			if (g_strcmp0(real_name, path) == 0){
				SAFE_FREE_CHAR(node->path);
				SAFE_FREE_CHAR(node->name);
				SAFE_FREE_CHAR(node->ext);
				SAFE_FREE_CHAR(node);
				*list = eina_list_remove_list(*list, l);
			}
			SAFE_FREE_CHAR(real_name);
		}
	}
}

bool mf_util_is_rotation_lock(void)
{
	MF_TRACE_BEGIN;
	int lock = -1;

	if (!vconf_get_bool(VCONFKEY_SETAPPL_AUTO_ROTATE_SCREEN_BOOL, &lock))
	{
		mf_debug("Rotation locked state[%d].", lock);
		return lock;
	}
	else
	{
		mf_debug("Get rotation lock state failed!");
		return false;
	}
}

