/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */





#ifndef __MYFILE_DEBUG_H_
#define __MYFILE_DEBUG_H_

#include <stdio.h>
#include <string.h>
#include "mf-log.h"

#ifdef MYFILE_USE_LOG

#ifdef LOG_TAG
#undef LOG_TAG
#endif

#define LOG_TAG		"MYFILES"
#include <dlog.h>

#define MYFILE_TRACE_DEBUG(fmt, arg...) do { LOGD("[%s][%d] "fmt "\n", __FILE__, __LINE__, ##arg); } while (0)
#define MYFILE_TRACE_ERROR(fmt, arg...) do { LOGE("[%s][%d] "fmt"\n", strrchr(__FILE__, '/')+1, __LINE__, ##arg); } while (0)

#define mf_func_line()			do { LOGD("[%s][%s][%d] \n", __FILE__, __func__, __LINE__); } while (0)
#define mf_debug(fmt, args...)	do { LOGD("[%s][%d] "fmt"\n", __func__, __LINE__, ##args); } while (0)

#define mf_error(fmt, args...)		do {\
						LOGE("[%s][%d] "fmt"\n", __func__, __LINE__, ##args); \
						MF_LOG_RECORD(fmt,##args); \
					   } while (0)
#define mf_warnig(fmt, args...)	do { LOGW("[%s][%d] "fmt"\n", __func__, __LINE__, ##args); } while (0)
#define mf_info(fmt, args...)		do { LOGI("[%s][%d] "fmt"\n", __func__, __LINE__, ##args); } while (0)
#define mf_assert(fmt, args...)	do { LOGE("[ASSERT][%s][%d] "fmt"\n", __func__, __LINE__, ##args); } while (0)

#else

#define MYFILE_TRACE_DEBUG(fmt, arg...)	do { printf("%s:%d: " fmt "\n", __FILE__, __LINE__, ##arg); } while (0)
#define MYFILE_TRACE_ERROR(fmt, arg...)	do { printf("[%s][%d] "fmt"\n", strrchr(__FILE__, '/')+1, __LINE__, ##arg); } while (0)

#define mf_debug(fmt, args...)	do { printf("[MYFILE][D][%40s:%4d] "fmt"\n", __func__, __LINE__, ##args); } while (0)
#define mf_func_line()			do { printf("[MYFILE][D][%40s:%4d] \n", __FILE__, __func__, __LINE__); } while (0)
#define mf_error(fmt, args...)	do { \
					printf("[MYFILE][E][%40s:%4d] "fmt"\n", __func__, __LINE__, ##args); \
					MF_LOG_RECORD(fmt,##args); \
				   } while (0)
#define mf_warnig(fmt, args...)	do { printf("[MYFILE][W][%40s:%4d] "fmt"\n", __func__, __LINE__, ##args); } while (0)
#define mf_info(fmt, args...)	do { printf("[MYFILE][I][%40s:%4d] "fmt"\n", __func__, __LINE__, ##args); } while (0)
#define mf_assert(fmt, args...)	do { printf("[MYFILE][A][%40s:%4d] "fmt"\n", __func__, __LINE__, ##args); } while (0)

#endif

#define mf_retvm_if(expr, val, fmt, arg...) do { \
			if (expr) { \
				MYFILE_TRACE_ERROR(fmt, ##arg); \
				MF_LOG_RECORD(fmt,##arg); \
				return (val); \
			} \
		} while (0)

#define mf_retv_if(expr, val) do { \
			if (expr) { \
				return (val); \
			} \
		} while (0)


#define mf_retm_if(expr, fmt, arg...) do { \
			if (expr) { \
				MYFILE_TRACE_ERROR(fmt, ##arg); \
				MF_LOG_RECORD(fmt,##arg); \
				return; \
			} \
		} while (0)

#define mf_ret_if(expr, fmt, arg...) do { \
			if (expr) { \
				MF_LOG_RECORD(fmt,##arg); \
				return; \
			} \
		} while (0)

#define MF_TRACE_BEGIN do {\
					{\
						LOGD("\n\033[0;35mENTER FUNCTION: %s. \033[0m\t%s:%d\n", \
						__FUNCTION__, (char *)(strrchr(__FILE__, '/')+1), __LINE__);\
					} \
				} while (0) ;

#define MF_TRACE_END  do {\
					{\
						LOGD("\n\033[0;35mEXIT FUNCTION: %s. \033[0m\t%s:%d\n", \
						__FUNCTION__, (char *)(strrchr(__FILE__, '/')+1), __LINE__);\
					} \
				} while (0) ;

#define MF_CHECK(expr) 				mf_retm_if(!(expr),"INVALID PARAM RETURN")
#define MF_CHECK_FALSE(expr) 			mf_retvm_if(!(expr),FALSE,"INVALID PARM RETURN FALSE")
#define MF_CHECK_VAL(expr, val) 		mf_retvm_if(!(expr),val,"INVALID PARM RETURN NULL")
#define MF_CHECK_NULL(expr) 			mf_retvm_if(!(expr),NULL,"INVALID PARM RETURN NULL")

#endif /* end of __MYFILE_DEBUG_H_ */
