/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */




#ifndef _MYFILE_FILESYSTEM_UTIL_H_
#define _MYFILE_FILESYSTEM_UTIL_H_

#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <linux/fs.h>
#include <Eina.h>

#include <unicode/udat.h>
#include <unicode/ustring.h>
#include <unicode/uloc.h>
#include <unicode/ucal.h>
#include <unicode/udatpg.h>
#include <unicode/utmscale.h>
#include <media_content.h>

#include "mf-dlog.h"

/*	File system related value definition	*/
#define MYFILE_FILE_NAME_LEN_MAX	255	/*charactor based, use g_utf8_strlen to check length*/
#define MYFILE_FILE_PATH_LEN_MAX	4096	/*charactor based, use g_utf8_strlen to check length*/
#define MYFILE_REPORT_SIZE			16384

#define MYFILE_FOLDER_SIZE			0;

/*	File system related String definition	*/
#define PHONE_FOLDER		"/opt/usr/media"
#define MEMORY_FOLDER		"/opt/storage/sdcard"
#define PHONE_PARENT		"/opt/usr"
#define PHONE_NAME		"media"
#define STORAGE_PARENT		"/opt/storage"
#define MMC_NAME		"sdcard"

#ifdef MYFILE_USB_OTG
#define OTG_FOLDER		"/opt/storage/usb"
#define OTG_NAME		"usb"
#endif
#define DEBUG_FOLDER		"SLP_debug"

#define MYFILE_NAME_PATTERN	"[\\:;*\"<>|?/]"

/*	File system define default folder	*/

#define DEFAULT_FOLDER_CAMERA_SHOTS		"Camera shots"
#define DEFAULT_FOLDER_DOWNLOADS		"Downloads"

#define DEFAULT_FOLDER_IMAGE			"Images"
#define SUB_FODER_WALLPAPER			"Wallpapers"

#define DEFAULT_FOLDER_VIDEO			"Videos"

#define DEFAULT_FOLDER_MUSIC			"Music"
#define SUB_FODER_FM				"FM radio"
#define SUB_FODER_VOICE_RECORD			"Voice recorder"

#define DEFAULT_FOLDER_ALERTS_AND_RINGTONES	"Alerts and ringtones"
#define SUB_FODER_ALERTS			"Alerts"
#define SUB_FODER_RINGTONES			"Ringtones"

#define DEFAULT_FOLDER_BOOKMARK	"Bookmark"
#define DEFAULT_FOLDER_RSS	"RSS"

#ifndef ICON_PATH
#define ICON_PATH "/usr/apps/org.tizen.myfile/res/images"
#endif

#ifndef DEFAULT_ICON
#define DEFAULT_ICON ICON_PATH"/myfile_icon_etc.png"
#endif

#define MF_ICON_FOLDER			ICON_PATH"/myfile_icon_folder.png"
#define MF_ICON_FOLDER_OPEN		ICON_PATH"/myfile_icon_folder_open.png"
#define MF_ICON_IMAGE			ICON_PATH"/myfile_icon_images.png"
#define MF_ICON_VIDEO			ICON_PATH"/myfile_icon_video.png"
#define MF_ICON_MUSIC			ICON_PATH"/myfile_icon_music.png"
#define MF_ICON_SOUND			ICON_PATH"/myfile_icon_music.png"
#define MF_ICON_PDF			ICON_PATH"/myfile_icon_pdf.png"
#define MF_ICON_DOC			ICON_PATH"/myfile_icon_word.png"
#define MF_ICON_PPT			ICON_PATH"/myfile_icon_ppt.png"
#define MF_ICON_EXCEL			ICON_PATH"/myfile_icon_excel.png"
#define MF_ICON_VOICE			ICON_PATH"/myfile_icon_music.png"
#define MF_ICON_HTML			ICON_PATH"/myfile_icon_html.png"
#define MF_ICON_FLASH			ICON_PATH"/myfile_icon_swf.png"
#define MF_ICON_TXT			ICON_PATH"/myfile_icon_text.png"
#define MF_ICON_VCONTACT		ICON_PATH"/myfile_icon_svg.png"
#define MF_ICON_VCALENDAR		ICON_PATH"/myfile_icon_svg.png"
#define MF_ICON_VNOTE			ICON_PATH"/myfile_icon_text.png"
#define MF_ICON_RSS			ICON_PATH"/myfile_icon_rss.png"
#define MF_ICON_JAVA			ICON_PATH"/myfile_icon_java.png"
#define MF_ICON_MEMORY_SUB		ICON_PATH"/myfile_icon_mmc_sub.png"
#define MF_ICON_MUSIC_PLAY		ICON_PATH"/myfile_icon_control_play.png"
#define MF_ICON_MUSIC_PAUSE		ICON_PATH"/myfile_icon_control_pause.png"
#define MF_ICON_PHONE			ICON_PATH"/myfile_icon_phone.png"
#define MF_ICON_MEMORY			ICON_PATH"/myfile_icon_mmc.png"
#define MF_ICON_EXTERNAL_DEVICE		ICON_PATH"/myfile_icon_USB.png"
#define MF_ICON_VIDEO_PLAY		ICON_PATH"/myfile_icon_video_play.png"
#define MF_ICON_ENTRY_FOLDER		ICON_PATH"/myfile_icon_entry_folder.png"
#define MF_ICON_ENTRY_FOLDER_PRESS	ICON_PATH"/myfile_icon_entry_folder_press.png"


#define MF_ICON_THUMBNAIL_EXCEL		ICON_PATH"/myfile_thumbnail_icon_excel.png"
#define MF_ICON_THUMBNAIL_FOLDER	ICON_PATH"/myfile_thumbnail_icon_folder.png"
#define MF_ICON_THUMBNAIL_FOLDER_OPEN	ICON_PATH"/myfile_thumbnail_icon_folder_open.png"
#define MF_ICON_THUMBNAIL_HTML		ICON_PATH"/myfile_thumbnail_icon_html.png"
#define MF_ICON_THUMBNAIL_MUSIC		ICON_PATH"/myfile_thumbnail_icon_music.png"
#define MF_ICON_THUMBNAIL_PDF		ICON_PATH"/myfile_thumbnail_icon_pdf.png"
#define MF_ICON_THUMBNAIL_PPT		ICON_PATH"/myfile_thumbnail_icon_ppt.png"
#define MF_ICON_THUMBNAIL_RINGTONE	ICON_PATH"/myfile_thumbnail_icon_ring tone.png"
#define MF_ICON_THUMBNAIL_RSS		ICON_PATH"/myfile_thumbnail_icon_rss.png"
#define MF_ICON_THUMBNAIL_FLASH		ICON_PATH"/myfile_thumbnail_icon_swf.png"
#define MF_ICON_THUMBNAIL_TXT		ICON_PATH"/myfile_thumbnail_icon_text.png"
#define MF_ICON_THUMBNAIL_VIDEO		ICON_PATH"/myfile_thumbnail_icon_video.png"
#define MF_ICON_THUMBNAIL_DOC		ICON_PATH"/myfile_thumbnail_icon_word.png"
#define MF_ICON_THUMBNAIL_VIDEO_PLAY	ICON_PATH"/myfile_thumbnail_icon_video_play.png"
#define MF_ICON_THUMBNAIL_UNKOWN	ICON_PATH"/myfile_thumbnail_icon_unkown.png"

#define MF_ICON_ITEM_PHONE		ICON_PATH"/U01_1st_icon_phone.png"
#define MF_ICON_ITEM_MMC		ICON_PATH"/U01_1st_icon_memory_card.png"
#define MF_ICON_ITEM_OTG		ICON_PATH"/U01_1st_icon_external_storage.png"
#define MF_ICON_ITEM_DB			ICON_PATH"/U01_1st_icon_dropbox.png"

typedef enum _SORT_OPTION fsSortOption;

enum _SORT_OPTION {
	MYFILE_SORT_BY_NONE = 0,		 /**< Sort by default */
	MYFILE_SORT_BY_NAME_A2Z,		 /**< Sort by file name ascending */
	MYFILE_SORT_BY_SIZE_S2L,		 /**< Sort by file size ascending */
	MYFILE_SORT_BY_DATE_O2R,		 /**< Sort by file date ascending */
	MYFILE_SORT_BY_TYPE_A2Z,				 /**< Sort by file type ascending */
	MYFILE_SORT_BY_NAME_Z2A,		 /**< Sort by file name descending */
	MYFILE_SORT_BY_SIZE_L2S,		 /**< Sort by file size descending */
	MYFILE_SORT_BY_DATE_R2O,		 /**< Sort by file date descending */
	MYFILE_SORT_BY_TYPE_Z2A,				 /**< Sort by file type descending */
	MYFILE_SORT_BY_MAX
} ;

typedef enum _mf_extension_state_e mf_extension_state_e;

enum _mf_extension_state_e {
	MF_EXTENSION_NONE = 0,
	MF_EXTENSION_SHOW,
	MF_EXTENSION_HIDE,
	MF_EXTENSION_MAX
} ;

typedef enum _FILE_TYPE fsFileType;

enum _FILE_TYPE {
	FILE_TYPE_NONE = 0,
	FILE_TYPE_DIR,			   /**< Folder category */
	FILE_TYPE_FILE,			   /**< File category */
	FILE_TYPE_IMAGE,	   /**< Image category */
	FILE_TYPE_VIDEO,	   /**< Video category */
	FILE_TYPE_MUSIC,	   /**< Music category */
	FILE_TYPE_SOUND,	   /**< Sound category */
	FILE_TYPE_PDF,		   /**< Pdf category */
	FILE_TYPE_DOC,		   /**< Word category */
	FILE_TYPE_PPT,		   /**< Powerpoint category */
	FILE_TYPE_EXCEL,	   /**< Excel category */
	FILE_TYPE_VOICE,	   /**< Voice category */
	FILE_TYPE_HTML,		   /**< Html category */
	FILE_TYPE_FLASH,	   /**< Flash category */
	FILE_TYPE_GAME,		   /**< Game category */
	FILE_TYPE_APP,		   /**< Application category */
	FILE_TYPE_THEME,	   /**< Theme category */
	FILE_TYPE_TXT,		   /**< Txt category */
	FILE_TYPE_VCONTACT,	   /**< Vcontact category */
	FILE_TYPE_VCALENDAR,	   /**< Vcalendar category */
	FILE_TYPE_VNOTE,	   /**< Vnote category */
	FILE_TYPE_VBOOKMARK,	   /**< Vbookmark category */
	FILE_TYPE_VIDEO_PROJECT,   /**< Video editor project category */
	FILE_TYPE_RADIO_RECORDED,  /**< radio recorded clips category */
	FILE_TYPE_MOVIE_MAKER,	   /**< Movie maker project category */
	FILE_TYPE_SVG,		   /**< Svg category */
	FILE_TYPE_RSS,		   /**< Rss reader file, *.opml */
	FILE_TYPE_CERTIFICATION,   /**< certification file, *.pem */
	FILE_TYPE_JAVA,		   /**< java file, *.jad, *.jar */
	FILE_TYPE_WGT,		   /**< wrt , *.wgt, *.wgt */
	FILE_TYPE_DRM,				/**< drm file , *.dcf */
	FILE_TYPE_ETC,		   /**< Other files category */
	FILE_TYPE_MAX
} ;

typedef enum _STORAGE MF_STORAGE;

enum _STORAGE {
	MYFILE_NONE = 0x00,
	MYFILE_PHONE = 0x01,
	MYFILE_MMC = 0x02,
	MYFILE_OTG = 0x04,
	MYFILE_MAX = 0xFF
};


typedef enum _SIZE_TYPE MF_SIZE_TYPE;
enum _SIZE_TYPE {
	SIZE_BYTE = 0,
	SIZE_KB,
	SIZE_MB,
	SIZE_GB
};

typedef enum __MF_SORT_BY_PRIORITY_SEQUENCE MF_SORT_BY_PRIORITY_SEQUENCE;
enum __MF_SORT_BY_PRIORITY_SEQUENCE {
	MF_SORT_BY_PRIORITY_TYPE_A2Z,
	MF_SORT_BY_PRIORITY_TYPE_Z2A,
	MF_SORT_BY_PRIORITY_DATE_O2R,
	MF_SORT_BY_PRIORITY_DATE_R2O,
	MF_SORT_BY_PRIORITY_SIZE_S2L,
	MF_SORT_BY_PRIORITY_SIZE_L2S,
};

/*	File operation error check options definition		*/
#define	MF_ERROR_CHECK_SRC_ARG_VALID		0x0001
#define	MF_ERROR_CHECK_DST_ARG_VALID		0x0002
#define	MF_ERROR_CHECK_SRC_EXIST			0x0004
#define	MF_ERROR_CHECK_DST_EXIST			0x0008
#define MF_ERROR_CHECK_SRC_PATH_VALID		0x0010
#define MF_ERROR_CHECK_DST_PATH_VALID		0x0020
#define	MF_ERROR_CHECK_SRC_PARENT_DIR_EXIST	0x0040
#define	MF_ERROR_CHECK_DST_PARENT_DIR_EXIST	0x0080
#define	MF_ERROR_CHECK_DUPLICATED			0x0100

/*	File system error definition	*/
#define MF_ERROR_MASKL16       0xFFFF

#define MF_ERROR_SET(X)        (X & MF_ERROR_MASKL16)

#define MID_CONTENTS_MGR_ERROR  0

#define MYFILE_ERR_NONE   (MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x00))    /**< No error */

/*1-10*/
#define MYFILE_ERR_SRC_ARG_INVALID		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x01))	/**< invalid src argument */
#define MYFILE_ERR_DST_ARG_INVALID		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x02))	/**< invalid dst argument */
#define MYFILE_ERR_FILE_DELETE_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x03))	/**< exception of delete file */
#define MYFILE_ERR_FILE_MOVE_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x04))	/**< exception of move file */
#define MYFILE_ERR_FILE_COPY_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x05))	/**< exception of copy file */
#define MYFILE_ERR_FILE_WRITE_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x06))	/**< exception of read file */
#define MYFILE_ERR_RENAME_FAIL			(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x07))    /**< exception of rename file */
#define MYFILE_ERR_FILE_NOT_FOUND		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x08))	/**< exception of file doesn't exist*/
#define MYFILE_ERR_DIR_OPEN_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x09))	/**< exception of dir open*/
#define MYFILE_ERR_DIR_CREATE_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x0a))	/**< exception of create dir */

/*11-20*/
#define MYFILE_ERR_DIR_DELETE_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x0b))	/**< exception of delete dir */
#define MYFILE_ERR_FILE_OPEN_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x0c))    /**< exception of rename dir */
#define MYFILE_ERR_DIR_COPY_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x0d))	/**< exception of copy dir */
#define MYFILE_ERR_DIR_MOVE_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x0e))	/**< exception of move dir */
#define MYFILE_ERR_DIR_FULL			(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x0f))	/**< exception of dir full */
#define MYFILE_ERR_DIR_TOO_DEEP			(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x10))	/**< exception of too deep dir */
#define MYFILE_ERR_DIR_NOT_FOUND		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x11))	/**< exception of dir doesn't exist*/
#define MYFILE_ERR_INVALID_DIR_NAME		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x12))	/**< exception of invalid dir name */
#define MYFILE_ERR_INVALID_DIR_PATH		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x13))	/**< exception of invalid dir path */
#define MYFILE_ERR_INVALID_FILE_NAME		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x14))	/**< exception of invalid file name */

/*21-30*/
#define MYFILE_ERR_INVALID_FILE_PATH		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x15))	/**< exception of invalid file path */
#define MYFILE_ERR_GET_MEMORY_STATUS_FAIL	(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x16))	/**< exception of statfs */
#define MYFILE_ERR_DUPLICATED_NAME		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x17))    /**< exception of duplicated dir name*/
#define MYFILE_ERR_SYSTEM_DIR			(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x18))	/**< exception of operating on system dir.*/
#define MYFILE_ERR_DIR_RECUR			(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x19))	/**< exception of copy/move a dir to its child */
#define MYFILE_ERR_ALLOCATE_MEMORY_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x1a))	/**< exception of memory allocation */
#define MYFILE_ERR_OUT_OF_RANGE			(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x1b))
#define MYFILE_ERR_INVALID_PATH			(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x1c))    /**< invalid path string */
#define MYFILE_ERR_ROOT_PATH			(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x1d))    /**< root path */
#define MYFILE_ERR_DCM_ENGINE_APPEND		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x1e))    /**< fail to append dcm data */

/*31-40*/
#define MYFILE_ERR_NOT_MMF_FILE			(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x1f))	/**< isn't mmf file */
#define MYFILE_ERR_SMAF_PERMISSION_DENY		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x20))	/**< smaf lock prohibits copy/move from mmc to phone */
#define MYFILE_ERR_DRM_PERMISSION_DENY		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x21))	    /**< can't copy/move drm file because of permission */
#define MYFILE_ERR_COPY_TO_SRC_DIR		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x22))	    /**< can't copy dir to source place*/
#define MYFILE_ERR_COPY_ROOT_DIR		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x23))	    /**< can't copy root dir*/
#define MYFILE_ERR_COPY_TO_SRC_FILE		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x24))	    /**< can't copy file to source place*/
#define MYFILE_ERR_MOVE_TO_SRC_DIR		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x25))	    /**< can't move dir to source place */
#define MYFILE_ERR_MOVE_ROOT_DIR		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x26))	    /**< can't move root dir */
#define MYFILE_ERR_MOVE_TO_SRC_FILE		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x27))	    /**< can't move file to source place*/
#define MYFILE_ERR_MOVE_FILE_USING		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x28))	    /**< can't move the file is being used*/

/*41-50*/
#define MYFILE_ERR_DELETE_ROOT_DIR		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x29))	    /**< can't delete root dir */
#define MYFILE_ERR_DELETE_SYSTEM_DIR		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x2a))	    /**< can't delete system dir */
#define MYFILE_ERR_RENAME_ROOT_DIR		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x2b))	    /**< can't rename root dir */
#define MYFILE_ERR_RENAME_SYSTEM_DIR		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x2c))	    /**< can't rename system dir */
#define MYFILE_ERR_EXCEED_MAX_LENGTH		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x2d))	    /**< length of file/dir path exceeds maximum length*/
#define MYFILE_ERR_LOW_MEMORY			(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x2e))	    /**< low memory*/
#define MYFILE_ERR_UNKNOWN_ERROR		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x2f))	    /**< unknow error*/
#define MYFILE_ERR_WRONG_FILE_TYPE		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x30))	    /**< wrong file type */
#define MYFILE_ERR_FILE_IS_BEING_USED		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x31))	    /**< file is being used */
#define MYFILE_ERR_SRC_NOT_EXIST		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x32))	/**< source not found */

/*51-60*/
#define MYFILE_ERR_DST_NOT_EXIST		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x33))	/**< destination not found */
#define MYFILE_ERR_CREATE_TEMP_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x34))	/**< create temp file failed */
#define MYFILE_ERR_GET_LOGIC_PATH_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x35))	/**< get logical path failed */
#define MYFILE_ERR_STORAGE_TYPE_ERROR		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x36))	/**< storage type error */
#define MYFILE_ERR_EXT_GET_ERROR		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x37))	/**< get ext type failed */
#define MYFILE_ERR_GET_PARENT_PATH_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x38))	/**< get parent path failed */
#define MYFILE_ERR_GET_STAT_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x39))	/**< get stat failed */
#define MYFILE_ERR_GENERATE_NAME_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x3a))	/**< generate name failed */
#define MYFILE_ERR_GET_CATEGORY_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x3b))	/**< get file category failed */
#define MYFILE_ERR_GET_VCONF_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x3c))	/**< get vconf value failed */

/*61-70*/
#define MYFILE_ERR_SETTING_RESET_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x3d))	/**< setting item reset  failed */
#define MYFILE_ERR_DIR_CLEAR_FAILED		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x3e))	/**< dir not clearly deleted */
#define MYFILE_ERR_SETTING_DELETE_FAILED	(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x3f))	/**< delete setting item failed */
#define MYFILE_ERR_GET_THUMBNAIL_FAILED		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x40))	/**< get file thumbnail failed */
#define MYFILE_ERR_CANCEL_PRESSED		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x41))	/**< cancel pressed while copy/move */
#define MYFILE_ERR_ACCESS_MODE			(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x42))	/**< access mode not satisfied */
#define MYFILE_ERR_FILE_READ_FAIL		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x43))	/**< exception of read file */
#define MYFILE_ERR_INVALID_ARG			(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x44))	       /**< argument of function is not valid */
#define MYFILE_ERR_NO_FREE_SPACE		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x45))	       /**< get free space failed */
#define MYFILE_ERR_GET_NAVI_FAILED		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x46))	       /**< get navigation bar failed */

/*71-80*/
#define MYFILE_ERR_STORAGE_INUSE_REMOVED	(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x47))	       /**< get navigation bar failed */
#define MYFILE_ERR_VCONF_GET_FAILED		(MID_CONTENTS_MGR_ERROR - MF_ERROR_SET(0x48))	       /**< get vconf value failed */

/*	File system related callback definition	*/
typedef int (*myfile_operation_cb) (const char *current_path,
				    const char *destination, int copied_size, FILE * fSource, FILE * fDestination, void *data);

typedef struct _FS_NODE_INFO fsNodeInfo;

struct _FS_NODE_INFO {
	char *path;
	char *name;
	UDate date;
	fsFileType type;
	char *ext;
	off_t size;
};


int mf_fs_oper_error(const char *src, const char *dst, int check_option);
void mf_fs_oper_print_node(fsNodeInfo *pNode);
/**********			File Attribute Related			**********/
int mf_file_attr_get_file_stat(const char *filename, fsNodeInfo **node);

int mf_file_attr_get_file_category(const char *filepath, fsFileType *category);
fsFileType mf_file_attr_get_file_type_by_mime(const char *file_path);

int mf_file_attr_is_dir(const char *filepath);
int mf_file_attr_get_store_type_by_full(const char *filepath, MF_STORAGE *store_type);
int mf_file_attr_get_file_ext(const char *filepath, char **file_ext);
int mf_file_attr_is_duplicated_name(const char *dir, const char *name);
int mf_file_attr_is_valid_name(const char *filename);
int mf_file_attr_is_right_dir_path(const char *dir_path);
int mf_file_attr_is_right_file_path(const char *file_path);
int mf_file_attr_get_parent_path(const char *path, char **parent_path);
int mf_file_attr_get_logical_path_by_full(const char *full_path, char **path);
void mf_file_attr_get_file_size_info(char **file_size, fsNodeInfo *pNode);

/**********			File Operation Related			**********/
int mf_fs_oper_read_dir(const char *path, Eina_List **dir_list, Eina_List **file_list);
void mf_fs_oper_sort_list(Eina_List **list, int sort_opt);
int mf_fs_oper_create_dir(const char *dir);
int mf_fs_oper_rename_file(const char *src, const char *dst);
int mf_file_attr_get_file_icon(const char *file_path, int *error_code, int view_type, const char **thumbnail, media_info_h *media_info);
const char *mf_file_attr_get_default_icon_by_type(fsFileType ftype);
const char *mf_file_attr_get_default_thumbnail_by_type(fsFileType ftype);
int mf_file_attr_get_path_level(const char *fullpath, int *level);
int mf_file_attr_is_system_dir(char *fullpath, bool *result);
int mf_file_attr_is_disk_link(const char *fullpath, bool *result);

#endif
