/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */




#ifndef __DEF_MYFILE_H_
#define __DEF_MYFILE_H_

#include <Ethumb.h>
#include <glib.h>
#include <Ecore.h>
#include <Ecore_X.h>
#include <Evas.h>
#include <stdbool.h>
#include <media_content.h>
#include <Elementary.h>
//#include <media-svc-types.h>

/* for SG */
#include <ui-gadget.h>
#include <app.h>

/* for dlog */
#include "mf-dlog.h"
#include "mf-util.h"
#include "mf-search.h"
#include "mf-copy.h"

#define CTRL_DISABLE_NONE					0x0000
#define CTRL_DISABLE_MOVE					0x0001
#define CTRL_DISABLE_COPY					0x0002
#define CTRL_DISABLE_DELETE					0x0004
#define CTRL_DISABLE_SEND					0x0008
#define CTRL_DISABLE_LIST_BY					0x0010
#define CTRL_DISABLE_CREATE					0x0020
#define CTRL_DISABLE_EDIT					0x0040
#define CTRL_DISABLE_COPY_MOVE					0x0080
#define CTRL_DISABLE_SEARCH					0x0100
#define CTRL_DISABLE_MOVE_HERE				0x0200
#define CTRL_DISABLE_COPY_HERE				0x0400
#define CTRL_DISABLE_CANCEL				0x0800
#define CTRL_DISABLE_MORE				0x1000

#define CTRL_DISABLE_EDIT_ALL				(CTRL_DISABLE_DELETE | CTRL_DISABLE_SEND | CTRL_DISABLE_MORE)
#define CTRL_DISABLE_DEFAULT_SEL			(CTRL_DISABLE_COPY)
#define CTRL_DISABLE_NOCONTENT_VIEW			(CTRL_DISABLE_EDIT | CTRL_DISABLE_LIST_BY | CTRL_DISABLE_SEARCH)
#define CTRL_DISABLE_USER_FOLDER_SEL			(CTRL_DISABLE_DELETE | CTRL_DISABLE_MORE)
#define CTRL_DISABLE_DEFAULT_ALL			(CTRL_DISABLE_LIST_BY | CTRL_DISABLE_MORE | CTRL_DISABLE_EDIT | CTRL_DISABLE_SEARCH)
#define CTRL_DISABLE_OTG_ROOT				~CTRL_DISABLE_SEARCH
#define CTRL_DISABLE_SYSFOLDER_SELECT			(CTRL_DISABLE_MOVE_HERE)
#define CTRL_DISABLE_OTG_OPT_ROOT			(CTRL_DISABLE_MOVE_HERE | CTRL_DISABLE_COPY_HERE | CTRL_DISABLE_CREATE)


#define myfile_ret_if(expr) do { \
	if (expr) { \
		mf_debug("!!! CHECK ERROR !!! (%s) -> %s() return!!!", #expr, __FUNCTION__); \
		return; \
	} \
} while (0)
#define myfile_retv_if(expr, val) do { \
	if (expr) { \
		mf_debug("!!! CHECK ERROR !!! (%s) -> %s() return !!!", #expr, __FUNCTION__); \
		return (val); \
	} \
} while (0)
#define myfile_retm_if(expr, fmt, arg...) do { \
	if (expr) { \
		mf_debug(fmt, ##arg); \
		mf_debug("!!! CHECK ERROR !!! (%s) -> %s() return !!!", #expr, __FUNCTION__); \
		return; \
	} \
} while (0)
#define myfile_retvm_if(expr, val, fmt, arg...) do { \
	if (expr) { \
		mf_debug(fmt, ##arg); \
		mf_debug("!!! CHECK ERROR !!! (%s) -> %s() return !!!", #expr, __FUNCTION__); \
		return (val); \
	} \
} while (0)

/***********	Global Definitions		***********/
typedef void (*mfCallBack) (void *, Evas_Object *, void *);

typedef struct _oper_record oper_record;

struct _oper_record {
	GString *path;
};

typedef struct _myfileNaviBar myfileNaviBar;

struct _myfileNaviBar {
	char *pNaviTitle;
	char *pNaviLabel;	/*indicate which storage is pointed by this navibar*/
	oper_record pre_operation;
	char *pCurrentPath;
	MORE_TYPE pre_more;	/*for tab function only*/
	bool naviFlagInUse;
	bool naviFlagEditStart;
	bool naviFlagEditJoin;	/*to indicate if pop in refresh needed*/
};

typedef struct _mfEditView mfEditView;
struct _mfEditView {
	Evas_Object *pBox;
	Evas_Object *pGenlist;
	Evas_Object *pGengrid;
	//Evas_Object *pSelectAllCheckBox;
	//Evas_Object *pSelectAllLayout;
	Evas_Object *pPathInfo;
	Evas_Object *pInfoBox;
	Eina_Bool bSelectAllChecked;
	int iTotalCount;
	int iCheckedCount;
};
#ifdef MYFILE_SPLIT_VIEW
typedef struct __mfSplitData mfSplitData;
struct __mfSplitData {
	Evas_Object *pMainLayout;
	Evas_Object *pPanes;

	Evas_Object *pSplitLeftGenlist;
	//Evas_Object *pSelectAllLayout;
	//Evas_Object *pSelectAllCheckBox;
	Elm_Object_Item *phone_item;
	Elm_Object_Item *mmc_item;
	Elm_Object_Item *otg_item;
	bool bFlagSplitViewCreate;
	bool bFlagPortraitViewCreate;
};
#endif

typedef struct _myfileMainWindow myfileMainWindow;
struct _myfileMainWindow {
	/* root window size */
	int root_w;
	int root_h;
	int root_x;
	int root_y;

	/* graphic data */
	Evas *evas;
	Evas_Object *pWindow;
	Evas_Object *pBackGround;
	Evas_Object *pConformant;
	Evas_Object *pMainLayout;
	Evas_Object *pTabBar;
	Eina_List *plistNaviBar;

	Evas_Object *pNormalPopup;
	Evas_Object *pContextPopup;
	Evas_Object *pEntry;
	Evas_Object *pEditField;
	Evas_Object *pBox;
	Ecore_Event_Handler *event;
	Ecore_Event_Handler *font_event;
	Evas_Object *pLabel;
	Evas_Object *pLabelLayout;
	Evas_Object *pProgressPopup;
	Evas_Object *pFinishPopup;
	Evas_Object *pMmcRemovedPopup;
	Evas_Object *pProgressLayout;
	Evas_Object *pOperationNotify;
	Evas_Object *pSearchEntry;
	Evas_Object *pSearchLabel;


	Evas_Object *pNaviBar;
	Evas_Object *pNaviLayout;
	Evas_Object *pNaviLeftBtn;
	Evas_Object *pNaviCtrlBar;
	Evas_Object *pNaviSearchBar;
	Evas_Object *pNaviBox;
	Evas_Object *pNaviGenlist;
	Evas_Object *pNaviGengrid;
	Evas_Object *pNaviPathInfo;
	Evas_Object *pNaviInfoBox;
	Evas_Object *pTrayBox;

	const Elm_Object_Item *pSearchViewSweepItem;
	const Elm_Object_Item *pPreSearchViewSweepItem;
	Ecore_Timer *pSearchViewTimer;
	Ecore_Timer *pPopupTimer;

#ifdef MYFILE_SPLIT_VIEW

	Evas_Object *pNaviSplitContent;
#endif
	Elm_Object_Item *pNaviItem;

#ifdef MYFILE_SPLIT_VIEW
	mfSplitData   sSplitData;
#endif
	mfEditView pEditView;
};

typedef struct _myfileStatus myfileStatus;

struct _myfileStatus {

	int more;		/** current mode **/
	int preMore;		/** the mode before operation **/
	int rotation_type;	/** current rotation type **/
	int pre_rotate_type;
	int folder_count;	/** current new created folders count **/
	int theme_type;		/** current theme type **/
	int iRadioValue;	/** current the radio box selected item value **/
	int flagViewType;	/** current view type, List view or Thumbnail view **/
	int iFolderSortType;	/* for list by operation */
	int iStorageState;	/** current storage state **/
	int iCtrlDisableItem;
	int iNFCState;
	int iSelectedSortType;
	int iExtensionState;
	int view_type;
	int preViewType;

	Eina_Bool flagRootView;
	Eina_Bool flagNoContent;
	Eina_Bool flagFolderSelect;
	Eina_Bool flagFileSelect;
	Eina_Bool flagStorageRemoved;
	Eina_Bool flagNaviPush;
	Eina_Bool flagIcuInit;
	Eina_Bool flagLCDLock;		/*lcd lock status*/
	Eina_Bool flagCtxPopShow;
	Eina_Bool flagSearchAll;
	Eina_Bool flagTabbarShow;
	Eina_Bool flagIME;

	Evas_Object *pRadioGroup;
	GString *path;		/* current path */
	char *upper_folder;
	char *search_filter;
	int navi_item_selected;
	char *monitor_path;
	Elm_Object_Item *pPreNaviItem;
	/* icu related */
	UDateTimePatternGenerator *generator;
	UDateFormat *formatter;

	mf_search_handle search_handler;

#ifdef MYFILE_SPLIT_VIEW
	bool flagNofiles;
	bool flagUpperSet;
	/* rotate related*/
	bool view_mode_start;
	bool view_mode_end;
	int rotation_angle;
	Ecore_Timer *rotation_timer;
#endif

#ifdef MYFILE_MOTION_FEATURE
	void (*doubletab_event_handler)(Evas_Object *data);
#endif
	Ecore_Idler *app_init_idler;
	Ecore_Idler *popup_del_idler;
	Ecore_Idler *navi_content_idler;
	Evas_Object *old_content;

	bool b_run_background; /* 20121008 added for fix PLM P121001-0404 temporary */
};


typedef struct _myfileFileOperation myfileFileOperation;
struct _myfileFileOperation {
	/* to rename */
	GString *to_rename;
	char *file_name_suffix;

	/*clicked item name */
	GString *clicked_item;

	/*mmc operation flag */
	gboolean bmmc_is_removing;

	/*image viewer SG */
	ui_gadget_h ugImageViewer;

	/*      operation error number  */
	int error_number;
	int error_code;

	/* pipe for inotify */
	void *conn;
	Ecore_Pipe *sync_pipe;
	gboolean same_storage;

	/*      refresh type */
	GString *source;
	GString *destination;
	gboolean refresh_type;

	Ecore_Pipe *inotify_pipe;

	/*  progress bar data record    */
	Evas_Object *progress_bar;
	Evas_Object *persent_label;
	Evas_Object *count_label;
	unsigned int current_count;
	unsigned long total_file_size;
	unsigned long finished_size;
	int progress_cancel;

	/**drm file path*/
	char *drm_file_path;
	Eina_List *search_result_list;
	Eina_List *default_dir_list;
	Eina_List *user_dir_list;
	Eina_List *folder_list;
	Eina_List *file_list;
	Eina_List *category_list;


	mf_cancel *pCancel;
	mf_fo_request *pRequest;
	mf_fo_msg *pMessage;
	GList *pSourceList;

	Elm_Object_Item *pGenlistItem;
	int iTotalCount;
	int iRequestType;
	Ecore_Timer *search_IME_hide_timer;
	Elm_Object_Item *rename_item;

#ifdef	MYFILE_USB_OTG
	Eina_List *otg_dev_list;
	Eina_List *otg_dev_removed_list;
#endif
	/**job handler**/
	Eina_List *job_pop_list;	/*to record the job handler for to_content_pop*/
	Ecore_Job *job_push; 	/*to record the job handler for push a new view*/
	bool iOperationSuccessFlag;	/*used for confirm whether the operation fininsh successfully, not canceled and failed*/
	char *pOperationMsg;
	Elm_Object_Item *idle_delete_item;
};

typedef struct _myfileFileRecord myfileFileRecord;

struct _myfileFileRecord {
	/* operation property */
	Eina_List *operation_property;

	/* value saver for bluetooth / protection */
	Eina_List *value_saver;
	Eina_List *selected_files;
	Eina_List *selected_dest_files;
	Eina_List *backup_file_list;
};

typedef struct _myfileSharedGadget myfileSharedGadget;
struct _myfileSharedGadget {
	int is_password_correct;
	ui_gadget_h ug;
	/*SGController  *sg_controller;for privacy lock*/
};

struct _myfileOtg {
	Evas_Object *rootpath_box;
	Evas_Object *rootpath_layout;
	Elm_Object_Item *rootpath_navi_item;
};
typedef struct _myfileOtg myfileOtg;

struct _myfileBundle {
	service_h recv_service;
	char *path;
	char *select_type;
	char *file_type;
	char *marked_mode;
	char *drm_type;
};

typedef struct _myfileBundle myfileBundle;

struct appdata {
	myfileMainWindow mf_MainWindow;
	myfileStatus mf_Status;
	myfileFileOperation mf_FileOperation;
	myfileFileRecord mf_FileRecordList;
	myfileSharedGadget mf_SharedGadget;
	myfileOtg mf_Otg;
	myfileBundle mf_Bundle;
};


/*handler list node struct*/
typedef struct _myfileEcoreHandleNode	myfileEcoreHandleNode;
struct _myfileEcoreHandleNode {
	Ecore_Job *handler;
	char *navi_label;
};


typedef enum _MF_ACTION mfAction;

enum _MF_ACTION {
	MFACTION_CLICK,
	MFACTION_FLICK
};

#define _EDJ(o) elm_layout_edje_get(o)
Evas_Object *mf_main_load_edj(Evas_Object * parent, const char *file, const char *group);
#define GET_SYS_STR(str) dgettext("sys_string", str)
#define GET_STR(str) _(str)

#endif /* __DEF_MYFILE_H_ */
