/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */



#ifndef	__DEF_MF_OTG_H__
#define __DEF_MF_OTG_H__

#include "mf-main.h"
#include "mf-fs-util.h"
#include "mf-resource.h"

enum _mf_otg_list {
	MF_OTG_LIST_NONE = 0,
	MF_OTG_LIST_DEV,
	MF_OTG_LIST_REMOVED,
	MF_OTG_LIST_MAX
};
typedef enum _mf_otg_list mf_otg_list_e;

enum _mf_otg_status {
	MF_OTG_STATUS_NONE = 0x00,
	MF_OTG_STATUS_EDIT = 0x01,
	MF_OTG_STATUS_USING = 0x02
};
typedef enum _mf_otg_list mf_otg_status_e;

enum _mf_otg_ex_opt {
	MF_OTG_OPT_NONE = 0x00,
	MF_OTG_OPT_REQ_POPUP_DEL = 0x01,
	MF_OTG_OPT_NOR_POPUP_DEL = 0x02,
	MF_OTG_OPT_TABBAR_ENABLE = 0x04,
	MF_OTG_OPT_THEME_RESET = 0x08

};
typedef enum _mf_otg_ex_opt mf_otg_ex_opt_e;

struct _mf_otg_node {
	char *name;		/*disk name*/
	bool flag_using;	/*if the disk is using*/
	bool flag_editstart;	/*if the edit mode changed from this disk*/
};
typedef struct _mf_otg_node mf_otg_node;

/**
 * mf_otg_list_clean
 *@data, the appdata which contain the otg lists : device list, removed list
 *@list_type, a variable for indicate list type: device list, removed list
 *clean the otg list whicn list_type indicate
 */
void mf_otg_list_clean(void *data, mf_otg_list_e list_type);

/**
 * mf_otg_init
 *@data, the appdata which contain the otg list
 *read the otg folder, save the device into the otg list
 */
int mf_otg_init(void *data);
/**
 * mf_otg_is_in_list
 *@data, the appdata which contain the otg list
 *@dev name: the device name
 *check if the device in the otg list
 */

bool mf_otg_is_in_list(void *data, const char *dev_name);

/**
 * mf_otg_insert_node
 *@data, the appdata which contain the otg list
 *@name: the new insert device name
 *insert the new insert device into the otg list
 */

int mf_otg_insert_node(void *data, const char *name);

/**
 * mf_otg_delete_node
 *@data, the appdata which contain the otg list
 *@name: the remove device name
 *delete the removed device node from the otg list
 */
int mf_otg_delete_node(void *data, const char *name);

/**
 * mf_otg_list_update
 *@data, the appdata which contain the otg list
 *@flag_inc_dec: true: increase devices. false: remove devices
 *update the otg list
 */
int mf_otg_list_update(void *data, bool flag_inc_dec);

/**
 * mf_otg_is_empty
 *@data, the appdata which contain the otg list
 *check if the otg list is empty
 *
 */
bool mf_otg_is_empty(void *data);

/**
 * mf_otg_get_node_in_use
 *@data, the appdata which contain the otg list
 *get the otg device name which is in used currently
 *
 */
char *mf_otg_get_node_in_use(void *data);

/**
 * mf_otg_set_node_using_flag
 *@data, the appdata which contain the otg list
 *@dev_name, the dest device name
 *@status, ture: in used; false: not in used
 *set the dest device in used flag
 *
 */
void mf_otg_set_node_using_flag(void *data, char *dev_name, bool status);

/**
 * mf_otg_clear_removed_device
 *@data, the appdata which contain the otg list
 *clear the removed otg list
 *
 */
void mf_otg_clear_removed_device(void *data);

/**
 * mf_otg_generate_removed_list
 *@data, the appdata which contain the otg list
 * get the devices which are not in the otg list now to generate the removed list
 *
 */
void mf_otg_generate_removed_list(void *data);

/**
 * mf_otg_dev_removed_update
 *@data, the appdata which contain the otg list
 *@opt, the option which need to check when update the view
 *	 MF_OTG_OPT_REQ_POPUP_DEL: delete the request popup
 *	 MF_OTG_OPT_NOR_POPUP_DEL: delete the normal popup
 *	 MF_OTG_OPT_TABBAR_ENABLE: set the tar bar enable
 *	 MF_OTG_OPT_THEME_RESET: reset the mainlayout theme
 *
 * the refresh operation when remove some devices but not remove all devices
 *
 */
void mf_otg_dev_removed_update(void *data, mf_otg_ex_opt_e opt);

/**
 * mf_otg_dev_all_removed_update
 *@data, the appdata which contain the otg list
 *@opt, the option which need to check when update the view
 *	 MF_OTG_OPT_REQ_POPUP_DEL: delete the request popup
 *	 MF_OTG_OPT_NOR_POPUP_DEL: delete the normal popup
 *	 MF_OTG_OPT_TABBAR_ENABLE: set the tar bar enable
 *	 MF_OTG_OPT_THEME_RESET: reset the mainlayout theme
 *
 * the refresh operation when remove all devices
 *
 */
void mf_otg_dev_all_removed_update(void *data, mf_otg_ex_opt_e opt);

/**
 * mf_otg_finalize
 *@data, the appdata which contain the otg list
 *
 * clear the otg list
 *
 */
void mf_otg_finalize(void *data);
void mf_otg_set_node_editstart_flag(void *data, char *dev_name, bool status);

#endif
