/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */





#ifndef __DEF_MYFILE_UTIL_H_
#define __DEF_MYFILE_UTIL_H_

#include <sys/time.h>
#include <stdbool.h>

#include <assert.h>
#include <stdlib.h>
#include <app.h>
#include <Elementary.h>
#include "mf-fo-common.h"
#include "mf-fs-util.h"


#define SAFE_FREE_CHAR(x) do {\
					if ((x) != NULL) {\
						free(x); \
						x = NULL;\
					} \
			     } while (0)

#define SAFE_FREE_GSTRING(x) do {\
					if ((x) != NULL) {\
						g_string_free(x, TRUE); \
						x = NULL;\
					} \
				} while (0)

#define CHAR_CHECK_NULL_GOTO(arg, dest) do {\
						if ((arg) == NULL) {\
							goto dest;\
						} \
					   } while (0)

#define GSTRING_CHECK_NULL_GOTO(arg, dest) do {\
						if ((arg) == NULL || (arg->str) == NULL) {\
							goto dest;\
						} \
					   } while (0)

#define SAFE_TIMER_DEL(x) do {\
				if ((x) != NULL) {\
					ecore_timer_del(x); \
					x = NULL;\
				} \
			  } while (0)

#define SAFE_DEL_NAVI_ITEM(x) do {\
					if ((x) != NULL) {\
						elm_object_item_del(x); \
						x = NULL;\
					} \
				  } while (0)

#define SAFE_DEL_ECORE_TIMER(timer) do { \
						if (timer) { \
							ecore_timer_del(timer);\
							timer = NULL; \
						} \
					} while (0)

#define SAFE_FREE_OBJ(x) do {\
					if ((x) != NULL) {\
						evas_object_del(x); \
						x = NULL;\
					} \
			     } while (0)

#define SAFE_STRCPY(dest, src) \
			do{if(!dest||!src)break;\
				strncpy (dest , src, sizeof(dest)-1);\
				dest[sizeof(dest)-1] = 0;	}while(0)

#define mf_ecore_idler_del(idler) do { \
				if(idler) { \
					ecore_idler_del(idler);\
					idler = NULL; \
				} \
			} while (0)

#define SAFE_FREE_ECORE_EVENT(event) do { \
						if (event) { \
							ecore_event_handler_del(event);\
							event = NULL; \
						} \
					} while (0)

typedef enum _MF_INTERNAL_NAME_ERR MF_INTERNAL_NAME_ERR;
enum _MF_INTERNAL_NAME_ERR {
	MF_INTERNAL_FILE_NAME_NULL = 1,
	MF_INTERNAL_FILE_NAME_EMPTY,
	MF_INTERNAL_FILE_NAME_IGNORE,
	MF_INTERNAL_FILE_NAME_CHUG,
	MF_INTERNAL_FILE_NAME_MAX_LENGTH,
	MF_INTERNAL_FILE_NAME_INVALID_CHAR,
};

typedef enum _MF_THEME_TYPE	MF_THEME_TYPE;
enum _MF_THEME_TYPE {
	THEME_INVALID = -1,
	THEME_NBEAT = 0,
	THEME_NBEAT_BLACK = 1,
	THEME_ERROR
};

typedef enum __mf_thumbnail_type mf_thumbnail_type;
enum __mf_thumbnail_type {
	MF_THUMBNAIL_TYPE_DEFAULT,
	MF_THUMBNAIL_TYPE_THUMBNAIL,
	MF_THUMBNAIL_TYPE_MAX
};

#define MYFILE_MAGIC_MAIN_CONTEXT           (0x1983cdaf)
#define MYFILE_MAGIC_DETAIL_LIST_ITEM       (0x1977abcd)
#define MYFILE_MAGIC_PIPE_DATA				 (0x0716ffcc)

#define MYFILE_FINISH_MMC_INIT_DATA      "db/Apps/FileManager/FinishMmcInitData"

#define MYFILE_MAGIC                 unsigned int  __magic
#define MYFILE_MAGIC_SET(d, m)       (d)->__magic = (m)
#define MYFILE_MAGIC_CHECK(d, m)     ((d) && ((d)->__magic == (m)))

#define MYFILE_DBUS_SIGNAL_INTERFACE	"app.fexplorer.dbus.Signal"
#define MYFILE_DBUS_SIGNAL_PATH		"/app/fexplorer/dbus"
#define MAX_LEN_VIB_DURATION 		0.5

typedef enum _MORE_TYPE MORE_TYPE;
enum _MORE_TYPE {			/* softkey / contextual popup */
	MORE_DEFAULT = 0,
	MORE_EDIT,
	MORE_CREATE_FOLDER,
	MORE_DELETE,
	MORE_IDLE_DELETE,
	MORE_DATA_COPYING,
	MORE_DATA_MOVING,
	MORE_INTERNAL_COPY_MOVE,
	MORE_INTERNAL_COPY,
	MORE_INTERNAL_MOVE,
	MORE_THUMBNAIL_RENAME,
	MORE_RENAME,
	MORE_SEARCH,
	MORE_SENDVIABLUETOOTH,
	MORE_SENDVIAEMAIL,
	MORE_SENDVIAMESSAGE,
	MORE_TYPE_MAX
};

typedef enum __mf_view_type_e mf_view_type_e;
enum __mf_view_type_e {
	mf_view_root = 0,
	mf_view_root_category,
	mf_view_normal,
};

typedef enum _LAUNCH_TYPE LAUNCH_TYPE;
enum _LAUNCH_TYPE {
	LAUNCH_TYPE_FORK = 0,
	LAUNCH_TYPE_FAKE,
	LAUNCH_TYPE_FAIL,
	LAUNCH_TYPE_DIR,
	LAUNCH_TYPE_UNSUPPORT,
	LAUNCH_TYPE_MAX
};


typedef enum _REPORT_TYPE REPORT_TYPE;
enum _REPORT_TYPE {
	MYFILE_REPORT_NONE = 1,
	MYFILE_REPORT_RECURSION_DETECT,
	MYFILE_REPORT_BOTH_ARE_SAME_FILE,
	MYFILE_REPORT_MAX,
};

typedef enum _MYFILE_FILE_NAME_TYPE MYFILE_FILE_NAME_TYPE;
enum _MYFILE_FILE_NAME_TYPE {
	FILE_NAME_WITH_BRACKETS,
	FILE_NAME_WITH_UNDERLINE,
	FILE_NAME_NONE,
};

typedef enum _MYFILE_CONTENT_TYPE MYFILE_CONTENT_TYPE;
enum _MYFILE_CONTENT_TYPE {
	MYFILE_TYPE_MIN,
	MYFILE_TYPE_GSTRING,
	MYFILE_TYPE_CHAR,
	MYFILE_TYPE_FSNODE,
	MYFILE_TYPE_MAX
};

struct _mtd {
	char *category;
	char *key;
	char *icon_path;
	char *exe_path;
	char *param;
	int is_direct_launch;
	int need_sort_param;
	int having_a_thumbnail;
};


typedef enum _MYFILE_VCONF_TYPE MYFILE_VCONF_TYPE;
enum _MYFILE_VCONF_TYPE {
	VCONF_TYPE_SORT_TYPE = 0,
	VCONF_TYPE_TIME_FORMAT,
	VCONF_TYPE_DATE_FORMAT,
	VCONF_TYPE_MASS_STORAGE,
	VCONF_TYPE_VIEW_STYLE,
	VCONF_TYPE_NFC_ENABLE,
	VCONF_TYPE_EXTENSION_STATE,
	VCONF_TYPE_MAX
};

void mf_callback_warning_popup_cb(void *data, Evas_Object *obj, void *event_info);
int keydown_cb(void *data, int type, void *event);

void ime_mkdir_callback_cb(void *data);
void ime_rename_callback_cb(void *data);
void add_panel_to_softkey(Evas_Object *obj, int mode);
void create_popup(void *data, char *header, Evas_Smart_Cb func);
void softkey_cb(void *data, Evas_Object *obj, void *event_info);
void tab_cb(void *data, Evas_Object *obj, void *event_info);
int keydown_cb(void *data, int type, void *event);
int copy_files(char *src, char *dst);

void file_selection_popup_cb(void *data, Evas_Object *obj, void *event_info);
int set_default_state_as(int state);

bool mf_util_check_forbidden_operation(void *data);
int mf_util_check_disk_space(void *data);

void operation_abort(void *data);
void myfile_recovery_after_cancel();
void myfile_mmc_in_out_cb(void *data);

int mf_util_refresh_screen(void *data);
int mfScreenReset(void *data);
int mf_util_is_mmc_on(int *mmc_card);
int mf_util_is_otg_on(int *otg);
int mf_util_get_eina_list_len(const Eina_List *list);
void mf_util_free_eina_list_with_data(Eina_List **list, MYFILE_CONTENT_TYPE type);
void mf_util_operation_alloc_failed(void *data);
void mf_util_action_storage_insert(void *data, char *pItemLabel);
int mf_util_get_vconf_value(MYFILE_VCONF_TYPE type, int *value);
void mf_util_set_sort_type(int value);
void mf_util_merge_eina_list_to_glist(const Eina_List *eSource, GList **gSource);
void mf_util_exception_func(void *data);
int mf_util_is_valid_name_check(const char *name);
void mf_util_ex_disk_list_update(void *data);
long mf_util_character_count_get(const char *original);
/**ecore_*** handler related**/
void mf_util_add_to_handler_list(void *data, Ecore_Job *handler, char *label);
void mf_util_delete_from_handler_list(void *data, char *label, bool flag_del);
void mf_util_clear_handler_list(void *data);
char *mf_util_upper_folder_name_get(void *data, GString *fullpath);
/**icu related**/
char *mf_util_get_icu_date(fsNodeInfo *pNode);
int mf_util_icu_init(void *data);
void mf_util_icu_finalize(void *data);
char *mf_util_icu_translate(void *data, UDate date);
void mf_util_set_pm_lock(void *data, Eina_Bool isLock);
Eina_List *mf_util_get_all_selected(void *data);
gboolean mf_util_is_file_selected(Eina_List **source, GString *path);
MF_THEME_TYPE mf_util_get_theme(void);
void mf_util_sort_the_file_list(void *data);
int mf_util_generate_file_list(void *data);
char *mf_util_get_text(const char *ID);
void mf_util_set_view_style(int value);
void mf_util_rotate_state_set(void *data, app_device_orientation_e  rotate_mode);
int mf_util_generate_root_view_file_list(Eina_List **list, int storage_state);
void mf_util_set_extension_state(int value);
const char *mf_util_search_markup_keyword(const char *string, char *searchword, bool *result);
void mf_util_set_recent_file(char *path);
Eina_List *mf_util_get_all_selected_items(void *data);
void mf_util_remove_item_from_list_by_location(Eina_List **list, int location);
void mf_util_generate_list(Eina_List **list, const char *path, int file_type);
bool mf_util_is_rotation_lock(void);

#endif /* __DEF_MYFILE_UTIL_H_ */
