/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */



#ifndef __DEF_MYFILE_WINSET_H_
#define __DEF_MYFILE_WENSET_H_

#include <stdio.h>
#include <assert.h>
#include <glib.h>


#include "mf-conf.h"
#include "mf-fs-util.h"
#include "mf-callback.h"
#include "mf-split-view.h"
#include "mf-gengrid.h"
#include "mf-dlog.h"
#include "mf-resource.h"


#define NAVI_MORE_BUTTON_PART		"toolbar_more_btn"
#define NAVI_BOTTOM_BUTTON_1_PART	"toolbar_button1"
#define NAVI_BOTTOM_BUTTON_2_PART	"toolbar_button2"
#define NAVI_BUTTON_STYLE		"naviframe/toolbar/default"
#define NAVI_BUTTON_EDIT		"naviframe/more/default"

#define CTRL_STYLE_TYPE		"controlbar"
#define TITLE_LEFT_BTN		"title_left_btn"
#define TITLE_RIGHT_BTN		"title_right_btn"
#define TITLE_BTN_STYLE	"elm/button/base/naviframe/title_icon"

/***********	Layout APIs	***********/
Evas_Object *mfCreateLayout(Evas_Object *parent, char *label, void(*_back_cb)(void *, Evas_Object *, void *), void *data);
void mfDeleteLayout(Evas_Object *pLayout);

/***********	TabBar APIs	***********/
Evas_Object *mf_tab_bar_create_tab_bar(Evas_Object *parent);
Elm_Object_Item *mf_tab_bar_append_item(Evas_Object *pTabBar, char *pTabIconPath, char *pTabBarLabel, void *user_data);
void mf_tab_bar_set_selected_item(Elm_Object_Item *pItem);
void mf_tab_bar_set_item_view(Evas_Object *parent, Evas_Object *tab_layout, Evas_Object *view);
void mf_tab_bar_set_item_disabled(Evas_Object *pTabBar, Eina_Bool disable);
Elm_Object_Item *mf_tab_bar_get_item_by_label(Evas_Object *pTabBar, char *pTabLabel);
int mf_tab_bar_items_count_get(Evas_Object *pTabBar);
void mf_tab_bar_items_del(Evas_Object *pTabBar);
void mf_tab_bar_generate(void *data);

/***********	Control Bar Definitions	***********/
#define	EDIT_SEG_COUNT	3
#define	OPER_SEG_COUNT	5

#define MF_CTRL_ICON_NEW_FOLDER_CREATE				ICON_PATH"/myfile_icon_create_folder.png"
#define MF_CTRL_ICON_LIST_BY					ICON_PATH"/myfile_icon_list_by.png"
#define MF_CTRL_ICON_EDIT					ICON_PATH"/myfile_icon_edit.png"
#define MF_CTRL_ICON_COPY					ICON_PATH"/myfile_icon_copy.png"
#define MF_CTRL_ICON_MOVE					ICON_PATH"/myfile_icon_move.png"
#define MF_CTRL_ICON_PASTE_HERE					ICON_PATH"/myfile_icon_paste_here.png"
#define MF_CTRL_ICON_MOVE_HERE					ICON_PATH"/myfile_icon_move_here.png"
#define MF_CTRL_ICON_DELETE					ICON_PATH"/myfile_icon_delete.png"
#define MF_CTRL_ICON_SEND					ICON_PATH"/myfile_icon_share.png"
#define MF_CTRL_ICON_STORE_HERE					ICON_PATH"/myfile_icon_store_here.png"
#define MF_CTRL_ICON_CANCEL					ICON_PATH"/myfile_icon_cancel.png"
#define MF_CTRL_ICON_SAVE					ICON_PATH"/myfile_icon_done.png"
#define MF_CTRL_ICON_SEARCH					ICON_PATH"/myfile_icon_search.png"
#define MF_TITLE_ICON_HOME	            			ICON_PATH"/Controlbar/U01_controlbar_cion_home.png"
#define MF_TITLE_ICON_HOME_PRESS	            		ICON_PATH"/Controlbar/U01_controlbar_cion_home_press.png"
#define MF_TITLE_ICON_UPPER	            			ICON_PATH"/Controlbar/U01_controlbar_cion_up_folder.png"
#define MF_TITLE_ICON_UPPER_PRESS	            		ICON_PATH"/Controlbar/U01_controlbar_cion_up_folder_press.png"
#define MF_TITLE_ICON_SELECT_ALL	            		ICON_PATH"/00_icon_select_all_web.png"
#define MF_TITLE_ICON_SELECT_ALL_PRESS	            		ICON_PATH"/00_icon_select_all_press_web.png"
#define MF_TITLE_ICON_SELECT_ALL_DIM	            		ICON_PATH"/00_icon_select_all_dim_web.png"

typedef enum _eCtrlBarButtonType eCtrlBarButtonType;
enum _eCtrlBarButtonType {
	CTRL_BUTTON_NEW_FOLDER_CREATE = 0,
	CTRL_BUTTON_LIST_BY,
	CTRL_BUTTON_EDIT,
	CTRL_BUTTON_COPY,
	CTRL_BUTTON_MOVE,
	CTRL_BUTTON_DELETE,
	CTRL_BUTTON_SEND,
	CTRL_BUTTON_PASTE_HERE,
	CTRL_BUTTON_MOVE_HERE,
	CTRL_BUTTON_CANCEL,
	CTRL_BUTTON_SAVE,
	CTRL_BUTTON_COPY_MOVE,
	CTRL_BUTTON_SEARCH,
	CTRL_BUTTON_MAX
};

/***********	Control Bar APIs	***********/
Evas_Object *mf_ctrl_bar_create(void *data, Evas_Object *parent);
void mf_ctrl_bar_item_set_disable(Evas_Object *ctrl_bar, int disable_item, bool disable);
void mf_ctrl_bar_reset(void *data);
int mf_ctrl_bar_item_get_disable(Evas_Object *ctrl_bar);


/***********	Navigation Bar Definition	************/
#define MF_POPUP_TIMEOUT_NORMAL		3
#define MF_POPUP_TIMEOUT_SHORT		2

typedef enum {
	NAVI_BAR_MIN = 0,
	NAVI_BAR_DEFAULT,	/*when MMC is on, show All/Phone/Memory Card together OR when copy/move show phone and mmc*/
	NAVI_BAR_TITLE,		/*when MMC is off, show "myfile" only*/
	NAVI_BAR_PATH_WITH_ONE_ACTION,
	NAVI_BAR_PATH_WITH_TWO_ACTION,
	NAVI_BAR_MAX
} eNaviBarStyle;


/***********	Navigation Bar API		************/
#define MF_NAVI_STYLE_ENABLE	"basic"
#define MF_NAVI_STYLE_DISABLE	"basic/instant"

typedef enum _MF_NAVIFRAME_ITEM_DEL_TYPE {
	MF_NAVIFRAME_ITEM_DEL_DIRECTLY,
	MF_NAVIFRAME_ITEM_DEL_BY_NAVI
}MF_NAVIFRAME_ITEM_DEL_TYPE;

typedef enum __mf_notify_popup_type {
	mf_notify_popup_normal,
	mf_notify_popup_layout
}mf_notify_popup_type;

typedef enum __mf_navi_btn_type mf_navi_btn_typ;
enum __mf_navi_btn_type {
	MF_NAVI_BTN_MORE = 0,
	MF_NAVI_BTN_SHARE,
	MF_NAVI_MORE_MAX
};


Evas_Object *mf_navi_bar_create_navi_bar(Evas_Object *parent);
Evas_Object *mfNaviBarConformGet(void *data);
void mf_navi_bar_set_ctrl_bar(struct appdata *data);
Evas_Object *mf_navi_bar_get_content(Evas_Object *pConform);
void mf_navi_bar_set_style(void *data, eNaviBarStyle navi_style, Evas_Object *NaviContent);
void mf_navi_bar_set_selected_count_label(void *data, int dir_count, int file_count);
void mf_navi_bar_create_default_view(void *data);
void mf_navi_bar_create_edit_view(void *data);
void mf_navi_bar_create_path_select_view(void *data);
void mf_navi_bar_create_path_select_view_otg_root(void *data);
void mf_navi_bar_create_search_view(void *data);
myfileNaviBar *mf_navi_bar_get_in_use(void *data);
void mf_navi_bar_object_reset(myfileNaviBar *pNavi_s);
bool mf_navi_bar_remove_list_item_by_label(void *data, const char *pNaviLabel);
myfileNaviBar *mf_navi_bar_get_struct_by_label(void *data, const char *pNaviLabel);
void mf_navi_bar_refresh_recovered_view(void *data, myfileNaviBar * pNaviStruct);
myfileNaviBar *mf_navi_bar_recover_list(void *data);
myfileNaviBar *mf_navi_bar_get_edit_start(void *data);
bool mf_navi_bar_is_navi_empty(void *data, char *label);
Evas_Object *mf_navi_bar_get_box_end(Evas_Object * pBox);
void mf_navi_bar_set_upper_item_disable(void *data, bool bFlag);
void mf_navi_bar_recover_info_box(void *data, myfileNaviBar *pNavi_s);
void mf_navi_bottom_item_del(Elm_Object_Item **pPreNaviItem);
Evas_Object *mf_navi_bar_get_edit_genlist(void *data);
Evas_Object *mf_navi_bar_upper_create(Evas_Object *parent);
void mf_navi_bar_remove_back_button(Elm_Object_Item *pNaviItem);
Evas_Object *mf_navi_bar_pathinfo_label_create(Evas_Object *parent, char *pathinfo);
void mfNaviBarSetContent(void *data, Evas_Object *pLayout, Evas_Object *NaviContent);
myfileNaviBar *mf_navi_bar_get_navi_from_navilist(Eina_List *navilist, const char *label);
void mf_navi_bar_transition_finished_cb(void *data, Evas_Object *obj, void *event_info);
void mf_navi_bar_remove_previous_contents(void *data, Evas_Object *pNaviBar);
void mf_navi_add_back_button(void *data);
void mf_navi_bar_del_item_directly(void *data);
Evas_Object *mf_navi_bar_content_create(void *data);
void mf_navi_bar_edit_list_update(void *data);
void mf_navi_bar_reset_panes_main(void *data);
void mf_navi_bar_create_rename_view(void *data);
void mf_navi_bar_title_set(void *data);
Evas_Object *mf_navi_bar_create_header_box(void *data);
void mf_navi_bar_reset_ctrl_button(void *data);
void mf_navi_bar_button_set_disable(Elm_Object_Item *navi_it, int disable_item, bool disable);
int mf_navi_bar_button_get_disable(Elm_Object_Item *navi_it);
void mf_navi_bar_set_toolbar_visible(Elm_Object_Item *navi_item, bool visible);
void mf_navi_bar_set_panes_navi_to_main(void *data);

/***********	Content Definition		************/
#define INTERVAL_THUMB_UPDATE	0.5

typedef struct {
	GString *m_ItemName;
	char *size;
	char *create_date;
	char *thumb_path;
	Eina_Bool m_checked;
	bool real_thumb_flag;
	int flagExpand;
	Evas_Object *pCheckBox;
	Elm_Object_Item *item;
	bool unregister_lang_mgr;
	media_info_h media;
	struct appdata *ap;
} mfItemData_s;

typedef enum __eMfViewStyle {
	MF_VIEW_STYLE_LIST,
	MF_VIEW_SYTLE_LIST_DETAIL,
	MF_VIEW_STYLE_THUMBNAIL,
}eMfViewStyle;

/***********	Content API			************/
void mf_genlist_get_list_selected_files(Evas_Object * pGenlist, Eina_List **list);
Evas_Object *mf_genlist_create_list(void *data, Evas_Object *parent);
Evas_Object *mfSearchResultListView(void *data);
Evas_Object *mf_genlist_create_list_new_folder_style(void *data);
void mf_genlist_sweep_item_recover(Evas_Object *obj);
void mf_genlist_create_list_default_style(Evas_Object *pGenlist, void *data, Eina_List *dir_list,
					  Eina_List *file_list);
int mf_util_generate_list_data(const char *path, Eina_List ** default_dir_list, Eina_List ** user_dir_list, Eina_List ** file_list);

void mf_genlist_update_default_view_item_style(void *data);
void mf_genlist_update_edit_view_item_style(void *data);
void mf_genlist_clear(Evas_Object *genlist);
void mf_genlist_rename_done(void *data, Evas_Object *obj, void *event_info);
#ifdef MYFILE_SPLIT_VIEW
Elm_Object_Item *mf_genlist_split_root_item_append(void *data, Evas_Object *pGenlist, const char *name);
void mf_genlist_split_left_genlist_style(void *data);
void mf_genlist_split_root_items_append(Evas_Object *pGenlist, void *data);
Elm_Object_Item *mf_genlist_split_left_items_append( void *data, Evas_Object *pGenlist, Elm_Object_Item *parent_item, Eina_List *dir_list);
#endif
Evas_Object *mf_genlist_create_list_rename_style(void *data);
void mf_genlist_create_search_itc();
void mf_genlist_get_list_selected_items(Evas_Object * pGenlist, Eina_List **list);
void mf_genlist_gl_sel(void *data, Evas_Object * obj, void *event_info);
void mf_genlist_create_data(mfItemData_s **m_TempItem, const char *name, void *data);


/***********	Popup Definition	************/
typedef struct _StorageData_s StorageData_s;

struct _StorageData_s{
	char *label;
	int index;
	bool unregister_lang_mgr;
	Elm_Object_Item *item;
	struct appdata *ap;
} ;

typedef enum {
	POPMODE_MIN = 0,
	POPMODE_TEXT,
	POPMODE_TITLE_TEXT,
	POPMODE_TEXT_NOT_DISABLED,
	POPMODE_TEXT_BTN_SEARCH,
	POPMODE_TEXT_TWO_BTN,
	POPMODE_TITLE_TEXT_TWO_BTN,
	POPMODE_TEXT_BTN,
	POPMODE_TITLE_TEXT_BTN,
	POPMODE_TITLE_TEXT_THREE_BTN,
	POPMODE_PROGRESSBAR,
	POPMODE_TITLE_LIST_BTN,
	POPMODE_STORAGE_BTN,
	POPMODE_VIEW_AS_LIST,
	POPMPDE_MAX
} ePopMode;

typedef enum {
	ELM_POPUP_NONE,
	ELM_POPUP_YES,
	ELM_POPUP_NO,
	ELM_POPUP_OK,
	ELM_POPUP_CANCLE,
	ELM_POPUP_AUTO_RENAME,
	ELM_POPUP_REPLACE,
	ELM_POPUP_PHONE,
	ELM_POPUP_MEMORY,
	ELM_POPUP_MOVE,
	ELM_POPUP_DELETE,
	ELM_POPUP_MAX
} ePopInfo;

typedef struct _myfilePopupListData {
	int index;
	bool unregister_lang_mgr;
	Elm_Object_Item *item;
	struct appdata *ap;
} myfilePopupListData;


/***********	Popup API		************/
Evas_Object *mf_popup_create_popup(void *data, ePopMode popupMode, char *title, const char *context, const char *first_btn_text, const char *second_btn_text,
				   const char *third_btn_text, Evas_Smart_Cb func, void *param);
Evas_Object *mf_popup_create_pb_popup(void *data, char *context, int file_count, void *func, void *param);
void mf_popup_indicator_popup(void *data, const char *text);

/***********	Search Bar API	************/
Evas_Object *mfSearchBarCreate(void *data);

/***********RenameView API***********/
void rename_item_add(mfItemData_s *data);
void mfTitleStyleSet(void *data, int titleStyle, char *title, char *leftbtn, char *rightbtn);
void mfTitleSet(void *data, char *title);
void mfTitleCallBackSet(void *data, int titleCbStyle);


/****Create New Folder API ***********/
void mf_navi_bar_create_new_folder_view(void *data);
void mfNewFolderNaviBarSetStyle(myfileNaviBar *pNavi_s, Evas_Object *NaviContent);


/****Search Related API	********/
void mf_search_bar_enter_search_routine(void *data, Evas_Object *obj, void *event_info);
void mf_search_bar_search_started_callback(void *data, Evas_Object *obj, void *event_info);
void mf_search_bar_category_search(void *data, int category);



void __mf_genlist_select_all_layout_mouse_down_cb(void *data, Evas * evas, Evas_Object * obj, void *event_info);
void __mf_genlist_select_all_check_changed_cb(void *data, Evas_Object * obj, void *event_info);
Evas_Object *mf_widget_create_select_all_layout(Evas_Object *parent);

void mf_genlist_gl_edit_item_selected(void *data);
void mf_genlist_gl_selected(void *data, Evas_Object * obj, void *event_info);
void mf_genlist_gl_mode_sweep_set(void *data, Evas_Object *obj, void *event_info);
void mf_genlist_gl_mode_sweep_unset(void *data, Evas_Object *obj, void *event_info);
void mf_genlist_gl_mode_cancel(void *data, Evas_Object * obj, void *event_info);
void mf_genlist_set_category_list_style();

/************* widget API   **************/

typedef enum __MF_ROTATE_TYPE {
	MF_ROTATE_PORTRAIT,
	MF_ROTATE_LANDSCAPE
}MF_ROTATE_TYPE;

typedef enum __MF_RM_NAVI_ITEM_TYPE {
	MF_RM_NAVI_ITEM_DIRECTLY,
	MF_RM_NAVI_ITEM_BY_EFFECT
}MF_RM_NAVI_ITEM_TYPE;

/*****************  Context POP  *************/
void mf_context_popup_create_more(void *data, Evas_Object *parent);
int mousedown_cb( void *data, int type, void *event );
void mf_context_popup_get_more_position(Evas_Object *obj, int *x, int *y);
void mf_context_popup_create_share(void *data, int share_mode, Evas_Object *parent);
void mf_context_popup_create_gengrid(void *data);

myfileNaviBar *mf_widget_phone_storage_init(void *data);
void mf_widget_refresh_view(void *data);
void mf_widget_set_main_layout_content(void *data);
void mf_widget_create(void *data);
void mf_widget_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h);
Evas_Object *mf_widget_create_select_all_layout(Evas_Object *parent);
Evas_Object *mf_widget_create_no_content(Evas_Object *parent);
Evas_Object *mf_widget_create_check_box(Evas_Object *parent);
Evas_Object *mf_widget_create_conform(Evas_Object * parent);
Evas_Object *mf_widget_create_layout(Evas_Object *parent, const char *edj, const char *grp_name);
Evas_Object *mf_widget_create_box(Evas_Object * parent);
Evas_Object *mf_widget_create_genlist(Evas_Object *parent);
Evas_Object *mf_widget_util_create_button(Evas_Object *parent, char *style,
				   char *caption, Evas_Object *icon,
				   void (*func) (void *, Evas_Object *, void *),
				   void *data);

void mf_widget_set_navi_blocking(void * data,Eina_Bool is_locked);
void mf_widget_unset_main_layout_content(Evas_Object *layout);
void mf_widget_panes_right_set(Evas_Object *panes, Evas_Object *content);
Evas_Object *mf_widget_create_editfield(Evas_Object *parent, Evas_Object **pEntry);
Evas_Object *mf_widget_button_create(Evas_Object *parent, const char *label);
Evas_Object *mf_widget_create_notify_popup(void *data,
				    Evas_Object *parent,
				    char *label,
				    mf_notify_popup_type type,
				    bool time_out,
				    void func(void *data, Evas_Object *obj, void *event_info));
void mf_widget_items_remove(Evas_Object *parent, int storage, int type);
Evas_Object *mf_widget_unset_part_content(Evas_Object *parent, const char *part);


/*************** Widget *********************/
#ifdef MYFILE_MOTION_FEATURE
void mf_widget_first_item_bring_in(void *data);
#endif

void mf_widget_create_entryfield(void *data,
			   Evas_Object *parent,
			   void (*changed_cb)(void *data, Evas_Object *obj, void *event_info),
			   void (*click_cb)(void *data, Evas_Object *obj, void *event_info));
Evas_Object *mf_widget_create_layout_main(Evas_Object * parent);
Evas_Object *mf_widget_create_panes(Evas_Object *parent);
Elm_Object_Item *mf_widget_item_append(Evas_Object *parent, fsNodeInfo *pNode, void *data);
Evas_Object *mf_widget_get_part_content(Evas_Object *parent, const char *part);
void mf_widget_item_remove(Evas_Object *parent, const char *path, int type);
void mf_widget_imf_hide_callback_register(Evas_Object *pConformant, void *data);
void mf_widget_imf_hide_callback_del(Evas_Object *pConformant);

#endif
