/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include "mf-widget.h"
#include "mf-callback.h"
#include "mf-share.h"
#include "mf-language-mgr.h"


static Evas_Coord touch_x;
static Evas_Coord touch_y;


/******************************
** Prototype    : _move_ctxpopup
** Description  : move the context popup to the suitable location
** Input        : void *data
**                Evas_Object *ctxpopup
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static void _move_ctxpopup(void *data, Evas_Object *parent, Evas_Object *ctxpopup )
{
	struct appdata* ap = (struct appdata *)data;
	elm_ctxpopup_direction_priority_set(ctxpopup, ELM_CTXPOPUP_DIRECTION_UP,
					    ELM_CTXPOPUP_DIRECTION_LEFT,
					    ELM_CTXPOPUP_DIRECTION_RIGHT,
					    ELM_CTXPOPUP_DIRECTION_DOWN);


	if (parent) {
		int x = 0;
		int y = 0;
		int w = 0;
		int h = 0;

		evas_object_geometry_get(parent, &x, &y, &w, &h);
		evas_object_move(ctxpopup, x + (w / 2), y + (h /2));
	} else {
		evas_object_move(ctxpopup, touch_x, touch_y);
	}
}


/******************************
** Prototype    : mousedown_cb
** Description  : callback function for the mouse down operation in  context
                  popup
** Input        : void *data
**                int type
**                void *event
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_context_popup_get_position(Evas_Object *obj, const char *label, int *x, int *y)
{
	mf_retm_if(obj == NULL, "obj is NULL");
	mf_retm_if(label == NULL, "label is NULL");
	Evas_Object *pCtrlBar = obj;

	Elm_Object_Item *pItem = NULL;
	int x_position = 0;
	int y_position = 0;
	int w = 0;

	pItem = elm_toolbar_first_item_get(pCtrlBar);

	while(pItem) {
		const char *button_label = elm_object_item_text_get(pItem);
		if (g_strcmp0(button_label, label) == 0) {
			Evas_Object *icon = elm_toolbar_item_object_get(pItem);
			evas_object_geometry_get(icon, &x_position, &y_position, &w, NULL);
			*x = x_position+w/2;
			*y = y_position;
			return;
		}
		pItem = elm_toolbar_item_next_get(pItem);
	}
}

int mousedown_cb( void *data, int type, void *event )
{
	struct appdata* ap = (struct appdata *)data;
	Evas_Coord x = 0;
	Evas_Coord y = 0;
	Evas_Coord w = 0;
	Elm_Object_Item *navi_it = ap->mf_MainWindow.pNaviItem;


	if (ap->mf_Status.navi_item_selected == MF_NAVI_BTN_MORE) {
		Evas_Object *button = elm_object_item_part_content_get(navi_it, NAVI_MORE_BUTTON_PART);
		evas_object_geometry_get(button, &x, &y, &w, NULL);
		touch_x = x+w/2;;
		touch_y = y;
		return 0;
	} else if (ap->mf_Status.more == MF_NAVI_BTN_SHARE) {
		Evas_Object *button = elm_object_item_part_content_get(navi_it, NAVI_BOTTOM_BUTTON_2_PART);

		mf_context_popup_get_position(button, GET_SYS_STR(LABEL_SHARE),&x, &y);
		mf_debug("x is [%d] y is [%d]", x, y);
		touch_x = x;
		touch_y = y;
		return 0;
	} else {
		return 0;
	}
}

/******************************
** Prototype    : mfContextPopupCreate
** Description  : Create the context popup
** Input        : void *data
**                eContextPopMode popupMode
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static void _ctxpopup_hide_cb(void *data, Evas_Object *obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(obj == NULL, "obj is NULL");
	mf_language_mgr_unregister_object_item_by_type(ITEM_TYPE_CTXPOP);
	evas_object_del(obj);
	obj = NULL;
	MF_TRACE_END;
}

void mf_context_popup_create_gengrid(void *data)
{
	mf_retm_if(data == NULL, "data is NULL");
	mfItemData_s *itemData = (mfItemData_s *)data;
	struct appdata* ap = (struct appdata *)itemData->ap;
	mf_retm_if(ap == NULL, "ap is NULL");

	SAFE_FREE_OBJ(ap->mf_MainWindow.pContextPopup);


	if (ap->mf_Status.more == MORE_DEFAULT) {
		Evas_Object *ctxpopup = elm_ctxpopup_add(ap->mf_MainWindow.pNaviBar);
		ap->mf_MainWindow.pContextPopup = ctxpopup;
		Elm_Object_Item *it = NULL;
		Evas_Object *parent = elm_object_item_part_content_get(itemData->item, "elm.swallow.icon");

		if (ap->mf_Status.view_type == mf_view_root) {
			it = elm_ctxpopup_item_append(ctxpopup, GET_SYS_STR(LABEL_DETAIL), NULL, mf_callback_detail_ctx_cb, itemData);
			mf_language_mgr_register_object_item(it, LABEL_DETAIL, ITEM_TYPE_CTXPOP);
		} else {
			it = elm_ctxpopup_item_append(ctxpopup, GET_SYS_STR(LABEL_RENAME), NULL, mf_callback_rename_create_cb, itemData);
			mf_language_mgr_register_object_item(it, LABEL_RENAME, ITEM_TYPE_CTXPOP);
			it = elm_ctxpopup_item_append(ctxpopup, GET_SYS_STR(LABEL_DELETE), NULL, mf_callback_delete_button_cb, itemData);
			mf_language_mgr_register_object_item(it, LABEL_DELETE, ITEM_TYPE_CTXPOP);

		}
		_move_ctxpopup(ap, parent, ctxpopup);
		evas_object_show(ctxpopup);

		if(ctxpopup)
		{
			evas_object_smart_callback_add(ctxpopup,"dismissed", _ctxpopup_hide_cb, ap);
		}
	}
}


void mf_context_popup_create_more(void *data, Evas_Object *parent)
{
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata* ap = (struct appdata *)data;
	int extension_state = ap->mf_Status.iExtensionState;


	SAFE_FREE_OBJ(ap->mf_MainWindow.pContextPopup);

	Evas_Object *ctxpopup = elm_ctxpopup_add(ap->mf_MainWindow.pNaviBar);
	ap->mf_MainWindow.pContextPopup = ctxpopup;
	Elm_Object_Item *it = NULL;

	if (ap->mf_Status.more == MORE_DEFAULT) {
		/*Search*/
		if (ap->mf_Status.view_type == mf_view_root) {
			it = elm_ctxpopup_item_append(ctxpopup, GET_STR(LABEL_VIEW_AS), NULL, mf_callback_view_as_cb, ap);
			mf_language_mgr_register_object_item(it, LABEL_VIEW_AS, ITEM_TYPE_CTXPOP);

			/*Sort by*/
			it = elm_ctxpopup_item_append(ctxpopup, GET_STR(LABEL_SORT_BY), NULL, mf_callback_list_by_view_cb, ap);
			mf_language_mgr_register_object_item(it, LABEL_SORT_BY, ITEM_TYPE_CTXPOP);
		} else if (ap->mf_Status.view_type == mf_view_root_category) {
			it = elm_ctxpopup_item_append(ctxpopup, GET_SYS_STR(LABEL_SEARCH), NULL,mf_search_bar_enter_search_routine, ap);
			mf_language_mgr_register_object_item(it, LABEL_SEARCH, ITEM_TYPE_CTXPOP);
			it = elm_ctxpopup_item_append(ctxpopup, GET_STR(LABEL_VIEW_AS), NULL, mf_callback_view_as_cb, ap);
			mf_language_mgr_register_object_item(it, LABEL_VIEW_AS, ITEM_TYPE_CTXPOP);

			/*Sort by*/
			it = elm_ctxpopup_item_append(ctxpopup, GET_STR(LABEL_SORT_BY), NULL, mf_callback_list_by_view_cb, ap);
			mf_language_mgr_register_object_item(it, LABEL_SORT_BY, ITEM_TYPE_CTXPOP);
			if (extension_state == MF_EXTENSION_SHOW) {
				it = elm_ctxpopup_item_append(ctxpopup, GET_STR(MF_LABEL_HIDE_EXTENSION), NULL, mf_callback_extension_cb, ap);
				mf_language_mgr_register_object_item(it, MF_LABEL_HIDE_EXTENSION, ITEM_TYPE_CTXPOP);
			} else {
				it = elm_ctxpopup_item_append(ctxpopup, GET_STR(MF_LABEL_SHOW_EXTENSION), NULL, mf_callback_extension_cb, ap);
				mf_language_mgr_register_object_item(it, MF_LABEL_SHOW_EXTENSION, ITEM_TYPE_CTXPOP);
			}
		}
		else {
			it = elm_ctxpopup_item_append(ctxpopup, GET_SYS_STR(LABEL_SEARCH), NULL,mf_search_bar_enter_search_routine, ap);
			mf_language_mgr_register_object_item(it, LABEL_SEARCH, ITEM_TYPE_CTXPOP);

			/*Create folder*/
			it = elm_ctxpopup_item_append(ctxpopup, GET_SYS_STR(MF_LABEL_CREATE_FOLDER), NULL,mf_callback_new_folder_create_cb, ap);
			mf_language_mgr_register_object_item(it, MF_LABEL_CREATE_FOLDER, ITEM_TYPE_CTXPOP);

			/*View as*/
			it = elm_ctxpopup_item_append(ctxpopup, GET_STR(LABEL_VIEW_AS), NULL, mf_callback_view_as_cb, ap);
			mf_language_mgr_register_object_item(it, LABEL_VIEW_AS, ITEM_TYPE_CTXPOP);

			/*Sort by*/
			it = elm_ctxpopup_item_append(ctxpopup, GET_STR(LABEL_SORT_BY), NULL, mf_callback_list_by_view_cb, ap);
			mf_language_mgr_register_object_item(it, LABEL_SORT_BY, ITEM_TYPE_CTXPOP);

			/*Extension*/
			if (extension_state == MF_EXTENSION_SHOW) {
				it = elm_ctxpopup_item_append(ctxpopup, GET_STR(MF_LABEL_HIDE_EXTENSION), NULL, mf_callback_extension_cb, ap);
				mf_language_mgr_register_object_item(it, MF_LABEL_HIDE_EXTENSION, ITEM_TYPE_CTXPOP);
			} else {
				it = elm_ctxpopup_item_append(ctxpopup, GET_STR(MF_LABEL_SHOW_EXTENSION), NULL, mf_callback_extension_cb, ap);
				mf_language_mgr_register_object_item(it, MF_LABEL_SHOW_EXTENSION, ITEM_TYPE_CTXPOP);
			}
		}
	} else if(ap->mf_Status.more == MORE_EDIT) {
		/*Copy*/
		it = elm_ctxpopup_item_append(ctxpopup, GET_SYS_STR(LABEL_COPY), NULL, mf_callback_copy_move_cb, ap);
		mf_language_mgr_register_object_item(it, LABEL_COPY, ITEM_TYPE_CTXPOP);
		/*Move*/
		it = elm_ctxpopup_item_append(ctxpopup, GET_SYS_STR(LABEL_MOVE), NULL,mf_callback_copy_move_cb, ap);
		mf_language_mgr_register_object_item(it, LABEL_MOVE, ITEM_TYPE_CTXPOP);

	} else if(ap->mf_Status.more == MORE_INTERNAL_MOVE || ap->mf_Status.more == MORE_INTERNAL_COPY) {
		it = elm_ctxpopup_item_append(ctxpopup, GET_SYS_STR(MF_LABEL_CREATE_FOLDER), NULL,mf_callback_new_folder_create_cb, ap);
		mf_language_mgr_register_object_item(it, MF_LABEL_CREATE_FOLDER, ITEM_TYPE_CTXPOP);
	}
	_move_ctxpopup(ap, parent, ctxpopup);
	evas_object_show(ctxpopup);

	if(ctxpopup)
	{
		evas_object_smart_callback_add(ctxpopup,"dismissed", _ctxpopup_hide_cb, ap);
	}
}


static void __mf_share_select(void *data, Evas_Object * obj, void *event_info)
{
	mf_retm_if(data == NULL, "data is NULL");
	mf_retm_if(event_info == NULL, "event_info is NULL");

	struct appdata* ap = (struct appdata *)data;
	Elm_Object_Item *item = (Elm_Object_Item *) event_info;
	if (item != NULL) {
		const char *label = NULL;
		label = elm_object_item_text_get(item);
		if (g_strcmp0(label, GET_SYS_STR(MF_POP_MSG_BT)) == 0) {
			ap->mf_Status.more = MORE_SENDVIABLUETOOTH;
		} else if (g_strcmp0(label, GET_SYS_STR(MF_POP_MSG_EMAIL)) == 0) {
			ap->mf_Status.more = MORE_SENDVIAEMAIL;

		} else if (g_strcmp0(label, GET_SYS_STR(MF_POP_MSG_MESSAGE)) == 0) {
			ap->mf_Status.more = MORE_SENDVIAMESSAGE;
		}

		mf_callback_send_response_cb(ap);
	}
}

void mf_context_popup_create_share(void *data, int share_mode, Evas_Object *parent)
{
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata* ap = (struct appdata *)data;


	SAFE_FREE_OBJ(ap->mf_MainWindow.pContextPopup);

	ap->mf_Status.navi_item_selected = MF_NAVI_BTN_SHARE;

	Evas_Object *ctxpopup = elm_ctxpopup_add(ap->mf_MainWindow.pNaviBar);
	ap->mf_MainWindow.pContextPopup = ctxpopup;
	Elm_Object_Item *it = NULL;

	it = elm_ctxpopup_item_append(ctxpopup, GET_SYS_STR(MF_POP_MSG_MESSAGE), NULL, __mf_share_select, ap);
	mf_language_mgr_register_object_item(it, MF_POP_MSG_MESSAGE, ITEM_TYPE_CTXPOP);
	it = elm_ctxpopup_item_append(ctxpopup, GET_SYS_STR(MF_POP_MSG_EMAIL), NULL, __mf_share_select, ap);
	mf_language_mgr_register_object_item(it, MF_POP_MSG_EMAIL, ITEM_TYPE_CTXPOP);
	it = elm_ctxpopup_item_append(ctxpopup, GET_SYS_STR(MF_POP_MSG_BT), NULL, __mf_share_select, ap);
	mf_language_mgr_register_object_item(it, MF_POP_MSG_BT, ITEM_TYPE_CTXPOP);



	_move_ctxpopup(ap, parent,ctxpopup);
	evas_object_show(ctxpopup);

	if(ctxpopup)
	{
		evas_object_smart_callback_add(ctxpopup,"dismissed", _ctxpopup_hide_cb, ap);
	}
}

