/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */




#include <Eina.h>
#include "mf-util.h"
#include "mf-widget.h"
#include "mf-callback.h"
#include "mf-resource.h"
#include "mf-language-mgr.h"

/******************************
** Prototype    : __mf_ctrl_bar_create_ctrl_bar
** Description  : Samsung
** Input        : Evas_Object *parent
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static Evas_Object *__mf_ctrl_bar_create_ctrl_bar(Evas_Object * parent)
{
	Evas_Object *pCtrlBar = NULL;

	if (parent == NULL) {
		return NULL;
	}

	pCtrlBar = elm_toolbar_add(parent);
	elm_toolbar_shrink_mode_set(pCtrlBar, ELM_TOOLBAR_SHRINK_EXPAND);
	elm_object_style_set(pCtrlBar, "naviframe");
	evas_object_show(pCtrlBar);
	return pCtrlBar;
}

/******************************
** Prototype    : __mf_ctrl_bar_set_label_list
** Description  : Samsung
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static Eina_List *__mf_ctrl_bar_set_label_list(void *data)
{
	mf_debug();
	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, false, "ap is NULL");
	Eina_List *pLabelList = NULL;

	switch (ap->mf_Status.more) {
	case MORE_DEFAULT:
		pLabelList = eina_list_append(pLabelList, MF_LABEL_EDIT);
		pLabelList = eina_list_append(pLabelList, LABEL_SORT_BY);
		pLabelList = eina_list_append(pLabelList, LABEL_SEARCH);
		pLabelList = eina_list_append(pLabelList, MF_LABEL_MORE);
		break;
	case MORE_EDIT:
		pLabelList = eina_list_append(pLabelList, LABEL_DELETE);
		pLabelList = eina_list_append(pLabelList, LABEL_SHARE);
		pLabelList = eina_list_append(pLabelList, LABEL_MOVE);
		pLabelList = eina_list_append(pLabelList, LABEL_COPY);
		pLabelList = eina_list_append(pLabelList, LABEL_CANCEL);
		break;
	case MORE_CREATE_FOLDER:
	case MORE_THUMBNAIL_RENAME:
	case MORE_RENAME:
		pLabelList = eina_list_append(pLabelList, LABEL_SAVE);
		pLabelList = eina_list_append(pLabelList, LABEL_CANCEL);
		break;
	case MORE_INTERNAL_COPY:
	case MORE_DATA_COPYING:
		pLabelList = eina_list_append(pLabelList, LABEL_COPY_HERE);
		pLabelList = eina_list_append(pLabelList, NULL);
		pLabelList = eina_list_append(pLabelList, MF_LABEL_CREATE_FOLDER);
		pLabelList = eina_list_append(pLabelList, NULL);
		pLabelList = eina_list_append(pLabelList, LABEL_CANCEL);
		break;
	case MORE_INTERNAL_MOVE:
	case MORE_DATA_MOVING:
		pLabelList = eina_list_append(pLabelList, LABEL_MOVE_HERE);
		pLabelList = eina_list_append(pLabelList, NULL);
		pLabelList = eina_list_append(pLabelList, MF_LABEL_CREATE_FOLDER);
		pLabelList = eina_list_append(pLabelList, NULL);
		pLabelList = eina_list_append(pLabelList, LABEL_CANCEL);
		break;
	case MORE_INTERNAL_COPY_MOVE:
		pLabelList = eina_list_append(pLabelList, LABEL_CANCEL);
		pLabelList = eina_list_append(pLabelList, LABEL_MOVE_HERE);
		pLabelList = eina_list_append(pLabelList, LABEL_COPY_HERE);
		pLabelList = eina_list_append(pLabelList, MF_LABEL_CREATE_FOLDER);
		break;
	default:
		return NULL;
	}
	return pLabelList;
}




/******************************
** Prototype    : __mf_ctrl_bar_set_label
** Description  : Samsung
** Input        : void *data
**                Eina_List **label_list
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static void __mf_ctrl_bar_set_label(void *data, Eina_List ** label_list)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	if (label_list != NULL) {
		if (*label_list != NULL) {
			eina_list_free(*label_list);
			//mf_util_free_eina_list_with_data(label_list, MYFILE_TYPE_CHAR);
			*label_list = NULL;
		}
	} else {
		return;
	}
	*label_list = __mf_ctrl_bar_set_label_list(ap);
}


static Eina_List *__mf_ctrl_bar_set_icon_list(void *data)
{
	mf_debug();
	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, false, "ap is NULL");
	Eina_List *pLabelList = NULL;

	switch (ap->mf_Status.more) {
	case MORE_DEFAULT:
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_EDIT));
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_LIST_BY));
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_SEARCH));
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_NEW_FOLDER_CREATE));
		break;
	case MORE_EDIT:
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_DELETE));
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_SEND));
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_MOVE));
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_COPY));
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_CANCEL));
		break;
	case MORE_CREATE_FOLDER:
	case MORE_THUMBNAIL_RENAME:
	case MORE_RENAME:
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_SAVE));
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_CANCEL));
		break;
	case MORE_INTERNAL_COPY:
	case MORE_DATA_COPYING:
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_PASTE_HERE));
		pLabelList = eina_list_append(pLabelList, NULL);
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_NEW_FOLDER_CREATE));
		pLabelList = eina_list_append(pLabelList, NULL);
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_CANCEL));
		break;
	case MORE_INTERNAL_MOVE:
	case MORE_DATA_MOVING:
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_MOVE_HERE));
		pLabelList = eina_list_append(pLabelList, NULL);
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_NEW_FOLDER_CREATE));
		pLabelList = eina_list_append(pLabelList, NULL);
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_CANCEL));
		break;
	case MORE_INTERNAL_COPY_MOVE:
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_CANCEL));
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_MOVE));
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_COPY));
		pLabelList = eina_list_append(pLabelList, strdup(MF_CTRL_ICON_NEW_FOLDER_CREATE));
		break;
	default:
		return NULL;
	}
	return pLabelList;
}


static void __mf_ctrl_bar_set_icon(void *data, Eina_List ** label_list)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	if (label_list != NULL) {
		if (*label_list != NULL) {
			mf_util_free_eina_list_with_data(label_list, MYFILE_TYPE_CHAR);
			*label_list = NULL;
		}
	} else {
		return;
	}
	*label_list = __mf_ctrl_bar_set_icon_list(ap);
}


static Eina_List *__mf_ctrl_bar_set_callback_list(void *data)
{
	mf_debug();
	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, NULL, "ap is NULL");
	Eina_List *pCallBackList = NULL;

	switch (ap->mf_Status.more) {
	case MORE_DEFAULT:
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_edit_cb);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_list_by_view_cb);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_search_bar_enter_search_routine);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_more_button_cb);
		break;
	case MORE_INTERNAL_COPY_MOVE:
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_cancel_cb);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_move_here_cb);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_paste_here_cb);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_new_folder_create_cb);
		break;
	case MORE_INTERNAL_COPY:
	case MORE_DATA_COPYING:
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_paste_here_cb);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) NULL);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_new_folder_create_cb);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) NULL);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_cancel_cb);
		break;
	case MORE_INTERNAL_MOVE:
	case MORE_DATA_MOVING:
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_move_here_cb);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) NULL);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_new_folder_create_cb);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) NULL);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_cancel_cb);
		break;
	case MORE_EDIT:
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_delete_cb);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_share_cb);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_copy_move_cb);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_copy_move_cb);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_cancel_cb);
		/*pCallBackList = eina_list_append(pCallBackList, (mfCallBack)mfCopyCB);*/
		/*pCallBackList = eina_list_append(pCallBackList, (mfCallBack)mfMoveCB);*/
		break;
	case MORE_RENAME:
	case MORE_THUMBNAIL_RENAME:
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_rename_save_cb);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_cancel_cb);
		break;
	case MORE_CREATE_FOLDER:
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_save_cb);
		pCallBackList = eina_list_append(pCallBackList, (mfCallBack) mf_callback_cancel_cb);
	default:
		break;
	}
	return pCallBackList;
}

/******************************
** Prototype    : __mf_ctrl_bar_set_callback
** Description  : Samsung
** Input        : void *data
**                Eina_List **callback_list
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static void __mf_ctrl_bar_set_callback(void *data, Eina_List ** callback_list)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	if (callback_list != NULL) {
		if (*callback_list != NULL) {
			eina_list_free(*callback_list);
			*callback_list = NULL;
		}
	} else {
		return;
	}
	*callback_list = __mf_ctrl_bar_set_callback_list(ap);
}



/******************************
** Prototype    : __mf_ctrl_bar_add_item
** Description  : Samsung
** Input        : void *data
**                Evas_Object *ctrl_bar
**                Eina_List *label_list
**                Eina_List *callback_list
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static bool __mf_ctrl_bar_add_item(void *data, Evas_Object * ctrl_bar, Eina_List * label_list, Eina_List * callback_list, Eina_List * button_list)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, false, "ap is NULL");

	Evas_Object *pCtrlBar = ctrl_bar;

	Eina_List *callback = callback_list;
	int label_count = 0;
	mfCallBack fun = NULL;
	int callback_count = 0;
	int index = 0;

	label_count = eina_list_count(label_list);
	callback_count = eina_list_count(callback_list);

/*
	if(label_count ==0 || callback_count == 0 || label_count != callback_count)
	{
		return false;
	}
	else*/

	Elm_Object_Item *pItem = NULL;
	Elm_Object_Item *pNext = NULL;

	pItem = elm_toolbar_first_item_get(pCtrlBar);

	while(pItem) {
		pNext = elm_toolbar_item_next_get(pItem);
		mf_language_mgr_unregister_object_item(pItem);
		elm_object_item_del(pItem);
		pItem = pNext;
	}

	for (index = 0; index < callback_count; index++) {
		char *lable = NULL;
		char *icon = NULL;
		fun = (mfCallBack) eina_list_nth(callback, index);
		lable = eina_list_nth(label_list, index);
		icon = eina_list_nth(button_list, index);
		Elm_Object_Item *it = elm_toolbar_item_append(pCtrlBar, NULL, mf_util_get_text(lable), fun, ap);
		if (lable == NULL) {
			elm_object_item_disabled_set(it, EINA_TRUE);
		}
		mf_language_mgr_register_object_item(it, lable, ITEM_TYPE_CTRL);
	}
	return true;
}

/******************************
** Prototype    : mf_ctrl_bar_create
** Description  : Samsung
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static void __mf_ctrl_bar_del_item(Evas_Object *pCtrlBar)
{
	mf_retm_if(pCtrlBar == NULL, "pCtrlBar is NULL");


	Elm_Object_Item *pItem = NULL;
	Elm_Object_Item *pNext = NULL;
	pItem = elm_toolbar_first_item_get(pCtrlBar);

	while(pItem) {
		printf("deleting item is [%s]\n", elm_object_item_text_get(pItem));
		pNext = elm_toolbar_item_next_get(pItem);
		mf_language_mgr_unregister_object_item(pItem);
		elm_object_item_del(pItem);
		pItem = pNext;
	}

}

void mf_ctrl_bar_reset(void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	Evas_Object *pCtrlBar = NULL;
	Eina_List *label_list = NULL;
	Eina_List *button_list = NULL;
	Eina_List *callback_list = NULL;

	if (ap->mf_MainWindow.pNaviCtrlBar) {
		__mf_ctrl_bar_del_item(ap->mf_MainWindow.pNaviCtrlBar);
	}

	pCtrlBar = ap->mf_MainWindow.pNaviCtrlBar;

	__mf_ctrl_bar_set_label(ap, &label_list);
	__mf_ctrl_bar_set_icon(ap, &button_list);
	__mf_ctrl_bar_set_callback(ap, &callback_list);

	__mf_ctrl_bar_add_item(ap, pCtrlBar, label_list, callback_list, button_list);

	eina_list_free(callback_list);
	//mf_util_free_eina_list_with_data(&label_list, MYFILE_TYPE_CHAR);
	eina_list_free(callback_list);

	evas_object_show(pCtrlBar);

	MF_TRACE_END;
}

Evas_Object *mf_ctrl_bar_create(void *data, Evas_Object *parent)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, NULL, "ap is NULL");
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");
	Evas_Object *pCtrlBar = NULL;
	Eina_List *label_list = NULL;
	Eina_List *button_list = NULL;
	Eina_List *callback_list = NULL;

	pCtrlBar = __mf_ctrl_bar_create_ctrl_bar(parent);
	mf_retvm_if(pCtrlBar == NULL, NULL, "pCtrlBar is NULL");

	__mf_ctrl_bar_set_label(ap, &label_list);
	__mf_ctrl_bar_set_icon(ap, &button_list);
	__mf_ctrl_bar_set_callback(ap, &callback_list);

	__mf_ctrl_bar_add_item(ap, pCtrlBar, label_list, callback_list, button_list);

	//mf_util_free_eina_list_with_data(&label_list, MYFILE_TYPE_CHAR);
	eina_list_free(label_list);
	eina_list_free(callback_list);
	MF_TRACE_END;
	return pCtrlBar;
}


/******************************
** Prototype    : mf_ctrl_bar_item_set_disable
** Description  : function for the Disable set of the Control Bar
** Input        : Evas_Object *ctrl_bar
**                int disable_item
**                bool disable
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_ctrl_bar_item_set_disable(Evas_Object *ctrl_bar, int disable_item, bool disable)
{
	mf_debug();
	Evas_Object *pCtrlBar = ctrl_bar;

	Elm_Object_Item *it = NULL;
	it = elm_toolbar_first_item_get(pCtrlBar);

	while(it) {

		const char *button_label = elm_object_item_text_get(it);

		if ((disable_item & CTRL_DISABLE_COPY) && g_strcmp0(button_label, GET_SYS_STR(LABEL_COPY)) == 0) {
			mf_debug();
			elm_object_item_disabled_set(it, disable);
		} else if ((disable_item & CTRL_DISABLE_MOVE) && g_strcmp0(button_label, GET_SYS_STR(LABEL_MOVE)) == 0) {
			mf_debug();
			elm_object_item_disabled_set(it, disable);
		} else if ((disable_item & CTRL_DISABLE_DELETE)
			   && g_strcmp0(button_label, GET_SYS_STR(LABEL_DELETE)) == 0) {
			mf_debug();
			elm_object_item_disabled_set(it, disable);
		} else if ((disable_item & CTRL_DISABLE_SEARCH) && g_strcmp0(button_label, GET_SYS_STR(LABEL_SEARCH)) == 0) {
			mf_debug();
			elm_object_item_disabled_set(it, disable);
		} else if ((disable_item & CTRL_DISABLE_SEND) && g_strcmp0(button_label, GET_SYS_STR(LABEL_SHARE)) == 0) {
			mf_debug();
			elm_object_item_disabled_set(it, disable);
		} else if ((disable_item & CTRL_DISABLE_LIST_BY) && g_strcmp0(button_label, GET_STR(LABEL_SORT_BY)) == 0) {
			mf_debug();
			elm_object_item_disabled_set(it, disable);
		} else if ((disable_item & CTRL_DISABLE_CREATE) && g_strcmp0(button_label, GET_SYS_STR(MF_LABEL_CREATE_FOLDER)) == 0) {
			mf_debug();
			elm_object_item_disabled_set(it, disable);
		} else if ((disable_item & CTRL_DISABLE_EDIT) && g_strcmp0(button_label, GET_SYS_STR(MF_LABEL_EDIT)) == 0) {
			mf_debug();
			elm_object_item_disabled_set(it, disable);
		} else if ((disable_item & CTRL_DISABLE_CANCEL) && g_strcmp0(button_label, GET_SYS_STR(LABEL_CANCEL)) == 0) {
			mf_debug();
			elm_object_item_disabled_set(it, disable);
		} else if ((disable_item & CTRL_DISABLE_COPY_HERE) && g_strcmp0(button_label, GET_STR(LABEL_COPY_HERE)) == 0) {
			mf_debug();
			elm_object_item_disabled_set(it, disable);
		} else if ((disable_item & CTRL_DISABLE_MOVE_HERE) && g_strcmp0(button_label, GET_STR(LABEL_MOVE_HERE)) == 0) {
			mf_debug();
			elm_object_item_disabled_set(it, disable);
		} else if ((disable_item & CTRL_DISABLE_MORE) && g_strcmp0(button_label, GET_SYS_STR(MF_LABEL_MORE)) == 0) {
			mf_debug();
			elm_object_item_disabled_set(it, disable);

		}
		it = elm_toolbar_item_next_get(it);
	}

}

int mf_ctrl_bar_item_get_disable(Evas_Object *ctrl_bar)
{
	Evas_Object *pCtrlBar = ctrl_bar;

	int disable_item = 0;

	Elm_Object_Item *pItem = NULL;
	pItem = elm_toolbar_first_item_get(pCtrlBar);

	while(pItem) {
		if (elm_object_item_disabled_get(pItem)) {
			const char *button_label = elm_object_item_text_get(pItem);

			if (g_strcmp0(button_label, GET_SYS_STR(LABEL_DELETE)) == 0) {
				disable_item |= CTRL_DISABLE_DELETE;
			} else if (g_strcmp0(button_label, GET_SYS_STR(LABEL_SHARE)) == 0) {
				disable_item |= CTRL_DISABLE_SEND;
			} else if (g_strcmp0(button_label, GET_SYS_STR(LABEL_COPY)) == 0) {
				disable_item |= CTRL_DISABLE_COPY;
			} else if (g_strcmp0(button_label, GET_SYS_STR(LABEL_MOVE)) == 0) {
				disable_item |= CTRL_DISABLE_MOVE;
			}
		}
		pItem = elm_toolbar_item_next_get(pItem);
	}

	return disable_item;
}

