/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */



#include "mf-widget.h"
#include "mf-util.h"
#include "mf-callback.h"
#include "mf-launch.h"
#include "mf-fm-svc-wrapper.h"
#include "mf-otg.h"
#include "mf-language-mgr.h"

Elm_Gengrid_Item_Class gic;
#define GENGRID_ITEM_WIDTH	72

static inline void __mf_gengrid_get_thumbnail(mfItemData_s *params)
{
	int error_code = 0;
	if (params->thumb_path == NULL) {
		if (ecore_file_is_dir(params->m_ItemName->str)) {
			if (params->ap->mf_Status.view_type == mf_view_root) {
				params->thumb_path = strdup(MF_ICON_THUMBNAIL_FOLDER);
			} else if (mf_fm_svc_wrapper_is_root_path(params->ap->mf_Status.path->str) == MYFILE_OTG) {
				params->thumb_path = strdup(MF_ICON_EXTERNAL_DEVICE);
			} else {
				params->thumb_path = strdup(MF_ICON_THUMBNAIL_FOLDER);
			}
			params->real_thumb_flag = TRUE;
		} else {
			const char *icon_path = NULL;
			mf_file_attr_get_file_icon(params->m_ItemName->str, &error_code, MF_ROTATE_LANDSCAPE, &icon_path, &params->media);
			if (icon_path && ecore_file_exists(icon_path)) {
				if (!g_strcmp0(icon_path, DEFAULT_ICON)) {
					params->thumb_path = strdup(MF_ICON_THUMBNAIL_UNKOWN);
				} else {
					params->thumb_path = strdup(icon_path);
				}
				params->real_thumb_flag = TRUE;
			} else {	/*must be image/video file*/
				fsFileType type = FILE_TYPE_NONE;
				type = mf_file_attr_get_file_type_by_mime(params->m_ItemName->str);

				if (type == FILE_TYPE_VIDEO) {
					params->thumb_path = strdup(MF_ICON_THUMBNAIL_VIDEO);
				} else if (type == FILE_TYPE_IMAGE) {
					params->thumb_path = strdup(MF_ICON_IMAGE);
				} else {
					params->thumb_path = strdup(MF_ICON_THUMBNAIL_UNKOWN);
				}
				params->real_thumb_flag = FALSE;
				error_code = MYFILE_ERR_GET_THUMBNAIL_FAILED;
			}
		}
	}  else if (params->thumb_path && ecore_file_exists(params->thumb_path)) {
		if (params->real_thumb_flag == FALSE) {
			error_code = MYFILE_ERR_GET_THUMBNAIL_FAILED;
		}
	} else if (!ecore_file_exists(params->thumb_path)) {
		mf_error("thumb_path[%s] is not exist", params->thumb_path);
		free(params->thumb_path);
		params->thumb_path = NULL;
		/*set default icon*/
		fsFileType type = FILE_TYPE_NONE;
		type = mf_file_attr_get_file_type_by_mime(params->m_ItemName->str);
		if (ecore_file_is_dir(params->m_ItemName->str)) {
			if (mf_fm_svc_wrapper_is_root_path(params->ap->mf_Status.path->str) == MYFILE_OTG) {
				params->thumb_path = strdup(MF_ICON_EXTERNAL_DEVICE);
			} else {
				params->thumb_path = strdup(MF_ICON_FOLDER);
			}

		} else {
			if (type == FILE_TYPE_VIDEO) {
				params->thumb_path = strdup(MF_ICON_THUMBNAIL_VIDEO);
			} else if (type == FILE_TYPE_IMAGE) {
				params->thumb_path = strdup(MF_ICON_IMAGE);
			} else {
				params->thumb_path = strdup(DEFAULT_ICON);
			}
		}
		params->real_thumb_flag = FALSE;
		error_code = MYFILE_ERR_GET_THUMBNAIL_FAILED;
	}
	if (error_code != 0) {
		mf_callback_create_thumbnail(params, mf_callback_thumb_created_cb);
	}
}

static void __mf_gengrid_icon_clicked(void *data, Evas_Object *obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	mf_retm_if(event_info == NULL, "event_info is NULL");
	struct appdata *ap = (struct appdata *)data;
	Elm_Object_Item *item = (Elm_Object_Item *) event_info;
	int ret = 0;

	if (ap->mf_Status.more == MORE_EDIT)
		return;
	if (item != NULL) {
		elm_gengrid_item_selected_set(item, EINA_FALSE);
		mfItemData_s *selected = (mfItemData_s *) elm_object_item_data_get(item);
		mf_retm_if(selected == NULL, "selected is NULL");
		mf_retm_if(selected->m_ItemName == NULL, "selected->m_ItemName is NULL");
		mf_retm_if(selected->m_ItemName->str == NULL, "selected->m_ItemName->str is NULL");
		if (mf_fm_svc_wrapper_is_dir(selected->m_ItemName)) {

			myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
			mf_retm_if(pNavi_s == NULL, "get navi structure failed");
			GString *new_path = NULL;
			int error_code = 0;
			bool result = FALSE;

			if (ap->mf_Status.more == MORE_SEARCH)
				ap->mf_Status.more = MORE_DEFAULT;


			new_path = g_string_new(selected->m_ItemName->str);

#ifdef MYFILE_USB_OTG
			/* if click items under otg root path, it means some disk take into use*/
			error_code = mf_file_attr_is_disk_link(new_path->str, &result);
			if (error_code == 0 && result == TRUE) {	/* a usb disk take into use */
				/*
				**we don't need to set the previous device flag to be false
				**because click cannot jump from one device to another
				*/
				mf_otg_set_node_using_flag(ap, new_path->str,TRUE);
			}
#endif
			/*set new path */
			if (ap->mf_Status.path != NULL) {
				g_string_free(ap->mf_Status.path, TRUE);
				ap->mf_Status.path = NULL;
			}
			ap->mf_Status.path = new_path;

			if (pNavi_s->pCurrentPath != NULL) {
				free(pNavi_s->pCurrentPath);
				pNavi_s->pCurrentPath = NULL;
			}
			pNavi_s->pCurrentPath = g_strdup(new_path->str);

			if (ap->mf_Status.view_type == mf_view_root) {
				ap->mf_Status.view_type = mf_view_normal;
#ifdef MYFILE_SPLIT_VIEW
				if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
					if (ap->mf_Status.view_type != mf_view_root_category && ap->mf_Status.view_type != mf_view_root) {
						mf_navi_bar_reset_panes_main(ap);
					}
				}
#endif
			}
			mf_widget_refresh_view(ap);
		} else {
			ret = mf_launch_service(ap, selected->m_ItemName->str);
		}
	}
	MF_TRACE_END;
}
static void __mf_gengrid_item_check_changed(void *data, Evas_Object *obj, void *event_info)
{
	MF_TRACE_BEGIN;

	mfItemData_s *itemData = (mfItemData_s *)data;
	struct appdata *ap = (struct appdata *)itemData->ap;

	itemData->m_checked = elm_check_state_get(obj);
	if (itemData->m_checked) {
		ap->mf_MainWindow.pEditView.iCheckedCount++;
	} else {
		ap->mf_MainWindow.pEditView.iCheckedCount--;
	}

	//if (ap->mf_MainWindow.pEditView.pSelectAllCheckBox) {

		if (ap->mf_MainWindow.pEditView.iTotalCount == ap->mf_MainWindow.pEditView.iCheckedCount) {
			ap->mf_MainWindow.pEditView.bSelectAllChecked = EINA_TRUE;
		} else {
			ap->mf_MainWindow.pEditView.bSelectAllChecked = EINA_FALSE;
		}
		//elm_check_state_pointer_set(ap->mf_MainWindow.pEditView.pSelectAllCheckBox, &ap->mf_MainWindow.pEditView.bSelectAllChecked);
	//}

	elm_gengrid_item_selected_set(itemData->item, itemData->m_checked);
	mf_genlist_gl_edit_item_selected(ap);
	MF_TRACE_END;
}

void mf_gengrid_select_all(void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "input parameter data error");

	Evas_Object *gengrid = ap->mf_MainWindow.pEditView.pGengrid;
	mfItemData_s *it_data = NULL;
	Elm_Object_Item *it = NULL;

	it = elm_gengrid_first_item_get(gengrid);
	while (it) {
		it_data = elm_object_item_data_get(it);
		if (it_data) {
			it_data->m_checked = ap->mf_MainWindow.pEditView.bSelectAllChecked;
			elm_check_state_set(it_data->pCheckBox, it_data->m_checked);
			elm_gengrid_item_selected_set(it_data->item, it_data->m_checked);
		}
		it = elm_gengrid_item_next_get(it);
	}

}

void mf_gengrid_edit_select(void *data, int *count)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "input parameter data error");

	Evas_Object *gengrid = ap->mf_MainWindow.pEditView.pGengrid;
	mfItemData_s *itemData = NULL;
	Elm_Object_Item *it = NULL;

	it = elm_gengrid_first_item_get(gengrid);
	while (it) {
		itemData = elm_object_item_data_get(it);
		if (itemData->m_checked) {
			ap->mf_Status.flagFileSelect = TRUE;
			(*count)++;
		}

		it = elm_gengrid_item_next_get(it);
	}
	MF_TRACE_END;
}

void mf_gengrid_create_grid_items(void *data, Evas_Object *grid, Eina_List *file_list)
{
	MF_TRACE_BEGIN;
	mf_retm_if(grid == NULL, "grid is NULL");
	mf_retm_if(file_list == NULL, "file_list is NULL");
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata*)data;

	Eina_List *l = NULL;
	void  *pData = NULL;
	mfItemData_s *m_TempItem = NULL;

	elm_gengrid_clear(grid);

	mf_debug("****************** count is [%d]", eina_list_count(file_list));
	EINA_LIST_FOREACH(file_list, l, pData) {
		char *real_name = NULL;

		fsNodeInfo *pNode = (fsNodeInfo *)pData;
		if (pNode == NULL)
			continue;
		real_name = g_strconcat(pNode->path, "/", pNode->name, NULL);

		if (real_name == NULL) {
			continue;
		}
		mf_debug("************** gengrid item is [%s]", real_name);
		m_TempItem = (mfItemData_s *) malloc(sizeof(mfItemData_s));
		if (m_TempItem == NULL) {
			free(real_name);
			real_name = NULL;
			continue;
		}
		Elm_Object_Item *it = NULL;

		m_TempItem->m_ItemName = g_string_new(real_name);
		m_TempItem->size = NULL;
		m_TempItem->create_date = NULL;
		m_TempItem->m_checked = FALSE;
		m_TempItem->pCheckBox = NULL;
		m_TempItem->thumb_path = NULL;
		m_TempItem->real_thumb_flag = FALSE;
		m_TempItem->unregister_lang_mgr = TRUE;
		m_TempItem->media = NULL;
		m_TempItem->ap = ap;

		if (ap->mf_Status.more == MORE_INTERNAL_COPY_MOVE || ap->mf_Status.more == MORE_INTERNAL_COPY
		    || ap->mf_Status.more == MORE_INTERNAL_MOVE || ap->mf_Status.more == MORE_DATA_COPYING || ap->mf_Status.more == MORE_DATA_MOVING){
			mf_debug();
			it = elm_gengrid_item_append(grid, &gic, m_TempItem, NULL, NULL);
			mf_language_mgr_register_gengrid_item(it);
		} else {
			mf_debug();
			it = elm_gengrid_item_append(grid, &gic, m_TempItem, __mf_gengrid_icon_clicked, ap);
			mf_language_mgr_register_gengrid_item(it);
		}
		m_TempItem->item = it;
		free(real_name);
		mf_debug();
	}
}

void mf_gengrid_refresh(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata *)data;

	mf_gengrid_create_grid_items(ap, ap->mf_MainWindow.pNaviGengrid, ap->mf_FileOperation.file_list);
}


void mf_gengrid_item_select(void *data, Evas_Object *pGengrid, int *file_count, int *dir_count)
{
	mf_retm_if(data == NULL, "data is NULL");
	mf_retm_if(pGengrid == NULL, "pGengrid is NULL");
	struct appdata *ap = (struct appdata *)data;
	Elm_Object_Item *it = NULL;
	mfItemData_s *itemData = NULL;
	//int error_code = 0;

	it = elm_gengrid_first_item_get(pGengrid);
	while (it) {
		itemData = elm_object_item_data_get(it);
		if (itemData->m_checked) {
			if (ecore_file_is_dir(itemData->m_ItemName->str)) {
				//bool systemfolder = FALSE;
				ap->mf_Status.flagFolderSelect = TRUE;
				(*dir_count)++;
			} else {
				ap->mf_Status.flagFileSelect = TRUE;
				(*file_count)++;
			}
		}

		it = elm_gengrid_item_next_get(it);

	}
}

void mf_gengrid_get_grid_selected_files(Evas_Object *gengrid, Eina_List **list)
{
	MF_TRACE_BEGIN;
	mf_retm_if(gengrid == NULL, "gengrid is NULL");

	mfItemData_s *itemData = NULL;
	Elm_Object_Item *it = NULL;
	GString *pTemp = NULL;

	it = elm_gengrid_first_item_get(gengrid);
	while (it) {
		itemData = elm_object_item_data_get(it);
		if (itemData->m_checked) {
			pTemp = g_string_new(itemData->m_ItemName->str);
			*list = eina_list_append(*list, pTemp);
		}
		it = elm_gengrid_item_next_get(it);
	}
	MF_TRACE_END;

}

void mf_gengrid_get_grid_selected_items(Evas_Object *gengrid, Eina_List **list)
{
	MF_TRACE_BEGIN;
	mf_retm_if(gengrid == NULL, "gengrid is NULL");

	mfItemData_s *itemData = NULL;
	Elm_Object_Item *it = NULL;

	it = elm_gengrid_first_item_get(gengrid);
	while (it) {
		itemData = elm_object_item_data_get(it);
		if (itemData->m_checked) {
			*list = eina_list_append(*list, itemData);
		}
		it = elm_gengrid_item_next_get(it);
	}
	MF_TRACE_END;

}

static Evas_Object *__mf_gengrid_item_icon_get(void *data, Evas_Object *obj, const char *part)
{
	MF_TRACE_BEGIN;
	mfItemData_s *params = (mfItemData_s *) data;
	struct appdata *ap = (struct appdata *)params->ap;
	mf_retvm_if(params == NULL, NULL, "param is NULL");
	mf_retvm_if(params->m_ItemName == NULL, NULL, "m_ItemName is NULL");
	mf_retvm_if(part == NULL, NULL, "part is NULL");
	mf_retvm_if(obj == NULL, NULL, "obj is NULL");
	mf_retvm_if(ap == NULL, NULL, "ap is NULL");

	if (!strcmp(part, "elm.swallow.icon")) {
		Evas_Object *bg = elm_bg_add(obj);
		__mf_gengrid_get_thumbnail(params);
		elm_bg_load_size_set(bg, MF_ICON_SIZE, MF_ICON_SIZE);

		if (bg) {
			elm_bg_file_set(bg, params->thumb_path, NULL);
			evas_object_size_hint_max_set(bg, MF_ICON_SIZE, MF_ICON_SIZE);
			evas_object_size_hint_aspect_set(bg, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
			evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
			evas_object_size_hint_align_set(bg, EVAS_HINT_FILL, EVAS_HINT_FILL);
			evas_object_show(bg);

		}
		fsFileType type = FILE_TYPE_NONE;
		type = mf_file_attr_get_file_type_by_mime(params->m_ItemName->str);
		if (type == FILE_TYPE_VIDEO) {
			Evas_Object *layout = NULL;;
			layout = elm_layout_add(obj);
			elm_object_focus_set(layout, EINA_FALSE);
			elm_layout_file_set(layout, EDJ_NAME, GRP_THUMBNAIL_ONLY);
			elm_object_part_content_set(layout, "thumbnail", bg);

			const char *play_icon_path = MF_ICON_THUMBNAIL_VIDEO_PLAY;
			Evas_Object *play_icon = NULL;
			play_icon = elm_image_add(obj);
			elm_object_focus_set(play_icon, EINA_FALSE);
			elm_image_file_set(play_icon, play_icon_path, NULL);
			evas_object_show(play_icon);
			elm_object_part_content_set(layout, "video_play", play_icon);
			return layout;

		}
		return bg;
	} else if (!strcmp(part, "elm.swallow.block") && params->ap->mf_Status.more == MORE_EDIT) {
		Evas_Object *layout = elm_layout_add(obj);
		elm_layout_theme_set(layout, "gengrid", "item", "block/default");
		evas_object_propagate_events_set(layout, 0);
		evas_object_event_callback_add(layout, EVAS_CALLBACK_MOUSE_DOWN, (Evas_Object_Event_Cb)mf_callback_rename_create_cb, params);
		return layout;
	} else if (!strcmp(part, "elm.swallow.end") && params->ap->mf_Status.more == MORE_EDIT)
     	{
		Evas_Object *ck = elm_check_add(obj);
		mf_retvm_if(ck == NULL, NULL, "ck is NULL");
		elm_object_focus_set(ck, EINA_FALSE);
		params->pCheckBox = ck;
		elm_object_style_set(ck, "grid");
		evas_object_propagate_events_set(ck, 1);
		mf_debug("params is [%p]params->m_checked is [%d]",params, params->m_checked);
		elm_check_state_set(ck, params->m_checked);
		elm_gengrid_item_selected_set(params->item, params->m_checked);
		evas_object_smart_callback_add(ck, "changed", __mf_gengrid_item_check_changed, params);
		evas_object_show(ck);
		return ck;
     	}
   	return NULL;
}

static char *__mf_gengrid_item_label_get(void *data, Evas_Object * obj, const char *part)
{
	MF_TRACE_BEGIN;
	mfItemData_s *params = (mfItemData_s *) data;
	mf_retv_if(params == NULL, NULL);

	if (strcmp(part, "elm.text") == 0) {
		/* supporting multi-lang for default folders */

		if (g_strcmp0(params->m_ItemName->str, PHONE_FOLDER) == 0) {
			return g_strdup(GET_SYS_STR(MF_LABEL_PHONE));
		} else if (g_strcmp0(params->m_ItemName->str, MEMORY_FOLDER) == 0) {
			return g_strdup(GET_SYS_STR(MF_LABEL_MMC));
		} else if (g_strcmp0(params->m_ItemName->str, OTG_FOLDER) == 0) {
			return g_strdup(GET_STR(MF_LABEL_OTG));
		}else {
			if (params->ap->mf_Status.more == MORE_SEARCH &&params->ap->mf_Status.search_filter) {
				char *markup_name = NULL;
				bool res = false;
				markup_name = (char *)mf_util_search_markup_keyword(ecore_file_file_get(params->m_ItemName->str), params->ap->mf_Status.search_filter, &res);
				if(res) {
					MF_TRACE_END;
					return g_strdup(markup_name);
				}
			}

			if (params->ap->mf_Status.iExtensionState == MF_EXTENSION_HIDE && !mf_file_attr_is_dir(params->m_ItemName->str)) {
				return mf_fm_svc_get_file_name(params->m_ItemName);

			} else {
				return g_strdup(ecore_file_file_get(params->m_ItemName->str));
			}
		}
	}  else {
		return strdup(_(""));
	}
	MF_TRACE_END;
}

static void __mf_gengrid_item_del(void *data, Evas_Object * obj)
{
	mfItemData_s *params = (mfItemData_s *) data;
	assert(params);
	if (params->m_ItemName) {
		g_string_free(params->m_ItemName, TRUE);
		params->m_ItemName = NULL;
	}
	if (params->thumb_path) {
		free(params->thumb_path);
		params->thumb_path = NULL;
	}
	if (params->create_date) {
		free(params->create_date);
		params->create_date = NULL;
	}
	if (params->size) {
		free(params->size);
		params->size = NULL;
	}
	if (params->media) {
		media_info_cancel_thumbnail(params->media);
		media_info_destroy(params->media);
		params->media = NULL;
	}
	free(params);
	return;
}



Evas_Object *mf_gengrid_create_grid (Evas_Object *parent)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");
	Evas_Object *grid = NULL;

	grid = elm_gengrid_add(parent);
	evas_object_size_hint_weight_set(grid, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(grid, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_focus_set(grid, EINA_FALSE);

	elm_gengrid_align_set(grid, 0.0, 0.0);
   	elm_gengrid_horizontal_set(grid, EINA_FALSE);
	elm_scroller_bounce_set(grid, EINA_FALSE, EINA_TRUE);
   	elm_gengrid_multi_select_set(grid, EINA_TRUE);

	gic.item_style = "default_gridtext";
	gic.func.text_get = __mf_gengrid_item_label_get;
	gic.func.content_get = __mf_gengrid_item_icon_get;
	gic.func.state_get = NULL;
	gic.func.del = __mf_gengrid_item_del;

	evas_object_show(grid);
	return grid;
}

void mf_gengrid_create_list_default_style(Evas_Object *pGengrid, void *data, Eina_List *dir_list,
				Eina_List *file_list)
{

	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	mf_retm_if(pGengrid == NULL, "pGengrid is NULL");
	/*0.    variable definition and parameter check*/
	struct appdata *ap = (struct appdata *)data;

	fsNodeInfo *pNode = NULL;
	Eina_List *l = NULL;

	EINA_LIST_FOREACH(dir_list, l, pNode) {
		mf_widget_item_append(pGengrid, pNode, ap);
	}
	/*	add file items into the genlist */

	EINA_LIST_FOREACH(file_list, l, pNode) {
		mf_widget_item_append(pGengrid, pNode, ap);
	}
}

static void __mf_gengrid_thumbs_longpressed(void *data, Evas_Object *obj,
				    void *event_info)
{
	mf_retm_if(event_info == NULL, "event_info is NULL");

	mfItemData_s *it_data = elm_object_item_data_get(event_info);
	mf_retm_if(it_data == NULL, "it_data is NULL");

	mf_context_popup_create_gengrid(it_data);
}
Evas_Object *mf_gengrid_create_list(void *data, Evas_Object *parent)
{

	MF_TRACE_BEGIN;
	mf_retvm_if(data == NULL, NULL, "data is NULL");
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");

	struct appdata *ap = (struct appdata *)data;
	Evas_Object *gengrid = NULL;
	Eina_List *file_list = NULL;
	Eina_List *dir_list = NULL;


	if (ap->mf_Status.view_type == mf_view_root_category) {
		file_list = ap->mf_FileOperation.category_list;
	} else {
		file_list = ap->mf_FileOperation.file_list;
		dir_list = ap->mf_FileOperation.folder_list;
	}

	gengrid = mf_gengrid_create_grid(parent);

	evas_object_smart_callback_add(gengrid, "longpressed",
				       __mf_gengrid_thumbs_longpressed, ap);
	if (MF_SCALE_FACTORY == 1) {
		elm_gengrid_item_size_set(gengrid, MF_HD_GENGRID_ITEM_WIDTH, MF_HD_GENGRID_ITEM_HEIGTH);
	} else {
		elm_gengrid_item_size_set(gengrid, MF_WVGA_GENGRID_ITEM_WIDTH, MF_WVGA_GENGRID_ITEM_HEIGTH);
	}

	if (gengrid == NULL) {
		return NULL;
	}
	evas_object_smart_callback_add(gengrid, "selected", __mf_gengrid_icon_clicked, ap);
	mf_gengrid_create_list_default_style(gengrid, ap, dir_list, file_list);
	MF_TRACE_END;
	return gengrid;
}

