/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */




#include <math.h>

#include <vconf-keys.h>
#include <vconf.h>
#include <media_content.h>

#include "mf-main.h"
#include "mf-widget.h"
#include "mf-util.h"
#include "mf-fm-svc-wrapper.h"
#include "mf-fs-util.h"
#include "mf-gadget.h"
#include "mf-dlog.h"
#include "mf-ta.h"
#include "mf-delete.h"
#include "mf-launch.h"
#include "mf-resource.h"
#include "mf-language-mgr.h"

#define NUM_OF_GENLIST_STYLES 4
#define NUM_OF_ITEMS 50
#define MF_DATE_FORMAT_DD_MM_YYYY "%d-%b-%Y "
#define MF_DATE_FORMAT_MM_DD_YYYY "%b-%d-%Y "
#define MF_DATE_FORMAT_YYYY_MM_DD "%Y-%b-%d "
#define MF_DATE_FORMAT_YYYY_DD_MM "%Y-%d-%b "
#define MF_TIME_FORMAT_12HOUR "%l:%M%p"
#define MF_TIME_FORMAT_24HOUR "%H:%M"
#define MF_ROUND_D(x, dig) (floor((x) * pow(10, dig+1) + 0.5) / pow(10, dig+1))

typedef struct {
	void *data;
	Evas_Object *pGenlist;
} renameData_s;

/****	Global definition	****/
Elm_Genlist_Item_Class itc, userfolderitc, categoryitc, newfolderitc, indexitc;
Elm_Genlist_Item_Class left_itc;
Elm_Genlist_Item_Class search_itc;
static Elm_Genlist_Item_Class itch;


/***	static function declare	***/
static char *__mf_genlist_gl_label_get(void *data, Evas_Object *obj, const char *part);
static Evas_Object *__mf_genlist_gl_default_icon_get(void *data, Evas_Object *obj, const char *part);
static Evas_Object *__mf_genlist_gl_edit_icon_get(void *data, Evas_Object *obj, const char *part);
static void __mf_genlist_gl_del(void *data, Evas_Object *obj);
static Eina_Bool __mf_genlist_gl_state_get(void *data, Evas_Object *obj, const char *part);
static inline void __mf_genlist_get_thumbnail(mfItemData_s *params);

static void __mf_genlist_reset_serch_sweep_item(void *data)
{
	struct appdata *ap = data;
	mf_retm_if(ap == NULL, "input ad is NULL");
	mf_retm_if(ap->mf_Status.more != MORE_SEARCH, "more is not MORE_SEARCH");
	mf_retm_if(ap->mf_MainWindow.pNaviGenlist == NULL, "input pNaviGenlist is NULL");
	mf_retm_if(ap->mf_MainWindow.pSearchViewSweepItem == NULL, "input pSearchViewSweepItem is NULL");
	const Elm_Object_Item *decorated_item = elm_genlist_decorated_item_get(ap->mf_MainWindow.pNaviGenlist);
	Elm_Object_Item *item = NULL;
	if(decorated_item) {
		mf_debug("it is decorate status");
		ap->mf_MainWindow.pSearchViewSweepItem = decorated_item;
		return;
	}

	for(item =  elm_genlist_first_item_get(ap->mf_MainWindow.pNaviGenlist); item != NULL; item = elm_genlist_item_next_get(item)) {
		if(item == ap->mf_MainWindow.pSearchViewSweepItem) {
			//elm_object_scroll_freeze_push(ap->mf_MainWindow.pNaviGenlist);
			elm_genlist_item_decorate_mode_set(item, "slide", EINA_TRUE);
			elm_genlist_item_select_mode_set(item, ELM_OBJECT_SELECT_MODE_NONE);
			//elm_object_scroll_freeze_pop(ap->mf_MainWindow.pNaviGenlist);
			return;
		}
	}
}

void mf_genlist_rename_done(void *data, Evas_Object *obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "input ad is NULL");

	if (ap->mf_FileOperation.rename_item && elm_genlist_item_flip_get(ap->mf_FileOperation.rename_item)) {
		//elm_genlist_item_flip_set(ap->mf_FileOperation.rename_item, EINA_FALSE);
		//elm_genlist_item_select_mode_set(ap->mf_FileOperation.rename_item, ELM_OBJECT_SELECT_MODE_DEFAULT);
	}
	mf_callback_rename_save_cb(ap, obj, event_info);
	MF_TRACE_END;
}


static Evas_Object *__mf_genlist_rename_editfiled_create(void *data, Evas_Object *parent)
{
	MF_TRACE_BEGIN;
	mfItemData_s *params = (mfItemData_s *) data;
	mf_retvm_if(params == NULL, NULL, "param is NULL");
	mf_retvm_if(params->m_ItemName == NULL, NULL, "m_ItemName is NULL");

	struct appdata *ap = (struct appdata *)params->ap;
	mf_retvm_if(ap == NULL, NULL, "input parameter data error");

	static Elm_Entry_Filter_Limit_Size limit_filter_data;
	Evas_Object *edit_field = NULL;
	Evas_Object *entry = NULL;
	GString *filename = NULL;

	edit_field = mf_widget_create_editfield(parent, &entry);

	evas_object_size_hint_weight_set(edit_field, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(edit_field, EVAS_HINT_FILL, EVAS_HINT_FILL);


	elm_entry_single_line_set(entry, EINA_TRUE);
	elm_entry_scrollable_set(entry, EINA_TRUE);
	elm_object_signal_emit(edit_field, "elm,state,eraser,show", "elm");
	//elm_editfield_eraser_set(edit_field, EINA_TRUE);

	elm_entry_autocapital_type_set(entry, ELM_AUTOCAPITAL_TYPE_NONE);
	elm_entry_input_panel_layout_set(entry, ELM_INPUT_PANEL_LAYOUT_NORMAL);
	params->ap->mf_MainWindow.pEntry = entry;
	params->ap->mf_MainWindow.pEditField = edit_field;

	if (params->ap->mf_FileOperation.to_rename != NULL) {
		g_string_free(params->ap->mf_FileOperation.to_rename, TRUE);
		params->ap->mf_FileOperation.to_rename = NULL;
	}
	params->ap->mf_FileOperation.to_rename = g_string_new((char *)params->m_ItemName->str);


	/* the below is sample code for control entry. It is not mandatory.*/

	/* set guide text */
	filename = mf_fm_svc_wrapper_get_file_name(ap->mf_FileOperation.to_rename);
	char *guide_text = NULL;
	SAFE_FREE_CHAR(ap->mf_FileOperation.file_name_suffix);
	if (!mf_file_attr_is_dir(ap->mf_FileOperation.to_rename->str)) {
		mf_debug();
		char *ext = NULL;
		char *name_without_ext = NULL;
		name_without_ext = g_strdup(ap->mf_FileOperation.to_rename->str);
		mf_file_attr_get_file_ext(ap->mf_FileOperation.to_rename->str, &ext);
		mf_debug("ext is %s", ext);
		if (ext && strlen(ext) != 0) {
			mf_debug();
			name_without_ext[strlen(name_without_ext) - strlen(ext) - 1] = '\0';
			ap->mf_FileOperation.file_name_suffix = strdup(ext);
			mf_debug("name_without_ext is [%s]\n", name_without_ext);
			if (strlen(name_without_ext)) {
				guide_text = elm_entry_utf8_to_markup(ecore_file_file_get(name_without_ext));
			} else {
				guide_text = elm_entry_utf8_to_markup(filename->str);
			}
		} else {
			guide_text = elm_entry_utf8_to_markup(filename->str);
		}

		SAFE_FREE_CHAR(ext);
		SAFE_FREE_CHAR(name_without_ext);
	} else {
		guide_text = elm_entry_utf8_to_markup(filename->str);
	}

	if (ap->mf_FileOperation.file_name_suffix) {
		limit_filter_data.max_char_count = (MYFILE_FILE_NAME_LEN_MAX - mf_util_character_count_get(ap->mf_FileOperation.file_name_suffix) - 1);
	} else {
		limit_filter_data.max_char_count = MYFILE_FILE_NAME_LEN_MAX;
	}

	limit_filter_data.max_byte_count = 0;
	elm_entry_markup_filter_append(ap->mf_MainWindow.pEntry, elm_entry_filter_limit_size, &limit_filter_data);
	elm_entry_input_panel_return_key_type_set(ap->mf_MainWindow.pEntry, ELM_INPUT_PANEL_RETURN_KEY_TYPE_DONE);

	elm_entry_entry_set(ap->mf_MainWindow.pEntry, guide_text);
	/*elm_entry_entry_set(entry, ecore_file_file_get(params->m_ItemName->str));*/
	elm_entry_cursor_end_set(entry);

	SAFE_FREE_CHAR(guide_text);
	SAFE_FREE_GSTRING(filename);

	evas_object_smart_callback_add(entry, "activated", mf_genlist_rename_done, params->ap);
	evas_object_smart_callback_add(entry, "changed", mf_callback_imf_changed_cb, params->ap);
	evas_object_smart_callback_add(entry, "maxlength,reached", mf_callback_max_len_reached_cb, params->ap);
	evas_object_smart_callback_add(entry, "preedit,changed",
				       mf_callback_imf_preedit_change_cb, ap);

	elm_object_signal_callback_add(edit_field, "elm,eraser,clicked", "elm", mf_callback_eraser_clicked_cb, entry);
	evas_object_show(entry);
	elm_object_focus_set(entry, EINA_TRUE);

	MF_TRACE_END;
	return edit_field;
}

static Eina_Bool __mf_genlist_timer_cb(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, EINA_FALSE, "ap is null");

	if (ap->mf_MainWindow.pPreSearchViewSweepItem)
		elm_genlist_item_update(ap->mf_MainWindow.pPreSearchViewSweepItem);

	if(ap->mf_MainWindow.pSearchViewTimer) {
		ecore_timer_del(ap->mf_MainWindow.pSearchViewTimer);
		ap->mf_MainWindow.pSearchViewTimer = NULL;
	}
	return false;
}

void mf_genlist_gl_mode_sweep_set(void *data, Evas_Object *obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(event_info == NULL, "event_info is NULL");
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is null");

#ifndef MYFILE_ADD_TO_HOME
#ifndef	MYFILE_DETAILS
	if (ap->mf_Status.view_type == mf_view_root)
		return;
#endif
#endif

	if (ap->mf_Status.more == MORE_RENAME) {
		mf_genlist_rename_done(ap, NULL, NULL);
		return;
	} else if(ap->mf_Status.more == MORE_EDIT) {
		return;
	}

	if (ap->mf_Status.more == MORE_RENAME) {
		mf_callback_destory_rename(ap);
	}
	/* Start genlist sweep */
	Elm_Object_Item *it = (Elm_Object_Item *)elm_genlist_decorated_item_get(obj);
	mf_debug("it=%p, event_info=%p, pSearchViewSweepItem=%p",
		it, event_info, ap->mf_MainWindow.pSearchViewSweepItem);
	if (it && (it != event_info)) {
		elm_genlist_item_decorate_mode_set(it, "slide", EINA_FALSE);
		elm_genlist_item_select_mode_set(it, ELM_OBJECT_SELECT_MODE_DEFAULT);
	} else {
		mf_debug("no decorated mode item");
	}

	elm_genlist_item_decorate_mode_set(event_info, "slide", EINA_TRUE);
	elm_genlist_item_select_mode_set(event_info, ELM_OBJECT_SELECT_MODE_NONE);

	if (ap->mf_Status.more == MORE_SEARCH && event_info) {
		if(ap->mf_MainWindow.pSearchViewTimer) {
			ecore_timer_del(ap->mf_MainWindow.pSearchViewTimer);
			ap->mf_MainWindow.pSearchViewTimer = NULL;
		}
		ap->mf_MainWindow.pSearchViewTimer = ecore_timer_add(0.3, __mf_genlist_timer_cb, ap);
		ap->mf_MainWindow.pPreSearchViewSweepItem = it;
		ap->mf_MainWindow.pSearchViewSweepItem = event_info;
	}

	MF_TRACE_END;
}

void mf_genlist_gl_mode_sweep_unset(void *data, Evas_Object *obj, void *event_info)
{
	MF_TRACE_BEGIN;

	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is null");
#ifndef MYFILE_ADD_TO_HOME
#ifndef	MYFILE_DETAILS
	if (ap->mf_Status.view_type == mf_view_root)
		return;
#endif
#endif

	if (!event_info)
		return;

	if (ap->mf_Status.more == MORE_RENAME) {
		mf_genlist_rename_done(ap, NULL, NULL);
		return;
	} else if(ap->mf_Status.more == MORE_EDIT) {
		return;
	}

	if (obj)
		//elm_object_scroll_freeze_push(obj);

	if (ap->mf_Status.more == MORE_SEARCH && ap->mf_MainWindow.pSearchViewSweepItem) {
		if(ap->mf_MainWindow.pSearchViewTimer) {
			ecore_timer_del(ap->mf_MainWindow.pSearchViewTimer);
			ap->mf_MainWindow.pSearchViewTimer = NULL;
		}
		ap->mf_MainWindow.pSearchViewTimer = ecore_timer_add(0.3, __mf_genlist_timer_cb, ap);
		ap->mf_MainWindow.pPreSearchViewSweepItem = ap->mf_MainWindow.pSearchViewSweepItem;
		ap->mf_MainWindow.pSearchViewSweepItem = NULL;
	}

	/* Finish genlist sweep */
	elm_genlist_item_decorate_mode_set(event_info, "slide", EINA_FALSE);
	elm_genlist_item_select_mode_set(event_info, ELM_OBJECT_SELECT_MODE_DEFAULT);
	MF_TRACE_END;
}

void mf_genlist_gl_mode_cancel(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is null");
#ifndef MYFILE_ADD_TO_HOME
#ifndef	MYFILE_DETAILS
	if (ap->mf_Status.view_type == mf_view_root)
		return;
#endif
#endif
	if (!obj)
		return;

	if (ap->mf_Status.more == MORE_RENAME) {
		mf_genlist_rename_done(ap, NULL, NULL);
		return;
	} else if(ap->mf_Status.more == MORE_EDIT) {
		return;
	}
	/* Get sweeped item */
	Elm_Object_Item *it = (Elm_Object_Item *) elm_genlist_decorated_item_get(obj);

	if (ap->mf_Status.more == MORE_SEARCH)
		ap->mf_MainWindow.pSearchViewSweepItem = NULL;

	/* Finish genlist sweep */
	if (it) {
		elm_genlist_item_decorate_mode_set(it, "slide", EINA_FALSE);
		elm_genlist_item_select_mode_set(it, ELM_OBJECT_SELECT_MODE_DEFAULT);
	}
	MF_TRACE_END;
}

/**	label related	**/
static char *__mf_genlist_gl_label_get(void *data, Evas_Object * obj, const char *part)
{

	mfItemData_s *params = (mfItemData_s *) data;
	mf_retv_if(params == NULL, NULL);
	mf_debug("part=%s", part);
	__mf_genlist_reset_serch_sweep_item(params->ap);
	if (strcmp(part, "elm.text") == 0) {
		/* supporting multi-lang for default folders */
		if (g_strcmp0(params->m_ItemName->str, PHONE_FOLDER) == 0) {
			return g_strdup(GET_SYS_STR(MF_LABEL_PHONE));
		} else if (g_strcmp0(params->m_ItemName->str, MEMORY_FOLDER) == 0) {
			return g_strdup(GET_SYS_STR(MF_LABEL_MMC));
		} else if (g_strcmp0(params->m_ItemName->str, OTG_FOLDER) == 0) {
			return g_strdup(GET_STR(MF_LABEL_OTG));
		} else {
			if (params->ap->mf_Status.more == MORE_SEARCH &&params->ap->mf_Status.search_filter) {
				char *markup_name = NULL;
				bool res = false;
				markup_name = (char *)mf_util_search_markup_keyword(ecore_file_file_get(params->m_ItemName->str), params->ap->mf_Status.search_filter, &res);
				if(res) {
					return g_strdup(markup_name);
				}
			}
			if (params->ap->mf_Status.iExtensionState == MF_EXTENSION_HIDE && !mf_file_attr_is_dir(params->m_ItemName->str)) {
				return mf_fm_svc_get_file_name(params->m_ItemName);

			} else {
				return g_strdup(ecore_file_file_get(params->m_ItemName->str));
			}
		}
	} else if (strcmp(part, "elm.uptitle.text") == 0) {
		return g_strdup(params->m_ItemName->str);
	} else if (strcmp(part, "elm.slide_base.text") == 0) {
		return g_strdup(ecore_file_file_get(params->m_ItemName->str));
	} else if (strcmp(part, "elm.slide.text.1") == 0) {
		return g_strdup(ecore_file_file_get(params->m_ItemName->str));
	} else if (strcmp(part, "elm.text.1") == 0) {
		if (g_strcmp0(params->m_ItemName->str, PHONE_FOLDER) == 0) {
			return g_strdup(GET_SYS_STR(MF_LABEL_PHONE));
		} else if (g_strcmp0(params->m_ItemName->str, MEMORY_FOLDER) == 0) {
			return g_strdup(GET_SYS_STR(MF_LABEL_MMC));
		} else if (g_strcmp0(params->m_ItemName->str, OTG_FOLDER) == 0) {
			return g_strdup(GET_STR(MF_LABEL_OTG));
		} else if (params->ap->mf_Status.more == MORE_SEARCH &&params->ap->mf_Status.search_filter) {
			char *markup_name = NULL;
			bool res = false;
			markup_name = (char *)mf_util_search_markup_keyword(ecore_file_file_get(params->m_ItemName->str), params->ap->mf_Status.search_filter, &res);
			if(res) {
				return g_strdup(markup_name);
			}
		}
		if (params->ap->mf_Status.iExtensionState == MF_EXTENSION_HIDE && !mf_file_attr_is_dir(params->m_ItemName->str)) {
			return mf_fm_svc_get_file_name(params->m_ItemName);

		} else {
			return g_strdup(ecore_file_file_get(params->m_ItemName->str));
		}
	} else if (strcmp(part, "elm.text.2") == 0) {

		if (params->ap->mf_Status.more == MORE_SEARCH &&params->ap->mf_Status.search_filter) {
			int root_len = 0;
			char *new_path = NULL;
			GString *parent_path = NULL;
			parent_path = mf_fm_svc_wrapper_get_file_parent_path(params->m_ItemName);
			if (parent_path) {
				switch(mf_fm_svc_wrapper_get_location(parent_path->str)) {
				case MYFILE_PHONE:
					root_len = strlen(PHONE_FOLDER);
					new_path = g_strconcat(GET_SYS_STR(MF_LABEL_PHONE), parent_path->str + root_len, NULL);
					break;
				case MYFILE_MMC:
					root_len = strlen(MEMORY_FOLDER);
					new_path = g_strconcat(GET_SYS_STR(MF_LABEL_MMC), parent_path->str + root_len, NULL);
					break;
				case MYFILE_OTG:
					root_len = strlen(OTG_FOLDER);
					new_path = g_strconcat(GET_STR(MF_LABEL_OTG), parent_path->str + root_len, NULL);
					break;
				default:
					break;
				}
				SAFE_FREE_GSTRING(parent_path);
			}

			return new_path;
		} else if (params->ap->mf_Status.more != MORE_SEARCH
							&& params->ap->mf_Status.view_type == mf_view_root_category) {
			int root_len = 0;
			char *new_path = NULL;
			GString *parent_path = NULL;
			parent_path = mf_fm_svc_wrapper_get_file_parent_path(params->m_ItemName);
			if (parent_path) {
				switch(mf_fm_svc_wrapper_get_location(parent_path->str)) {
				case MYFILE_PHONE:
					root_len = strlen(PHONE_FOLDER);
					new_path = g_strconcat(GET_SYS_STR(MF_LABEL_PHONE), parent_path->str + root_len, NULL);
					break;
				case MYFILE_MMC:
					root_len = strlen(MEMORY_FOLDER);
					new_path = g_strconcat(GET_SYS_STR(MF_LABEL_MMC), parent_path->str + root_len, NULL);
					break;
				case MYFILE_OTG:
					root_len = strlen(OTG_FOLDER);
					new_path = g_strconcat(GET_SYS_STR(MF_LABEL_OTG), parent_path->str + root_len, NULL);
					break;
				default:
					break;
				}
				SAFE_FREE_GSTRING(parent_path);
			}

			return new_path;
		} else {
			if (params->ap->mf_Status.flagViewType != MF_VIEW_SYTLE_LIST_DETAIL) {
				int iSortTypeValue = 0;
				mf_util_get_vconf_value(VCONF_TYPE_SORT_TYPE, &iSortTypeValue);
				if (iSortTypeValue == MYFILE_SORT_BY_SIZE_S2L || iSortTypeValue == MYFILE_SORT_BY_SIZE_L2S) {
					if (params->size) {
						return g_strdup(params->size);
					} else {
						return NULL;
					}
				} else if (iSortTypeValue == MYFILE_SORT_BY_DATE_O2R || iSortTypeValue == MYFILE_SORT_BY_DATE_R2O) {
					if (params->create_date) {
						return g_strdup(params->create_date);
					} else {
						return NULL;
					}
				} else {
					return NULL;
				}
			} else {
				if (ecore_file_is_dir(params->m_ItemName->str)) {
					Eina_List *file_list = NULL;
					Eina_List *folder_list = NULL;
					int ret = 0;
					ret = mf_fs_oper_read_dir(params->m_ItemName->str, &folder_list, &file_list);
					if (ret == MYFILE_ERR_NONE) {
						int count = 0;
						count = eina_list_count(file_list) + eina_list_count(folder_list);
						mf_util_free_eina_list_with_data(&file_list, MYFILE_TYPE_FSNODE);
						mf_util_free_eina_list_with_data(&folder_list, MYFILE_TYPE_FSNODE);
						return g_strdup_printf("%d items", count);
					} else {
						return g_strdup(_(""));
					}
				} else {
					if (params->size) {
						return g_strdup(params->size);
					} else {
						return NULL;
					}
				}
			}

		}
	} else if (strcmp(part, "elm.text.3") == 0) {
		if (params->create_date) {
			return g_strdup(params->create_date);
		} else {
			return NULL;
		}

	} else {
		return g_strdup(_(""));
	}
}

/**	icon related	**/
static Evas_Object *__mf_genlist_gl_default_icon_get(void *data, Evas_Object * obj, const char *part)
{
	Evas_Object *layout = NULL;
	mfItemData_s *params = (mfItemData_s *) data;

	mf_retvm_if(params == NULL, NULL, "param is NULL");
	mf_retvm_if(params->m_ItemName == NULL, NULL, "m_ItemName is NULL");
	mf_retvm_if(part == NULL, NULL, "part is NULL");
	mf_retvm_if(obj == NULL, NULL, "obj is NULL");
	mf_debug("part=%s", part);
	__mf_genlist_reset_serch_sweep_item(params->ap);

	if (!strcmp(part, "elm.icon") || !strcmp(part, "elm.icon.1")) {
		Evas_Object *thumb = NULL;
		Evas_Object *ic_memory = NULL;
		MF_STORAGE is_in_mmc = FALSE;
		char *key = NULL;
		const char *memory_icon_path = MF_ICON_MEMORY_SUB;

		__mf_genlist_get_thumbnail(params);
		layout = mf_widget_create_layout(obj, EDJ_NAME, GRP_THUMBNAIL_ONLY);
		evas_object_size_hint_aspect_set(layout, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
		if (layout) {
			thumb = elm_image_add(layout);
			elm_object_focus_set(thumb, EINA_FALSE);
			if (thumb == NULL) {
				mf_error("thumb is NULL");
				return NULL;
			}
			elm_image_file_set(thumb, params->thumb_path, NULL);
			elm_object_part_content_set(layout, "thumbnail", thumb);

			mf_file_attr_get_store_type_by_full(params->m_ItemName->str, &is_in_mmc);
			if (is_in_mmc == MYFILE_MMC) {
				ic_memory = elm_image_add(layout);
				elm_object_focus_set(ic_memory, EINA_FALSE);
				elm_image_file_set(ic_memory, memory_icon_path, key);
				evas_object_show(ic_memory);
				elm_object_part_content_set(layout, "myfile.memory_icon_multi.sub", ic_memory);
			}

			fsFileType type = FILE_TYPE_NONE;
			type = mf_file_attr_get_file_type_by_mime(params->m_ItemName->str);
			if (type == FILE_TYPE_VIDEO) {
				const char *play_icon_path = MF_ICON_VIDEO_PLAY;
				Evas_Object *play_icon = NULL;
				play_icon = elm_image_add(layout);
				elm_object_focus_set(play_icon, EINA_FALSE);
				elm_image_file_set(play_icon, play_icon_path, key);
				evas_object_show(play_icon);
				elm_object_part_content_set(layout, "video_play", play_icon);
			}
		}
	} else if (!strcmp(part, "elm.icon.2")) {
		if (params->ap->mf_Status.more == MORE_EDIT) {
			return NULL;
		}
		Evas_Object *entry_button = NULL;
		Evas_Object *entry_icon = NULL;
		entry_button = elm_button_add(obj);
		elm_object_style_set(entry_button, "circle/empty");

		entry_icon = elm_image_add(entry_button);
		char *play_icon = strdup(MF_ICON_ENTRY_FOLDER);
		elm_image_file_set(entry_icon, play_icon, NULL);
		evas_object_size_hint_aspect_set(entry_icon, EVAS_ASPECT_CONTROL_BOTH, 1, 1);
		elm_object_part_content_set(entry_button, "icon", entry_icon);
		evas_object_propagate_events_set(entry_button, EINA_FALSE);
		evas_object_smart_callback_add(entry_button, "pressed", mf_callback_entry_button_pressed_cb, play_icon);
		evas_object_smart_callback_add(entry_button, "unpressed", mf_callback_entry_button_unpressed_cb, play_icon);
		evas_object_smart_callback_add(entry_button, "clicked", (Evas_Smart_Cb)mf_callback_entry_cb, (void*)params);
		return entry_button;
	}
	else if (!strcmp(part, "elm.base.swallow.detail")) {
	} else if (!strcmp(part, "elm.slide.swallow.1")) {
		if (params->ap->mf_Status.view_type == mf_view_root && params->ap->mf_Status.more != MORE_SEARCH) {
		}
		else
		{
			layout = elm_button_add(obj);
			elm_object_style_set(layout, "sweep/multiline");
			elm_object_text_set(layout, GET_SYS_STR(LABEL_RENAME));
			mf_language_mgr_register_object(layout, OBJ_TYPE_ELM_OBJECT, NULL, LABEL_RENAME);
			evas_object_smart_callback_add(layout, "clicked", (Evas_Smart_Cb) mf_callback_rename_button_callback, data);
		}
	} else if (!strcmp(part, "elm.slide.swallow.2")) {

		if (params->ap->mf_Status.view_type == mf_view_root && params->ap->mf_Status.more != MORE_SEARCH) {

		} else {
			layout = elm_button_add(obj);
			elm_object_style_set(layout, "sweep/delete");
			elm_object_text_set(layout, GET_SYS_STR(LABEL_DELETE));
			mf_language_mgr_register_object(layout, OBJ_TYPE_ELM_OBJECT, NULL, LABEL_DELETE);
			evas_object_smart_callback_add(layout, "clicked", mf_callback_delete_button_cb, data);
		}
	} else if (!strcmp(part, "elm.slide.swallow.3")) {
		layout = elm_button_add(obj);
		elm_object_style_set(layout, "sweep/delete");
		elm_object_text_set(layout, GET_SYS_STR(LABEL_DELETE));
		mf_language_mgr_register_object(layout, OBJ_TYPE_ELM_OBJECT, NULL, LABEL_DELETE);
		evas_object_smart_callback_add(layout, "clicked", mf_callback_delete_button_cb, data);
	} else if (!strcmp(part, "elm.slide.swallow.4")) {
		layout = elm_button_add(obj);
		elm_object_style_set(layout, "sweep/delete");
		elm_object_text_set(layout, GET_SYS_STR(LABEL_DELETE));
		mf_language_mgr_register_object(layout, OBJ_TYPE_ELM_OBJECT, NULL, LABEL_DELETE);
		evas_object_smart_callback_add(layout, "clicked", mf_callback_delete_button_cb, data);
	} else if (!strcmp(part, "elm.flip.content")) {	/* this is used when the rename mode is enabled.*/
		Evas_Object *edit_field = NULL;
		edit_field = __mf_genlist_rename_editfiled_create(params, obj);
		evas_object_propagate_events_set(edit_field, EINA_FALSE);
		return edit_field;
	} else if (!strcmp(part, "elm.flip.icon")) {
		Evas_Object *cancel_btn = mf_widget_util_create_button(obj, NULL, LABEL_CANCEL, NULL,
				mf_callback_cancel_cb, params->ap);
		evas_object_show(cancel_btn);
		return cancel_btn;
	}
	return layout;
}

static void __mf_genlist_item_check_changed_callback(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mfItemData_s *params = (mfItemData_s *) data;
	mf_retm_if(params == NULL, "param is NULL");

	struct appdata *ap = (struct appdata *)params->ap;
	mf_retm_if(ap == NULL, "input parameter data error");
	mf_retm_if(obj == NULL, "obj is NULL");

	if (ap->mf_Status.more == MORE_RENAME) {
		mf_genlist_rename_done(ap, NULL, NULL);
		params->m_checked = !params->m_checked;
		elm_check_state_pointer_set(obj, &params->m_checked);
		return;
	}

	if (params->m_checked) {
		ap->mf_MainWindow.pEditView.iCheckedCount++;
	} else {
		ap->mf_MainWindow.pEditView.iCheckedCount--;
	}


	if (ap->mf_MainWindow.pEditView.iTotalCount == ap->mf_MainWindow.pEditView.iCheckedCount) {
		ap->mf_MainWindow.pEditView.bSelectAllChecked = EINA_TRUE;
	} else {
		ap->mf_MainWindow.pEditView.bSelectAllChecked = EINA_FALSE;
	}
	mf_genlist_gl_edit_item_selected(ap);
	MF_TRACE_END;
}

static void __mf_genlist_selected_all_items_set(Evas_Object *genlist, int value)
{
	MF_TRACE_BEGIN;
	mf_retm_if(genlist == NULL, "genlist is NULL");

	mfItemData_s *it_data;
	Elm_Object_Item *it;

	it = elm_genlist_first_item_get(genlist);
	while (it) {
		it_data = elm_object_item_data_get(it);
		if (it_data)
			it_data->m_checked = value;
		//elm_check_state_pointer_set(it_data->pCheckBox, it_data->m_checked);
		elm_genlist_item_fields_update(it, "elm.edit.icon.1", ELM_GENLIST_ITEM_FIELD_CONTENT);
		it = elm_genlist_item_next_get(it);
	}
	MF_TRACE_END;
}

static void __mf_genlist_check_select_all(void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "input parameter data error");

	if (ap->mf_MainWindow.pEditView.bSelectAllChecked) {
		ap->mf_MainWindow.pEditView.iCheckedCount = ap->mf_MainWindow.pEditView.iTotalCount;
	} else {
		ap->mf_MainWindow.pEditView.iCheckedCount = 0;
	}

	if (ap->mf_Status.flagViewType == MF_VIEW_STYLE_LIST || ap->mf_Status.flagViewType == MF_VIEW_SYTLE_LIST_DETAIL) {
		__mf_genlist_selected_all_items_set(ap->mf_MainWindow.pEditView.pGenlist, ap->mf_MainWindow.pEditView.bSelectAllChecked);
	}else {
		mf_gengrid_select_all(ap);
	}
	mf_genlist_gl_edit_item_selected(ap);
	MF_TRACE_END;
}


void __mf_genlist_select_all_layout_mouse_down_cb(void *data, Evas * evas, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "input parameter data error");

	if (ap->mf_Status.more == MORE_RENAME) {
		mf_genlist_rename_done(ap, NULL, NULL);
		return;
	}

	ap->mf_MainWindow.pEditView.bSelectAllChecked = !ap->mf_MainWindow.pEditView.bSelectAllChecked;
	//elm_check_state_pointer_set(ap->mf_MainWindow.pEditView.pSelectAllCheckBox, &ap->mf_MainWindow.pEditView.bSelectAllChecked);

	__mf_genlist_check_select_all(ap);
	MF_TRACE_END;
}


void __mf_genlist_select_all_check_changed_cb(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "input parameter data error");

	if (ap->mf_Status.more == MORE_RENAME) {
		mf_genlist_rename_done(ap, NULL, NULL);
		ap->mf_MainWindow.pEditView.bSelectAllChecked = !ap->mf_MainWindow.pEditView.bSelectAllChecked;
		//elm_check_state_pointer_set(ap->mf_MainWindow.pEditView.pSelectAllCheckBox, &ap->mf_MainWindow.pEditView.bSelectAllChecked);
		return;
	}

	__mf_genlist_check_select_all(ap);

	MF_TRACE_END;
}

static Evas_Object *__mf_genlist_gl_edit_icon_get(void *data, Evas_Object * obj, const char *part)
{
	Evas_Object *layout = NULL;
	mfItemData_s *params = (mfItemData_s *) data;
	struct appdata *ap = (struct appdata *)params->ap;

	mf_retvm_if(params == NULL, NULL, "param is NULL");
	mf_retvm_if(params->m_ItemName == NULL, NULL, "m_ItemName is NULL");
	mf_retvm_if(part == NULL, NULL, "part is NULL");
	mf_retvm_if(obj == NULL, NULL, "obj is NULL");
	mf_retvm_if(ap == NULL, NULL, "ap is NULL");

	if (elm_genlist_decorate_mode_get(obj)) {
		if (!strcmp(part, "elm.edit.icon.1")) {
			Evas_Object *check = NULL;
			check = elm_check_add(obj);
			elm_object_focus_set(check, EINA_FALSE);
			evas_object_propagate_events_set(check, EINA_FALSE);
			elm_check_state_pointer_set(check, &params->m_checked);
			evas_object_smart_callback_add(check, "changed", __mf_genlist_item_check_changed_callback, params);
			return check;
		} else if (!strcmp(part, "elm.edit.icon.2")) {
			layout = elm_button_add(obj);
			elm_object_style_set(layout, "rename");
			evas_object_propagate_events_set(layout, EINA_FALSE);
			evas_object_smart_callback_add(layout, "clicked", mf_callback_rename_button_callback, params);
			return layout;
		}
	}
	if (!strcmp(part, "elm.icon") || !strcmp(part, "elm.icon.1")) {
		Evas_Object *thumb = NULL;
		Evas_Object *ic_memory = NULL;
		MF_STORAGE is_in_mmc = FALSE;
		char *key = NULL;
		const char *memory_icon_path = MF_ICON_MEMORY_SUB;

		__mf_genlist_get_thumbnail(params);
#ifdef USE_IMAGE_OBJECT
		thumb = evas_object_image_add(evas_object_evas_get(obj));
#else
		thumb = elm_image_add(obj);
		elm_object_focus_set(thumb, EINA_FALSE);
#endif
		if (thumb == NULL)
			mf_error("thumb is NULL");

		if (thumb) {
#ifdef USE_IMAGE_OBJECT
			int load_err = EVAS_LOAD_ERROR_NONE;
			evas_object_image_file_set(thumb, params->thumb_path, NULL);
			load_err = evas_object_image_load_error_get(thumb);

			if (load_err != EVAS_LOAD_ERROR_NONE) {
				evas_object_del(thumb);
				mf_error("load error code : %d", load_err);
			} else {
				evas_object_image_filled_set(thumb, EINA_TRUE);
			}
#else
			elm_image_file_set(thumb, params->thumb_path, NULL);
#endif
			layout = elm_layout_add(obj);
			elm_object_focus_set(layout, EINA_FALSE);
			elm_layout_file_set(layout, EDJ_NAME, GRP_THUMBNAIL_ONLY);
			elm_object_part_content_set(layout, "thumbnail", thumb);

			mf_file_attr_get_store_type_by_full(params->m_ItemName->str, &is_in_mmc);
			if (is_in_mmc == MYFILE_MMC) {
				ic_memory = elm_image_add(obj);
				elm_object_focus_set(ic_memory, EINA_FALSE);
				elm_image_file_set(ic_memory, memory_icon_path, key);
				evas_object_show(ic_memory);
				elm_object_part_content_set(layout, "myfile.memory_icon_multi.sub", ic_memory);
			}
			fsFileType type = FILE_TYPE_NONE;
			type = mf_file_attr_get_file_type_by_mime(params->m_ItemName->str);
			if (type == FILE_TYPE_VIDEO) {
				const char *play_icon_path = MF_ICON_VIDEO_PLAY;
				Evas_Object *play_icon = NULL;
				play_icon = elm_image_add(layout);
				elm_object_focus_set(play_icon, EINA_FALSE);
				elm_image_file_set(play_icon, play_icon_path, key);
				evas_object_show(play_icon);
				elm_object_part_content_set(layout, "video_play", play_icon);
			}
		}
	} else if (!strcmp(part, "elm.icon.2")) {
		if (params->ap->mf_Status.more == MORE_EDIT || (params->ap->mf_Status.preMore == MORE_EDIT && params->ap->mf_Status.more == MORE_RENAME)) {
			return NULL;
		}
		Evas_Object *entry_button = NULL;
		Evas_Object *entry_icon = NULL;
		entry_button = elm_button_add(obj);
		elm_object_style_set(entry_button, "circle/empty");

		entry_icon = elm_image_add(entry_button);
		char *play_icon = strdup(MF_ICON_ENTRY_FOLDER);
		elm_image_file_set(entry_icon, play_icon, NULL);
		evas_object_size_hint_aspect_set(entry_icon, EVAS_ASPECT_CONTROL_BOTH, 1, 1);
		elm_object_part_content_set(entry_button, "icon", entry_icon);
		evas_object_propagate_events_set(entry_button, EINA_FALSE);
		evas_object_smart_callback_add(entry_button, "clicked", (Evas_Smart_Cb)mf_callback_entry_cb, (void*)params);
		evas_object_smart_callback_add(entry_button, "pressed", mf_callback_entry_button_pressed_cb, play_icon);
		evas_object_smart_callback_add(entry_button, "unpressed", mf_callback_entry_button_unpressed_cb, play_icon);
		return entry_button;
	} else if (!strcmp(part, "elm.swallow.end")) {
		if (ap->mf_Status.rotation_type != MF_ROTATE_LANDSCAPE) {
			//bool systemfolder = false;
			//int error_code = 0;
#if 0
			error_code = mf_fm_svc_wrapper_get_folder_foldersystem(params->m_ItemName, &systemfolder);
			if (systemfolder == FALSE) {
#endif
			/*__mf_genlist_add_rename_button*/
			layout = elm_button_add(obj);
			//elm_object_focus_set(layout, EINA_FALSE);
			elm_object_style_set(layout, "rename");
			evas_object_propagate_events_set(layout, EINA_FALSE);
			evas_object_smart_callback_add(layout, "clicked", mf_callback_rename_button_callback, params);
			/*evas_object_event_callback_add(layout, EVAS_CALLBACK_MOUSE_UP, mf_callback_rename_button_callback, params);*/
			//}
		}
	} else if (!strcmp(part, "elm.flip.content")) {	/* this is used when the rename mode is enabled.*/
		Evas_Object *edit_field = NULL;
		edit_field = __mf_genlist_rename_editfiled_create(params, obj);
		evas_object_propagate_events_set(edit_field, EINA_FALSE);
		return edit_field;
	} else if (!strcmp(part, "elm.flip.icon")) {
		Evas_Object *cancel_btn = mf_widget_util_create_button(obj, NULL, LABEL_CANCEL, NULL,
				mf_callback_cancel_cb, params->ap);
		evas_object_show(cancel_btn);
		return cancel_btn;
	}

	return layout;
}


static Evas_Object *__mf_genlist_gl_new_folder_icon_get(void *data, Evas_Object * obj, const char *part)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, NULL, "ap is NULL");

	if (!strcmp(part, "elm.icon")) {
		int error_code = 0;
		char *fileName = NULL;
		char *fullpath = (char *)malloc(sizeof(char) * MYFILE_FILE_PATH_LEN_MAX);
		if (fullpath == NULL) {
			return NULL;
		}
		GString *dest_fullpath;
		static Elm_Entry_Filter_Limit_Size limit_filter;
		Evas_Object *entry = ap->mf_MainWindow.pEntry;

		Evas_Object *edit_field = ap->mf_MainWindow.pEditField;

		limit_filter.max_char_count = MYFILE_FILE_NAME_LEN_MAX;
		limit_filter.max_byte_count = 0;

		memset(fullpath, 0, MYFILE_FILE_PATH_LEN_MAX);
		fileName = elm_entry_markup_to_utf8(GET_SYS_STR(MF_LABEL_FOLDER));
		snprintf(fullpath, (MYFILE_FILE_PATH_LEN_MAX), "%s/%s", (char *)ap->mf_Status.path->str, fileName);

		dest_fullpath = g_string_new(fullpath);

		elm_entry_markup_filter_append(ap->mf_MainWindow.pEntry, elm_entry_filter_limit_size, &limit_filter);
		evas_object_smart_callback_add(ap->mf_MainWindow.pEntry, "maxlength,reached", mf_callback_max_len_reached_cb, ap);

		if (mf_fm_svc_wrapper_detect_duplication(dest_fullpath)) {
			mf_debug("dectetion result is TRUE\ndest_fullpath is %s\n", dest_fullpath->str);
			error_code = mf_fm_svc_wrapper_file_auto_rename(ap, dest_fullpath, FILE_NAME_WITH_BRACKETS, &dest_fullpath);
			mf_debug("rename result is %s\n", dest_fullpath->str);
			memset(fullpath, 0, strlen(fullpath));
			int len = strlen(ap->mf_Status.path->str) + 1;
			strncpy(fullpath, dest_fullpath->str + len, MYFILE_FILE_PATH_LEN_MAX);
			/*memset(fileName,0,strlen(fileName));*/

			SAFE_FREE_CHAR(fileName);
			fileName = elm_entry_markup_to_utf8(fullpath);
		}

		/*elm_entry_entry_set(entry, ecore_file_file_get(params->m_ItemName->str));*/

		elm_entry_entry_set(entry, fileName);
		elm_entry_cursor_end_set(entry);

		SAFE_FREE_CHAR(fullpath);
		SAFE_FREE_GSTRING(dest_fullpath);
		SAFE_FREE_CHAR(fileName);
		return edit_field;
	}
	return NULL;
}

static Evas_Object *__mf_genlist_gl_rename_icon_get(void *data, Evas_Object * obj, const char *part)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, NULL, "ap is NULL");

	if (!strcmp(part, "elm.icon")) {
		static Elm_Entry_Filter_Limit_Size limit_filter_data;
		Evas_Object *edit_field = NULL;
		Evas_Object *entry = NULL;
		GString *filename = NULL;

		edit_field = mf_widget_create_editfield(obj, &entry);

		evas_object_size_hint_weight_set(edit_field, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(edit_field, EVAS_HINT_FILL, EVAS_HINT_FILL);


		elm_entry_single_line_set(entry, EINA_TRUE);
		elm_entry_scrollable_set(entry, EINA_TRUE);
		elm_object_signal_emit(edit_field, "elm,state,eraser,show", "elm");
		//elm_editfield_eraser_set(edit_field, EINA_TRUE);

		elm_entry_autocapital_type_set(entry, ELM_AUTOCAPITAL_TYPE_NONE);
		elm_entry_input_panel_layout_set(entry, ELM_INPUT_PANEL_LAYOUT_NORMAL);
		ap->mf_MainWindow.pEntry = entry;
		ap->mf_MainWindow.pEditField = edit_field;


		/* set guide text */
		filename = mf_fm_svc_wrapper_get_file_name(ap->mf_FileOperation.to_rename);
		char *guide_text = NULL;
		SAFE_FREE_CHAR(ap->mf_FileOperation.file_name_suffix);
		if (!mf_file_attr_is_dir(ap->mf_FileOperation.to_rename->str)) {
			mf_debug();
			char *ext = NULL;
			char *name_without_ext = NULL;
			name_without_ext = g_strdup(ap->mf_FileOperation.to_rename->str);
			mf_file_attr_get_file_ext(ap->mf_FileOperation.to_rename->str, &ext);
			mf_debug("ext is %s", ext);
			if (ext && strlen(ext) != 0) {
				mf_debug();
				name_without_ext[strlen(name_without_ext) - strlen(ext) - 1] = '\0';
				ap->mf_FileOperation.file_name_suffix = strdup(ext);
				mf_debug("name_without_ext is [%s]\n", name_without_ext);
				if (strlen(name_without_ext)) {
					guide_text = elm_entry_utf8_to_markup(ecore_file_file_get(name_without_ext));
				} else {
					guide_text = elm_entry_utf8_to_markup(filename->str);
				}
			} else {
				guide_text = elm_entry_utf8_to_markup(filename->str);
			}

			SAFE_FREE_CHAR(ext);
			SAFE_FREE_CHAR(name_without_ext);
		} else {
			guide_text = elm_entry_utf8_to_markup(filename->str);
		}

		if (ap->mf_FileOperation.file_name_suffix) {
			limit_filter_data.max_char_count = (MYFILE_FILE_NAME_LEN_MAX - mf_util_character_count_get(ap->mf_FileOperation.file_name_suffix) - 1);
		} else {
			limit_filter_data.max_char_count = MYFILE_FILE_NAME_LEN_MAX;
		}

		limit_filter_data.max_byte_count = 0;
		elm_entry_markup_filter_append(ap->mf_MainWindow.pEntry, elm_entry_filter_limit_size, &limit_filter_data);


		elm_entry_entry_set(ap->mf_MainWindow.pEntry, guide_text);
		/*elm_entry_entry_set(entry, ecore_file_file_get(params->m_ItemName->str));*/
		elm_entry_cursor_end_set(entry);

		SAFE_FREE_CHAR(guide_text);
		SAFE_FREE_GSTRING(filename);

		//evas_object_smart_callback_add(entry, "activated", mf_genlist_rename_done, ap);
		evas_object_smart_callback_add(entry, "changed", mf_callback_imf_changed_cb, ap);
		evas_object_smart_callback_add(entry, "maxlength,reached", mf_callback_max_len_reached_cb, ap);
		evas_object_smart_callback_add(entry, "preedit,changed",
					       mf_callback_imf_preedit_change_cb, ap);
		elm_object_signal_callback_add(edit_field, "elm,eraser,clicked", "elm", mf_callback_eraser_clicked_cb, entry);
		evas_object_show(entry);
		elm_object_focus_set(entry, EINA_TRUE);
		return edit_field;
	}
	return NULL;
}

/**	delete related	**/
static void __mf_genlist_gl_del(void *data, Evas_Object * obj)
{
	MF_TRACE_BEGIN;
	mfItemData_s *params = (mfItemData_s *) data;
	assert(params);
	if (params->ap->mf_Status.more == MORE_RENAME) {
		if (params->item == params->ap->mf_FileOperation.rename_item) {
			mf_callback_destory_rename(params->ap);
		}
	}
	if (params->m_ItemName) {
		g_string_free(params->m_ItemName, TRUE);
		params->m_ItemName = NULL;
	}
	if (params->thumb_path) {
		free(params->thumb_path);
		params->thumb_path = NULL;
	}
	if (params->create_date) {
		free(params->create_date);
		params->create_date = NULL;
	}
	if (params->size) {
		free(params->size);
		params->size = NULL;
	}
	if (params->flagExpand) {
		params->flagExpand = false;
	}
	if (params->media) {
		media_info_cancel_thumbnail(params->media);
		media_info_destroy(params->media);
		params->media = NULL;
	}
	free(params);
	MF_TRACE_END;
	return;
}

/**	state related	**/
static Eina_Bool __mf_genlist_gl_state_get(void *data, Evas_Object * obj, const char *part)
{
	return EINA_FALSE;
}

/**	select related	**/
/**	this function will be splited into several functions	**/
void mf_genlist_gl_selected(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, NULL, "ap is NULL");

	Elm_Object_Item *item = (Elm_Object_Item *) event_info;
	if (item != NULL) {
		mfItemData_s *selected = (mfItemData_s *) elm_object_item_data_get(item);

		elm_genlist_item_selected_set(item, FALSE);

		if (ap->mf_Status.flagNaviPush) {
			mf_debug("the transition of the naviframe is not finished");
			return;
		}
		if (ap->mf_Status.more == MORE_RENAME) {
			if (elm_genlist_item_flip_get(item)) {
				MF_TRACE_END;
				return;
			} else {
				mf_callback_rename_save_cb(ap, NULL, NULL);
				return;
			}
		} else if (ap->mf_Status.more != MORE_EDIT) {
#ifdef MYFILE_SPLIT_VIEW
			if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
				mf_split_view_left_genlist_refresh(ap);
			}
#endif
			mf_callback_click_cb(data, MFACTION_CLICK, selected->m_ItemName);
		}
	}
	MF_TRACE_END;
}
#ifdef MYFILE_SPLIT_VIEW

static void mf_genlist_split_left_items_sel(void *data, Evas_Object *obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;

	mfItemData_s *selected = (mfItemData_s *) elm_object_item_data_get(event_info);
	elm_genlist_item_selected_set(event_info, EINA_FALSE);
	if (ap->mf_Status.flagNaviPush) {
		mf_debug("the transition of the naviframe is not finished");
		return;
	}
	Eina_Bool expanded = EINA_FALSE;
	if (event_info) {
		expanded = elm_genlist_item_expanded_get(event_info);
		selected->flagExpand = !expanded;
		mf_error("selected->flagExpand is [%d]", selected->flagExpand);
		elm_object_item_data_set(selected->item, selected);
		elm_genlist_item_fields_update(selected->item, "elm.icon", ELM_GENLIST_ITEM_FIELD_CONTENT);
		elm_genlist_item_expanded_set(event_info, !expanded);
	}
	MF_TRACE_END;
}

Elm_Object_Item *mf_genlist_split_left_items_append( void *data, Evas_Object *pGenlist, Elm_Object_Item *parent_item, Eina_List *dir_list)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, NULL, "input parameter data error");

	mf_retvm_if(pGenlist == NULL, NULL, "input parameter pGenlist error");
	mf_retvm_if(parent_item == NULL, NULL, "input parameter parent_item error");

	fsNodeInfo *pNode = NULL;
	mfItemData_s *m_TempItem = NULL;
	Eina_List *l = NULL;
	Elm_Object_Item *expand = NULL;

	int real_len = 0;
	int current_len = 0;
	bool flagExpand = false;

	EINA_LIST_FOREACH(dir_list, l, pNode) {
		char *real_name = NULL;
		real_name = g_strconcat(pNode->path, "/", pNode->name, NULL);
		if (real_name == NULL) {
			continue;
		}

		m_TempItem = (mfItemData_s *) malloc(sizeof(mfItemData_s));
		if (m_TempItem == NULL) {
			free(real_name);
			real_name = NULL;
			continue;
		}


		Elm_Object_Item *it = NULL;
		m_TempItem->m_ItemName = g_string_new(real_name);
		m_TempItem->size = NULL;
		m_TempItem->create_date = NULL;
		m_TempItem->m_checked = FALSE;
		m_TempItem->pCheckBox = NULL;
		m_TempItem->thumb_path = NULL;
		m_TempItem->flagExpand = false;
		m_TempItem->real_thumb_flag = FALSE;
		m_TempItem->unregister_lang_mgr = TRUE;
		m_TempItem->media = NULL;
		m_TempItem->ap = ap;

		if (ap->mf_Status.flagViewType == MF_VIEW_SYTLE_LIST_DETAIL) {
			char *tempdate = mf_util_icu_translate(ap, pNode->date);
			if (tempdate == NULL)
				m_TempItem->create_date = g_strdup("N/A");
			else {
				m_TempItem->create_date = g_strdup(tempdate);
				free(tempdate);
			}

		}

		it = elm_genlist_item_append(pGenlist, &left_itc, m_TempItem, parent_item, ELM_GENLIST_ITEM_TREE, mf_genlist_split_left_items_sel, ap);
		real_len = strlen(real_name);
		current_len = strlen(ap->mf_Status.path->str);
		if (real_len > current_len && !flagExpand) {
			expand = NULL;
		} else {
			if (strncmp(real_name, ap->mf_Status.path->str, strlen(real_name)) == 0) {
				mf_debug("real_name is [%s] currnet path is [%s]", real_name, ap->mf_Status.path->str);
				expand = it;
				flagExpand = true;
			}
		}
		m_TempItem->item = it;
		free(real_name);
	}
	MF_TRACE_END;
	return expand;
}


Elm_Object_Item *mf_genlist_split_root_item_append(void *data, Evas_Object *pGenlist, const char *name)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(data == NULL, NULL, "input parameter data error");
	mf_retvm_if(pGenlist == NULL, NULL, "input parameter pGenlist error");

	struct appdata *ap = (struct appdata *)data;

	mfItemData_s *m_TempItem = NULL;
	Elm_Object_Item *it = NULL;

	mf_genlist_data_init(ap, &m_TempItem, name);
	m_TempItem->unregister_lang_mgr = TRUE;

	it = elm_genlist_item_append(pGenlist, &left_itc, m_TempItem, NULL, ELM_GENLIST_ITEM_TREE, mf_genlist_split_left_items_sel, ap);

	m_TempItem->item = it;
	return it;
}

void mf_genlist_split_root_items_append(Evas_Object *pGenlist, void *data)
{
	MF_TRACE_BEGIN;
	/*0.    variable definition and parameter check*/
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "input parameter data error");

	mf_retm_if(pGenlist == NULL, "input parameter pGenlist error");

	/*4.    append items to the genlist*/
	Elm_Object_Item *it = NULL;

	//mf_genlist_data_init(ap, &m_TempItem, PHONE_FOLDER);

	it = mf_genlist_split_root_item_append(ap, pGenlist, PHONE_FOLDER);
	//it = elm_genlist_item_append(pGenlist, &left_itc, m_TempItem, NULL, ELM_GENLIST_ITEM_NONE, mf_genlist_gl_sel, m_TempItem);

	//m_TempItem->item = it;
	ap->mf_MainWindow.sSplitData.phone_item = it;

	if (ap->mf_Status.iStorageState & MYFILE_MMC) {
		it = mf_genlist_split_root_item_append(ap, pGenlist, MEMORY_FOLDER);
		ap->mf_MainWindow.sSplitData.mmc_item = it;

	}

	if (ap->mf_Status.iStorageState & MYFILE_OTG) {

		it = mf_genlist_split_root_item_append(ap, pGenlist, OTG_FOLDER);
		ap->mf_MainWindow.sSplitData.otg_item = it;

	}
	MF_TRACE_END;

}

void mf_genlist_split_left_genlist_style(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "input parameter data error");
	left_itc.item_style = "1text.2/expandable_n_depth";
	left_itc.func.text_get = __mf_genlist_gl_label_get;
	left_itc.func.content_get  = __mf_genlist_gl_default_icon_get;
	left_itc.func.state_get = __mf_genlist_gl_state_get;
	left_itc.func.del = __mf_genlist_gl_del;

	elm_genlist_decorate_mode_set(ap->mf_MainWindow.sSplitData.pSplitLeftGenlist, EINA_FALSE);

}

#endif
/**	edit mode related	**/
void mf_genlist_gl_sel(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mfItemData_s *selected = data;

	struct appdata *ap = (struct appdata *)selected->ap;
	mf_retm_if(ap == NULL, "input parameter data error");

	if (ap->mf_Status.more != MORE_EDIT) {
		return;
	}

	Evas_Object *genlist = elm_object_item_widget_get(selected->item);

	if (genlist && elm_genlist_decorate_mode_get(genlist)) {

		selected->m_checked = !selected->m_checked;
		if (selected->m_checked) {
			ap->mf_MainWindow.pEditView.iCheckedCount++;
		} else {
			ap->mf_MainWindow.pEditView.iCheckedCount--;
		}
		//if (ap->mf_MainWindow.pEditView.pSelectAllCheckBox) {
			if (ap->mf_MainWindow.pEditView.iTotalCount == ap->mf_MainWindow.pEditView.iCheckedCount) {
				ap->mf_MainWindow.pEditView.bSelectAllChecked = EINA_TRUE;
			} else {
				ap->mf_MainWindow.pEditView.bSelectAllChecked = EINA_FALSE;
			}
			//elm_check_state_pointer_set(ap->mf_MainWindow.pEditView.pSelectAllCheckBox, &ap->mf_MainWindow.pEditView.bSelectAllChecked);
		//}
		elm_genlist_item_fields_update(selected->item, "elm.edit.icon.1", ELM_GENLIST_ITEM_FIELD_CONTENT);
	}
	mf_genlist_gl_edit_item_selected(ap);
	MF_TRACE_END;
}

void mf_genlist_genlist_item_select(void *data, Evas_Object *pGenlist, int *file_count, int *dir_count)
{
	mf_retm_if(data == NULL, "data is NULL");
	mf_retm_if(pGenlist == NULL, "pGenlist is NULL");

	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mfItemData_s *itemData = NULL;
	Elm_Object_Item *it = NULL;
	//int error_code = 0;
	it = elm_genlist_first_item_get(pGenlist);
	while (it) {
		itemData = elm_object_item_data_get(it);
		if (itemData->m_checked) {
			if (itemData->m_ItemName == NULL || itemData->m_ItemName->str == NULL) {
				continue;
			}
			if (ecore_file_is_dir(itemData->m_ItemName->str)) {
				//bool systemfolder = FALSE;
				ap->mf_Status.flagFolderSelect = TRUE;
				(*dir_count)++;
			} else {
				ap->mf_Status.flagFileSelect = TRUE;
				(*file_count)++;
			}
		}

		it = elm_genlist_item_next_get(it);

	}
	MF_TRACE_END;
}

void mf_genlist_gl_edit_item_selected(void *data)
{
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata *)data;

	if (ap->mf_Status.more != MORE_EDIT)
		return;
	Evas_Object *notify_popup = NULL;
	char *label = NULL;
	int count_dir = 0;
	int count_file = 0;

	ap->mf_Status.flagFolderSelect = FALSE;
	ap->mf_Status.flagFileSelect = FALSE;
	if (ap->mf_Status.flagViewType == MF_VIEW_STYLE_LIST || ap->mf_Status.flagViewType == MF_VIEW_SYTLE_LIST_DETAIL) {
		Evas_Object *pGenlist = ap->mf_MainWindow.pEditView.pGenlist;
		mf_genlist_genlist_item_select(ap, pGenlist, &count_file, &count_dir);

	} else {
		Evas_Object *pGengrid = ap->mf_MainWindow.pEditView.pGengrid;
		mf_gengrid_item_select(ap, pGengrid, &count_file, &count_dir);
	}

	if (ap->mf_Status.flagFolderSelect) {
		mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_EDIT_ALL, true);
		mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_USER_FOLDER_SEL, false);
	} else if (ap->mf_Status.flagFileSelect){
		mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_EDIT_ALL, false);
	} else {
		mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_EDIT_ALL, true);
	}

	char lable_file[MYFILE_LABEL_STRING_LENGTH] = { '\0', };
	int count = 0;
	/*1 TODO: need to update for multi-language*/
	count = count_file + count_dir;
	snprintf(lable_file, sizeof(lable_file), "%d", (count));
	label = g_strconcat(GET_SYS_STR(MF_LABEL_SELECTED), " (", lable_file, ")", NULL);

	if (count > 0) {
		notify_popup = mf_widget_create_notify_popup(ap, ap->mf_MainWindow.pNaviLayout, label, mf_notify_popup_normal, true, NULL);
		ap->mf_MainWindow.pLabelLayout = notify_popup;
	}else {
		SAFE_FREE_OBJ(ap->mf_MainWindow.pLabelLayout);
	}
	SAFE_FREE_CHAR(label);
}


/**	button related	**/


static inline void __mf_genlist_get_thumbnail(mfItemData_s *params)
{
	int error_code = 0;
	if (params->thumb_path == NULL) {
		if (ecore_file_is_dir(params->m_ItemName->str)) {
			if (params->ap->mf_Status.view_type == mf_view_root) {
				params->thumb_path = strdup(MF_ICON_FOLDER);
			} else if (mf_fm_svc_wrapper_is_root_path(params->ap->mf_Status.path->str) == MYFILE_OTG) {
				params->thumb_path = strdup(MF_ICON_EXTERNAL_DEVICE);
			} else {
				if (params->flagExpand == true) {
					params->thumb_path = strdup(MF_ICON_FOLDER_OPEN);
				} else {
					params->thumb_path = strdup(MF_ICON_FOLDER);
				}
			}
			params->real_thumb_flag = TRUE;
		} else {
			const char *icon_path = NULL;
			int thumbnail_type = 0;
			thumbnail_type = mf_file_attr_get_file_icon(params->m_ItemName->str, &error_code, MF_ROTATE_PORTRAIT, &icon_path, &params->media);
			if (icon_path && ecore_file_exists(icon_path)) {
				params->thumb_path = strdup(icon_path);
				params->real_thumb_flag = TRUE;
			} else {	/*must be image/video file*/
				fsFileType type = FILE_TYPE_NONE;
				type = mf_file_attr_get_file_type_by_mime(params->m_ItemName->str);

				if (type == FILE_TYPE_VIDEO) {
					params->thumb_path = strdup(MF_ICON_VIDEO);
				} else if (type == FILE_TYPE_IMAGE) {
					params->thumb_path = strdup(MF_ICON_IMAGE);
				} else {
					params->thumb_path = strdup(DEFAULT_ICON);
				}
				params->real_thumb_flag = FALSE;
				error_code = MYFILE_ERR_GET_THUMBNAIL_FAILED;
			}
		}
	} else if (params->thumb_path && ecore_file_exists(params->thumb_path)) {

		if (ecore_file_is_dir(params->m_ItemName->str)) {
			if (mf_fm_svc_wrapper_is_root_path(params->ap->mf_Status.path->str) == MYFILE_OTG) {
				params->thumb_path = strdup(MF_ICON_EXTERNAL_DEVICE);
			} else {
				if (params->flagExpand == true) {
					params->thumb_path = strdup(MF_ICON_FOLDER_OPEN);
				} else {
					params->thumb_path = strdup(MF_ICON_FOLDER);
				}
			}
		}
		if (params->real_thumb_flag == FALSE) {
			error_code = MYFILE_ERR_GET_THUMBNAIL_FAILED;
		}
	} else if (!ecore_file_exists(params->thumb_path)) {
		mf_error("thumb_path[%s] is not exist", params->thumb_path);
		free(params->thumb_path);
		params->thumb_path = NULL;

		fsFileType type = FILE_TYPE_NONE;
		type = mf_file_attr_get_file_type_by_mime(params->m_ItemName->str);
		if (ecore_file_is_dir(params->m_ItemName->str)) {
			if (mf_fm_svc_wrapper_is_root_path(params->ap->mf_Status.path->str) == MYFILE_OTG) {
				params->thumb_path = strdup(MF_ICON_EXTERNAL_DEVICE);
			} else {
				if (params->flagExpand == true) {
					params->thumb_path = strdup(MF_ICON_FOLDER_OPEN);
				} else {
					params->thumb_path = strdup(MF_ICON_FOLDER);
				}
			}

		} else {
			if (type == FILE_TYPE_VIDEO) {
				params->thumb_path = strdup(MF_ICON_VIDEO);
			} else if (type == FILE_TYPE_IMAGE) {
				params->thumb_path = strdup(MF_ICON_IMAGE);
			} else {
				params->thumb_path = strdup(DEFAULT_ICON);
			}
		}
		/*set default icon*/
		params->real_thumb_flag = FALSE;
		error_code = MYFILE_ERR_GET_THUMBNAIL_FAILED;
	}
	if (error_code != 0) {
		mf_callback_create_thumbnail(params, mf_callback_thumb_created_cb);
	}
}


void mf_genlist_update_edit_view_item_style(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "input parameter data error");
	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "get navigation bar in use failed");

	userfolderitc.decorate_item_style = NULL;
	userfolderitc.func.text_get = __mf_genlist_gl_label_get;
	userfolderitc.func.content_get = __mf_genlist_gl_edit_icon_get;
	userfolderitc.func.del = __mf_genlist_gl_del;
	userfolderitc.decorate_all_item_style = "edit_default";

	itc.decorate_item_style = NULL;
	itc.func.text_get = __mf_genlist_gl_label_get;
	itc.func.content_get = __mf_genlist_gl_edit_icon_get;
	itc.func.del = __mf_genlist_gl_del;
	itc.decorate_all_item_style = "edit_default";

	categoryitc.decorate_item_style = NULL;
	categoryitc.func.text_get = __mf_genlist_gl_label_get;
	categoryitc.func.content_get = __mf_genlist_gl_edit_icon_get;
	categoryitc.func.del = __mf_genlist_gl_del;
	categoryitc.decorate_all_item_style = "edit_default";

}


void mf_genlist_set_category_list_style()
{
	categoryitc.item_style = "2text.2icon.4";
	categoryitc.decorate_item_style = "mode/slide2";
	categoryitc.func.text_get = __mf_genlist_gl_label_get;
	categoryitc.func.content_get  = __mf_genlist_gl_default_icon_get;
	categoryitc.func.state_get = __mf_genlist_gl_state_get;
	categoryitc.func.del = __mf_genlist_gl_del;
	categoryitc.decorate_all_item_style = "edit_default";
}
void mf_genlist_data_init(void *data, mfItemData_s **itemData, const char *item_name)
{

	MF_TRACE_BEGIN;
	/*0.    variable definition and parameter check*/
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "input parameter data error");


	*itemData = (mfItemData_s *)malloc(sizeof(mfItemData_s));
	if (*itemData == NULL) {
		mf_debug("itemData malloc failed");
	} else {
		memset(*itemData, 0, sizeof(mfItemData_s));
		if (item_name) {
			(*itemData)->m_ItemName = g_string_new(item_name);
		} else {
			(*itemData)->m_ItemName = NULL;
		}
		(*itemData)->m_checked = false;
		(*itemData)->size = NULL;
		(*itemData)->create_date = NULL;
		(*itemData)->thumb_path = 0;
		(*itemData)->pCheckBox = NULL;
		(*itemData)->item = NULL;
		(*itemData)->real_thumb_flag = false;
		(*itemData)->flagExpand = false;
		(*itemData)->media = NULL;
		(*itemData)->ap = ap;
	}
}


void mf_genlist_update_default_view_item_style(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "input parameter data error");
	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "get navigation bar in use failed");

	categoryitc.decorate_item_style = userfolderitc.decorate_item_style = itc.decorate_item_style = "mode/slide";
	categoryitc.func.text_get = userfolderitc.func.text_get = itc.func.text_get = __mf_genlist_gl_label_get;
	categoryitc.func.content_get = userfolderitc.func.content_get = itc.func.content_get = __mf_genlist_gl_default_icon_get;
	categoryitc.func.state_get = userfolderitc.func.state_get = itc.func.state_get = __mf_genlist_gl_state_get;
	categoryitc.func.del = userfolderitc.func.del = itc.func.del = __mf_genlist_gl_del;

	elm_genlist_decorate_mode_set(ap->mf_MainWindow.pNaviGenlist, EINA_FALSE);
}

void mf_genlist_create_search_itc()
{
	MF_TRACE_BEGIN;
	search_itc.item_style = "2text.1icon.4.tb";
	search_itc.decorate_item_style = "mode/slide";
	search_itc.func.text_get = __mf_genlist_gl_label_get;
	search_itc.func.content_get = __mf_genlist_gl_default_icon_get;
	search_itc.func.state_get = __mf_genlist_gl_state_get;
	search_itc.func.del = __mf_genlist_gl_del;
	search_itc.decorate_all_item_style = "edit_default";

}


void mf_genlist_create_data(mfItemData_s **m_TempItem, const char *name, void *data)
{
	mf_retm_if(m_TempItem == NULL, "m_TempItem is NULL");
	mf_retm_if(name == NULL, "name is NULL");

	*m_TempItem = (mfItemData_s *) calloc(1, sizeof(mfItemData_s));
	if (*m_TempItem == NULL) {
		return;
	}

	(*m_TempItem)->m_ItemName = g_string_new(name);
	(*m_TempItem)->size = NULL;
	(*m_TempItem)->create_date = NULL;
	(*m_TempItem)->m_checked = FALSE;
	(*m_TempItem)->pCheckBox = NULL;
	(*m_TempItem)->thumb_path = NULL;
	(*m_TempItem)->real_thumb_flag = FALSE;
	(*m_TempItem)->media = NULL;
	(*m_TempItem)->ap = (struct appdata *)data;

}
void mf_genlist_create_list_default_style(Evas_Object *pGenlist, void *data, Eina_List *dir_list,
					  Eina_List *file_list)
{
	MF_TRACE_BEGIN;
	/*0.    variable definition and parameter check*/
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "input parameter data error");

	mf_retm_if(pGenlist == NULL, "input parameter pGenlist error");
	/*0.5.  data list varaible set*/
	/*1.    item style set */
	MF_TA_ACUM_ITEM_BEGIN("        init genlist item class", 0);

	userfolderitc.decorate_all_item_style = "edit_default";
	itc.decorate_all_item_style = "edit_default";

	if (ap->mf_Status.flagViewType == MF_VIEW_SYTLE_LIST_DETAIL) {
		userfolderitc.item_style = itc.item_style = "3text.1icon.1";

	} else {
		userfolderitc.item_style = itc.item_style = "1text.1icon.2";

	}
	if (ap->mf_Status.view_type == mf_view_root) {
	} else {
		userfolderitc.decorate_item_style = itc.decorate_item_style = "mode/slide2";
	}
	userfolderitc.func.text_get = itc.func.text_get = __mf_genlist_gl_label_get;
	userfolderitc.func.content_get = itc.func.content_get = __mf_genlist_gl_default_icon_get;
	userfolderitc.func.state_get = itc.func.state_get = __mf_genlist_gl_state_get;
	userfolderitc.func.del = itc.func.del = __mf_genlist_gl_del;

	MF_TA_ACUM_ITEM_END("        init genlist item class", 0);
	/*3.    check if we need give no content view*/
	/*4.    append items to the genlist*/
	fsNodeInfo *pNode = NULL;
	Eina_List *l = NULL;

	MF_TA_ACUM_ITEM_BEGIN("        append genlist items", 0);

	mf_debug("dir_list count is [%d] file is [%d]", eina_list_count(dir_list), eina_list_count(file_list));
	/*      add default folder items into the genlist       */
	EINA_LIST_FOREACH(dir_list, l, pNode) {
		mf_widget_item_append(pGenlist, pNode, ap);
	}
	/*      add file items into the genlist */
	mf_debug("view_type is [%d]", ap->mf_Status.view_type);
	if (ap->mf_Status.view_type != mf_view_root) {
		EINA_LIST_FOREACH(file_list, l, pNode) {
			mf_widget_item_append(pGenlist, pNode, ap);
		}
	}
	MF_TA_ACUM_ITEM_END("        append genlist items", 0);
	MF_TRACE_BEGIN;
}

Evas_Object *mf_genlist_create_list(void *data,Evas_Object *parent)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(data == NULL, NULL, "data is NULL");
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");

	struct appdata *ap = (struct appdata *)data;
	Evas_Object *genlist = NULL;
	Eina_List *file_list = NULL;
	Eina_List *dir_list = NULL;

	/*	generate raw data list*/

	if (ap->mf_Status.view_type == mf_view_root_category) {
		file_list = ap->mf_FileOperation.category_list;
	} else {
		file_list = ap->mf_FileOperation.file_list;
		dir_list = ap->mf_FileOperation.folder_list;
	}

/*      create Genlist*/
	MF_TA_ACUM_ITEM_BEGIN("        create genlist", 0);
	genlist = mf_widget_create_genlist(parent);
	if (genlist == NULL) {
		return NULL;
	}
	MF_TA_ACUM_ITEM_END("        create genlist", 0);
	mf_debug("More is [%d]", ap->mf_Status.more);
	switch (ap->mf_Status.more) {
	MF_TA_ACUM_ITEM_BEGIN("        register smart callback for genlist", 0);
	case MORE_DEFAULT:
			evas_object_smart_callback_add(genlist, "drag,start,right", mf_genlist_gl_mode_sweep_set, ap);
			evas_object_smart_callback_add(genlist, "drag,start,left", mf_genlist_gl_mode_sweep_unset, ap);
			evas_object_smart_callback_add(genlist, "drag,start,up", mf_genlist_gl_mode_cancel, ap);
			evas_object_smart_callback_add(genlist, "drag,start,down", mf_genlist_gl_mode_cancel, ap);
	case MORE_INTERNAL_COPY:
	case MORE_INTERNAL_MOVE:
	case MORE_INTERNAL_COPY_MOVE:
	case MORE_DATA_COPYING:
	case MORE_DATA_MOVING:
		evas_object_smart_callback_add(genlist, "selected", mf_genlist_gl_selected, ap);
		MF_TA_ACUM_ITEM_END("        register smart callback for genlist", 0);
		mf_genlist_create_list_default_style(genlist, ap, dir_list, file_list);
		break;
	default:
		break;
	}
/*4.    add watcher:*/
/*5.    clear temporary data*/
	MF_TRACE_END;
	return genlist;
}

void mf_genlist_get_list_selected_files(Evas_Object * pGenlist, Eina_List **list)
{
	MF_TRACE_BEGIN;
	mf_retm_if(pGenlist == NULL, "pGenlist is NULL");
	Elm_Object_Item *item = NULL;
	mfItemData_s *itemData = NULL;
	GString *pTemp = NULL;

	item = elm_genlist_first_item_get(pGenlist);
	while (item) {
		itemData = elm_object_item_data_get(item);
		if (itemData->m_checked) {
			pTemp = g_string_new(itemData->m_ItemName->str);
			*list = eina_list_append(*list, pTemp);
		}
		item = elm_genlist_item_next_get(item);
	}
	MF_TRACE_END;
}

void mf_genlist_get_list_selected_items(Evas_Object * pGenlist, Eina_List **list)
{
	MF_TRACE_BEGIN;
	mf_retm_if(pGenlist == NULL, "pGenlist is NULL");
	Elm_Object_Item *item = NULL;
	mfItemData_s *itemData = NULL;

	item = elm_genlist_first_item_get(pGenlist);
	while (item) {
		itemData = elm_object_item_data_get(item);
		if (itemData->m_checked) {
			*list = eina_list_append(*list, itemData);
		}
		item = elm_genlist_item_next_get(item);
	}
	MF_TRACE_END;
}

Evas_Object *mf_genlist_create_list_new_folder_style(void *data)
{

	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, NULL, "ap is NULL");

	Evas_Object *genlist = NULL;
	Evas_Object *pNaviBar = NULL;

	pNaviBar = ap->mf_MainWindow.pNaviBar;

	genlist = elm_genlist_add(pNaviBar);
	//elm_genlist_homogeneous_set(genlist, EINA_TRUE);
	elm_object_focus_set(genlist, EINA_FALSE);

	newfolderitc.item_style = "dialogue/bg/1icon";
	newfolderitc.func.text_get = NULL;
	newfolderitc.func.content_get = __mf_genlist_gl_new_folder_icon_get;
	newfolderitc.func.state_get = NULL;
	newfolderitc.func.del = NULL;

	elm_genlist_item_append(genlist, &newfolderitc, ap, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);

	return genlist;
}

Evas_Object *mf_genlist_create_list_rename_style(void *data)
{
	mf_retvm_if(data == NULL, NULL, "data is NULL");

	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, NULL, "ap is NULL");

	Evas_Object *genlist = NULL;
	Evas_Object *pNaviBar = NULL;

	pNaviBar = ap->mf_MainWindow.pNaviBar;

	genlist = elm_genlist_add(pNaviBar);

	//elm_genlist_homogeneous_set(genlist, EINA_TRUE);
	elm_object_focus_set(genlist, EINA_FALSE);

	newfolderitc.item_style = "dialogue/bg/1icon";
	newfolderitc.func.text_get = NULL;
	newfolderitc.func.content_get = __mf_genlist_gl_rename_icon_get;
	newfolderitc.func.state_get = NULL;
	newfolderitc.func.del = NULL;

	elm_genlist_item_append(genlist, &newfolderitc, ap, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);

	return genlist;
}


void mf_genlist_clear(Evas_Object *genlist)
{
	assert(genlist);

	Elm_Object_Item *it;
	it = elm_genlist_first_item_get(genlist);
	while (it) {
		elm_object_item_del(it);
		it = elm_genlist_first_item_get(genlist);
	}

}

void mf_genlist_sweep_item_recover(Evas_Object *obj)
{

	mf_retm_if(obj == NULL, "obj is NULL");
	Elm_Object_Item *it = (Elm_Object_Item *) elm_genlist_decorated_item_get(obj);

	/* Finish genlist sweep*/
	if (it) {
		elm_genlist_item_decorate_mode_set(it, "slide", EINA_FALSE);
		elm_genlist_item_select_mode_set(it, ELM_OBJECT_SELECT_MODE_DEFAULT);
		elm_genlist_item_update(it);
	}

}

static char *__get_title_gl_text(void *data, Evas_Object *obj, const char *part)
{
	MF_TRACE_BEGIN;
	if (data)
		return g_strdup((char *)data);
	else
		return NULL;
}
static void __del_title_gl(void *data, Evas_Object * obj)
{
	SAFE_FREE_CHAR(data);
}

Evas_Object *mf_genlist_create_path_info(Evas_Object *parent, char *info)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");

	Evas_Object *genlist = NULL;
	Elm_Object_Item *git=NULL;

	genlist = elm_genlist_add(parent);
	//elm_genlist_homogeneous_set(genlist, EINA_TRUE);

	itch.item_style = "groupindex";
	itch.func.text_get = __get_title_gl_text;
	itch.func.del = __del_title_gl;

	git = elm_genlist_item_append(genlist, &itch, info, NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);
	elm_genlist_item_select_mode_set(git, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
	MF_TRACE_END;

	return genlist;
}
