/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include "mf-widget.h"
#include "mf-callback.h"
#include "mf-fm-svc-wrapper.h"
#include "mf-gengrid.h"
#include "mf-util.h"
#include "mf-ta.h"
#include "mf-resource.h"
#include "mf-language-mgr.h"
#include "mf-tray-item.h"

void __mf_navi_bar_reset_navi_obj(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	ap->mf_MainWindow.pNaviBox = NULL;
	ap->mf_MainWindow.pNaviInfoBox = NULL;
	ap->mf_MainWindow.pTrayBox = NULL;

}
Evas_Object *__mf_navi_bar_get_upper_genlist(Evas_Object *parent)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");
	Evas_Object *upper_layout = NULL;
	Evas_Object *upper_genlist = NULL;

	upper_layout = mf_navi_bar_get_box_end(parent);
	mf_retvm_if(upper_layout == NULL, NULL, "upper_layout is NULL");

	upper_genlist = elm_object_part_content_get(upper_layout, "upper");
	mf_retvm_if(upper_genlist == NULL, NULL, "upper_genlist is NULL");
	MF_TRACE_END;
	return upper_genlist;
}

Evas_Object *mf_navi_bar_create_navi_bar(Evas_Object * parent)
{
	Evas_Object *navi_bar;
	assert(parent);
	navi_bar = elm_naviframe_add(parent);
	elm_naviframe_prev_btn_auto_pushed_set(navi_bar, EINA_FALSE);
	evas_object_show(navi_bar);
	return navi_bar;
}

/*	external functions	*/
/******************************
** Prototype    : mfNaviBarCreateView
** Description  :
** Input        : struct appdata* data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
Evas_Object *mf_navi_bar_create_search_bar(Evas_Object * parent, void *data)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retv_if(ap == NULL, NULL);
	mf_retv_if(parent == NULL, NULL);

	Evas_Object *sb = NULL;
	Evas_Object *en = NULL;

	sb = elm_layout_add(parent);

	elm_layout_theme_set(sb, "layout", "searchbar", "cancel_button");

	Evas_Object *cancel_btn = elm_button_add(sb);
	elm_object_style_set(cancel_btn, "searchbar/default");
	elm_object_text_set(cancel_btn, GET_SYS_STR(LABEL_CANCEL));
	mf_language_mgr_register_object(cancel_btn, OBJ_TYPE_ELM_OBJECT, NULL, LABEL_CANCEL);
	evas_object_smart_callback_add(cancel_btn, "clicked", mf_callback_cancel_cb, ap);

	elm_object_part_content_set(sb, "button_cancel", cancel_btn);
	elm_object_signal_emit(sb, "cancel,show", "");
	elm_object_signal_emit(sb, "elm,state,eraser,show", "elm");

	en = elm_entry_add(sb);
	elm_entry_scrollable_set(en, EINA_TRUE);
	elm_entry_single_line_set(en, EINA_TRUE);
	elm_entry_cnp_mode_set(en, ELM_CNP_MODE_PLAINTEXT);
	elm_object_part_content_set(sb, "elm.swallow.content", en);
	evas_object_data_set(sb, "entry", en);

	elm_entry_input_panel_layout_set(en, ELM_INPUT_PANEL_LAYOUT_NORMAL);

	evas_object_size_hint_weight_set(sb, EVAS_HINT_EXPAND, 0);
	evas_object_size_hint_align_set(sb, EVAS_HINT_FILL, 0.0);

	evas_object_smart_callback_add(en, "maxlength,reached", mf_callback_max_len_reached_cb, ap);
	evas_object_smart_callback_add(en, "changed", mf_callback_imf_changed_cb, ap);
	evas_object_smart_callback_add(en, "activated", mf_search_bar_search_started_callback, ap);

	elm_object_signal_callback_add(sb, "elm,eraser,clicked", "elm", mf_callback_eraser_clicked_cb, en);

	static Elm_Entry_Filter_Limit_Size limit_filter_data;
	limit_filter_data.max_char_count = 0;
	limit_filter_data.max_byte_count = MYFILE_FILE_NAME_LEN_MAX;
	elm_entry_markup_filter_append(en, elm_entry_filter_limit_size, &limit_filter_data);

	evas_object_show(sb);
	elm_object_focus_set(en, EINA_TRUE);
	ap->mf_MainWindow.pSearchEntry = en;
	return sb;


}



Evas_Object *mf_navi_bar_get_edit_genlist(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retv_if(ap == NULL, NULL);
	return ap->mf_MainWindow.pEditView.pGenlist;
}

Evas_Object *mfNaviBarConformGet(void *data)
{
	mf_retv_if(data == NULL, NULL);
	struct appdata *ap = (struct appdata *)data;
	return ap->mf_MainWindow.pNaviLayout;
}

Evas_Object *mf_navi_bar_get_box_end(Evas_Object * pBox)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(pBox == NULL, NULL, "pBox is NULL");
	Eina_List *pBoxChildren = NULL;
	pBoxChildren = elm_box_children_get(pBox);
	mf_retvm_if(pBoxChildren == NULL, NULL, "pBoxChildren is NULL");

	int len = eina_list_count(pBoxChildren);
	if (len > 0) {
		MF_TRACE_END;
		return eina_list_nth(pBoxChildren, len - 1);
	}
	MF_TRACE_END;
	return NULL;
}

/******************************
** Prototype    : mfNaviBarSetSegment
** Description  :
** Input        : struct appdata* data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_navi_bar_set_ctrl_bar(struct appdata *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");
	Evas_Object *NaviBar = ap->mf_MainWindow.pNaviBar;
	mf_retm_if(NaviBar == NULL, "Navi bar is NULL");
	Evas_Object *pCtrlBar = NULL;

	pCtrlBar = mf_ctrl_bar_create(ap, NaviBar);
	mf_retm_if(pCtrlBar == NULL, "mf_ctrl_bar_create failed");

	ap->mf_MainWindow.pNaviCtrlBar = pCtrlBar;
	elm_object_item_part_content_set(ap->mf_MainWindow.pNaviItem, CTRL_STYLE_TYPE, pCtrlBar);

	evas_object_show(pCtrlBar);
	MF_TRACE_END;
}


Evas_Object *mf_navi_bar_content_create(void *data)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(data == NULL, NULL, "data is NULL");

	struct appdata *ap = (struct appdata *)data;
	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retvm_if(pNavi_s == NULL, NULL, "pNavi_s is NULL");
	mf_retvm_if(ap->mf_Status.path == NULL, NULL, "ap->mf_Status.path is NULL");
	mf_retvm_if(ap->mf_Status.path->str == NULL, NULL, "ap->mf_Status.path->str is NULL");

	Evas_Object *pContent = NULL;
	Eina_List *file_list = NULL;
	Eina_List *dir_list = NULL;
	int dir_list_len = 0;
	int file_list_len = 0;
	int error_code = 0;

	if (ap->mf_Status.view_type == mf_view_root) {
		if (ap->mf_FileOperation.folder_list) {
			mf_util_free_eina_list_with_data(&(ap->mf_FileOperation.folder_list), MYFILE_TYPE_FSNODE);
		}
		if (ap->mf_FileOperation.file_list) {
			mf_util_free_eina_list_with_data(&(ap->mf_FileOperation.file_list), MYFILE_TYPE_FSNODE);
		}

		mf_util_generate_root_view_file_list(&(ap->mf_FileOperation.folder_list), ap->mf_Status.iStorageState);
		ap->mf_Status.flagNoContent = EINA_FALSE;
		mf_debug("count is [%d]", eina_list_count(ap->mf_FileOperation.folder_list));
	} else if (ap->mf_Status.view_type == mf_view_root_category) {
		if (eina_list_count(ap->mf_FileOperation.category_list) > 0) {
			ap->mf_Status.flagNoContent = EINA_FALSE;
			mf_util_sort_the_file_list(ap);
		} else {
			ap->mf_Status.flagNoContent = EINA_TRUE;
		}
	} else {
		MF_TA_ACUM_ITEM_BEGIN("      mf_util_generate_file_list", 0);
		error_code = mf_util_generate_file_list(ap);
		MF_TA_ACUM_ITEM_END("      mf_util_generate_file_list", 0);
		if (error_code != MYFILE_ERR_NONE) {
			/*Todo: we need to free all the Eina_List*/
			return NULL;
		}
		MF_TA_ACUM_ITEM_BEGIN("      mf_util_sort_the_file_list", 0);

		/*	sort the list by sort_type*/
		mf_util_sort_the_file_list(ap);
		MF_TA_ACUM_ITEM_END("      mf_util_sort_the_file_list", 0);

		MF_TA_ACUM_ITEM_BEGIN("      No content check", 0);
		file_list = ap->mf_FileOperation.file_list;
		dir_list = ap->mf_FileOperation.folder_list;

		dir_list_len = eina_list_count(dir_list);
		file_list_len = eina_list_count(file_list);

		if ((dir_list_len + file_list_len) > 0) {
			ap->mf_Status.flagNoContent = EINA_FALSE;
		} else {
			ap->mf_Status.flagNoContent = EINA_TRUE;
		}
		MF_TA_ACUM_ITEM_END("      No content check", 0);
	}
	ap->mf_MainWindow.pNaviGenlist = NULL;
	ap->mf_MainWindow.pNaviGengrid = NULL;

	if (ap->mf_Status.flagNoContent) {
		MF_TA_ACUM_ITEM_BEGIN("      No content create", 0);
		pContent = mf_widget_create_no_content(ap->mf_MainWindow.pNaviBar);
		MF_TA_ACUM_ITEM_END("      No content create", 0);
	} else {
		if (ap->mf_Status.flagViewType == MF_VIEW_STYLE_THUMBNAIL) {
			pContent = mf_gengrid_create_list(ap, ap->mf_MainWindow.pNaviBar);
			ap->mf_MainWindow.pNaviGengrid = pContent;
		} else {
			MF_TA_ACUM_ITEM_BEGIN("      genlist view create", 0);
			pContent = mf_genlist_create_list(ap, ap->mf_MainWindow.pNaviBar);
			MF_TA_ACUM_ITEM_END("      genlist view create", 0);
			ap->mf_MainWindow.pNaviGenlist = pContent;
		}
	}
	if (ap->mf_Status.more == MORE_DEFAULT) {
		MF_TA_ACUM_ITEM_BEGIN("      Add dir monitor", 0);
		mf_callback_add_dir_watch(ap->mf_Status.path->str, ap);
		MF_TA_ACUM_ITEM_END("      Add dir monitor", 0);
	}

	MF_TRACE_END;
	return pContent;

}
/******************************
** Prototype    : mfNaviBarSetContent
** Description  :
** Input        : Evas_Object *pLayout
**                Evas_Object *NaviContent
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static int __mf_navi_bar_reset_content_idler(void *data)
{
	MF_TRACE_BEGIN;

	struct appdata *ap = (struct appdata *)data;
	if (ap) {
		SAFE_FREE_OBJ(ap->mf_Status.old_content);
		ap->mf_Status.navi_content_idler = NULL;
	}
	MF_TRACE_END;

	return ECORE_CALLBACK_CANCEL;
}

void mfNaviBarSetContent(void *data, Evas_Object *pLayout, Evas_Object *NaviContent)
{
	mf_retm_if(data == NULL, "data is NULL");
	mf_retm_if(pLayout == NULL, "pConform is NULL");
	mf_retm_if(NaviContent == NULL, "NaviContent is NULL");

	struct appdata *ap = (struct appdata *)data;
	Evas_Object *unUsed = mf_widget_unset_part_content(pLayout, "content");
	evas_object_hide(unUsed);
	ap->mf_Status.old_content = unUsed;
	mf_ecore_idler_del(ap->mf_Status.navi_content_idler);
	ap->mf_Status.navi_content_idler = ecore_idler_add((Ecore_Task_Cb)__mf_navi_bar_reset_content_idler, ap);
	/*if bs, use idler here*/
	elm_object_part_content_set(pLayout, "content", NaviContent);
}

Evas_Object *mf_navi_bar_get_content(Evas_Object *pConform)
{
	return elm_object_content_get(pConform);
}

/******************************
** Prototype    : mf_navi_bar_set_style
** Description  :
** Input        : struct appdata* data
**                eNaviBarStyle navi_style
**                Evas_Object *NaviContent
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_navi_bar_title_set(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata *)data;
	int more = ap->mf_Status.more;
	char *path = ap->mf_Status.path->str;
	Elm_Object_Item *navi_it = ap->mf_MainWindow.pNaviItem;
	Evas_Object *pNavi = ap->mf_MainWindow.pNaviBar;

	if (more == MORE_RENAME || more == MORE_CREATE_FOLDER ||  more == MORE_THUMBNAIL_RENAME) {
		// do nothing
	} else {
		myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
		if (more != MORE_EDIT && more != MORE_SEARCH) {
			if (ap->mf_Status.view_type != mf_view_root) {
				Evas_Object *home_btn = elm_button_add(pNavi);
				//elm_object_style_set(home_btn, TITLE_BTN_STYLE);
				elm_object_style_set(home_btn, "naviframe/title_icon");
				Evas_Object *home_ic = elm_image_add(pNavi);
				elm_image_file_set(home_ic, MF_TITLE_ICON_HOME, NULL);
				elm_image_resizable_set(home_ic, EINA_TRUE, EINA_TRUE);
				elm_object_content_set(home_btn, home_ic);
				mf_language_mgr_register_object(home_btn, OBJ_TYPE_ELM_OBJECT, NULL, MF_LABEL_HOME);
				evas_object_smart_callback_add(home_btn, "clicked", mf_callback_home_button_cb, ap);
				evas_object_smart_callback_add(home_btn, "pressed", mf_callback_home_button_pressed_cb, home_ic);
				evas_object_smart_callback_add(home_btn, "unpressed", mf_callback_home_button_unpressed_cb, home_ic);
				evas_object_show(home_btn);
				elm_object_item_part_content_set(navi_it, TITLE_LEFT_BTN, home_btn);

				Evas_Object *up_btn = elm_button_add(pNavi);
				//elm_object_style_set(up_btn, TITLE_BTN_STYLE);
				elm_object_style_set(up_btn, "naviframe/title_icon");
				Evas_Object *up_ic = elm_image_add(pNavi);
				elm_image_file_set(up_ic, MF_TITLE_ICON_UPPER, NULL);
				elm_image_resizable_set(up_ic, EINA_TRUE, EINA_TRUE);
				elm_object_content_set(up_btn, up_ic);
				mf_language_mgr_register_object(up_btn, OBJ_TYPE_ELM_OBJECT, NULL, MF_LABEL_UPPER);
				evas_object_smart_callback_add(up_btn, "clicked", mf_callback_upper_click_cb, ap);
				evas_object_smart_callback_add(up_btn, "pressed", mf_callback_upper_button_pressed_cb, up_ic);
				evas_object_smart_callback_add(up_btn, "unpressed", mf_callback_upper_button_unpressed_cb, up_ic);
				evas_object_show(up_btn);
				elm_object_item_part_content_set(navi_it, TITLE_RIGHT_BTN, up_btn);

				elm_naviframe_item_title_visible_set(navi_it, EINA_TRUE);
			}

		} else {
			elm_naviframe_item_title_visible_set(navi_it, EINA_TRUE);
		}

		Evas_Object *label = elm_label_add(pNavi);
		// set the label style
		elm_object_style_set(label, "naviframe_title");
		// set the label for the title slide mode
		elm_label_slide_mode_set(label, ELM_LABEL_SLIDE_MODE_AUTO);
		elm_label_wrap_width_set(label, 1);
		elm_label_ellipsis_set(label, EINA_TRUE);
		// set the label text
		if (more == MORE_EDIT) {
			elm_object_text_set(label, GET_SYS_STR(MF_LABEL_EDIT));
		} else {
			elm_object_text_set(label, pNavi_s->pNaviTitle);
		}
		evas_object_show(label);
		elm_object_item_part_content_set(navi_it, "elm.swallow.title", label);
	}
	MF_TRACE_END;

}
void mf_navi_bar_set_style(void *data, eNaviBarStyle navi_style, Evas_Object *NaviContent)
{
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata *)data;

	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	Evas_Object *NaviBar = ap->mf_MainWindow.pNaviBar;
	mf_retm_if(NaviBar == NULL, "Navi bar is NULL");
	Evas_Object *pConform = ap->mf_MainWindow.pNaviLayout;
	mf_retm_if(pConform == NULL, "Conformant is NULL");
	Evas_Object *l_button = ap->mf_MainWindow.pNaviLeftBtn;
	Elm_Object_Item *navi_it = NULL;

	char *title = pNavi_s->pNaviTitle;
	mfNaviBarSetContent(ap, pConform, NaviContent);

	switch (navi_style) {
	case NAVI_BAR_DEFAULT:	/*when MMC is off, show "myfile" only*/
		ap->mf_Status.pPreNaviItem = ap->mf_MainWindow.pNaviItem;
		mf_language_mgr_unregister_object_item_by_type(ITEM_TYPE_CTRL);
		navi_it = elm_naviframe_item_push(NaviBar, NULL, NULL, NULL, pConform, MF_NAVI_STYLE_ENABLE);

		ap->mf_MainWindow.pNaviItem = navi_it;
		break;
	case NAVI_BAR_PATH_WITH_ONE_ACTION:
		ap->mf_Status.pPreNaviItem = ap->mf_MainWindow.pNaviItem;
		mf_language_mgr_unregister_object_item_by_type(ITEM_TYPE_CTRL);
		navi_it = elm_naviframe_item_push(NaviBar, NULL, l_button, NULL, pConform, MF_NAVI_STYLE_ENABLE);
		ap->mf_MainWindow.pNaviItem = navi_it;
		break;
	default:
		break;
	}
}


/******************************
** Prototype    : __mf_navi_bar_select_count_label_timeout_cb
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event_info
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/



void mf_navi_bar_set_upper_item_disable(void *data, bool bFlag)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");
	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "get navigation bar in use failed");

	if (!mf_fm_svc_wrapper_is_root_path(ap->mf_Status.path->str)) {
		Elm_Object_Item *pUpperItem = NULL;
		Evas_Object *upper_genlist = NULL;

		upper_genlist = __mf_navi_bar_get_upper_genlist(ap->mf_MainWindow.pNaviInfoBox);
		pUpperItem = elm_genlist_first_item_get(upper_genlist);
		if (pUpperItem) {
			mfItemData_s *itemData = elm_object_item_data_get(pUpperItem);

			if (itemData && itemData->m_ItemName && itemData->m_ItemName->str
			    && g_strcmp0(itemData->m_ItemName->str, ap->mf_Status.upper_folder) == 0) {
				elm_object_item_disabled_set(pUpperItem, bFlag);
			}
		}
	}
}

Evas_Object *__mf_navi_bar_backbutton_create(Evas_Object *parent, Evas_Object *win)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");
	mf_retvm_if(win == NULL, NULL, "win is NULL");

	Evas_Object *btn = NULL;
	btn = elm_button_add(parent);
	elm_object_style_set(btn, "naviframe/end_btn/default");
	evas_object_show(btn);
	return btn;

}
Evas_Object *mf_navi_bar_pathinfo_label_create(Evas_Object *parent, char *pathinfo)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");
	mf_retvm_if(pathinfo == NULL, NULL, "pathinfo is NULL");
	Evas_Object *label = NULL;

	label = elm_label_add(parent);
	elm_object_focus_set(label, EINA_FALSE);
	evas_object_size_hint_weight_set(label, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(label, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_text_set(label, pathinfo);
	evas_object_show(label);
	SAFE_FREE_CHAR(pathinfo);
	MF_TRACE_END;
	return label;
}

Evas_Object *__mf_navi_bar_pathinfo_create(Evas_Object *parent, const char  *content)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");
	mf_retvm_if(content == NULL, NULL, "content is NULL");

	Evas_Object *layout = NULL;

	layout = elm_layout_add(parent);
	elm_object_focus_set(layout, EINA_FALSE);
	elm_layout_file_set(layout, EDJ_NAME, GRP_PATH_INFO);
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(layout, EVAS_HINT_FILL, EVAS_HINT_FILL);

	//elm_object_part_content_set(layout, "info", content);
	elm_object_part_text_set(layout, "info", content);
	evas_object_show(layout);
	MF_TRACE_END;
	return layout;
}

void mf_navi_add_back_button(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	struct appdata *ap = (struct appdata *)data;

	Evas_Object *pBackButton = NULL;

	pBackButton = __mf_navi_bar_backbutton_create(ap->mf_MainWindow.pNaviBar, ap->mf_MainWindow.pWindow);
	if (pBackButton) {
		elm_object_item_part_content_set(ap->mf_MainWindow.pNaviItem, "prev_btn", pBackButton);
		evas_object_smart_callback_add(pBackButton, "clicked", mf_callback_backbutton_clicked_cb, ap);
	}
}

void mf_navi_remove_home_upper_button(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	struct appdata *ap = (struct appdata *)data;

	if (ap->mf_Status.view_type != mf_view_root) {
		Evas_Object *btn = NULL;
		btn = elm_object_item_part_content_unset(ap->mf_MainWindow.pNaviItem, TITLE_LEFT_BTN);
		SAFE_FREE_OBJ(btn);
		btn = elm_object_item_part_content_unset(ap->mf_MainWindow.pNaviItem, TITLE_RIGHT_BTN);
		SAFE_FREE_OBJ(btn);
	}
}

Evas_Object *mf_navi_bar_upper_create(Evas_Object *parent)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");

	Evas_Object *upper_genlist = NULL;
	upper_genlist = elm_genlist_add(parent);
	mf_retvm_if(upper_genlist == NULL, NULL, "upper_genlist is NULL");
	elm_object_focus_set(upper_genlist, EINA_FALSE);
	evas_object_size_hint_weight_set(upper_genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(upper_genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_scroller_bounce_set(upper_genlist, EINA_FALSE, EINA_FALSE);
	evas_object_show(upper_genlist);
	MF_TRACE_END;
	return upper_genlist;

}

void mf_navi_bar_remove_back_button(Elm_Object_Item *pNaviItem)
{
	MF_TRACE_BEGIN;
	mf_retm_if(pNaviItem == NULL, "pNaviItem is NULL");
	Evas_Object *back_btn = NULL;

	back_btn = elm_object_item_part_content_get(pNaviItem, "prev_btn");
	if (back_btn != NULL) {
		elm_object_item_part_content_unset(pNaviItem, "prev_btn");
		evas_object_hide(back_btn);
		evas_object_del(back_btn);
	}
	MF_TRACE_END;
}

Evas_Object *mf_navi_bar_create_header_box(void *data)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(data == NULL, NULL, "data is NULL");

	struct appdata *ap = (struct appdata *)data;
	Evas_Object *box = NULL;
	Evas_Object *pathinfo_layout = NULL;
	char *pathinfo = NULL;
	char *info = NULL;
	if (!mf_fm_svc_wrapper_is_root_path(ap->mf_Status.path->str)) {
		myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
		mf_retvm_if(pNavi_s == NULL, NULL, "pNavi_s is NULL");

		//box = mf_widget_create_box(ap->mf_MainWindow.pNaviBox);
		//evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_FILL);
		//evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);
		//elm_box_clear(box);




		pathinfo = mf_fm_svc_path_info_get(ap->mf_Status.path->str);
		info = mf_fm_svc_path_info_translate(pathinfo, MF_PATH_INFO_MAX_LENGTH_PORTRAIT);


		//Evas_Object *genlist = mf_genlist_create_path_info(box, info);
		pathinfo_layout = __mf_navi_bar_pathinfo_create(ap->mf_MainWindow.pNaviBox, info);
		evas_object_size_hint_weight_set(pathinfo_layout, EVAS_HINT_EXPAND, 0);
		evas_object_size_hint_align_set(pathinfo_layout, EVAS_HINT_FILL, 0.0);
		//evas_object_show(genlist);

#if 0
		upper_layout = elm_layout_add(box);
		elm_object_focus_set(upper_layout, EINA_FALSE);
		ret = elm_layout_file_set(upper_layout, EDJ_NAME, "upper_folder");
		evas_object_size_hint_weight_set(upper_layout, EVAS_HINT_FILL, EVAS_HINT_FILL);//(upper_layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(upper_layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
		evas_object_show(upper_layout);
		elm_box_pack_end(box, upper_layout);

#endif
		//elm_box_pack_start(box, pathinfo_layout);
		evas_object_show(pathinfo_layout);
	}
	MF_TRACE_END;
	return pathinfo_layout;
}

void mf_navi_bar_set_ctrl_button(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata *)data;
	Elm_Object_Item *navi_it = ap->mf_MainWindow.pNaviItem;
	Evas_Object *more_bt = NULL;
	Evas_Object *bt_1 = NULL;
	Evas_Object *bt_2 = NULL;
	switch(ap->mf_Status.more) {
	case MORE_DEFAULT:
		more_bt = mf_widget_util_create_button(ap->mf_MainWindow.pNaviBar, NAVI_BUTTON_EDIT, MF_LABEL_MORE, NULL, mf_callback_more_button_cb, ap);
		if (ap->mf_Status.view_type == mf_view_root) {
			bt_1 = mf_widget_util_create_button(ap->mf_MainWindow.pNaviBar, NAVI_BUTTON_STYLE, MF_LABEL_SEARCH, NULL, mf_search_bar_enter_search_routine, ap);
		} else {
			bt_1 = mf_widget_util_create_button(ap->mf_MainWindow.pNaviBar, NAVI_BUTTON_STYLE, MF_LABEL_EDIT, NULL, mf_callback_edit_cb, ap);
		}
		break;
	case MORE_EDIT:
		more_bt = mf_widget_util_create_button(ap->mf_MainWindow.pNaviBar, NAVI_BUTTON_EDIT, MF_LABEL_MORE, NULL, mf_callback_more_button_cb, ap);
		bt_1 = mf_widget_util_create_button(ap->mf_MainWindow.pNaviBar, NAVI_BUTTON_STYLE, LABEL_DELETE, NULL, mf_callback_delete_cb, ap);
		bt_2 = mf_widget_util_create_button(ap->mf_MainWindow.pNaviBar, NAVI_BUTTON_STYLE, LABEL_SHARE, NULL, mf_callback_share_cb, ap);
		break;
	case MORE_INTERNAL_COPY:
	case MORE_DATA_COPYING:
		if (ap->mf_Status.view_type != mf_view_root) {
			more_bt = mf_widget_util_create_button(ap->mf_MainWindow.pNaviBar, NAVI_BUTTON_EDIT, MF_LABEL_MORE, NULL, mf_callback_more_button_cb, ap);
		}
		bt_1 = mf_widget_util_create_button(ap->mf_MainWindow.pNaviBar, NAVI_BUTTON_STYLE, LABEL_COPY_HERE, NULL, mf_callback_paste_here_cb, ap);
		break;
	case MORE_INTERNAL_MOVE:
	case MORE_DATA_MOVING:
		if (ap->mf_Status.view_type != mf_view_root) {
			more_bt = mf_widget_util_create_button(ap->mf_MainWindow.pNaviBar, NAVI_BUTTON_EDIT, MF_LABEL_MORE, NULL, mf_callback_more_button_cb, ap);
		}
		bt_1 = mf_widget_util_create_button(ap->mf_MainWindow.pNaviBar, NAVI_BUTTON_STYLE, LABEL_MOVE_HERE, NULL, mf_callback_move_here_cb, ap);
		break;
	default:
		break;
	}
	if (more_bt) {
		elm_object_item_part_content_set(navi_it, NAVI_MORE_BUTTON_PART, more_bt);
	}
	if (bt_1) {
		elm_object_item_part_content_set(navi_it, NAVI_BOTTOM_BUTTON_1_PART, bt_1);
	}
	if (bt_2) {
		elm_object_item_part_content_set(navi_it, NAVI_BOTTOM_BUTTON_2_PART, bt_2);
	}

}
void mf_navi_bar_reset_ctrl_button(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata *)data;
	Elm_Object_Item *navi_it = ap->mf_MainWindow.pNaviItem;
	mf_retm_if(navi_it == NULL, "navi_it is NULL");

	Evas_Object *more_bt = NULL;
	Evas_Object *bt_1 = NULL;
	Evas_Object *bt_2 = NULL;
	more_bt = elm_object_item_part_content_unset(navi_it, NAVI_MORE_BUTTON_PART);
	SAFE_FREE_OBJ(more_bt);
	bt_1 = elm_object_item_part_content_unset(navi_it, NAVI_BOTTOM_BUTTON_1_PART);
	SAFE_FREE_OBJ(bt_1);
	bt_2 = elm_object_item_part_content_unset(navi_it, NAVI_BOTTOM_BUTTON_2_PART);
	SAFE_FREE_OBJ(bt_2);
	mf_navi_bar_set_ctrl_button(ap);
}
void mf_navi_bar_create_default_view(void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;

	GString *title = NULL;
	Evas_Object *newContent = NULL;
	Evas_Object *box = NULL;
	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "get navigation bar in use failed");
	mf_retm_if(ap->mf_MainWindow.pNaviBar == NULL, "ap->mf_MainWindow.pNaviBar is NULL");

	__mf_navi_bar_reset_navi_obj(ap);

	MF_TA_ACUM_ITEM_BEGIN("      create view box", 0);
	ap->mf_MainWindow.pNaviBox = mf_widget_create_box(ap->mf_MainWindow.pNaviBar);
	MF_TA_ACUM_ITEM_END("      create view box", 0);

	ap->mf_MainWindow.pNaviLayout = mf_widget_create_layout(ap->mf_MainWindow.pNaviBox, EDJ_NAME, "selectinfo_layout");
	/*set content*/
	MF_TA_ACUM_ITEM_BEGIN("      create genlist", 0);
	newContent = mf_navi_bar_content_create(ap);
	MF_TA_ACUM_ITEM_END("      create genlist", 0);
	/*set title     segment or title*/
	evas_object_show(newContent);
	MF_TA_ACUM_ITEM_BEGIN("      set title", 0);
	if (ap->mf_Status.view_type == mf_view_root) {
		title = g_string_new(GET_SYS_STR(LABEL_MYFILE));
	} else if (mf_fm_svc_wrapper_is_root_path(ap->mf_Status.path->str)) {
		title = g_string_new(GET_SYS_STR(LABEL_MYFILE));
	} else {
		title = mf_fm_svc_wrapper_get_file_name(ap->mf_Status.path);
	}
	MF_TA_ACUM_ITEM_END("      set title", 0);

	MF_TA_ACUM_ITEM_BEGIN("      set content of conform", 0);

	if (title != NULL) {
		pNavi_s->pNaviTitle = g_strdup(title->str);
		g_string_free(title, TRUE);
		title = NULL;
	}

	/*navigation view integration*/
	mfNaviBarSetContent(ap, ap->mf_MainWindow.pNaviLayout, newContent);
	MF_TA_ACUM_ITEM_END("      set content of conform", 0);

	MF_TA_ACUM_ITEM_BEGIN("      set conform to box", 0);
	elm_box_pack_end(ap->mf_MainWindow.pNaviBox, ap->mf_MainWindow.pNaviLayout);
	MF_TA_ACUM_ITEM_END("      set conform to box", 0);

	MF_TA_ACUM_ITEM_BEGIN("      create header box", 0);
	if (ap->mf_Status.view_type == mf_view_normal) {
		box = mf_navi_bar_create_header_box(ap);
		if (box) {
			ap->mf_MainWindow.pNaviInfoBox = box;
			elm_box_pack_start(ap->mf_MainWindow.pNaviBox, box);
		}
	}
	MF_TA_ACUM_ITEM_END("      create header box", 0);

	MF_TA_ACUM_ITEM_BEGIN("      push view to navibar", 0);

	ap->mf_Status.pPreNaviItem = ap->mf_MainWindow.pNaviItem;

	mf_language_mgr_unregister_object_item_by_type(ITEM_TYPE_CTRL);
	ap->mf_MainWindow.pNaviItem = elm_naviframe_item_push(ap->mf_MainWindow.pNaviBar, NULL, NULL, NULL, ap->mf_MainWindow.pNaviBox, MF_NAVI_STYLE_ENABLE);

	MF_TA_ACUM_ITEM_END("      push view to navibar", 0);

	MF_TA_ACUM_ITEM_BEGIN("      add backbutton", 0);
	if (ap->mf_Status.more != MORE_INTERNAL_COPY && ap->mf_Status.more != MORE_INTERNAL_MOVE) {
		mf_navi_add_back_button(ap);
	}
	MF_TA_ACUM_ITEM_END("      add backbutton", 0);

	/*add control bar for navigation bar*/
	MF_TA_ACUM_ITEM_BEGIN("      add control bar", 0);
	//mf_navi_bar_set_ctrl_bar(ap);

	mf_navi_bar_set_ctrl_button(data);
	MF_TA_ACUM_ITEM_END("      add control bar", 0);

	MF_TA_ACUM_ITEM_BEGIN("      set control bar", 0);
	if (ap->mf_Status.flagNoContent == 1) {
		mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_NOCONTENT_VIEW, true);
		//mf_ctrl_bar_item_set_disable(ap->mf_MainWindow.pNaviCtrlBar, CTRL_DISABLE_NOCONTENT_VIEW, TRUE);
	} else {
		mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_NOCONTENT_VIEW, FALSE);
		//mf_ctrl_bar_item_set_disable(ap->mf_MainWindow.pNaviCtrlBar, CTRL_DISABLE_NOCONTENT_VIEW, FALSE);
	}
#ifdef MYFILE_USB_OTG
	if(mf_fm_svc_wrapper_is_root_path(ap->mf_Status.path->str) == MYFILE_OTG) {
		ap->mf_Otg.rootpath_box = ap->mf_MainWindow.pNaviBox;
		ap->mf_Otg.rootpath_layout = ap->mf_MainWindow.pNaviLayout;
		ap->mf_Otg.rootpath_navi_item = ap->mf_MainWindow.pNaviItem;
		mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_OTG_ROOT, TRUE);
		//mf_ctrl_bar_item_set_disable(ap->mf_MainWindow.pNaviCtrlBar, CTRL_DISABLE_OTG_ROOT, TRUE);
	}
#endif
	MF_TA_ACUM_ITEM_END("      set control bar", 0);
	mf_navi_bar_title_set(ap);
	/*temp data free*/
	MF_TRACE_END;
}

/******************************
** Prototype    : mf_navi_bar_create_edit_view
** Description  :
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_navi_bar_remove_path_info(void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	if (!mf_fm_svc_wrapper_is_root_path(ap->mf_Status.path->str)) {
		elm_box_unpack(ap->mf_MainWindow.pNaviBox, ap->mf_MainWindow.pNaviPathInfo);
		evas_object_hide(ap->mf_MainWindow.pNaviPathInfo);
		//evas_object_del(pNavi_s->pNaviPathInfo);
		ap->mf_MainWindow.pEditView.pPathInfo = ap->mf_MainWindow.pNaviPathInfo;
	} else {
		ap->mf_MainWindow.pEditView.pPathInfo = NULL;
	}
	MF_TRACE_END;
}

void mf_navi_bar_recover_path_info(void *data, myfileNaviBar *pNavi_s)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	if (ap->mf_MainWindow.pEditView.pPathInfo) {
		ap->mf_MainWindow.pNaviPathInfo = ap->mf_MainWindow.pEditView.pPathInfo;
		elm_box_pack_start(ap->mf_MainWindow.pNaviBox, ap->mf_MainWindow.pNaviPathInfo);
		evas_object_show(ap->mf_MainWindow.pEditView.pPathInfo);
	}

	MF_TRACE_END;
	return;
}

void mf_navi_bar_remove_info_box(void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	if (!mf_fm_svc_wrapper_is_root_path(ap->mf_Status.path->str)) {
		elm_box_unpack(ap->mf_MainWindow.pNaviBox, ap->mf_MainWindow.pNaviInfoBox);
		evas_object_hide(ap->mf_MainWindow.pNaviInfoBox);
		ap->mf_MainWindow.pEditView.pInfoBox = ap->mf_MainWindow.pNaviInfoBox;
	} else {
		ap->mf_MainWindow.pEditView.pInfoBox = NULL;
	}
	MF_TRACE_END;
}

void mf_navi_bar_recover_info_box(void *data, myfileNaviBar *pNavi_s)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	if (ap->mf_MainWindow.pEditView.pInfoBox) {
		ap->mf_MainWindow.pNaviInfoBox = ap->mf_MainWindow.pEditView.pInfoBox;
		elm_box_pack_start(ap->mf_MainWindow.pNaviBox, ap->mf_MainWindow.pNaviInfoBox);
		evas_object_show(ap->mf_MainWindow.pEditView.pInfoBox);
	}

	MF_TRACE_END;
	return;
}

void mf_navi_bar_edit_list_update(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	struct appdata *ap = (struct appdata *)data;

	if (ap->mf_Status.flagViewType == MF_VIEW_STYLE_LIST || ap->mf_Status.flagViewType == MF_VIEW_SYTLE_LIST_DETAIL) {
		if (ap->mf_MainWindow.pNaviGenlist) {
			mf_genlist_sweep_item_recover(ap->mf_MainWindow.pNaviGenlist);
			mf_genlist_update_edit_view_item_style(ap);
			ap->mf_MainWindow.pEditView.pGenlist = ap->mf_MainWindow.pNaviGenlist;
			elm_genlist_decorate_mode_set(ap->mf_MainWindow.pNaviGenlist, EINA_TRUE);
		}
	} else {
		Eina_List *file_list = NULL;
		Eina_List *dir_list = NULL;

		if (ap->mf_Status.view_type == mf_view_root_category) {
			file_list = ap->mf_FileOperation.category_list;
		} else {
			file_list = ap->mf_FileOperation.file_list;
			dir_list = ap->mf_FileOperation.folder_list;
		}

		if (ap->mf_MainWindow.pNaviGengrid) {
			elm_gengrid_clear(ap->mf_MainWindow.pNaviGengrid);
			mf_gengrid_create_list_default_style(ap->mf_MainWindow.pNaviGengrid, ap, dir_list, file_list);
			ap->mf_MainWindow.pEditView.pGengrid = ap->mf_MainWindow.pNaviGengrid;
		}
	}

}

void mf_navi_bar_create_edit_view(void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	//Evas_Object *pSelectAllLayout = NULL;
	/*get navigation bar to create view*/
	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "get navigation bar in use failed");

	ap->mf_MainWindow.pEditView.pBox = ap->mf_MainWindow.pNaviBox;

	mf_navi_bar_remove_info_box(ap);
		/*create conformant*/
	mf_navi_remove_home_upper_button(ap);

	Evas_Object *select_all_btn = elm_button_add(ap->mf_MainWindow.pNaviBar);
	elm_object_style_set(select_all_btn, "naviframe/title_icon");
	Evas_Object *select_all_ic = elm_image_add(ap->mf_MainWindow.pNaviBar);
	elm_image_file_set(select_all_ic, MF_TITLE_ICON_SELECT_ALL, NULL);
	elm_image_resizable_set(select_all_ic, EINA_TRUE, EINA_TRUE);
	elm_object_content_set(select_all_btn, select_all_ic);
	evas_object_smart_callback_add(select_all_btn, "clicked", (Evas_Smart_Cb)__mf_genlist_select_all_layout_mouse_down_cb, ap);
	evas_object_smart_callback_add(select_all_btn, "pressed", (Evas_Smart_Cb)mf_callback_select_all_button_pressed_cb, select_all_ic);
	evas_object_smart_callback_add(select_all_btn, "unpressed", (Evas_Smart_Cb)mf_callback_select_all_button_unpressed_cb, select_all_ic);
	evas_object_show(select_all_btn);
	elm_object_item_part_content_set(ap->mf_MainWindow.pNaviItem, TITLE_RIGHT_BTN, select_all_btn);

	mf_navi_bar_title_set(ap);
	mf_navi_bar_reset_ctrl_button(ap);


	if (ap->mf_Status.flagViewType != MF_VIEW_STYLE_THUMBNAIL) {
		Elm_Object_Item *it;
		it = elm_genlist_first_item_get(ap->mf_MainWindow.pNaviGenlist);
		while (it) {
			elm_genlist_item_fields_update(it, "elm.icon.2", ELM_GENLIST_ITEM_FIELD_CONTENT);
			it = elm_genlist_item_next_get(it);
		}
	}


	//mf_ctrl_bar_reset(ap);
	mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_EDIT_ALL, TRUE);
	//mf_ctrl_bar_item_set_disable(ap->mf_MainWindow.pNaviCtrlBar, CTRL_DISABLE_EDIT_ALL, true);
	/**    Todo:   should we add segment status set here?*/
	/*temp data free*/
	MF_TRACE_END;
}

/******************************
** Prototype    : mf_navi_bar_create_path_select_view
** Description  :
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_navi_bar_create_path_select_view(void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	GString *title = NULL;
	Evas_Object *newContent = NULL;
	Evas_Object *box = NULL;
	/*get navigation bar to create view*/
	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "get navigation bar in use failed");

	/*set pre_status*/
	pNavi_s->pre_more = ap->mf_Status.more;
	/*create conformant*/

	__mf_navi_bar_reset_navi_obj(ap);

	ap->mf_MainWindow.pNaviBox = mf_widget_create_box(ap->mf_MainWindow.pNaviBar);
	ap->mf_MainWindow.pNaviLayout = mf_widget_create_layout(ap->mf_MainWindow.pNaviBox, EDJ_NAME, "selectinfo_layout");
	/*set content*/
	newContent = mf_navi_bar_content_create(ap);

	/*set title     segment or title*/
	if (ap->mf_Status.view_type == mf_view_root) {
		title = g_string_new(GET_SYS_STR(LABEL_MYFILE));
	} else if (mf_fm_svc_wrapper_is_root_path(ap->mf_Status.path->str)) {
		title = g_string_new(GET_SYS_STR(LABEL_MYFILE));
	} else {
		title = mf_fm_svc_wrapper_get_file_name(ap->mf_Status.path);
	}

	if (title != NULL) {

		pNavi_s->pNaviTitle = g_strdup(title->str);
		g_string_free(title, TRUE);
		title = NULL;
	}
	mfNaviBarSetContent(ap, ap->mf_MainWindow.pNaviLayout, newContent);
	elm_box_pack_end(ap->mf_MainWindow.pNaviBox, ap->mf_MainWindow.pNaviLayout);
	if (ap->mf_Status.view_type == mf_view_normal) {
		box = mf_navi_bar_create_header_box(ap);
		if (box) {
			ap->mf_MainWindow.pNaviInfoBox = box;
			elm_box_pack_start(ap->mf_MainWindow.pNaviBox, box);
		}
	}
	/*Navigation bar Path Select view integration*/

	MF_TA_ACUM_ITEM_BEGIN("      push view to navibar", 0);
	ap->mf_Status.pPreNaviItem = ap->mf_MainWindow.pNaviItem;

	mf_language_mgr_unregister_object_item_by_type(ITEM_TYPE_CTRL);
	ap->mf_MainWindow.pNaviItem = elm_naviframe_item_push(ap->mf_MainWindow.pNaviBar, NULL, NULL, NULL, ap->mf_MainWindow.pNaviBox, MF_NAVI_STYLE_ENABLE);
	MF_TA_ACUM_ITEM_END("	   push view to navibar", 0);

	mf_navi_add_back_button(ap);

	/*add control bar for navigation bar*/
	//mf_navi_bar_set_ctrl_bar(ap);
	mf_navi_bar_set_ctrl_button(data);
#ifdef MYFILE_USB_OTG
	/*control bar disable set if in otg root path*/
	if (mf_fm_svc_wrapper_is_root_path(ap->mf_Status.path->str) == MYFILE_OTG) {
		mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_OTG_OPT_ROOT, TRUE);
	}
#endif
	if (ap->mf_Status.view_type == mf_view_root || ap->mf_Status.view_type == mf_view_root_category) {
		mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_OTG_ROOT, TRUE);
	}
	mf_navi_bar_title_set(ap);
	MF_TRACE_END;

	/**    Todo:   should we add segment status set here?*/
	/*temp data free*/
}

void mf_navi_bar_create_path_select_view_otg_root(void *data)
{

	struct appdata *ap = (struct appdata *)data;
	GString *title = NULL;
	Evas_Object *newContent = NULL;

	/*get navigation bar to create view*/
	myfileNaviBar *pNavi_s = mf_navi_bar_get_struct_by_label(ap, GET_STR(MF_LABEL_OTG));
	mf_retm_if(pNavi_s == NULL, "get navigation bar in use failed");

	/*set pre_status*/
	pNavi_s->pre_more = ap->mf_Status.more;
	/*create conformant*/
	__mf_navi_bar_reset_navi_obj(ap);

	ap->mf_MainWindow.pNaviLayout = mf_widget_create_layout(ap->mf_MainWindow.pNaviBar, EDJ_NAME, "selectinfo_layout");

	ap->mf_MainWindow.pNaviBox = ap->mf_MainWindow.pNaviLayout;
	/*set content*/
	newContent = mf_navi_bar_content_create(ap);

	/*set title     segment or title*/
	if (ap->mf_Status.view_type == mf_view_root) {
		title = g_string_new(GET_SYS_STR(LABEL_MYFILE));
	} else if (mf_fm_svc_wrapper_is_root_path(ap->mf_Status.path->str)) {
		title = g_string_new(GET_SYS_STR(LABEL_MYFILE));
	} else {
		title = mf_fm_svc_wrapper_get_file_name(ap->mf_Status.path);
	}

	if (title != NULL) {

		pNavi_s->pNaviTitle = g_strdup(title->str);
		g_string_free(title, TRUE);
		title = NULL;
	}
	/*Navigation bar Path Select view integration*/
	mf_navi_bar_set_style(ap, NAVI_BAR_PATH_WITH_ONE_ACTION, newContent);
	/*add control bar for navigation bar*/
	//mf_navi_bar_set_ctrl_bar(ap);
	mf_navi_add_back_button(ap);
	mf_navi_bar_set_ctrl_button(data);
	mf_navi_bar_title_set(ap);
#ifdef MYFILE_USB_OTG
	/*control bar disable set if in otg root path*/
	if(mf_fm_svc_wrapper_is_root_path(ap->mf_Status.path->str) == MYFILE_OTG) {
		mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_OTG_ROOT, TRUE);
		//mf_ctrl_bar_item_set_disable(ap->mf_MainWindow.pNaviCtrlBar, CTRL_DISABLE_OTG_ROOT, TRUE);
	}

#endif
	/**    Todo:   should we add segment status set here?*/
	/*temp data free*/
}


/******************************
** Prototype    : mf_navi_bar_create_search_view
** Description  :
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
#ifdef MYFILE_SPLIT_VIEW
void mf_navi_bar_set_panes_navi_to_main(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata*)data;
	if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {

		mf_widget_unset_main_layout_content(ap->mf_MainWindow.sSplitData.pMainLayout);
		/***1. unset naviframe from panes/right layout **/
		mf_split_view_unset_right_content(ap);
		/***2. unset panes from main layout **/
		mf_widget_unset_main_layout_content(ap->mf_MainWindow.pMainLayout);


		/***3. set naviframe to main layout **/
		elm_object_part_content_set(ap->mf_MainWindow.pMainLayout, "elm.swallow.content", ap->mf_MainWindow.pNaviBar);
	}
	MF_TRACE_END;
}

void mf_navi_bar_reset_panes_main(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata*)data;


	mf_widget_unset_main_layout_content(ap->mf_MainWindow.pMainLayout);
	elm_object_part_content_set(ap->mf_MainWindow.sSplitData.pMainLayout, "elm.swallow.content", ap->mf_MainWindow.pNaviBar);

	mf_widget_panes_right_set(ap->mf_MainWindow.sSplitData.pPanes, ap->mf_MainWindow.sSplitData.pMainLayout);
	/**** 3. set panes to main layout **/
	elm_object_part_content_set(ap->mf_MainWindow.pMainLayout, "elm.swallow.content", ap->mf_MainWindow.sSplitData.pPanes);
	mf_split_view_show(ap);
	MF_TRACE_END;
}
#endif
void mf_navi_bar_create_search_view(void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;

	GString *title = NULL;
	Evas_Object *path_label = NULL;
#ifdef MYFILE_SPLIT_VIEW
	if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
		mf_navi_bar_set_panes_navi_to_main(ap);
	}
#endif

	__mf_navi_bar_reset_navi_obj(ap);

	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "get navigation bar in use failed");
	Evas_Object *pSearchViewLayout = NULL;
	//ap->mf_MainWindow.pNaviLayout = mf_widget_create_layout(ap->mf_MainWindow.pNaviBox, EDJ_NAME, "selectinfo_layout");
	pSearchViewLayout = mf_widget_create_layout(ap->mf_MainWindow.pNaviBar, EDJ_NAME, "selectinfo_layout");
	ap->mf_MainWindow.pNaviLayout = pSearchViewLayout;

	ap->mf_MainWindow.pNaviSearchBar = mf_navi_bar_create_search_bar(pSearchViewLayout, ap);
	ap->mf_MainWindow.pNaviBox = mf_widget_create_box(ap->mf_MainWindow.pNaviBar);

	/*set content*/
	/*2. create path label for the start of search view*/
	/*2.1 create path label*/

	path_label = elm_entry_add(pSearchViewLayout);

	elm_object_style_set(path_label, "default");
	elm_entry_single_line_set(path_label, EINA_FALSE);
	elm_object_focus_set(path_label, EINA_FALSE);

	evas_object_show(path_label);
	/*2.2 contribute label content*/
	char *path_in_label = mf_fm_svc_wrapper_translate_path(ap->mf_Status.path->str, MF_TRANS_OPTION_LABEL);
	char *label_content = g_strconcat(GET_SYS_STR(MF_LABEL_SEARCH), ":", "<br>", path_in_label, NULL);
	/*2.3 set style for path_label*/
	elm_entry_editable_set(path_label, EINA_FALSE);
	elm_entry_entry_set(path_label, label_content);

	/*set title segment or title*/
	title = g_string_new(GET_SYS_STR(MF_LABEL_SEARCH));
	if (title != NULL) {
		pNavi_s->pNaviTitle = g_strdup(title->str);
		g_string_free(title, TRUE);
		title = NULL;
	}
	/*navigation view integration*/
	elm_object_part_content_set(pSearchViewLayout, "content", path_label);
	elm_box_pack_end(ap->mf_MainWindow.pNaviBox, pSearchViewLayout);
	elm_box_pack_start(ap->mf_MainWindow.pNaviBox, ap->mf_MainWindow.pNaviSearchBar);


	ap->mf_Status.pPreNaviItem = ap->mf_MainWindow.pNaviItem;

	mf_language_mgr_unregister_object_item_by_type(ITEM_TYPE_CTRL);
	ap->mf_MainWindow.pNaviItem = elm_naviframe_item_push(ap->mf_MainWindow.pNaviBar, NULL, NULL, NULL, ap->mf_MainWindow.pNaviBox, MF_NAVI_STYLE_ENABLE);


	/*hide Tab Bar in search view*/

	/*add control bar for navigation bar*/
	/*temp data free*/
	if (path_in_label != NULL) {
		free(path_in_label);
		path_in_label = NULL;
	}
	if (label_content != NULL) {
		free(label_content);
		label_content = NULL;
	}
	mf_navi_bar_title_set(ap);
	evas_object_show(ap->mf_MainWindow.pSearchEntry);
	elm_object_focus_set(ap->mf_MainWindow.pSearchEntry, EINA_TRUE);
	mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);

	MF_TRACE_END;
}

static void __new_folder_entryfield_create(Evas_Object *parent, void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");
	mf_retm_if(parent == NULL, "parent is NULL");
	int error_code = 0;
	char *fileName = NULL;
	GString *dest_fullpath;
	static Elm_Entry_Filter_Limit_Size limit_filter;
	Evas_Object *entry = NULL;
	Evas_Object *edit_field = NULL;
	char *fullpath = (char *)malloc(sizeof(char) * MYFILE_FILE_PATH_LEN_MAX);
	if (fullpath == NULL) {
		MF_TRACE_END;
		return;
	}
	mf_widget_create_entryfield(ap, parent, mf_callback_new_folder_changed_cb, NULL);
	entry = ap->mf_MainWindow.pEntry;

	edit_field = ap->mf_MainWindow.pEditField;

	limit_filter.max_char_count = MYFILE_FILE_NAME_LEN_MAX;
	limit_filter.max_byte_count = 0;

	memset(fullpath, 0, MYFILE_FILE_PATH_LEN_MAX);
	fileName = elm_entry_markup_to_utf8(GET_SYS_STR(MF_LABEL_FOLDER));
	snprintf(fullpath, (MYFILE_FILE_PATH_LEN_MAX), "%s/%s", (char *)ap->mf_Status.path->str, fileName);

	dest_fullpath = g_string_new(fullpath);

	elm_entry_markup_filter_append(ap->mf_MainWindow.pEntry, elm_entry_filter_limit_size, &limit_filter);
	evas_object_smart_callback_add(ap->mf_MainWindow.pEntry, "maxlength,reached", mf_callback_max_len_reached_cb, ap);
	evas_object_smart_callback_add(ap->mf_MainWindow.pEntry, "preedit,changed",
				       mf_callback_imf_preedit_change_cb, ap);

	if (mf_fm_svc_wrapper_detect_duplication(dest_fullpath)) {
		mf_debug("dectetion result is TRUE\ndest_fullpath is %s\n", dest_fullpath->str);
		error_code = mf_fm_svc_wrapper_file_auto_rename(ap, dest_fullpath, FILE_NAME_WITH_BRACKETS, &dest_fullpath);
		mf_debug("rename result is %s\n", dest_fullpath->str);
		memset(fullpath, 0, strlen(fullpath));
		int len = strlen(ap->mf_Status.path->str) + 1;
		strncpy(fullpath, dest_fullpath->str + len, MYFILE_FILE_PATH_LEN_MAX);
		/*memset(fileName,0,strlen(fileName));*/

		SAFE_FREE_CHAR(fileName);
		fileName = elm_entry_markup_to_utf8(fullpath);
	}

	/*elm_entry_entry_set(entry, ecore_file_file_get(params->m_ItemName->str));*/
	mf_widget_imf_hide_callback_register(ap->mf_MainWindow.pConformant, entry);

	elm_entry_entry_set(entry, fileName);
	elm_entry_cursor_end_set(entry);

	SAFE_FREE_CHAR(fullpath);
	SAFE_FREE_GSTRING(dest_fullpath);
	SAFE_FREE_CHAR(fileName);
	MF_TRACE_END;

}
void mf_navi_bar_create_new_folder_view(void *data)
{

	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");

	Evas_Object *newContent = NULL;

	Evas_Object *pNaviBar = NULL;
	GString *title = NULL;
#ifdef MYFILE_SPLIT_VIEW
	if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
		mf_navi_bar_set_panes_navi_to_main(ap);
	}
#endif

	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "pNavi_s is NULL");

	__mf_navi_bar_reset_navi_obj(ap);

	pNaviBar = ap->mf_MainWindow.pNaviBar;
	ap->mf_MainWindow.pNaviBox = NULL;
	ap->mf_MainWindow.pNaviLayout= NULL;

	ap->mf_MainWindow.pNaviBox = mf_widget_create_box(ap->mf_MainWindow.pNaviBar);
	newContent = mf_widget_create_layout(ap->mf_MainWindow.pNaviBox, EDJ_NAME, GRP_NEW_FOLDER);
	__new_folder_entryfield_create(newContent, ap);
	evas_object_size_hint_weight_set(newContent, EVAS_HINT_EXPAND, 0);
	evas_object_size_hint_align_set(newContent, EVAS_HINT_FILL, 0.0);
	Evas_Object *pad = mf_widget_create_layout(ap->mf_MainWindow.pNaviBox, EDJ_NAME, "PAD");
	elm_object_part_content_set(newContent, "content", ap->mf_MainWindow.pEditField);
	elm_box_pack_start(ap->mf_MainWindow.pNaviBox, newContent);
	elm_box_pack_end(ap->mf_MainWindow.pNaviBox, pad);
	//newContent = mf_genlist_create_list_new_folder_style(ap);
	/*set title segment or title*/
	/*navigation view integration*/

	Elm_Object_Item *navi_it = NULL;
	//mf_navi_bar_set_style(ap, NAVI_BAR_DEFAULT, newContent);
	ap->mf_Status.pPreNaviItem = ap->mf_MainWindow.pNaviItem;
	mf_language_mgr_unregister_object_item_by_type(ITEM_TYPE_CTRL);
	navi_it = elm_naviframe_item_push(pNaviBar, NULL, NULL, NULL, ap->mf_MainWindow.pNaviBox, NULL);

	ap->mf_MainWindow.pNaviItem = navi_it;

	//mf_navi_add_back_button(ap);

	Evas_Object *l_bt = NULL;
	Evas_Object *r_bt = NULL;

	l_bt = elm_button_add(ap->mf_MainWindow.pNaviBar);
	elm_object_focus_set(l_bt, EINA_FALSE);
	elm_object_style_set(l_bt, "naviframe/title/default");
	elm_object_text_set(l_bt, GET_SYS_STR(LABEL_SAVE));
	mf_language_mgr_register_object(l_bt, OBJ_TYPE_ELM_OBJECT, NULL, LABEL_SAVE);
	evas_object_smart_callback_add(l_bt, "clicked", mf_callback_save_cb, ap);
	evas_object_show(l_bt);
	elm_object_item_part_content_set(navi_it, "title_toolbar_button1", l_bt);
	//elm_object_item_part_content_set(ap->mf_MainWindow.pNaviItem, TITLE_LEFT_BTN, l_bt);

#if 1
	r_bt = elm_button_add(ap->mf_MainWindow.pNaviBar);
	elm_object_focus_set(r_bt, EINA_FALSE);
	elm_object_style_set(r_bt, "naviframe/end_btn/default");
	//elm_object_style_set(r_bt, TITLE_BTN_STYLE);
	//elm_object_text_set(r_bt, GET_SYS_STR(LABEL_CANCEL));
	//mf_language_mgr_register_object(r_bt, OBJ_TYPE_ELM_OBJECT, NULL, LABEL_CANCEL);
	evas_object_smart_callback_add(r_bt, "clicked", mf_callback_cancel_cb, ap);
	evas_object_show(r_bt);
	elm_object_item_part_content_set(ap->mf_MainWindow.pNaviItem, "title_prev_btn", r_bt);
#endif
	elm_object_item_signal_emit(navi_it, "elm,state,sip,shown", "");

	elm_object_focus_set(ap->mf_MainWindow.pEntry, EINA_TRUE);


}

static void __rename_entryfield_create(Evas_Object *parent, void *data)
{
	MF_TRACE_BEGIN;
	MF_TRACE_BEGIN;
	mfItemData_s *params = (mfItemData_s *) data;
	mf_retvm_if(params == NULL, NULL, "param is NULL");
	mf_retvm_if(params->m_ItemName == NULL, NULL, "m_ItemName is NULL");

	struct appdata *ap = (struct appdata *)params->ap;
	mf_retvm_if(ap == NULL, NULL, "input parameter data error");

	static Elm_Entry_Filter_Limit_Size limit_filter_data;
	Evas_Object *edit_field = NULL;
	Evas_Object *entry = NULL;
	GString *filename = NULL;

	mf_widget_create_entryfield(ap, parent, mf_callback_imf_changed_cb, NULL);

	entry = ap->mf_MainWindow.pEntry;

	edit_field = ap->mf_MainWindow.pEditField;

	evas_object_size_hint_weight_set(edit_field, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(edit_field, EVAS_HINT_FILL, EVAS_HINT_FILL);


	elm_entry_single_line_set(entry, EINA_TRUE);
	elm_entry_scrollable_set(entry, EINA_TRUE);
	elm_object_signal_emit(edit_field, "elm,state,eraser,show", "elm");
	//elm_editfield_eraser_set(edit_field, EINA_TRUE);

	elm_entry_autocapital_type_set(entry, ELM_AUTOCAPITAL_TYPE_NONE);
	elm_entry_input_panel_layout_set(entry, ELM_INPUT_PANEL_LAYOUT_NORMAL);

	if (params->ap->mf_FileOperation.to_rename != NULL) {
		g_string_free(params->ap->mf_FileOperation.to_rename, TRUE);
		params->ap->mf_FileOperation.to_rename = NULL;
	}
	params->ap->mf_FileOperation.to_rename = g_string_new((char *)params->m_ItemName->str);


	/* the below is sample code for control entry. It is not mandatory.*/

	/* set guide text */
	filename = mf_fm_svc_wrapper_get_file_name(ap->mf_FileOperation.to_rename);
	char *guide_text = NULL;
	SAFE_FREE_CHAR(ap->mf_FileOperation.file_name_suffix);
	if (!mf_file_attr_is_dir(ap->mf_FileOperation.to_rename->str)) {
		mf_debug();
		char *ext = NULL;
		char *name_without_ext = NULL;
		name_without_ext = g_strdup(ap->mf_FileOperation.to_rename->str);
		mf_file_attr_get_file_ext(ap->mf_FileOperation.to_rename->str, &ext);
		mf_debug("ext is %s", ext);
		if (ext && strlen(ext) != 0) {
			mf_debug();
			name_without_ext[strlen(name_without_ext) - strlen(ext) - 1] = '\0';
			ap->mf_FileOperation.file_name_suffix = strdup(ext);
			mf_debug("name_without_ext is [%s]\n", name_without_ext);
			if (strlen(name_without_ext)) {
				guide_text = elm_entry_utf8_to_markup(ecore_file_file_get(name_without_ext));
			} else {
				guide_text = elm_entry_utf8_to_markup(filename->str);
			}
		} else {
			guide_text = elm_entry_utf8_to_markup(filename->str);
		}

		SAFE_FREE_CHAR(ext);
		SAFE_FREE_CHAR(name_without_ext);
	} else {
		guide_text = elm_entry_utf8_to_markup(filename->str);
	}

	if (ap->mf_FileOperation.file_name_suffix) {
		limit_filter_data.max_char_count = (MYFILE_FILE_NAME_LEN_MAX - mf_util_character_count_get(ap->mf_FileOperation.file_name_suffix) - 1);
	} else {
		limit_filter_data.max_char_count = MYFILE_FILE_NAME_LEN_MAX;
	}

	limit_filter_data.max_byte_count = 0;
	elm_entry_markup_filter_append(ap->mf_MainWindow.pEntry, elm_entry_filter_limit_size, &limit_filter_data);
	elm_entry_input_panel_return_key_type_set(ap->mf_MainWindow.pEntry, ELM_INPUT_PANEL_RETURN_KEY_TYPE_DONE);

	elm_entry_entry_set(ap->mf_MainWindow.pEntry, guide_text);
	/*elm_entry_entry_set(entry, ecore_file_file_get(params->m_ItemName->str));*/
	elm_entry_cursor_end_set(entry);

	SAFE_FREE_CHAR(guide_text);
	SAFE_FREE_GSTRING(filename);

	evas_object_smart_callback_add(entry, "activated", mf_genlist_rename_done, params->ap);
	evas_object_smart_callback_add(entry, "maxlength,reached", mf_callback_max_len_reached_cb, params->ap);
	evas_object_smart_callback_add(entry, "preedit,changed",
				       mf_callback_imf_preedit_change_cb, ap);
	mf_widget_imf_hide_callback_register(ap->mf_MainWindow.pConformant, entry);

	evas_object_show(entry);
	elm_object_focus_set(entry, EINA_TRUE);

	MF_TRACE_END;

}

void mf_navi_bar_create_rename_view(void *data)
{

	MF_TRACE_BEGIN;
	mfItemData_s *params = (mfItemData_s *) data;

	struct appdata *ap = (struct appdata *)params->ap;
	mf_retm_if(ap == NULL, "ap is NULL");

	Evas_Object *newContent = NULL;

	Evas_Object *pNaviBar = NULL;
	GString *title = NULL;
#ifdef MYFILE_SPLIT_VIEW
	if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
		mf_navi_bar_set_panes_navi_to_main(ap);
	}
#endif

	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "pNavi_s is NULL");

	pNaviBar = ap->mf_MainWindow.pNaviBar;
	SAFE_FREE_GSTRING(ap->mf_FileOperation.to_rename);

	ap->mf_FileOperation.to_rename = g_string_new((char *)params->m_ItemName->str);

	ap->mf_MainWindow.pNaviBox = NULL;
	ap->mf_MainWindow.pNaviLayout= NULL;

	ap->mf_MainWindow.pNaviBox = mf_widget_create_box(ap->mf_MainWindow.pNaviBar);
	newContent = mf_widget_create_layout(ap->mf_MainWindow.pNaviBox, EDJ_NAME, GRP_NEW_FOLDER);
	__rename_entryfield_create(newContent, params);
	evas_object_size_hint_weight_set(newContent, EVAS_HINT_EXPAND, 0);
	evas_object_size_hint_align_set(newContent, EVAS_HINT_FILL, 0.0);
	Evas_Object *pad = mf_widget_create_layout(ap->mf_MainWindow.pNaviBox, EDJ_NAME, "PAD");
	elm_object_part_content_set(newContent, "content", ap->mf_MainWindow.pEditField);
	elm_box_pack_start(ap->mf_MainWindow.pNaviBox, newContent);
	elm_box_pack_end(ap->mf_MainWindow.pNaviBox, pad);

	title = g_string_new(GET_SYS_STR(LABEL_RENAME));

	if (title != NULL) {
		pNavi_s->pNaviTitle = g_strdup(title->str);
		g_string_free(title, TRUE);
		title = NULL;
	}
	/*navigation view integration*/
	ap->mf_Status.pPreNaviItem = ap->mf_MainWindow.pNaviItem;
	mf_language_mgr_unregister_object_item_by_type(ITEM_TYPE_CTRL);
	ap->mf_MainWindow.pNaviItem = elm_naviframe_item_push(pNaviBar, NULL, NULL, NULL, ap->mf_MainWindow.pNaviBox, NULL);


	Evas_Object *l_bt = NULL;
	Evas_Object *r_bt = NULL;

	l_bt = elm_button_add(ap->mf_MainWindow.pNaviBar);
	elm_object_focus_set(l_bt, EINA_FALSE);
	elm_object_style_set(l_bt, "naviframe/title/default");
	elm_object_text_set(l_bt, GET_SYS_STR(LABEL_SAVE));
	mf_language_mgr_register_object(l_bt, OBJ_TYPE_ELM_OBJECT, NULL, LABEL_SAVE);
	evas_object_smart_callback_add(l_bt, "clicked", mf_callback_save_cb, ap);
	evas_object_show(l_bt);
	elm_object_item_part_content_set(ap->mf_MainWindow.pNaviItem, "title_toolbar_button1", l_bt);

	r_bt = elm_button_add(ap->mf_MainWindow.pNaviBar);
	elm_object_focus_set(r_bt, EINA_FALSE);
	elm_object_style_set(r_bt, "naviframe/end_btn/default");
	evas_object_smart_callback_add(r_bt, "clicked", mf_callback_cancel_cb, ap);
	evas_object_show(r_bt);
	elm_object_item_part_content_set(ap->mf_MainWindow.pNaviItem, "title_prev_btn", r_bt);
	elm_object_item_signal_emit(ap->mf_MainWindow.pNaviItem, "elm,state,sip,shown", "");
	MF_TRACE_END;
}


myfileNaviBar *mf_navi_bar_get_in_use(void *data)
{

	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, NULL, "ap is NULL");
	Eina_List *plistNavi_s = ap->mf_MainWindow.plistNaviBar;
	mf_retvm_if(plistNavi_s == NULL, NULL, "plistNavi_s is NULL");

	myfileNaviBar *ret = NULL;
	Eina_List *l = NULL;
	void *item;

	EINA_LIST_FOREACH(plistNavi_s, l, item) {
		ret = (myfileNaviBar *) item;

		mf_retvm_if(ret == NULL, NULL, "item is NULL");

		if (ret->naviFlagInUse == TRUE) {
			break;
		} else {
			ret = NULL;
		}
	}

	return ret;
}

myfileNaviBar *mf_navi_bar_get_edit_start(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	assert(ap);
	Eina_List *plistNavi_s = ap->mf_MainWindow.plistNaviBar;
	assert(plistNavi_s);

	myfileNaviBar *ret = NULL;
	Eina_List *l = NULL;
	void *item;

	EINA_LIST_FOREACH(plistNavi_s, l, item) {
		ret = (myfileNaviBar *) item;
		if (ret->naviFlagEditStart == TRUE) {
			break;
		} else {
			ret = NULL;
		}
	}
	return ret;
}

void mf_navi_bar_object_reset(myfileNaviBar *pNavi_s)
{
	mf_retm_if(pNavi_s == NULL, "Navi structure is NULL");
	pNavi_s->pNaviTitle = NULL;

	if (pNavi_s->pCurrentPath != NULL)
		free(pNavi_s->pCurrentPath);
	pNavi_s->pCurrentPath = NULL;

	if (pNavi_s->pNaviLabel != NULL)
		free(pNavi_s->pNaviLabel);
	pNavi_s->pNaviLabel = NULL;
}

/******************************
** Prototype    : mfNaviBarListItemRemove
** Description  : remove the navi bar item from the list by label
** Input        : void *data
**                const char* pNaviLabel
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
bool mf_navi_bar_remove_list_item_by_label(void *data, const char *pNaviLabel)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, false, "ap is NULL");
	mf_retvm_if(pNaviLabel == NULL, false, "Navi Bar Label is NULL");

	bool bEditStartFlag = FALSE;
	bool bInUseFlag = FALSE;
	mf_debug();
	myfileNaviBar *pNavi_s = mf_navi_bar_get_struct_by_label(ap, pNaviLabel);

	if (pNavi_s == NULL) {
		mf_debug();
		return bEditStartFlag;
	}
	Eina_List *plistNavi = eina_list_data_find_list(ap->mf_MainWindow.plistNaviBar, pNavi_s);
	ap->mf_MainWindow.plistNaviBar = eina_list_remove_list(ap->mf_MainWindow.plistNaviBar, plistNavi);

	mf_debug();
	if (pNavi_s->naviFlagEditStart == TRUE) {
		bEditStartFlag = TRUE;
	}
	if (pNavi_s->naviFlagInUse == TRUE) {
		bInUseFlag = TRUE;
	}
	mf_debug();
	mf_navi_bar_object_reset(pNavi_s);

	if (bInUseFlag == TRUE) {
		pNavi_s = (myfileNaviBar *) eina_list_nth(ap->mf_MainWindow.plistNaviBar, 0);
		pNavi_s->naviFlagInUse = TRUE;
	}
	mf_debug();
	return bEditStartFlag;
}

myfileNaviBar *mf_navi_bar_get_struct_by_label(void *data, const char *pNaviLabel)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, NULL, "ap is NULL");
	mf_retvm_if(pNaviLabel == NULL, NULL, "Navi Bar Label is NULL");

	Eina_List *l = NULL;
	void *item = NULL;

	myfileNaviBar *ret = NULL;
	EINA_LIST_FOREACH(ap->mf_MainWindow.plistNaviBar, l, item) {
		if (g_strcmp0(((myfileNaviBar *) item)->pNaviLabel, pNaviLabel) == 0) {
			ret = (myfileNaviBar *) item;
			break;
		}
	}
	return ret;
}

myfileNaviBar *mf_navi_bar_recover_list(void *data)
{
	mf_debug();
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	/*0     set state to be Default*/
	ap->mf_Status.more = MORE_DEFAULT;
	/*1     recover all the navigation bar*/
	Eina_List *l = NULL;
	void *item = NULL;
	myfileNaviBar *pNavi_s = NULL;
	myfileNaviBar *pNavi_toshow = NULL;
	bool flag_pop_todo = FALSE;
	EINA_LIST_FOREACH(ap->mf_MainWindow.plistNaviBar, l, item) {
		flag_pop_todo = TRUE;
		pNavi_s = (myfileNaviBar *) item;
		/*2.0   recover the pre_status*/
		pNavi_s->pre_more = ap->mf_Status.more;
		/*2.1   recover the path just before when edit click*/
		pNavi_s->pCurrentPath = g_strdup(pNavi_s->pre_operation.path->str);
		/*2.2   free the backup data structure*/
		g_string_free(pNavi_s->pre_operation.path, TRUE);
		pNavi_s->pre_operation.path = NULL;
		memset(&(pNavi_s->pre_operation), 0, sizeof(oper_record));
		/*2.3   set navi related status*/
		if (pNavi_s->naviFlagInUse == TRUE) {
			/*2.1-1 set pNavi_toshow for refreshing*/
			pNavi_toshow = pNavi_s;
		}
	}
	return pNavi_toshow;
}

void mf_navi_bar_refresh_recovered_view(void *data, myfileNaviBar * pNaviStruct)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");
	mf_retm_if(pNaviStruct == NULL, "Navi Bar Struct is NULL");

	if (ap->mf_Status.path != NULL) {
		g_string_free(ap->mf_Status.path, TRUE);
		ap->mf_Status.path = NULL;
	}
	ap->mf_Status.path = g_string_new(pNaviStruct->pCurrentPath);
	mf_debug("ap->mf_Status.path is [%s]", ap->mf_Status.path->str);

	/*insert new view*/
	mf_widget_refresh_view(ap);

	MF_TRACE_END;
}

bool mf_navi_bar_is_navi_empty(void *data, char *label)
{
	struct appdata *ap = (struct appdata *)data;
	assert(ap);
	assert(label);


	Elm_Object_Item *top_it= elm_naviframe_top_item_get(ap->mf_MainWindow.pNaviBar);
	Evas_Object *top_view = elm_object_item_content_get(top_it);

	if (top_view == NULL)
		return TRUE;
	else
		return FALSE;
}

void mf_navi_bar_transition_finished_cb(void *data, Evas_Object *obj, void *event_info)
{
 	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	mf_retm_if(obj == NULL, "obj is NULL");
	struct appdata *ap = (struct appdata *)data;
	Elm_Object_Item *it = ap->mf_Status.pPreNaviItem;

	SAFE_DEL_NAVI_ITEM(it);
	ap->mf_Status.pPreNaviItem = NULL;
	mf_widget_set_navi_blocking(ap, EINA_FALSE);
        evas_object_smart_callback_del(obj, "transition,finished", mf_navi_bar_transition_finished_cb);
 	MF_TRACE_END;
}

void mf_navi_bar_remove_previous_contents(void *data, Evas_Object *pNaviBar)
{
 	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	mf_retm_if(pNaviBar == NULL, "pNaviBar is NULL");
	struct appdata *ap = (struct appdata *)data;

	if (ap->mf_Status.pPreNaviItem != NULL) {
		mf_widget_set_navi_blocking(data, EINA_TRUE);
		evas_object_smart_callback_add(pNaviBar, "transition,finished", mf_navi_bar_transition_finished_cb, data);
	}
 	MF_TRACE_END;
}

void mf_navi_bottom_item_del(Elm_Object_Item **pPreNaviItem)
{
	MF_TRACE_BEGIN;
	mf_retm_if(*pPreNaviItem == NULL, "pPreNaviItem is NULL");
	SAFE_DEL_NAVI_ITEM(*pPreNaviItem);
	*pPreNaviItem = NULL;
 	MF_TRACE_END;
}

myfileNaviBar *mf_navi_bar_get_navi_from_navilist(Eina_List *navilist, const char *label)
{

	MF_TRACE_BEGIN;
	mf_retvm_if(navilist == NULL, NULL, "navilist is NULL");
	mf_retvm_if(label == NULL, NULL, "label is NULL");

	void *data = NULL;
	myfileNaviBar *pNavi = NULL;
	Eina_List *l = NULL;
	EINA_LIST_FOREACH(navilist, l, data) {
		if (data) {
			pNavi = (myfileNaviBar *)data;
			if (pNavi && pNavi->pNaviLabel && g_strcmp0(label, pNavi->pNaviLabel) == 0)
				return pNavi;
			else
				continue;
		}
	}
	return NULL;
}


void mf_navi_bar_del_item_directly(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata *)data;

	if (ap->mf_Status.flagNaviPush) {
		if(ap->mf_Status.pPreNaviItem)
		{
			SAFE_DEL_NAVI_ITEM(ap->mf_Status.pPreNaviItem);
			mf_widget_set_navi_blocking(ap, EINA_FALSE);
			evas_object_smart_callback_del(ap->mf_MainWindow.pNaviBar, "transition,finished", mf_navi_bar_transition_finished_cb);
	       }
	}
}

void mf_navi_bar_button_set_disable(Elm_Object_Item *navi_it, int disable_item, bool disable)
{
	mf_retm_if(navi_it == NULL, "navi_it is NULL");
	Evas_Object *btn = NULL;
	if (disable_item & CTRL_DISABLE_EDIT) {
		btn = elm_object_item_part_content_get(navi_it, NAVI_BOTTOM_BUTTON_1_PART);
		const char *button_label = elm_object_text_get(btn);
		if (g_strcmp0(button_label, GET_SYS_STR(MF_LABEL_EDIT)) == 0)
			elm_object_disabled_set(btn, disable);

	}
	if (disable_item & CTRL_DISABLE_DELETE) {
		btn = elm_object_item_part_content_get(navi_it, NAVI_BOTTOM_BUTTON_1_PART);
		const char *button_label = elm_object_text_get(btn);
		if (g_strcmp0(button_label, GET_SYS_STR(LABEL_DELETE)) == 0)
			elm_object_disabled_set(btn, disable);

	}
	if (disable_item & CTRL_DISABLE_SEND) {
		btn = elm_object_item_part_content_get(navi_it, NAVI_BOTTOM_BUTTON_2_PART);
		const char *button_label = elm_object_text_get(btn);
		if (g_strcmp0(button_label, GET_SYS_STR(LABEL_SHARE)) == 0)
			elm_object_disabled_set(btn, disable);

	}
	if (disable_item & CTRL_DISABLE_COPY_HERE) {
		btn = elm_object_item_part_content_get(navi_it, NAVI_BOTTOM_BUTTON_1_PART);
		const char *button_label = elm_object_text_get(btn);
		if (g_strcmp0(button_label, GET_STR(LABEL_COPY_HERE)) == 0)
			elm_object_disabled_set(btn, disable);

	}
	if (disable_item & CTRL_DISABLE_MOVE_HERE) {
		btn = elm_object_item_part_content_get(navi_it, NAVI_BOTTOM_BUTTON_1_PART);
		const char *button_label = elm_object_text_get(btn);
		if (g_strcmp0(button_label, GET_STR(LABEL_MOVE_HERE)) == 0)
			elm_object_disabled_set(btn, disable);

	}
	if (disable_item & CTRL_DISABLE_CREATE) {
		btn = elm_object_item_part_content_get(navi_it, NAVI_BOTTOM_BUTTON_2_PART);
		const char *button_label = elm_object_text_get(btn);
		if (g_strcmp0(button_label, GET_SYS_STR(MF_LABEL_CREATE_FOLDER)) == 0)
			elm_object_disabled_set(btn, disable);

	}
	if (disable_item & CTRL_DISABLE_MORE) {
		btn = elm_object_item_part_content_get(navi_it, NAVI_MORE_BUTTON_PART);
		elm_object_disabled_set(btn, disable);
	}

}

int mf_navi_bar_button_get_disable(Elm_Object_Item *navi_it)
{
	mf_retvm_if(navi_it == NULL, 0, "navi_it is NULL");
	int disable_item = 0;
	const char *button_label = NULL;

	Evas_Object *btn = NULL;
	btn = elm_object_item_part_content_get(navi_it, NAVI_BOTTOM_BUTTON_1_PART);
	button_label = elm_object_text_get(btn);
	if(elm_object_disabled_get(btn)) {
		if (g_strcmp0(button_label, GET_SYS_STR(LABEL_DELETE)) == 0) {
			disable_item |= CTRL_DISABLE_DELETE;
		}
	}

	btn = elm_object_item_part_content_get(navi_it, NAVI_BOTTOM_BUTTON_2_PART);
	button_label = elm_object_text_get(btn);
	if(elm_object_disabled_get(btn)) {
		if (g_strcmp0(button_label, GET_SYS_STR(LABEL_SHARE)) == 0) {
			disable_item |= CTRL_DISABLE_SEND;
		}
	}

	return disable_item;
}

void mf_navi_bar_set_toolbar_visible(Elm_Object_Item *navi_item, bool visible)
{
	mf_retm_if(navi_item == NULL, "navi_item is NULL");

	mf_debug("visible: %d", visible);
	if(visible) {
		elm_object_item_signal_emit(navi_item, "elm,state,toolbar,open", "");
	} else {
		elm_object_item_signal_emit(navi_item, "elm,state,toolbar,close", "");
	}
}


