/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <vconf-keys.h>
#include <syspopup_caller.h>
#include <bundle.h>
#include <status.h>

#include "mf-util.h"
#include "mf-widget.h"
#include "mf-fm-svc-wrapper.h"
#include "mf-callback.h"
#include "mf-language-mgr.h"

#define ITEM_COUNT	6
#define ITEM_MAX_COUNT	4
#define SHARE_ITEM_COUNT	4
#define MF_POPUP_STYLE_MIN_MENUSTYLE "menustyle"
#define MF_POPUP_MENUSTYLE_WIDTH (614*elm_config_scale_get())
#define MF_POPUP_MENUSTYLE_HEIGHT(x) ((113*x-1)*elm_config_scale_get())
#define MF_POPUP_MENUSTYLE_HEIGHT_MAX (408*elm_config_scale_get())
Elm_Genlist_Item_Class listby_itc, share_itc, storage_itc;

typedef struct {
	int index;
	bool unregister_lang_mgr;
	Elm_Object_Item *item;
	struct appdata *ap;
} ListByData_s;

static int __mf_popup_get_list_by_selected_item()
{
	int iSortTypeValue = 0;
	mf_util_get_vconf_value(VCONF_TYPE_SORT_TYPE, &iSortTypeValue);

	int ret = -1;
	switch (iSortTypeValue) {
	case	MYFILE_SORT_BY_DATE_O2R:
		ret = 3;
		break;
	case	MYFILE_SORT_BY_DATE_R2O:
		ret = 2;
		break;
	case	MYFILE_SORT_BY_TYPE_A2Z:
		ret = 5;
		break;
	case	MYFILE_SORT_BY_NAME_A2Z:
		ret = 0;
		break;
	case	MYFILE_SORT_BY_NAME_Z2A:
		ret = 1;
		break;
	case	MYFILE_SORT_BY_SIZE_L2S:
		ret = 4;
		break;
	case	MYFILE_SORT_BY_SIZE_S2L:
	case	MYFILE_SORT_BY_TYPE_Z2A:
	default:
		ret = 0;
		break;
	}
	return ret;
}

static int __mf_popup_get_list_by_opt(int index)
{
	//int iSortTypeValue = 0;
	//mf_util_get_vconf_value(VCONF_TYPE_SORT_TYPE, &iSortTypeValue);

	fsSortOption ret = MYFILE_SORT_BY_NONE;
	switch (index) {
	case 0:
		ret = MYFILE_SORT_BY_NAME_A2Z;
		break;
	case 1:
		ret = MYFILE_SORT_BY_NAME_Z2A;
		break;
	case 2:
		ret = MYFILE_SORT_BY_DATE_R2O;
		break;
	case 3:
		ret = MYFILE_SORT_BY_DATE_O2R;
		break;
	case 4:
		ret = MYFILE_SORT_BY_SIZE_L2S;
		break;
	case 5:
		ret = MYFILE_SORT_BY_TYPE_A2Z;
		break;
	default:
		ret = MYFILE_SORT_BY_DATE_R2O;
		break;
	}
	return ret;
}

static int __mf_popup_get_view_as_type(int index)
{
	eMfViewStyle ret = MF_VIEW_STYLE_LIST;
	switch (index) {
	case 0:
		ret = MF_VIEW_STYLE_LIST;
		break;
	case 1:
		ret = MF_VIEW_SYTLE_LIST_DETAIL;
		break;
	case 2:
		ret = MF_VIEW_STYLE_THUMBNAIL;
		break;
	default:
		ret = MF_VIEW_STYLE_LIST;
		break;
	}
	return ret;
}

static void __mf_popup_listby_genlist_del(void *data, Evas_Object * obj)
{
	MF_TRACE_BEGIN;
	ListByData_s *params = (ListByData_s *) data;
	mf_retm_if(params == NULL, "params is NULL");

	if(params->unregister_lang_mgr)
		mf_language_mgr_unregister_genlist_item(params->item);
	SAFE_FREE_CHAR(params);
}

static char *__mf_popup_get_genlist_label(void *data, Evas_Object * obj, const char *part)
{
	ListByData_s *params = (ListByData_s *) data;
	assert(params);
	struct appdata *ap = params->ap;
	assert(ap);
	assert(part);

	char *ret = NULL;
	if (!strcmp(part, "elm.text")) {
		//listby_opt = __mf_popup_get_list_by_opt(params->index);
		switch (params->index) {
		case 0:
			ret = g_strdup(GET_STR(MF_LABEL_NAME_HA_TO_Z));
			break;
		case 1:
			ret = g_strdup(GET_STR(MF_LABEL_NAME_HZ_TO_A));
			break;
		case 2:
			ret = g_strdup(GET_STR(MF_LABEL_DATE_HMOST_RECENT));
			break;
		case 3:
			ret = g_strdup(GET_STR(MF_LABEL_DATE_HOLDEST));
			break;
		case 4:
			ret = g_strdup(GET_SYS_STR(MF_LABEL_SIZE));
			break;
		case 5:
			ret = g_strdup(GET_SYS_STR(MF_LABEL_TYPE));		      /**< Sort by file size descending */
			break;
		#if 0
		case MYFILE_SORT_BY_DATE_R2O:			      /**< Sort by file date descending */
			ret = g_strdup(GET_SYS_STR(MF_LABEL_DATE));
			break;
		case MYFILE_SORT_BY_TYPE_Z2A:				/**< Sort by file type descending */
			ret = g_strdup(GET_SYS_STR(MF_LABEL_TYPE));
			break;
		#endif
		default:
			break;
		}
		return ret;
	}
	return NULL;
}

static char *__mf_popup_view_as_genlist_label_get(void *data, Evas_Object * obj, const char *part)
{
	ListByData_s *params = (ListByData_s *) data;
	assert(params);
	struct appdata *ap = params->ap;
	assert(ap);
	assert(part);

	eMfViewStyle view_as_type = MF_VIEW_STYLE_LIST;

	char *ret = NULL;
	if (!strcmp(part, "elm.text")) {
		view_as_type = __mf_popup_get_view_as_type(params->index);
		switch (view_as_type) {
		case MF_VIEW_STYLE_LIST:			      /**< Sort by file name ascending */
			ret = g_strdup(GET_SYS_STR(MF_LABEL_LIST));
			break;
		case MF_VIEW_SYTLE_LIST_DETAIL:			      /**< Sort by file size ascending */
			ret = g_strdup(GET_STR(LABEL_LIST_DETAIL_VIEW));
			break;
		case MF_VIEW_STYLE_THUMBNAIL:			      /**< Sort by file date ascending */
			ret = g_strdup(GET_STR(MF_LABEL_THUMBNAILS));
			break;
		default:
			break;
		}
		return ret;
	}
	return NULL;
}

static Evas_Object* __mf_popup_get_genlist_icon(void *data, Evas_Object * obj, const char *part)
{
	ListByData_s *params = (ListByData_s *) data;
	assert(params);
	struct appdata *ap = params->ap;
	assert(ap);
	assert(part);

	if (!strcmp(part, "elm.icon")) {
		Evas_Object *radio = NULL;
		radio = elm_radio_add(obj);
		elm_object_focus_set(radio, EINA_FALSE);
		elm_radio_state_value_set(radio, params->index);
		elm_radio_group_add(radio, ap->mf_Status.pRadioGroup);
		elm_radio_value_set(radio, ap->mf_Status.iRadioValue);
		evas_object_show(radio);
		return radio;
	}
	return NULL;
}

static void __mf_popup_genlist_select(void *data, Evas_Object * obj, void *event_info)
{
	assert(data);
	assert(event_info);
	ListByData_s *params = NULL;
	Elm_Object_Item *item = (Elm_Object_Item *) event_info;

	if (item != NULL) {

		elm_genlist_item_selected_set(item, FALSE);
		params = (ListByData_s *) elm_object_item_data_get(item);
		if (params != NULL && params->ap != NULL) {
			fsSortOption iListBySortType = MYFILE_SORT_BY_NONE;
			params->ap->mf_Status.iSelectedSortType = params->index;
			elm_radio_value_set(params->ap->mf_Status.pRadioGroup, params->index);
			iListBySortType = __mf_popup_get_list_by_opt(params->index);
			mf_util_set_sort_type(iListBySortType);
			mf_callback_list_by_response_cb(params->ap);
		}
	}
}


static void __mf_popup_view_as_genlist_select(void *data, Evas_Object * obj, void *event_info)
{
	assert(data);
	assert(event_info);
	ListByData_s *params = NULL;
	int iViewAsType = 0;
	Elm_Object_Item *item = (Elm_Object_Item *) event_info;
	if (item != NULL) {
		params = (ListByData_s *) elm_object_item_data_get(item);
		if (params != NULL && params->ap != NULL) {
			elm_radio_value_set(params->ap->mf_Status.pRadioGroup, params->index);
			iViewAsType = __mf_popup_get_view_as_type(params->index);
			params->ap->mf_Status.flagViewType = iViewAsType;
			mf_util_set_view_style(iViewAsType);
			mf_callback_popup_del_idler_cb(params->ap);
			if (params->ap->mf_Status.flagNoContent != EINA_TRUE) {
				mf_widget_refresh_view(params->ap);
			}
		}
	}
}
static int __mf_popup_get_view_as_selected_item()
{
	int iViewAs = 0;

	mf_util_get_vconf_value(VCONF_TYPE_VIEW_STYLE, &iViewAs);

	int ret = -1;
	switch (iViewAs) {
	case	MF_VIEW_STYLE_LIST:
		ret = 0;
		break;
	case	MF_VIEW_SYTLE_LIST_DETAIL:
		ret = 1;
		break;
	case	MF_VIEW_STYLE_THUMBNAIL:
		ret = 2;
		break;
	default:
		ret = 0;
		break;
	}
	return ret;
}

#ifdef MYFILE_SPLIT_VIEW
static void __mf_popup_storage_genlist_del(void *data, Evas_Object * obj)
{
	MF_TRACE_BEGIN;
	StorageData_s *params = (StorageData_s *) data;
	mf_retm_if(params == NULL, "params is NULL");

	if(params->unregister_lang_mgr)
		mf_language_mgr_unregister_genlist_item(params->item);
	SAFE_FREE_CHAR(params->label);
	SAFE_FREE_CHAR(params);
}

static char *__mf_popup_get_storage_genlist_label(void *data, Evas_Object * obj, const char *part)
{
	MF_TRACE_BEGIN;
	StorageData_s *params = (StorageData_s *) data;
	mf_retvm_if(params == NULL, NULL, "params is NULL");
	struct appdata *ap = params->ap;
	mf_retvm_if(ap == NULL, NULL, "ap is NULL");
	assert(part);

	if (!strcmp(part, "elm.text")) {
		MF_TRACE_END;
		return strdup(params->label);
	}
	MF_TRACE_END;
	return NULL;
}

static Evas_Object* __mf_popup_get_storage_genlist_icon(void *data, Evas_Object * obj, const char *part)
{
	MF_TRACE_BEGIN;
	StorageData_s *params = (StorageData_s *) data;
	mf_retvm_if(params == NULL, NULL, "params is NULL");
	struct appdata *ap = params->ap;
	mf_retvm_if(ap == NULL, NULL, "ap is NULL");
	assert(part);

	if (!strcmp(part, "elm.icon")) {
		Evas_Object *radio = NULL;
		radio = elm_radio_add(obj);
		elm_object_focus_set(radio, EINA_FALSE);
		elm_radio_state_value_set(radio, params->index);
		elm_radio_group_add(radio, ap->mf_Status.pRadioGroup);
		elm_radio_value_set(radio, ap->mf_Status.iRadioValue);
		evas_object_show(radio);
		MF_TRACE_END;
		return radio;
	}
	MF_TRACE_END;
	return NULL;
}

static void __mf_popup_storage_genlist_select(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	mf_retm_if(event_info == NULL, "event_info is NULL");
	StorageData_s *params = NULL;
	Elm_Object_Item *item = (Elm_Object_Item *) event_info;
	if (item != NULL) {
		params = (StorageData_s *) elm_object_item_data_get(item);
		if (params != NULL && params->ap != NULL) {
		}
	}
}
#endif
/******************************
** Prototype    : mf_popup_create_pb_popup
** Description  :
** Input        : void *data
**                char *context
**                int file_count
**                void*func
**                void* param
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
Evas_Object *mf_popup_create_pb_popup(void *data, char *context, int file_count, void *func, void *param)
{

	struct appdata *ap;

	ap = (struct appdata *)data;
	assert(ap);

	char count[256] = { 0 };
	Evas_Object *popup, *progressbar;
	Evas_Object *layout;

	snprintf(count, sizeof(count), "%s0/%d", count, file_count);

	popup = elm_popup_add(ap->mf_MainWindow.pMainLayout);
	elm_object_focus_set(popup, EINA_FALSE);

	layout = elm_layout_add(popup);
	elm_object_focus_set(layout, EINA_FALSE);
	elm_layout_file_set(layout, EDJ_NAME, "popup_center_progressview");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	ap->mf_MainWindow.pProgressLayout = layout;

	progressbar = elm_progressbar_add(popup);
	elm_object_focus_set(progressbar, EINA_FALSE);
	ap->mf_FileOperation.progress_bar = progressbar;
	elm_object_style_set(progressbar, "list_progress");
	elm_progressbar_horizontal_set(progressbar, EINA_TRUE);
	evas_object_size_hint_align_set(progressbar, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(progressbar, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_progressbar_value_set(progressbar, 0.0);
	evas_object_show(progressbar);

	elm_object_part_content_set(layout, "elm.swallow.content", progressbar);
	edje_object_part_text_set(elm_layout_edje_get(layout), "elm.title", context);
	edje_object_part_text_set(elm_layout_edje_get(layout), "elm.text.left", "0%");
	edje_object_part_text_set(elm_layout_edje_get(layout), "elm.text.right", count);

	elm_object_content_set(popup, layout);

	Evas_Object *btn1 = mf_widget_button_create(popup, GET_SYS_STR(LABEL_CANCEL));
	elm_object_style_set(btn1, "popup_button/default");
	evas_object_smart_callback_add(btn1, "clicked", func, param);
	elm_object_part_content_set(popup, "button1", btn1);

	evas_object_show(popup);

	return popup;
}

/******************************
** Prototype    : mf_popup_create_popup
** Description  :
** Input        : void *data
**                ePopMode popupMode
**                char *title
**                char *context
**                char *first_btn_text
**                char *second_btn_text
**                char *third_btn_text
**                Evas_Smart_Cb func
**                void* param
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
#ifdef MYFILE_SPLIT_VIEW
static void mf_popup_storage_item_append(void *data, Evas_Object *popup, Evas_Object *genlist)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	mf_retm_if(popup == NULL, "popup is NULL");
	mf_retm_if(genlist == NULL, "genlist is NULL");

	struct appdata *ap = (struct appdata *)data;
	Elm_Object_Item *it = NULL;

	StorageData_s *item_data = malloc(sizeof(StorageData_s));
	if (item_data == NULL) {
		mf_util_operation_alloc_failed(ap);
		MF_TRACE_END;
		return;
	}

	item_data->label= strdup(GET_SYS_STR(MF_LABEL_PHONE));
	item_data->index = MYFILE_PHONE;

	item_data->unregister_lang_mgr = TRUE;
	item_data->ap = ap;
	it = elm_genlist_item_append(genlist, &storage_itc, (void *)item_data, NULL,
				     ELM_GENLIST_ITEM_NONE, __mf_popup_storage_genlist_select, popup);
	item_data->item = it;
	mf_language_mgr_register_genlist_item(it);

	if (ap->mf_Status.iStorageState & MYFILE_MMC) {
		StorageData_s *item_data = malloc(sizeof(StorageData_s));
		if (item_data == NULL) {
			mf_util_operation_alloc_failed(ap);
			MF_TRACE_END;
			return;
		}
		item_data->label = strdup(GET_SYS_STR(MF_LABEL_MMC));
		mf_debug("item_data->label is [%s]", item_data->label);
		item_data->index = MYFILE_MMC;
		item_data->ap = ap;
		it = elm_genlist_item_append(genlist, &storage_itc, (void *)item_data, NULL,
					     ELM_GENLIST_ITEM_NONE, __mf_popup_storage_genlist_select, popup);
		item_data->item = it;
		mf_language_mgr_register_genlist_item(it);
	}
	if (ap->mf_Status.iStorageState & MYFILE_OTG) {
		StorageData_s *item_data = malloc(sizeof(StorageData_s));
		if (item_data == NULL) {
			mf_util_operation_alloc_failed(ap);
			MF_TRACE_END;
			return;
		}
		item_data->label = strdup(GET_STR(MF_LABEL_OTG));
		mf_debug("item_data->label is [%s]", item_data->label);
		item_data->index = MYFILE_OTG;
		item_data->ap = ap;
		it = elm_genlist_item_append(genlist, &storage_itc, (void *)item_data, NULL,
					     ELM_GENLIST_ITEM_NONE, __mf_popup_storage_genlist_select, popup);
		item_data->item = it;
		mf_language_mgr_register_genlist_item(it);
	}
	MF_TRACE_END;
}
#endif


static Evas_Object *__mf_popup_box_set(Evas_Object *parent, Evas_Object *content, int item_cnt)
{
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");
	mf_retvm_if(content == NULL, NULL, "content is NULL");
	mf_retvm_if(item_cnt < 0, NULL, "content is NULL");

	Evas_Object *box = elm_box_add(parent);
	if (item_cnt > ITEM_MAX_COUNT) {
		evas_object_size_hint_min_set(box, MF_POPUP_MENUSTYLE_WIDTH,
					      MF_POPUP_MENUSTYLE_HEIGHT(ITEM_MAX_COUNT));
	} else {
		evas_object_size_hint_min_set(box, MF_POPUP_MENUSTYLE_WIDTH,
					      MF_POPUP_MENUSTYLE_HEIGHT(item_cnt));
	}

	evas_object_show(content);
	elm_box_pack_end(box, content);
	return box;
}

Evas_Object *mf_ug_popup_create_search(void *data, Evas_Smart_Cb func, void *param)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, NULL, "ap is NULL");
	Evas_Object *popup = NULL;
	Evas_Object *btn1 = NULL;
	Evas_Object *box = NULL;
	Evas_Object *content = NULL;
	Evas_Object *label = NULL;

	popup = elm_popup_add(ap->mf_MainWindow.pWindow);
	elm_object_style_set(popup,"min_menustyle");
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	ap->mf_MainWindow.pNormalPopup = popup;

	btn1 = elm_button_add(popup);
	elm_object_text_set(btn1, GET_SYS_STR(LABEL_CANCEL));
	elm_object_part_content_set(popup, "button1", btn1);
	evas_object_smart_callback_add(btn1, "clicked", func, param);

	box = elm_box_add(popup);
	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	content = elm_layout_add(popup);
	elm_layout_file_set(content, EDJ_NAME, "search_path");

	evas_object_size_hint_weight_set(content, EVAS_HINT_EXPAND,
			EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(content, EVAS_HINT_FILL, EVAS_HINT_FILL);

	label = elm_label_add(content);
	ap->mf_MainWindow.pSearchLabel = label;
	elm_object_style_set(label, "popup/default");
	elm_label_wrap_width_set(label, 720);
	elm_label_ellipsis_set(label, EINA_TRUE);
	evas_object_size_hint_weight_set(label, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(label, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_text_set(label, "Test");


	elm_object_part_content_set(content, "path", label);
	evas_object_show(label);
	elm_box_pack_end(box, content);

	evas_object_size_hint_min_set(box, 720, 300);
	evas_object_size_hint_max_set(box, 720, 300);

	evas_object_show(btn1);
	evas_object_show(content);
	evas_object_show(box);

	elm_object_content_set(popup, box);

	evas_object_show(popup);
	return popup;
}

Evas_Object *mf_popup_create_popup(void *data, ePopMode popupMode, char *title, const char *context, const char *first_btn_text, const char *second_btn_text,
				   const char *third_btn_text, Evas_Smart_Cb func, void *param)
{
	mf_debug();
	Evas_Object *popup;
	struct appdata *ap;
	Evas_Object *genlist;
	Evas_Object *radio_group = NULL;
	Evas_Object *box = NULL;
	Evas_Object *btn1 = NULL;
	Evas_Object *btn2 = NULL;
	Evas_Object *btn3 = NULL;
	int index = 0;
	int listby_selected= 0;
	int context_items = 0;
#ifdef MYFILE_SPLIT_VIEW
	int storage = MYFILE_PHONE;
#endif
	ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, NULL, "ap is NULL");

	popup = elm_popup_add(ap->mf_MainWindow.pWindow);
	elm_object_focus_set(popup, EINA_FALSE);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	if (title) {
		elm_object_part_text_set(popup, "title,text", title);
		mf_language_mgr_register_object(popup, OBJ_TYPE_EDJE_OBJECT, "title,text", title);
	}
	if (context && popupMode != POPMODE_PROGRESSBAR) {
		mf_debug();
		elm_object_text_set(popup, context);
	}

	switch (popupMode) {
	case POPMODE_TEXT:
	case POPMODE_TITLE_TEXT:
		elm_popup_timeout_set(popup, 3);
		if (func != NULL) {
			evas_object_smart_callback_add(popup, "response", func, param);
		}
		break;
	case POPMODE_TEXT_NOT_DISABLED:
		if (func != NULL) {
			evas_object_smart_callback_add(popup, "response", func, NULL);
		}
		break;

	case POPMODE_TEXT_TWO_BTN:
	case POPMODE_TITLE_TEXT_TWO_BTN:
		btn1 = mf_widget_button_create(popup, first_btn_text);
		elm_object_style_set(btn1, "popup_button/default");
		btn2 = mf_widget_button_create(popup, second_btn_text);
		elm_object_style_set(btn2, "popup_button/default");
		if (g_strcmp0(GET_SYS_STR(LABEL_DELETE), first_btn_text) == 0) {
			elm_object_style_set(btn1, "sweep/delete");
			elm_object_style_set(btn2, "sweep/multiline");
		}
		evas_object_smart_callback_add(btn1, "clicked", func, param);
		evas_object_smart_callback_add(btn2, "clicked", func, param);
		elm_object_part_content_set(popup, "button1", btn1);
		elm_object_part_content_set(popup, "button2", btn2);
		break;

	case POPMODE_TEXT_BTN:
	case POPMODE_TITLE_TEXT_BTN:
		btn1 = mf_widget_button_create(popup, GET_SYS_STR(MF_BUTTON_LABEL_OK));
		elm_object_style_set(btn1, "popup_button/default");
		evas_object_smart_callback_add(btn1, "clicked", func, param);
		elm_object_part_content_set(popup, "button1", btn1);
		break;
	case POPMODE_TEXT_BTN_SEARCH:
		btn1 = mf_widget_button_create(popup, GET_SYS_STR(LABEL_CANCEL));
		elm_object_style_set(btn1, "popup_button/default");
		evas_object_smart_callback_add(btn1, "clicked", func, param);
		elm_object_part_content_set(popup, "button1", btn1);
		break;

	case POPMODE_TITLE_TEXT_THREE_BTN:

		elm_object_style_set(popup, "verticalbuttonstyle");
		evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

		btn1 = mf_widget_button_create(popup, GET_SYS_STR(MF_LABEL_REPLACE));
		elm_object_style_set(btn1, "popup_button/default");
		btn2 = mf_widget_button_create(popup, GET_STR(MF_BUTTON_LABEL_RENAME_AUTOMATICALLY));
		elm_object_style_set(btn2, "popup_button/default");
		btn3 = mf_widget_button_create(popup, GET_SYS_STR(LABEL_CANCEL));
		elm_object_style_set(btn3, "popup_button/default");

		evas_object_smart_callback_add(btn1, "clicked", mf_callback_operation_request_replace_cb, param);
		evas_object_smart_callback_add(btn2, "clicked", mf_callback_operation_request_rename_cb, param);
		evas_object_smart_callback_add(btn3, "clicked", mf_callback_operation_request_cancel_cb, param);
		elm_object_part_content_set(popup, "button1", btn1);
		elm_object_part_content_set(popup, "button2", btn2);
		elm_object_part_content_set(popup, "button3", btn3);
		break;

	case POPMODE_TITLE_LIST_BTN:

		elm_object_style_set(popup, MF_POPUP_STYLE_MIN_MENUSTYLE);

		btn1 = mf_widget_button_create(popup, GET_SYS_STR(LABEL_CANCEL));
		elm_object_style_set(btn1, "popup_button/default");
		evas_object_smart_callback_add(btn1, "clicked", func, param);
		elm_object_part_content_set(popup, "button1", btn1);

		//btn2 = mf_widget_button_create(popup, GET_STR(MF_LABEL_DESCENDING));
		//elm_object_style_set(btn2, "popup_button/default");
		//evas_object_smart_callback_add(btn2, "clicked", func, param);
		//elm_object_part_content_set(popup, "button2", btn2);

		if (ap->mf_Status.pRadioGroup) {
			evas_object_del(ap->mf_Status.pRadioGroup);
			ap->mf_Status.pRadioGroup = NULL;
		}

		radio_group = elm_radio_add(ap->mf_MainWindow.pWindow);
		elm_object_focus_set(radio_group, EINA_FALSE);
		elm_radio_value_set(radio_group, 0);
		evas_object_hide(radio_group);
		ap->mf_Status.pRadioGroup = radio_group;

		listby_selected = __mf_popup_get_list_by_selected_item();
		ap->mf_Status.iRadioValue = listby_selected;

		listby_itc.item_style = "1text.1icon.2";
		listby_itc.func.text_get = __mf_popup_get_genlist_label;
		listby_itc.func.content_get = __mf_popup_get_genlist_icon;
		listby_itc.func.state_get = NULL;
		listby_itc.func.del = __mf_popup_listby_genlist_del;
		genlist = elm_genlist_add(popup);
		//elm_genlist_homogeneous_set(genlist, EINA_TRUE);
		elm_object_focus_set(genlist, EINA_FALSE);
		evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);
		for (index = 0; index < ITEM_COUNT; index++) {
			Elm_Object_Item *it = NULL;
			ListByData_s *item_data = malloc(sizeof(ListByData_s));
			if (item_data == NULL)
				continue;

			item_data->index = index;
			item_data->unregister_lang_mgr = TRUE;
			item_data->ap = ap;
			it = elm_genlist_item_append(genlist, &listby_itc, (void *)item_data, NULL,
						     ELM_GENLIST_ITEM_NONE, __mf_popup_genlist_select, popup);

			item_data->item = it;
			mf_language_mgr_register_genlist_item(it);
		}

		box = __mf_popup_box_set(popup, genlist, ITEM_COUNT);
		elm_object_content_set(popup, box);
		evas_object_smart_callback_add(popup, "response", func, param);
		break;
#ifdef MYFILE_SPLIT_VIEW
	case POPMODE_STORAGE_BTN:
		if (ap->mf_Status.pRadioGroup ) {
			evas_object_del(ap->mf_Status.pRadioGroup);
			ap->mf_Status.pRadioGroup = NULL;
		}

		radio_group = elm_radio_add(ap->mf_MainWindow.pWindow);
		elm_object_focus_set(radio_group, EINA_FALSE);
		elm_radio_value_set(radio_group, 0);
		evas_object_hide(radio_group);
		ap->mf_Status.pRadioGroup = radio_group;

		storage = mf_fm_svc_wrapper_is_root_path(ap->mf_Status.path->str);
		ap->mf_Status.iRadioValue = storage;

		elm_object_style_set(popup, "menustyle");

		btn1 = mf_widget_button_create(popup, GET_SYS_STR(MF_POP_MSG_CLOSE));
		elm_object_style_set(btn1, "popup_button/default");
		evas_object_smart_callback_add(btn1, "clicked", func, param);
		elm_object_part_content_set(popup, "button1", btn1);

		storage_itc.item_style = "1text.1icon.2";
		storage_itc.func.text_get = __mf_popup_get_storage_genlist_label;
		storage_itc.func.content_get = __mf_popup_get_storage_genlist_icon;
		storage_itc.func.state_get = NULL;
		storage_itc.func.del = __mf_popup_storage_genlist_del;
		genlist = elm_genlist_add(popup);
		//elm_genlist_homogeneous_set(genlist, EINA_TRUE);
		elm_object_focus_set(genlist, EINA_FALSE);
		evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);
		mf_popup_storage_item_append(ap, popup, genlist);
		elm_object_content_set(popup, genlist);
		evas_object_smart_callback_add(popup, "response", func, param);
		break;
#endif
	case POPMODE_VIEW_AS_LIST:
		elm_object_style_set(popup, MF_POPUP_STYLE_MIN_MENUSTYLE);

		btn1 = mf_widget_button_create(popup, GET_SYS_STR(LABEL_CANCEL));
		elm_object_style_set(btn1, "popup_button/default");
		evas_object_smart_callback_add(btn1, "clicked", func, param);
		elm_object_part_content_set(popup, "button1", btn1);

		if (ap->mf_Status.pRadioGroup) {
			evas_object_del(ap->mf_Status.pRadioGroup);
			ap->mf_Status.pRadioGroup = NULL;
		}

		radio_group = elm_radio_add(ap->mf_MainWindow.pWindow);
		elm_object_focus_set(radio_group, EINA_FALSE);
		elm_radio_value_set(radio_group, 0);
		evas_object_hide(radio_group);
		ap->mf_Status.pRadioGroup = radio_group;

		listby_selected = __mf_popup_get_view_as_selected_item();
		ap->mf_Status.iRadioValue = listby_selected;

		listby_itc.item_style = "1text.1icon.2";
		listby_itc.func.text_get = __mf_popup_view_as_genlist_label_get;
		listby_itc.func.content_get = __mf_popup_get_genlist_icon;
		listby_itc.func.state_get = NULL;
		listby_itc.func.del = __mf_popup_listby_genlist_del;
		genlist = elm_genlist_add(popup);
		//elm_genlist_homogeneous_set(genlist, EINA_TRUE);
		elm_object_focus_set(genlist, EINA_FALSE);
		evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);
		for (index = 0; index < 3; index++) {
			Elm_Object_Item *it = NULL;
			ListByData_s *item_data = malloc(sizeof(ListByData_s));
			if (item_data == NULL)
				continue;

			item_data->index = index;
			item_data->unregister_lang_mgr = TRUE;
			item_data->ap = ap;
			it = elm_genlist_item_append(genlist, &listby_itc, (void *)item_data, NULL,
						     ELM_GENLIST_ITEM_NONE, __mf_popup_view_as_genlist_select, popup);
			item_data->item = it;
			mf_language_mgr_register_genlist_item(it);
		}

		box = __mf_popup_box_set(popup, genlist, 3);
		elm_object_content_set(popup, box);
		evas_object_smart_callback_add(popup, "response", func, param);
	default:
		break;
	}
	evas_object_show(popup);
	return popup;
}

void mf_popup_indicator_popup(void *data, const char *text)
{
	mf_retm_if(text == NULL, "text is NULL");
	int ret = status_message_post(text);
	mf_debug("status_message_post()... [0x%x]!", ret);
	if (ret != 0) {
		mf_debug("status_message_post()... [0x%x]!", ret);
	} else {

	}
	return ;

}
