/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */



#include <Eina.h>
#include <Ecore_X.h>

#include "mf-widget.h"
#include "mf-gengrid.h"
#include "mf-util.h"
#include "mf-dlog.h"
#include "mf-fm-svc-wrapper.h"
#include "mf-launch.h"
#include "mf-language-mgr.h"
#include "mf-tray-item.h"

#define MF_SEARCH_OPTION_DEF (MF_SEARCH_OPT_DIR | MF_SEARCH_OPT_FILE)
#define MF_SEARCH_ROOT_NUM 1
#define MF_SEARCH_TIMER_INTERVAL 0.5

extern Elm_Genlist_Item_Class itc, userfolderitc;
extern Elm_Genlist_Item_Class categoryitc;

extern Elm_Gengrid_Item_Class gic;
extern Elm_Gengrid_Item_Class search_itc;
extern int flagSearchMsg;
extern pthread_mutex_t gLockSearchMsg;
extern pthread_cond_t gCondSearchMsg;

Elm_Gengrid_Item_Class g_search_all_itc;
Elm_Gengrid_Item_Class search_all_itc;

typedef struct {
	char *size;
	char *create_date;
} mf_search_detail_infor_s;

static void __mf_search_bar_click_item(struct appdata *data, GString * path)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = data;
	assert(ap);
	if (ap->mf_Status.path == NULL || ap->mf_Status.path->str == NULL) {
		return;
	}
	GString *new_path = NULL;


	int error_code = 0;
	int mmc_card = 0;

	if (ap->mf_Status.more == MORE_RENAME) {
		mf_genlist_rename_done(ap, NULL, NULL);
		return;
	}
	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "get navi structure failed");
	if (ap->mf_Status.search_handler > 0) {
		mf_search_stop(ap->mf_Status.search_handler);
		mf_search_finalize(&ap->mf_Status.search_handler);
	}

	error_code = mf_util_is_mmc_on(&mmc_card);

	if (mf_fm_svc_wrapper_is_dir(path)) {

		if (ap->mf_FileOperation.search_result_list) {
			mf_util_free_eina_list_with_data(&(ap->mf_FileOperation.search_result_list), MYFILE_TYPE_CHAR);
		}
		elm_object_focus_set(ap->mf_MainWindow.pSearchEntry, EINA_FALSE);

		ap->mf_Status.view_type = mf_view_normal;
		ap->mf_Status.more = MORE_DEFAULT;
		new_path = g_string_new(path->str);
		mf_debug("path is %s\n", ap->mf_Status.path->str);

		/*set new path*/
		g_string_free(ap->mf_Status.path, TRUE);
		ap->mf_Status.path = NULL;

		ap->mf_Status.path = new_path;

		if (pNavi_s->pCurrentPath != NULL) {
			free(pNavi_s->pCurrentPath);
			pNavi_s->pCurrentPath = NULL;
		}
		pNavi_s->pCurrentPath = g_strdup(new_path->str);

		mf_widget_refresh_view(ap);
	} else {
		struct timeval mytime;
		gettimeofday(&mytime, NULL);
		mf_debug("[myfiles click time] %ld sec %ld usec \n", mytime.tv_sec, mytime.tv_usec);

		int ret = 0;
		ret = mf_launch_service(ap, path->str);
		mf_debug("ret is %d\n", ret);
		if (ret) {
			ap->mf_MainWindow.pNormalPopup =
				mf_popup_create_popup(ap, POPMODE_TEXT, NULL, GET_SYS_STR(MF_LABEL_UNSUPPORT_FILE_TYPE), NULL, NULL, NULL, NULL, NULL);
		}
	}
	MF_TRACE_END;
}

static void __mf_search_bar_item_sel(void *data, Evas_Object * obj, void *event_info)
{
	mf_debug("Start");
	assert(data);

	Elm_Object_Item *item = (Elm_Object_Item *) event_info;
	if (item != NULL) {
		mfItemData_s *selected = (mfItemData_s *) elm_object_item_data_get(item);

		elm_genlist_item_selected_set(item, FALSE);
		__mf_search_bar_click_item(data, selected->m_ItemName);
	}
}

static void __mf_search_bar_grid_item_sel(void *data, Evas_Object * obj, void *event_info)
{
	Elm_Object_Item *item = (Elm_Object_Item *) event_info;
	if (item != NULL) {
		mfItemData_s *selected = (mfItemData_s *) elm_object_item_data_get(item);
		mf_retm_if(selected == NULL, "selected is NULL");
		mf_retm_if(selected->m_ItemName == NULL, "selected->m_ItemName is NULL");
		mf_retm_if(selected->m_ItemName->str == NULL, "selected->m_ItemName->str is NULL");

		__mf_search_bar_click_item(data, selected->m_ItemName);
	}
}

static void __mf_search_bar_sel_search_all(void *data, Evas_Object * obj, void *event_info)
{
	mf_search_bar_search_started_callback(data, NULL, NULL);
}

static Evas_Object *_gl_content_get(void *data, Evas_Object * obj, const char *part)
{

	mf_debug("part is [%s]", part);
	if (!strcmp(part, "elm.icon")) {
		Evas_Object *btn = elm_button_add(obj);
		elm_object_text_set(btn, GET_STR(MF_LABEL_SEARCH_ALL));
		evas_object_smart_callback_add(btn, "clicked", __mf_search_bar_sel_search_all, data);

		return btn;
	} else if(!strcmp(part, "elm.swallow.icon")) {
		Evas_Object *btn = elm_button_add(obj);
		elm_object_text_set(btn, GET_STR(MF_LABEL_SEARCH_ALL));
		evas_object_smart_callback_add(btn, "clicked", __mf_search_bar_sel_search_all, data);
		return btn;
	}
	return NULL;
}


void mf_search_bar_search_all_item_append(void *data)
{

	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "input data error");

	if (ap->mf_Status.flagViewType == MF_VIEW_STYLE_THUMBNAIL) {

		Elm_Gengrid_Item_Class *itc = elm_gengrid_item_class_new();
		itc->item_style = "default_gridtext";
		itc->func.text_get = NULL;
		itc->func.content_get = _gl_content_get;
		itc->func.state_get = NULL;
		itc->func.del = NULL;

		elm_gengrid_item_append(ap->mf_MainWindow.pNaviGengrid, itc, ap, NULL, NULL);

	} else {
		Elm_Genlist_Item_Class *itc = elm_genlist_item_class_new();
		itc->item_style = "1icon";
		itc->func.text_get = NULL;
		itc->func.content_get = _gl_content_get;
		itc->func.state_get = NULL;
		itc->func.del = NULL;
		elm_genlist_item_append(ap->mf_MainWindow.pNaviGenlist, itc, ap, NULL,
					     ELM_GENLIST_ITEM_NONE, NULL, NULL);
	}
	ap->mf_Status.flagSearchAll = EINA_TRUE;
}
void mf_search_bar_item_append(void *data, void *user_data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)user_data;
	mf_retm_if(ap == NULL, "input data error");
	char *item_name = (char *)data;
	mf_retm_if(item_name == NULL, "input item_name error");
	mfItemData_s *m_TempItem = NULL;

	GString *search_path = g_string_new(item_name);

	g_string_free(search_path, TRUE);
	search_path = NULL;
	if (ap->mf_Status.view_type == mf_view_root_category && ap->mf_Status.more != MORE_SEARCH) {
		mf_util_generate_list(&ap->mf_FileOperation.category_list, item_name, FILE_TYPE_FILE);
	}
	m_TempItem = (mfItemData_s *) malloc(sizeof(mfItemData_s));

	if (m_TempItem == NULL) {
		return;
	}
	memset(m_TempItem, 0, sizeof(mfItemData_s));

	m_TempItem->m_ItemName = g_string_new(item_name);
	m_TempItem->size = NULL;
	m_TempItem->create_date = NULL;
	m_TempItem->m_checked = FALSE;
	m_TempItem->pCheckBox = NULL;
	m_TempItem->thumb_path = NULL;
	m_TempItem->real_thumb_flag = FALSE;
	m_TempItem->unregister_lang_mgr = TRUE;
	m_TempItem->media = NULL;
	m_TempItem->ap = ap;

	/*delete timer if exists*/

	if (ap->mf_Status.flagViewType == MF_VIEW_STYLE_THUMBNAIL) {
		Elm_Object_Item *git = NULL;
		git = elm_gengrid_item_append(ap->mf_MainWindow.pNaviGengrid, &gic, m_TempItem, __mf_search_bar_grid_item_sel, ap);
		mf_language_mgr_register_gengrid_item(git);
		m_TempItem->item = git;
	}
	else
	{
		Elm_Object_Item *it = NULL;
		if (ap->mf_Status.view_type == mf_view_root_category && ap->mf_Status.more != MORE_SEARCH) {
			it = elm_genlist_item_append(ap->mf_MainWindow.pNaviGenlist, &categoryitc, m_TempItem, NULL,
						     ELM_GENLIST_ITEM_NONE, mf_genlist_gl_sel, m_TempItem);
		} else {
			it = elm_genlist_item_append(ap->mf_MainWindow.pNaviGenlist, &search_itc, m_TempItem, NULL,
						     ELM_GENLIST_ITEM_NONE, __mf_search_bar_item_sel, ap);
		}
		m_TempItem->item = it;
	}

	mf_debug("get is [%d]", elm_genlist_items_count(ap->mf_MainWindow.pNaviGenlist));
	MF_TRACE_END;

}

static void __mf_search_bar_genlist_del_cb(void *data, Evas * e, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");
	if(ap->mf_MainWindow.pSearchViewTimer) {
		ecore_timer_del(ap->mf_MainWindow.pSearchViewTimer);
		ap->mf_MainWindow.pSearchViewTimer = NULL;
	}
	MF_TRACE_END;
}

static void __mf_search_bar_result_cb(mf_search_result_t *result, void *user_data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(result == NULL, "result is NULL");

	struct appdata *ap = (struct appdata *)user_data;
	mf_retm_if(ap == NULL, "ap is NULL");

	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "pNavi_s is NULL");

	Evas_Object *pSearchBarLabel = NULL;

	if (ap->mf_Status.view_type != mf_view_root_category) {
		elm_object_focus_set(ap->mf_MainWindow.pSearchEntry, EINA_TRUE);
	}

#ifdef MYFILE_SPLIT_VIEW
	if (ap->mf_FileOperation.search_result_list) {
		mf_util_free_eina_list_with_data(&(ap->mf_FileOperation.search_result_list), MYFILE_TYPE_CHAR);
	}
#endif
	if (ap->mf_Status.view_type == mf_view_root_category && ap->mf_Status.more != MORE_SEARCH) {
		if (ap->mf_FileOperation.category_list) {
			mf_util_free_eina_list_with_data(&(ap->mf_FileOperation.category_list), MYFILE_TYPE_FSNODE);
		}
	}

	if (g_list_length(result->dir_list) + g_list_length(result->file_list) == 0) {

		Evas_Object *parent = NULL;
		parent = ap->mf_MainWindow.pNaviLayout;
		ap->mf_Status.flagNoContent = EINA_TRUE;
		pSearchBarLabel = mf_widget_create_no_content(parent);
		elm_object_part_text_set(pSearchBarLabel, "elm.text", GET_SYS_STR(LABEL_NO_RESULT));

		if (ap->mf_Status.view_type != mf_view_root_category) {
			elm_object_part_content_set(parent, "content", pSearchBarLabel);
		} else {
			elm_object_part_content_set(parent, "content", pSearchBarLabel);
			Evas_Object *btn = mf_widget_get_part_content(ap->mf_MainWindow.pNaviBar, NAVI_BOTTOM_BUTTON_1_PART);
			elm_object_disabled_set(btn, EINA_TRUE);
		}

	} else {
		ap->mf_Status.flagNoContent = EINA_FALSE;
		if (ap->mf_Status.view_type == mf_view_root_category && ap->mf_Status.more != MORE_SEARCH) {
			mf_genlist_set_category_list_style();
			mf_navi_bar_title_set(ap);
		}
		if (ap->mf_Status.flagViewType == MF_VIEW_STYLE_LIST || ap->mf_Status.flagViewType == MF_VIEW_SYTLE_LIST_DETAIL) {
			if (ap->mf_MainWindow.pNaviGenlist == NULL) {
				ap->mf_MainWindow.pNaviGenlist = elm_genlist_add(ap->mf_MainWindow.pNaviLayout);
			}
			elm_genlist_clear(ap->mf_MainWindow.pNaviGenlist);

			elm_genlist_mode_set(ap->mf_MainWindow.pNaviGenlist, ELM_LIST_COMPRESS);
			elm_object_focus_set(ap->mf_MainWindow.pNaviGenlist, EINA_FALSE);
			evas_object_size_hint_weight_set(ap->mf_MainWindow.pNaviGenlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
			evas_object_size_hint_align_set(ap->mf_MainWindow.pNaviGenlist, EVAS_HINT_FILL, EVAS_HINT_FILL);

			//elm_genlist_homogeneous_set(ap->mf_MainWindow.pNaviGenlist, EINA_TRUE);
			evas_object_smart_callback_add(ap->mf_MainWindow.pNaviGenlist, "drag,start,right", mf_genlist_gl_mode_sweep_set, ap);
			evas_object_smart_callback_add(ap->mf_MainWindow.pNaviGenlist, "drag,start,left", mf_genlist_gl_mode_sweep_unset, ap);
			evas_object_smart_callback_add(ap->mf_MainWindow.pNaviGenlist, "drag,start,up", mf_genlist_gl_mode_cancel, ap);
			evas_object_smart_callback_add(ap->mf_MainWindow.pNaviGenlist, "drag,start,down", mf_genlist_gl_mode_cancel, ap);
			//evas_object_smart_callback_add(ap->mf_MainWindow.pNaviGenlist, "drag,stop", mf_genlist_gl_mode_sweep_stop, NULL);
			if (ap->mf_Status.view_type == mf_view_root_category && ap->mf_Status.more != MORE_SEARCH) {
				evas_object_smart_callback_add(ap->mf_MainWindow.pNaviGenlist, "selected", mf_genlist_gl_selected, ap);
			}
			evas_object_event_callback_add(ap->mf_MainWindow.pNaviGenlist,EVAS_CALLBACK_FREE, __mf_search_bar_genlist_del_cb, ap);
			evas_object_show(ap->mf_MainWindow.pNaviGenlist);
			mfNaviBarSetContent(ap, ap->mf_MainWindow.pNaviLayout, ap->mf_MainWindow.pNaviGenlist);
		} else {
			if (ap->mf_MainWindow.pNaviGengrid == NULL) {
				ap->mf_MainWindow.pNaviGengrid = mf_gengrid_create_grid(ap->mf_MainWindow.pNaviLayout);
			}
			elm_gengrid_clear(ap->mf_MainWindow.pNaviGengrid);
			if (MF_SCALE_FACTORY == 1) {
				elm_gengrid_item_size_set(ap->mf_MainWindow.pNaviGengrid, MF_HD_GENGRID_ITEM_WIDTH, MF_HD_GENGRID_ITEM_HEIGTH);
			} else {
				elm_gengrid_item_size_set(ap->mf_MainWindow.pNaviGengrid, MF_WVGA_GENGRID_ITEM_WIDTH, MF_WVGA_GENGRID_ITEM_HEIGTH);
			}
			evas_object_show(ap->mf_MainWindow.pNaviGengrid);
			mfNaviBarSetContent(ap, ap->mf_MainWindow.pNaviLayout, ap->mf_MainWindow.pNaviGengrid);
		}
		if (result->dir_list != NULL) {
			g_list_foreach(result->dir_list, mf_search_bar_item_append, ap);
		}
		if (result->file_list != NULL) {
			g_list_foreach(result->file_list, mf_search_bar_item_append, ap);
		}
		MF_TRACE_END;
	}
}

static void __mf_search_bar_stop_cb(void *data, Evas_Object * obj, void *event_info)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "input data error");

	Evas_Object *btn = (Evas_Object *)obj;
	const char *label = elm_object_text_get(btn);

	if (g_strcmp0(label, GET_SYS_STR(LABEL_CANCEL)) == 0 ) {
		if (ap->mf_FileOperation.sync_pipe != NULL) {
			ecore_pipe_del(ap->mf_FileOperation.sync_pipe);
			ap->mf_FileOperation.sync_pipe = NULL;
		}

		__mf_search_bar_result_cb(((ms_handle_t *) ap->mf_Status.search_handler)->result, ap);

		if (ap->mf_Status.search_handler > 0) {
			mf_search_stop(ap->mf_Status.search_handler);
		}

		evas_object_del(ap->mf_MainWindow.pNormalPopup);
		ap->mf_MainWindow.pNormalPopup = NULL;

		if (ap->mf_Status.search_handler > 0) {
			mf_search_finalize(&ap->mf_Status.search_handler);
		}
	}
}

static int __mf_search_bar_idle_search_msg_cope_finished(void *data)
{
	pthread_mutex_lock(&gLockSearchMsg);
	if (flagSearchMsg == 0) {
		flagSearchMsg = 1;
		pthread_cond_signal(&gCondSearchMsg);
	}
	pthread_mutex_unlock(&gLockSearchMsg);

	return ECORE_CALLBACK_CANCEL;
}

static void __mf_search_bar_pipe_cb(void *data, void *buffer, unsigned int nbyte)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	if (ap == NULL) {
		mf_debug("input ap is NULL");
		goto MF_CONTINURE_SEARCH;
	}

	mf_search_pipe_msg *pSearchMsg = (mf_search_pipe_msg *) buffer;
	if (pSearchMsg == NULL) {
		mf_debug("received message is NULL");
		goto MF_CONTINURE_SEARCH;
	}

	if (pSearchMsg->mf_sp_msg_type == MF_SEARCH_PIPE_MSG_RESULT_REPORT) {
		mf_debug("result get");
	} else if (pSearchMsg->mf_sp_msg_type == MF_SEARCH_PIPE_MSG_ROOT_CHANGE) {
		mf_debug("path change	%s", pSearchMsg->current_path);
		char *new_desc = mf_fm_svc_wrapper_translate_path(pSearchMsg->current_path, MF_TRANS_OPTION_POPUP);
		elm_object_text_set(ap->mf_MainWindow.pSearchLabel, new_desc);
		if (pSearchMsg->current_path) {
			free(pSearchMsg->current_path);
			pSearchMsg->current_path = NULL;
		}
		if (new_desc != NULL) {
			free(new_desc);
			new_desc = NULL;
		}
	} else if (pSearchMsg->mf_sp_msg_type == MF_SEARCH_PIPE_MSG_FINISHED) {
		evas_object_del(ap->mf_MainWindow.pNormalPopup);
		ap->mf_MainWindow.pNormalPopup = NULL;
		__mf_search_bar_result_cb((mf_search_result_t *) pSearchMsg->report_result, data);
	}

MF_CONTINURE_SEARCH:
	ecore_idler_add((Ecore_Task_Cb)__mf_search_bar_idle_search_msg_cope_finished, NULL);
	MF_TRACE_END;
}

void mf_search_bar_category_search_start(void *data, int category)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata *)data;
	int root_num = MF_SEARCH_ROOT_NUM;

	mf_genlist_create_search_itc();

	root_num = 1;
	const char *SearchRoot[3] = {0};
	SearchRoot[0] = PHONE_FOLDER;
	if (ap->mf_Status.iStorageState & MYFILE_MMC) {
		SearchRoot[root_num] = MEMORY_FOLDER;
		root_num++;
	}
	if (ap->mf_Status.iStorageState & MYFILE_OTG) {
		SearchRoot[root_num] = OTG_FOLDER;
		root_num++;
	}

	char *new_desc = NULL;
	/*Start Search routine*/
	if (!mf_search_start(ap->mf_Status.search_handler, SearchRoot, root_num, \
			     NULL, MF_SEARCH_OPT_FILE, (void *)ap, mf_tray_item_type, category)) {
		/*generate the popup used to show search path
		**it's sure that new_desc is not NULL even if original path is NULL*/
		new_desc = mf_fm_svc_wrapper_translate_path(ap->mf_Status.path->str, MF_TRANS_OPTION_POPUP);
		ap->mf_MainWindow.pNormalPopup = mf_ug_popup_create_search(ap, __mf_search_bar_stop_cb, ap);
			//mf_popup_create_popup(ap, POPMODE_TEXT_BTN_SEARCH, NULL, new_desc, NULL, NULL, NULL, __mf_search_bar_stop_cb, ap);
	} else {
		mf_debug("search failed");
	}

	if (new_desc != NULL) {
		free(new_desc);
		new_desc = NULL;
	}

	MF_TRACE_END;
}
static bool __mf_search_bar_idle_search_start(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	if (pNavi_s == NULL) {
		ap->mf_FileOperation.search_IME_hide_timer = NULL;
		mf_debug("pNavi_s is NULL");
		return ECORE_CALLBACK_CANCEL;
	}

	int root_num = 0;
	mf_genlist_create_search_itc();

	char *text = NULL;
	const char *SearchRoot[3] = {0};
	if (ap->mf_Status.more == MORE_SEARCH) {
		text = g_strdup(elm_object_text_get(ap->mf_MainWindow.pSearchEntry));
	}

	if (ap->mf_Status.view_type == mf_view_root || ap->mf_Status.view_type == mf_view_root_category) {
		SearchRoot[0] = PHONE_FOLDER;
		root_num++;
		if (ap->mf_Status.iStorageState & MYFILE_MMC) {
			SearchRoot[root_num] = MEMORY_FOLDER;
			root_num++;
		}
		if (ap->mf_Status.iStorageState & MYFILE_OTG) {
			SearchRoot[root_num] = OTG_FOLDER;
			root_num++;
		}
	} else {
		if (ap->mf_Status.flagSearchAll == EINA_FALSE) {
			SearchRoot[0] = ap->mf_Status.path->str;
			root_num++;
		} else {
			int root = mf_fm_svc_wrapper_is_root_path(ap->mf_Status.path->str);
			switch (root) {
			case MYFILE_PHONE:
				SearchRoot[0] = PHONE_FOLDER;
				break;
			case MYFILE_MMC:
				SearchRoot[0] = MEMORY_FOLDER;
				break;
			case MYFILE_OTG:
				SearchRoot[0] = OTG_FOLDER;
				break;
			default:
				SearchRoot[0] = PHONE_FOLDER;
				break;
			}
			root_num++;
		}
	}
	mf_debug("search root is %s text is [%s]", ap->mf_Status.path->str, text);
	char *new_desc = NULL;
	/*Start Search routine*/
	if (!mf_search_start(ap->mf_Status.search_handler, SearchRoot, root_num, \
			     ((text) ? text : NULL), MF_SEARCH_OPTION_DEF, (void *)ap, NULL, mf_tray_item_category_none)) {
		/*generate the popup used to show search path
		**it's sure that new_desc is not NULL even if original path is NULL*/
		new_desc = mf_fm_svc_wrapper_translate_path(ap->mf_Status.path->str, MF_TRANS_OPTION_POPUP);
		ap->mf_MainWindow.pNormalPopup = mf_ug_popup_create_search(ap, __mf_search_bar_stop_cb, ap);
			//mf_popup_create_popup(ap, POPMODE_TEXT_BTN_SEARCH, NULL, new_desc, NULL, NULL, NULL, __mf_search_bar_stop_cb, ap);
	}

	if (new_desc != NULL) {
		free(new_desc);
		new_desc = NULL;
	}

	if (text != NULL) {
		free(text);
		text = NULL;
	}

	ap->mf_FileOperation.search_IME_hide_timer = NULL;
	return ECORE_CALLBACK_CANCEL;
}

void mf_search_bar_enter_search_routine(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	SAFE_FREE_OBJ(ap->mf_MainWindow.pContextPopup);

	if (ap->mf_Status.more == MORE_RENAME) {
		mf_genlist_rename_done(ap, NULL, NULL);
		return;
	}

	if (ap->mf_Status.more != MORE_DEFAULT) {
		return;
	}
	if (ap->mf_Status.flagNaviPush) {
		mf_debug("the transition of the naviframe is not finished");
		return;
	}

	if (ap->mf_FileOperation.search_IME_hide_timer != NULL) {
		ecore_timer_del(ap->mf_FileOperation.search_IME_hide_timer);
		ap->mf_FileOperation.search_IME_hide_timer = NULL;
	}

	ap->mf_Status.more = MORE_SEARCH;
	ap->mf_Status.flagSearchAll = EINA_FALSE;

	mf_navi_bar_create_search_view(ap);
	MF_TRACE_END;
}

void mf_search_bar_category_search(void *data, int category)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "input data error");
	SAFE_FREE_CHAR(ap->mf_Status.search_filter);
	if (ap->mf_Status.search_handler > 0)
		mf_search_finalize(&ap->mf_Status.search_handler);
	int ret = mf_search_init(&ap->mf_Status.search_handler);
	mf_retm_if(ret < 0, "Fail to mf_search_init()");

	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "input data error");

	evas_object_del(mf_widget_unset_part_content(ap->mf_MainWindow.pNaviLayout, "content"));
	Evas_Object *tray = mf_widget_unset_part_content(ap->mf_MainWindow.pNaviLayout, "swallow.shortcut");
	SAFE_FREE_OBJ(tray);
	ap->mf_MainWindow.pNaviGengrid = NULL;
	ap->mf_MainWindow.pNaviGenlist = NULL;

	if (ap->mf_FileOperation.sync_pipe != NULL) {
		ecore_pipe_del(ap->mf_FileOperation.sync_pipe);
		ap->mf_FileOperation.sync_pipe = NULL;
	}
	ap->mf_FileOperation.sync_pipe = ecore_pipe_add(__mf_search_bar_pipe_cb, ap);

	if (ap->mf_FileOperation.sync_pipe == NULL) {
		mf_debug("add pipe failed");
	}
	/*this is to init global variable to ensure the first message can be transmitted correctly*/
	/*flagSearchMsg is to indicate the condition wait to sync data of threads*/
	pthread_mutex_lock(&gLockSearchMsg);
	flagSearchMsg = 1;
	pthread_mutex_unlock(&gLockSearchMsg);

	/*delete guide text label in the box*/
	mf_search_bar_category_search_start(ap, category);
	mf_debug("enter pressed %d", category);
	MF_TRACE_END;

}
void mf_search_bar_search_started_callback(void *data, Evas_Object * obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "input data error");

	char *text = g_strdup(elm_object_text_get(ap->mf_MainWindow.pSearchEntry));
	if (text == NULL || strlen(text) == 0 || strlen(g_strstrip(text)) == 0) {
		g_free(text);
		text = NULL;
		SAFE_FREE_CHAR(ap->mf_Status.search_filter);
		return;
	}

	g_free(text);
	text = NULL;

	SAFE_FREE_CHAR(ap->mf_Status.search_filter);
	ap->mf_Status.search_filter = g_strdup(elm_object_text_get(ap->mf_MainWindow.pSearchEntry));
	if (ap->mf_Status.more == MORE_RENAME) {
		mf_genlist_rename_done(ap, NULL, NULL);
		return;
	}

	if (ap->mf_Status.search_handler > 0)
		mf_search_finalize(&ap->mf_Status.search_handler);
	int ret = mf_search_init(&ap->mf_Status.search_handler);
	mf_retm_if(ret < 0, "Fail to mf_search_init()");

	myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
	mf_retm_if(pNavi_s == NULL, "input data error");

	/*hide the IME*/
	elm_object_focus_set(ap->mf_MainWindow.pSearchEntry, EINA_FALSE);

	/*delete guide text label in the box*/
	evas_object_del(mf_widget_unset_part_content(ap->mf_MainWindow.pNaviLayout, "content"));

	ap->mf_MainWindow.pNaviGengrid = NULL;
	ap->mf_MainWindow.pNaviGenlist = NULL;

	if (ap->mf_FileOperation.sync_pipe != NULL) {
		ecore_pipe_del(ap->mf_FileOperation.sync_pipe);
		ap->mf_FileOperation.sync_pipe = NULL;
	}
	ap->mf_FileOperation.sync_pipe = ecore_pipe_add(__mf_search_bar_pipe_cb, ap);

	if (ap->mf_FileOperation.sync_pipe == NULL) {
		mf_debug("add pipe failed");
	}
	/*this is to init global variable to ensure the first message can be transmitted correctly*/
	/*flagSearchMsg is to indicate the condition wait to sync data of threads*/
	pthread_mutex_lock(&gLockSearchMsg);
	flagSearchMsg = 1;
	pthread_mutex_unlock(&gLockSearchMsg);

	elm_object_focus_set(ap->mf_MainWindow.pSearchEntry, EINA_FALSE);
	/*delete guide text label in the box*/
	ap->mf_FileOperation.search_IME_hide_timer = ecore_timer_add(MF_SEARCH_TIMER_INTERVAL, (Ecore_Task_Cb)__mf_search_bar_idle_search_start, ap);
	mf_debug("enter pressed");
	MF_TRACE_END;

	return;
}
