/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include "mf-main.h"
#include "mf-widget.h"
#include "mf-callback.h"
#include "mf-fm-svc-wrapper.h"
//double wscale = (double)(((double)w) / ((double)GL_WIN_WIDTH));
#define LEFT_SCALE		0.4
#define LEFT_W			(800*LEFT_SCALE)
#define LEFT_H			(480)

#ifdef MYFILE_SPLIT_VIEW

static void _gl_con(void *data, Evas_Object *obj, void *event_info);
static void _gl_exp_ndepth(void *data, Evas_Object *obj, void *event_info);

void mf_split_view_unset_right_content(void *data)
{

	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata*)data;
	Evas_Object *unset = NULL;

	unset = mf_widget_unset_part_content(ap->mf_MainWindow.sSplitData.pPanes, "right");
	if (unset)
		evas_object_hide(unset);
	MF_TRACE_END;
}


void mf_split_view_path_info_update(Evas_Object *label, const char *fullpath, Eina_Bool root)
{
	MF_TRACE_BEGIN;
	mf_retm_if(label == NULL, "label is NULL");
	mf_retm_if(fullpath == NULL, "path_info is NULL");
	char *pathinfo = NULL;
	char *info = NULL;

	if (root) {
		info = g_strdup("Root");
	} else {
		pathinfo = mf_fm_svc_path_info_get(fullpath);
		info = mf_fm_svc_path_info_translate(pathinfo, MF_PATH_INFO_MAX_LENGTH_LANDSCAPE);
	}

	elm_object_part_text_set(label, "info", info);
	MF_TRACE_END;
}


void mf_split_view_init(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata*)data;
	Evas_Object *pLeftGenlist = NULL;
	/*** create the panes */

	ap->mf_MainWindow.sSplitData.pPanes = mf_widget_create_panes(ap->mf_MainWindow.pMainLayout);
	evas_object_show(ap->mf_MainWindow.sSplitData.pPanes);

	ap->mf_MainWindow.sSplitData.pMainLayout = mf_widget_create_layout_main(ap->mf_MainWindow.sSplitData.pPanes);

	/**Set left size*/
	elm_panes_content_left_size_set(ap->mf_MainWindow.sSplitData.pPanes, LEFT_SCALE);


	/** create the left folder genlist **/
	pLeftGenlist = mf_widget_create_genlist(ap->mf_MainWindow.sSplitData.pPanes);
	evas_object_smart_callback_add(pLeftGenlist, "expanded", _gl_exp_ndepth, ap);
	evas_object_smart_callback_add(pLeftGenlist, "contracted", _gl_con, ap);

	//mf_widget_resize(pLeftGenlist, LEFT_W, LEFT_H);
	ap->mf_MainWindow.sSplitData.pSplitLeftGenlist = pLeftGenlist;
	evas_object_show(pLeftGenlist);

	elm_object_part_content_set(ap->mf_MainWindow.sSplitData.pPanes, "left", pLeftGenlist);

	ap->mf_MainWindow.sSplitData.bFlagSplitViewCreate = TRUE;
	MF_TRACE_END;

}
void mf_split_view_destory(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata*)data;

	//SAFE_FREE_OBJ(ap->mf_MainWindow.sSplitData.pSelectAllCheckBox);
	//SAFE_FREE_OBJ(ap->mf_MainWindow.sSplitData.pSelectAllLayout);
	SAFE_FREE_OBJ(ap->mf_MainWindow.sSplitData.pSplitLeftGenlist);
	SAFE_FREE_OBJ(ap->mf_MainWindow.sSplitData.pPanes);
	SAFE_FREE_OBJ(ap->mf_MainWindow.sSplitData.pMainLayout);
	ap->mf_MainWindow.sSplitData.bFlagSplitViewCreate = FALSE;
	MF_TRACE_END;
}

void mf_split_view_hide(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata*)data;
	evas_object_hide(ap->mf_MainWindow.sSplitData.pSplitLeftGenlist);
	evas_object_hide(ap->mf_MainWindow.sSplitData.pPanes);
	evas_object_hide(ap->mf_MainWindow.sSplitData.pMainLayout);
	MF_TRACE_END;
}

void mf_split_view_show(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata*)data;
	evas_object_show(ap->mf_MainWindow.sSplitData.pSplitLeftGenlist);
	evas_object_show(ap->mf_MainWindow.sSplitData.pPanes);
	evas_object_show(ap->mf_MainWindow.sSplitData.pMainLayout);
	MF_TRACE_END;
}

void mf_split_view_genlist_append(void *data, Evas_Object *pGenlist, GString *fullpath, Elm_Object_Item *parent_item)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata*)data;
	Elm_Object_Item *parent = NULL;
	Eina_List *dir_list = NULL;
	Eina_List *file_list = NULL;
	mfItemData_s *it_data;
	int iSortTypeValue = 0;
	it_data = elm_object_item_data_get(parent_item);
	mf_retm_if(it_data == NULL, "it_data is NULL");
	mf_retm_if(it_data->m_ItemName == NULL, "it_data->m_ItemName is NULL");
	mf_retm_if(it_data->m_ItemName->str == NULL, "it_data->m_ItemName->str is NULL");

	mf_debug("fullpath is [%s] item is [%s]", fullpath->str, it_data->m_ItemName->str);

	mf_fm_svc_wrapper_get_file_list(it_data->m_ItemName->str, &dir_list, &file_list);


	mf_util_get_vconf_value(VCONF_TYPE_SORT_TYPE, &iSortTypeValue);

	mf_fs_oper_sort_list(&file_list, iSortTypeValue);

	if (iSortTypeValue == MYFILE_SORT_BY_NAME_A2Z || iSortTypeValue == MYFILE_SORT_BY_NAME_Z2A || iSortTypeValue == MYFILE_SORT_BY_DATE_R2O
	    || iSortTypeValue == MYFILE_SORT_BY_DATE_O2R) {
		mf_fs_oper_sort_list(&dir_list, iSortTypeValue);
	} else {
		mf_fs_oper_sort_list(&dir_list, ap->mf_Status.iFolderSortType);
	}
	if (dir_list == NULL || eina_list_count(dir_list) == 0) {
		MF_TRACE_END;
		return;
	}

	if (g_string_equal(fullpath, it_data->m_ItemName)) {
		it_data->flagExpand = true;
		elm_object_item_data_set(it_data->item, it_data);

		elm_genlist_item_update(it_data->item);
		mf_genlist_split_left_items_append(ap, pGenlist, parent_item, dir_list);

		MF_TRACE_END;
		return;
	} else {
		it_data->flagExpand = true;
		elm_object_item_data_set(it_data->item, it_data);

		elm_genlist_item_update(it_data->item);
		parent = mf_genlist_split_left_items_append(ap, pGenlist, parent_item, dir_list);
		mf_split_view_genlist_append(ap, pGenlist, fullpath, parent);
	}
	MF_TRACE_END;

}

Elm_Object_Item *mf_split_view_locate_item_get(void *data, int locate)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(data == NULL, NULL, "data is NULL");
	struct appdata *ap = (struct appdata*)data;

	Elm_Object_Item *item = NULL;

	switch (locate) {
	case MYFILE_MMC:
		item = ap->mf_MainWindow.sSplitData.mmc_item;
		break;
	case MYFILE_OTG:
		item = ap->mf_MainWindow.sSplitData.otg_item;
		break;
	case MYFILE_PHONE:
	default:
		item = ap->mf_MainWindow.sSplitData.phone_item;
		break;
	}
	return item;
}

static void _gl_con(void *data, Evas_Object *obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	if (ap->mf_Status.flagNaviPush) {
		mf_debug("the transition of the naviframe is not finished");
		return;
	}
	if (ap->mf_Status.more != MORE_EDIT) {
		Elm_Object_Item *item = event_info;
		elm_genlist_item_subitems_clear(item);
		SAFE_FREE_OBJ(ap->mf_MainWindow.pContextPopup);
		mfItemData_s *selected = (mfItemData_s *) elm_object_item_data_get(item);
		mf_error("g_strcmp0(ap->mf_Status.path->str, selected->m_ItemName->str) %d", g_strcmp0(ap->mf_Status.path->str, selected->m_ItemName->str));
		if (g_strcmp0(ap->mf_Status.path->str, selected->m_ItemName->str)) {
			myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
			SAFE_FREE_GSTRING(ap->mf_Status.path);
			ap->mf_Status.path = g_string_new(selected->m_ItemName->str);
			SAFE_FREE_CHAR(pNavi_s->pCurrentPath);
			pNavi_s->pCurrentPath = g_strdup(selected->m_ItemName->str);
			if (ap->mf_Status.more == MORE_DEFAULT) {
				mf_navi_bar_create_default_view(ap);
				mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);
				mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_OTG_ROOT, true);
				//mf_ctrl_bar_item_set_disable(ap->mf_MainWindow.pNaviCtrlBar, CTRL_DISABLE_OTG_ROOT, TRUE);
			} else if (ap->mf_Status.more == MORE_INTERNAL_COPY_MOVE || ap->mf_Status.more == MORE_INTERNAL_COPY || ap->mf_Status.more == MORE_INTERNAL_MOVE
				   || ap->mf_Status.more == MORE_DATA_COPYING || ap->mf_Status.more == MORE_DATA_MOVING) {
				mf_navi_bar_create_path_select_view(ap);
				mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);
				mf_navi_bar_button_set_disable(ap->mf_MainWindow.pNaviItem, CTRL_DISABLE_OTG_ROOT, true);
				//mf_ctrl_bar_item_set_disable(ap->mf_MainWindow.pNaviCtrlBar, CTRL_DISABLE_OTG_ROOT, TRUE);
			}
		}

	}
	MF_TRACE_END;
}

static void _gl_exp_ndepth(void *data, Evas_Object *obj, void *event_info)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	Elm_Object_Item *item = event_info;

	if (item != NULL) {
		mfItemData_s *selected = (mfItemData_s *) elm_object_item_data_get(item);
		//elm_genlist_item_selected_set(item, FALSE);

		if (ap->mf_Status.flagNaviPush) {
			mf_debug("the transition of the naviframe is not finished");
			return;
		}
		SAFE_FREE_OBJ(ap->mf_MainWindow.pContextPopup);
		if (ap->mf_Status.more == MORE_RENAME) {
			if (elm_genlist_item_flip_get(item)) {
				MF_TRACE_END;
				return;
			} else {
				mf_callback_rename_save_cb(ap, NULL, NULL);
				return;
			}
		} else if (ap->mf_Status.more != MORE_EDIT) {
			if (g_string_equal(selected->m_ItemName, ap->mf_Status.path)) {

				myfileNaviBar *pNavi_s = mf_navi_bar_get_in_use(ap);
				SAFE_FREE_GSTRING(ap->mf_Status.path);
				ap->mf_Status.path = g_string_new(selected->m_ItemName->str);
				SAFE_FREE_CHAR(pNavi_s->pCurrentPath);
				pNavi_s->pCurrentPath = g_strdup(selected->m_ItemName->str);

				mf_split_view_genlist_append(ap, ap->mf_MainWindow.sSplitData.pSplitLeftGenlist,
							     ap->mf_Status.path, item);
			} else {
				mf_split_view_genlist_append(ap, ap->mf_MainWindow.sSplitData.pSplitLeftGenlist,
							     ap->mf_Status.path, item);
				if (!g_string_equal(selected->m_ItemName, ap->mf_Status.path)) {
					mf_callback_left_click_cb(data, selected->m_ItemName, item);
				}

			}

			}
	}

	MF_TRACE_END;
}

void mf_split_view_left_genlist_refresh(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata*)data;
	Evas_Object *pGenlist = NULL;

	GString *fullpath = ap->mf_Status.path;
	mf_retm_if(fullpath == NULL, "fullpath is NULL");
	//Eina_List *dir_list = NULL;


	mf_widget_unset_part_content(ap->mf_MainWindow.sSplitData.pPanes, "left");
	SAFE_FREE_OBJ(ap->mf_MainWindow.sSplitData.pSplitLeftGenlist);
	/** create the left folder genlist **/
	pGenlist = mf_widget_create_genlist(ap->mf_MainWindow.sSplitData.pPanes);
	evas_object_smart_callback_add(pGenlist, "expanded", _gl_exp_ndepth, ap);
	evas_object_smart_callback_add(pGenlist, "contracted", _gl_con, ap);

	//mf_widget_resize(pLeftGenlist, LEFT_W, LEFT_H);
	ap->mf_MainWindow.sSplitData.pSplitLeftGenlist = pGenlist;

	//elm_genlist_mode_set(pGenlist, ELM_LIST_SCROLL); /* Back to default */
	//elm_scroller_policy_set(pGenlist,EINA_TRUE, EINA_TRUE);
	evas_object_show(pGenlist);

	elm_object_part_content_set(ap->mf_MainWindow.sSplitData.pPanes, "left", pGenlist);


	//pGenlist = ap->mf_MainWindow.sSplitData.pSplitLeftGenlist;
	mf_retm_if(pGenlist == NULL, "pGenlist is NULL");
	Elm_Object_Item *parent_item = NULL;
	mf_genlist_clear(pGenlist);
	mf_genlist_split_left_genlist_style(ap);
	mf_genlist_split_root_items_append(pGenlist, ap);


	int locate = mf_fm_svc_wrapper_get_location(fullpath->str);

	parent_item = mf_split_view_locate_item_get(ap, locate);

	mf_split_view_genlist_append(ap, pGenlist, fullpath, parent_item);
		//error_code = mf_fm_svc_wrapper_get_file_list(path, &dir_list, file_list);
	MF_TRACE_END;
}

bool mf_split_view_refresh(void *data)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(data == NULL, FALSE, "data is NULL");
	struct appdata *ap = (struct appdata*)data;
	//update the left genlist items
	mf_split_view_left_genlist_refresh(ap);

	MF_TRACE_END;
	return TRUE;

}
#endif
