/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <app.h>

#include "mf-widget.h"
#include "mf-callback.h"
#include "mf-fm-svc-wrapper.h"
#include "mf-util.h"
#include "mf-language-mgr.h"
#include "mf-ta.h"

/******************************
** Prototype    : mf_tab_bar_create_tab_bar
** Description  : Samsung
** Input        : Evas_Object *parent
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
Evas_Object *mf_tab_bar_create_tab_bar(Evas_Object *parent)
{
	MF_TRACE_BEGIN;
	mf_retv_if(parent == NULL, NULL);
	Evas_Object *toolbar = elm_toolbar_add(parent);
	elm_toolbar_shrink_mode_set(toolbar, ELM_TOOLBAR_SHRINK_EXPAND);
	elm_object_style_set(toolbar, "tabbar");
	//elm_toolbar_homogeneous_set(toolbar, EINA_FALSE);
	elm_toolbar_transverse_expanded_set(toolbar, EINA_TRUE);
	//set_rotate_cb_for_winset(__rotate_tabbar_cb, toolbar);
	MF_TRACE_END;
	return toolbar;
}

Evas_Object *mf_tab_bar_create_tab_layout(Evas_Object *tab_bar)
{
	Evas_Object *layout = elm_layout_add(tab_bar);
	elm_layout_theme_set(layout, "layout", "tabbar", "default");
	return layout;

}
/******************************
** Prototype    : mf_tab_bar_append_item
** Description  : Samsung
** Input        : Evas_Object *pTabBar
**                char *pTabIconPath
**                char *pTabBarLabel
**                Evas_Object *pTabView
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
Elm_Object_Item *mf_tab_bar_append_item(Evas_Object *pTabBar, char *pTabIconPath, char *pTabBarLabel, void *user_data)
{
	mf_retv_if(pTabBar == NULL, NULL);
	Elm_Object_Item *pItem = NULL;
	//pItem = elm_toolbar_item_append(pTabBar, pTabIconPath, pTabBarLabel, mf_callback_tab_cb, user_data);
	return pItem;
}



/******************************
** Prototype    : mf_tab_bar_set_item_view
** Description  : Samsung
** Input        : Elm_Object_Item *it
**                Evas_Object * view
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_tab_bar_set_item_view(Evas_Object *parent, Evas_Object *tab_layout, Evas_Object *view)
{
	MF_TRACE_BEGIN;
	mf_retm_if(parent == NULL, "parent is NULL");
	mf_retm_if(tab_layout == NULL, "tab_layout is NULL");
	mf_retm_if(view == NULL, "view is NULL");
	Evas_Object *layout = tab_layout;
	Evas_Object *unset = elm_object_part_content_get(layout, "elm.swallow.content");
	if (unset) {
		mf_widget_unset_part_content(layout, "elm.swallow.content");
		evas_object_hide(unset);
	}

	elm_object_part_content_set(layout, "elm.swallow.content", view);
	MF_TRACE_END;
}


/******************************
** Prototype    : mf_tab_bar_get_item_by_label
** Description  : Samsung
** Input        : Evas_Object *pTabBar
**                char *pTabLabel
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
Elm_Object_Item *mf_tab_bar_get_item_by_label(Evas_Object *pTabBar, char *pTabLabel)
{
	mf_retvm_if(pTabBar == NULL, NULL, "pTabBar is NULL");
	mf_retvm_if(pTabLabel == NULL, NULL, "pTabLabel is NULL");
	Elm_Object_Item *pItem = NULL;

	pItem = elm_toolbar_first_item_get(pTabBar);
	while(pItem) {
		if (g_strcmp0(pTabLabel, elm_object_item_text_get(pItem)) == 0) {
			return pItem;
		}
		pItem = elm_toolbar_item_next_get(pItem);
	}
	return NULL;
}


/******************************
** Prototype    : mf_tab_bar_remove_item_by_label
** Description  : Samsung
** Input        : Evas_Object *pTabBar
**                char *pTabLabel
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_tab_bar_remove_item_by_label(Evas_Object *pTabBar, char *pTabLabel)
{
	Elm_Object_Item *pItem = NULL;
	pItem = elm_toolbar_first_item_get(pTabBar);

	while(pItem) {
		if (g_strcmp0(pTabLabel, elm_object_item_text_get(pItem)) == 0) {
			mf_language_mgr_unregister_object_item(pItem);
			elm_object_item_del(pItem);
			return;
		}
		pItem = elm_toolbar_item_next_get(pItem);
	}
}


/******************************
** Prototype    : mf_tab_bar_set_selected_item
** Description  : Samsung
** Input        : Elm_Object_Item* pItem
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_tab_bar_set_selected_item(Elm_Object_Item *pItem)
{
	mf_retm_if(pItem == NULL, "pItem is NULL");
	elm_toolbar_item_selected_set(pItem, EINA_TRUE);
}


/******************************
** Prototype    : mf_tab_bar_set_item_disabled
** Description  : Samsung
** Input        : Evas_Object *pTabBar
**                Eina_Bool disable
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_tab_bar_set_item_disabled(Evas_Object *pTabBar, Eina_Bool disable)
{
	Elm_Object_Item *pCurrentItem = NULL;
	if (pTabBar != NULL) {

		pCurrentItem = elm_toolbar_first_item_get(pTabBar);
		while(pCurrentItem) {
			elm_object_item_disabled_set(pCurrentItem, disable);
			pCurrentItem = elm_toolbar_item_next_get(pCurrentItem);
		}
	}
}


int mf_tab_bar_items_count_get(Evas_Object *pTabBar)
{
	mf_retvm_if(pTabBar == NULL, 0, "pTabBar is NULL");

	Elm_Object_Item *pCurrentItem = NULL;
	int count = 0;

	pCurrentItem = elm_toolbar_first_item_get(pTabBar);
	while(pCurrentItem) {
		count++;
		pCurrentItem = elm_toolbar_item_next_get(pCurrentItem);
	}

	return count;
}

void mf_tab_bar_items_del(Evas_Object *pTabBar)
{
	mf_retm_if(pTabBar == NULL, "pTabBar does not exist");
	Elm_Object_Item *it = NULL;
	Elm_Object_Item *pNext = NULL;

	it = elm_toolbar_first_item_get(pTabBar);
	while(it) {
		pNext = elm_toolbar_item_next_get(it);
		mf_language_mgr_unregister_object_item(it);
		elm_object_item_del(it);
		it = pNext;
	}
}

void mf_tab_bar_generate(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata*)data;

	Evas_Object *pContent = NULL;
	Elm_Object_Item *pItem = NULL;

	MF_TA_ACUM_ITEM_END("	   storage check", 0);
	if (ap->mf_Status.iStorageState > MYFILE_PHONE) {
		MF_TA_ACUM_ITEM_BEGIN("      create_tabbar", 0);
		Evas_Object *parent = ap->mf_MainWindow.pNaviBar;
		/*create tab bar**/
		MF_TA_ACUM_ITEM_BEGIN("      mf_tab_bar_create_tab_bar", 0);
		if (ap->mf_MainWindow.pTabBar == NULL) {
			ap->mf_MainWindow.pTabBar = mf_tab_bar_create_tab_bar(parent);
		}
		MF_TA_ACUM_ITEM_END("      mf_tab_bar_create_tab_bar", 0);

		MF_TA_ACUM_ITEM_BEGIN("      tab bar content set", 0);

		pContent = ap->mf_MainWindow.pTabBar;
		MF_TA_ACUM_ITEM_BEGIN("      tab bar phone item add", 0);

		MF_TA_ACUM_ITEM_BEGIN("        set phone item", 0);
		if(!mf_tab_bar_get_item_by_label(pContent, GET_SYS_STR(MF_LABEL_PHONE))) {
			pItem = mf_tab_bar_append_item(pContent, MF_ICON_PHONE, GET_SYS_STR(MF_LABEL_PHONE), ap);
			mf_language_mgr_register_object_item(pItem, MF_LABEL_PHONE, ITEM_TYPE_TAB);
		}
		MF_TA_ACUM_ITEM_END("        set phone item", 0);
		MF_TA_ACUM_ITEM_END("      tab bar phone item add", 0);

		MF_TA_ACUM_ITEM_BEGIN("      tab bar MMC item add", 0);
		if (ap->mf_Status.iStorageState & MYFILE_MMC) {
			MF_TA_ACUM_ITEM_BEGIN("        set mmc item", 0);
			if(!mf_tab_bar_get_item_by_label(pContent, GET_SYS_STR(MF_LABEL_MMC))) {
				pItem = mf_tab_bar_append_item(pContent, MF_ICON_MEMORY, GET_SYS_STR(MF_LABEL_MMC), ap);
				mf_language_mgr_register_object_item(pItem, MF_LABEL_MMC, ITEM_TYPE_TAB);
			}
			MF_TA_ACUM_ITEM_END("        set mmc item", 0);
		}
		MF_TA_ACUM_ITEM_END("      tab bar MMC item add", 0);
#ifdef MYFILE_USB_OTG
		MF_TA_ACUM_ITEM_BEGIN("      tab bar OTG item add", 0);
		if (ap->mf_Status.iStorageState & MYFILE_OTG) {
			if(!mf_tab_bar_get_item_by_label(pContent, GET_STR(MF_LABEL_OTG))) {
				pItem = mf_tab_bar_append_item(pContent, MF_ICON_MEMORY, GET_STR(MF_LABEL_OTG), ap);
				mf_language_mgr_register_object_item(pItem, MF_LABEL_OTG, ITEM_TYPE_TAB);
			}
		}
		MF_TA_ACUM_ITEM_END("      tab bar OTG item add", 0);
#endif
		MF_TA_ACUM_ITEM_END("      tab bar content set", 0);

		elm_toolbar_select_mode_set(pContent, ELM_OBJECT_SELECT_MODE_ALWAYS);

		MF_TA_ACUM_ITEM_END("      create_tabbar", 0);
	}
	MF_TRACE_END;
}

