/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */




#include "mf-main.h"
#include "mf-callback.h"
#include "mf-search.h"
#include "mf-split-view.h"
#include "mf-widget.h"
#include "mf-fm-svc-wrapper.h"
#include "mf-resource.h"
#include "mf-ta.h"
#include "mf-language-mgr.h"

extern Elm_Genlist_Item_Class itc;
extern Elm_Genlist_Item_Class userfolderitc;
extern Elm_Genlist_Item_Class categoryitc;
extern Elm_Gengrid_Item_Class gic;



Evas_Object *mf_widget_create_genlist(Evas_Object *parent)
{
	Evas_Object *genlist = NULL;
	genlist = elm_genlist_add(parent);
	//elm_genlist_homogeneous_set(genlist, EINA_TRUE);
	//elm_object_focus_set(genlist, EINA_FALSE);
	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);
	return genlist;
}

Evas_Object *mf_widget_create_box(Evas_Object * parent)
{
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");

	Evas_Object *box = NULL;
	box = elm_box_add(parent);
	mf_retvm_if(box == NULL, NULL, "box is NULL");
	elm_object_focus_set(box, EINA_FALSE);
	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_FILL);
	evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_box_clear(box);
	evas_object_show(box);
	return box;

}

Evas_Object *mf_widget_create_layout(Evas_Object *parent, const char *edj, const char *grp_name)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");
	mf_retvm_if(edj == NULL, NULL, "edj is NULL");
	mf_retvm_if(grp_name == NULL, NULL, "grp_name is NULL");

	Evas_Object *layout = NULL;
	layout = elm_layout_add(parent);
	mf_retvm_if(layout == NULL, NULL, "layout is NULL");
	elm_object_focus_set(layout, EINA_FALSE);
	elm_layout_file_set(layout, edj, grp_name);
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(layout);
	MF_TRACE_END;
	return layout;

}

Evas_Object *mf_widget_create_conform(Evas_Object * parent)
{
	mf_retv_if(parent == NULL, NULL);

	Evas_Object *conform;

	conform = elm_conformant_add(parent);
	evas_object_size_hint_weight_set(conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(conform);

	return conform;
}

Evas_Object *mf_widget_create_check_box(Evas_Object *parent)
{
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");

	Evas_Object *checkbox = NULL;
	checkbox = elm_check_add(parent);
	elm_object_focus_set(checkbox, EINA_FALSE);
	evas_object_propagate_events_set(checkbox, EINA_FALSE);
	evas_object_show(checkbox);
	return checkbox;
}

Evas_Object *mf_widget_create_no_content(Evas_Object *parent)
{
	Evas_Object *nocontent = elm_layout_add(parent);
	elm_layout_theme_set(nocontent, "layout", "nocontents", "text");
	elm_object_focus_set(nocontent, EINA_FALSE);
	elm_object_part_text_set(nocontent, "elm.text", GET_SYS_STR(MF_LABEL_NO_FILES));
	mf_language_mgr_register_object(nocontent, OBJ_TYPE_EDJE_OBJECT, "elm.text", MF_LABEL_NO_FILES);
	return nocontent;
}

Evas_Object *mf_widget_create_select_all_layout(Evas_Object *parent)
{

	mf_retvm_if(parent == NULL, NULL, "parent is NULL");

	Evas_Object *pSelectedAllLayout = NULL;
	pSelectedAllLayout = elm_layout_add(parent);
	elm_object_focus_set(pSelectedAllLayout, EINA_FALSE);
	elm_layout_theme_set(pSelectedAllLayout, "genlist", "item", "select_all/default");
	evas_object_size_hint_weight_set(pSelectedAllLayout, EVAS_HINT_EXPAND, EVAS_HINT_FILL);
	evas_object_size_hint_align_set(pSelectedAllLayout, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_part_text_set(pSelectedAllLayout, "elm.text", GET_SYS_STR(MF_LABEL_SELECT_ALL));
	mf_language_mgr_register_object(pSelectedAllLayout, OBJ_TYPE_EDJE_OBJECT, "elm.text", MF_LABEL_SELECT_ALL);
	evas_object_show(pSelectedAllLayout);
	return pSelectedAllLayout;
}

Evas_Object *mf_widget_create_editfield(Evas_Object *parent, Evas_Object **pEntry)
{

	mf_retvm_if(parent == NULL, NULL, "parent is NULL");
	Evas_Object *layout = elm_layout_add(parent);
	elm_layout_theme_set(layout, "layout", "editfield", "default");
	Evas_Object *entry = elm_entry_add(parent);
	elm_object_style_set(entry, "default");
	elm_entry_cnp_mode_set(entry, ELM_CNP_MODE_PLAINTEXT);
	elm_object_part_content_set(layout, "elm.swallow.content", entry);
	*pEntry = entry;
	return layout;
}

Evas_Object *mf_widget_create_search_bar(Evas_Object *parent, Evas_Object **pEntry)
{
	Evas_Object *searchbar_layout = elm_layout_add(parent);
	elm_layout_theme_set(searchbar_layout, "layout", "searchbar", "default");
	Evas_Object *entry = elm_entry_add(searchbar_layout);
	elm_entry_scrollable_set(entry, EINA_TRUE);
	elm_entry_single_line_set(entry, EINA_TRUE);
	elm_entry_cnp_mode_set(entry, ELM_CNP_MODE_PLAINTEXT);
	elm_object_part_content_set(searchbar_layout, "elm.swallow.content", entry);
	*pEntry = entry;
	return searchbar_layout;
}

Evas_Object *mf_widget_create_panes(Evas_Object *parent)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");

	Evas_Object *panes = NULL;
	panes = elm_panes_add(parent);
	mf_retvm_if(panes == NULL, NULL, "panes is NULL");
	elm_object_focus_set(panes, EINA_FALSE);
	evas_object_size_hint_weight_set(panes, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(panes, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_panes_fixed_set(panes, EINA_FALSE);
	evas_object_show(panes);
	MF_TRACE_END;
	return panes;
}

Evas_Object *mf_widget_create_layout_main(Evas_Object * parent)
{
	Evas_Object *layout;

	mf_retv_if(parent == NULL, NULL);

	layout = elm_layout_add(parent);
	mf_retvm_if(layout == NULL, NULL, "Failed elm_layout_add.\n");
	elm_object_focus_set(layout, EINA_FALSE);

	MF_TA_ACUM_ITEM_BEGIN("      elm_layout_theme_set", 0);
	elm_layout_theme_set(layout, "layout", "application", "default");
	MF_TA_ACUM_ITEM_END("      elm_layout_theme_set", 0);

	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
	//elm_win_resize_object_add(parent, layout);

	evas_object_show(layout);

	return layout;
}


Evas_Object *mf_widget_create_cancel_button_on_search(Evas_Object *parent)
{
	Evas_Object *cancel_btn = elm_button_add(parent);
	elm_object_part_content_set(parent, "button_cancel", cancel_btn);
	elm_object_style_set(cancel_btn, "searchbar/default");
	elm_object_text_set(cancel_btn, GET_SYS_STR(LABEL_CANCEL));
	mf_language_mgr_register_object(cancel_btn, OBJ_TYPE_ELM_OBJECT, NULL, LABEL_CANCEL);
	return cancel_btn;
}

void mf_widget_item_remove(Evas_Object *parent, const char *path, int type)
{
	mf_debug("path is [%s]", path);
	mf_retm_if(parent == NULL, "parent is NULL");
	mf_retm_if(path == NULL, "path is NULL");

	mfItemData_s *itemData = NULL;
	Elm_Object_Item *it = NULL;

	if (type == MF_VIEW_STYLE_THUMBNAIL) {
		it = elm_gengrid_first_item_get(parent);
	} else {
		it = elm_genlist_first_item_get(parent);
	}
	while (it) {
		itemData = elm_object_item_data_get(it);
		if (itemData->m_ItemName == NULL || itemData->m_ItemName->str == NULL) {
			continue;
		}
		mf_debug("itemData->m_ItemName->str is [%s]", itemData->m_ItemName->str);
		if (g_strcmp0(path, itemData->m_ItemName->str) == 0) {
			elm_object_item_del(it);
			break;
		}

		if (type == MF_VIEW_STYLE_THUMBNAIL) {
			it = elm_gengrid_item_next_get(it);
		} else {
			it = elm_genlist_item_next_get(it);
		}
	}

}

void mf_widget_items_remove(Evas_Object *parent, int storage, int type)
{
	mf_retm_if(parent == NULL, "parent is NULL");

	mfItemData_s *itemData = NULL;
	Elm_Object_Item *it = NULL;
	Elm_Object_Item *next_it = NULL;

	if (type == MF_VIEW_STYLE_THUMBNAIL) {
		it = elm_gengrid_first_item_get(parent);
	} else {
		it = elm_genlist_first_item_get(parent);
	}
	while (it) {
		itemData = elm_object_item_data_get(it);
		if (itemData->m_ItemName == NULL || itemData->m_ItemName->str == NULL) {
			continue;
		}
		if (mf_fm_svc_wrapper_get_location(itemData->m_ItemName->str) == storage){
			if (type == MF_VIEW_STYLE_THUMBNAIL) {
				next_it = elm_gengrid_item_next_get(it);
			} else {
				next_it = elm_genlist_item_next_get(it);
			}
			elm_object_item_del(it);
			it = next_it;
		} else {
			if (type == MF_VIEW_STYLE_THUMBNAIL) {
				it = elm_gengrid_item_next_get(it);
			} else {
				it = elm_genlist_item_next_get(it);
			}
		}
	}

}

Elm_Object_Item *mf_widget_item_append(Evas_Object *parent, fsNodeInfo *pNode, void *data)
{
	mf_retvm_if(parent == NULL, NULL, "pGenlist is NULL");
	mf_retvm_if(pNode == NULL, NULL, "pNode is NULL");
	mf_retvm_if(pNode->path == NULL, NULL, "pNode->path is NULL");
	mf_retvm_if(pNode->name == NULL, NULL, "pNode->name is NULL");
	mf_retvm_if(data == NULL, NULL, "data is NULL");

	char *real_name = NULL;
	mfItemData_s *m_TempItem = NULL;
	struct appdata *ap = (struct appdata *)data;

	real_name = g_strconcat(pNode->path, "/", pNode->name, NULL);
	if (real_name == NULL) {
		return NULL;
	}

	mf_genlist_create_data(&m_TempItem, real_name, data);

	if (m_TempItem == NULL) {
		free(real_name);
		real_name = NULL;
		return NULL;
	}

	if (ap->mf_Status.view_type == mf_view_root) {
		if(g_strcmp0(real_name, PHONE_FOLDER) == 0) {
			m_TempItem->thumb_path = strdup(MF_ICON_ITEM_PHONE);
			m_TempItem->real_thumb_flag = true;
		} else if(g_strcmp0(real_name, MEMORY_FOLDER) == 0) {
			m_TempItem->thumb_path = strdup(MF_ICON_ITEM_MMC);
			m_TempItem->real_thumb_flag = true;
		} else if(g_strcmp0(real_name, OTG_FOLDER) == 0) {
			m_TempItem->thumb_path = strdup(MF_ICON_ITEM_MMC);
			m_TempItem->real_thumb_flag = true;
		}
		/*else if(g_strcmp0(real_name, DROP_BOX_FOLDER) == 0) {
			m_TempItem->thumb_path = strdup(MF_ICON_ITEM_DB);
		}*/
	}

	Elm_Object_Item *it = NULL;

	if (ap->mf_Status.flagViewType == MF_VIEW_SYTLE_LIST_DETAIL) {
		char *tempdate = mf_util_icu_translate(ap, pNode->date);
		if (tempdate == NULL)
			m_TempItem->create_date = g_strdup("N/A");
		else {
			m_TempItem->create_date = g_strdup(tempdate);
			free(tempdate);
		}
		mf_file_attr_get_file_size_info(&(m_TempItem->size), pNode);
	}
	if (ap->mf_Status.flagViewType == MF_VIEW_STYLE_LIST || ap->mf_Status.flagViewType == MF_VIEW_SYTLE_LIST_DETAIL) {
		if (ap->mf_Status.view_type == mf_view_root_category) {
			it = elm_genlist_item_append(parent, &categoryitc, m_TempItem, NULL, ELM_GENLIST_ITEM_NONE, mf_genlist_gl_sel, m_TempItem);
		} else {
			it = elm_genlist_item_append(parent, &userfolderitc, m_TempItem, NULL, ELM_GENLIST_ITEM_NONE, mf_genlist_gl_sel, m_TempItem);
		}
	} else {
		it = elm_gengrid_item_append(parent, &gic, m_TempItem, NULL, NULL);
	}

	m_TempItem->item = it;
	free(real_name);
	return it;
}

Evas_Object *mf_widget_get_part_content(Evas_Object *parent, const char *part)
{
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");
	mf_retvm_if(part == NULL, NULL, "part is NULL");

	Evas_Object *content = NULL;
	content = elm_object_part_content_get(parent, part);
	return content;
}

Evas_Object *mf_widget_unset_part_content(Evas_Object *parent, const char *part)
{
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");
	mf_retvm_if(part == NULL, NULL, "part is NULL");

	Evas_Object *content = NULL;
	content = elm_object_part_content_unset(parent, part);
	return content;
}

Evas_Object *mf_widget_util_create_button(Evas_Object *parent, char *style,
				   char *caption, Evas_Object *icon,
				   void (*func) (void *, Evas_Object *, void *),
				   void *data)
{
	if (!parent)
		return NULL;

	Evas_Object *btn;

	btn = elm_button_add(parent);

	if (style) {
		elm_object_style_set(btn, style);
	}

	if (caption) {
		elm_object_text_set(btn, mf_util_get_text(caption));
		mf_language_mgr_register_object(btn, OBJ_TYPE_ELM_OBJECT, NULL, caption);
	}

	if (icon)
		elm_object_content_set(btn, icon);

	elm_object_focus_allow_set(btn, EINA_FALSE);
	evas_object_propagate_events_set(btn, EINA_FALSE);

	evas_object_smart_callback_add(btn, "clicked", func, (void *)data);

	return btn;
}

static Eina_Bool __mf_widget_notify_popup_timer_cb(void *data)
{
	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, EINA_FALSE, "ap is null");

	mf_widget_unset_part_content(ap->mf_MainWindow.pNaviLayout,
					    "sel_contents");

	SAFE_FREE_OBJ(ap->mf_MainWindow.pLabelLayout);
	ap->mf_MainWindow.pPopupTimer = NULL;
	return false;
}

Evas_Object *mf_widget_create_notify_popup(void *data,
				    Evas_Object *parent,
				    char *label,
				    mf_notify_popup_type type,
				    bool time_out,
				    void func(void *data, Evas_Object *obj, void *event_info))
{
	MF_TRACE_BEGIN;
	mf_retvm_if(data == NULL, NULL, "data is NULL");
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");
	mf_retvm_if(label == NULL, NULL, "label is NULL");

	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, NULL, "ap is NULL");

	Evas_Object *countlabel = NULL;
	if (ap->mf_MainWindow.pLabelLayout && ap->mf_MainWindow.pPopupTimer) {
	 	elm_object_part_text_set(ap->mf_MainWindow.pLabelLayout, "elm.text", label);
		countlabel = ap->mf_MainWindow.pLabelLayout;
	} else {
		mf_widget_unset_part_content(ap->mf_MainWindow.pNaviLayout,
						    "sel_contents");
		SAFE_FREE_OBJ(ap->mf_MainWindow.pLabelLayout);
		countlabel = elm_layout_add(parent);
		evas_object_size_hint_weight_set(countlabel,
						 EVAS_HINT_EXPAND,
						 EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(countlabel,
						EVAS_HINT_FILL,
						EVAS_HINT_FILL);
		elm_layout_theme_set(countlabel, "standard", "selectioninfo", "center_text");
		elm_object_part_text_set(countlabel, "elm.text", label);
		elm_object_part_content_set(parent,
					    "sel_contents",
					    countlabel);
		elm_object_signal_emit(countlabel, "show,selection,info", "elm");
	}

	SAFE_DEL_ECORE_TIMER(ap->mf_MainWindow.pPopupTimer);
	if (time_out) {
		ap->mf_MainWindow.pPopupTimer = ecore_timer_add(3, __mf_widget_notify_popup_timer_cb, ap);
	}

	evas_object_show(countlabel);
	MF_TRACE_END;
	return countlabel;


}

void mf_widget_create_entryfield(void *data,
			   Evas_Object *parent,
			   void (*changed_cb)(void *data, Evas_Object *obj, void *event_info),
			   void (*click_cb)(void *data, Evas_Object *obj, void *event_info))
{
	struct appdata *ap = (struct appdata *)data;
	mf_retm_if(ap == NULL, "ap is NULL");
	Evas_Object *entry = NULL;

	Evas_Object *edit_field = mf_widget_create_editfield(parent, &entry);

	elm_entry_single_line_set(entry, EINA_TRUE);
	elm_entry_scrollable_set(entry, EINA_TRUE);

	elm_object_signal_emit(edit_field, "elm,state,eraser,show", "elm");

	evas_object_size_hint_weight_set(edit_field, 1, 1);
	evas_object_size_hint_align_set(edit_field, 1, 0.5);
	//evas_object_show(edit_field);
	ap->mf_MainWindow.pEditField = edit_field;

	ap->mf_MainWindow.pEntry = entry;

	evas_object_smart_callback_add(entry, "changed", changed_cb, ap);
	elm_object_signal_callback_add(edit_field, "elm,eraser,clicked", "elm", mf_callback_eraser_clicked_cb, entry);

	evas_object_show(entry);
}

Evas_Object *mf_widget_button_create(Evas_Object *parent, const char *label)
{
	Evas_Object *btn = NULL;
	btn = elm_button_add(parent);
	if (label) {
		elm_object_text_set(btn, label);
		mf_language_mgr_register_object(btn, OBJ_TYPE_ELM_OBJECT, NULL, label);
	}
	return btn;

}
void mf_widget_panes_right_set(Evas_Object *panes, Evas_Object *content)
{
	mf_retm_if(panes == NULL, "panes is NULL");
	mf_retm_if(content == NULL, "content is NULL");
	elm_object_part_content_set(panes, "right", content);
}

void mf_widget_panes_left_set(Evas_Object *panes, Evas_Object *content)
{
	mf_retm_if(panes == NULL, "panes is NULL");
	mf_retm_if(content == NULL, "content is NULL");
	elm_object_part_content_set(panes, "left", content);
}


#ifdef MYFILE_SPLIT_VIEW
void mf_widget_unset_main_layout_content(Evas_Object *layout)
{
	Evas_Object *unSet = NULL;
	unSet = mf_widget_unset_part_content(layout, "elm.swallow.content");
	evas_object_hide(unSet);

}

void mf_widget_unset_tabbar_layout_content(Evas_Object *layout)
{
	mf_widget_unset_part_content(layout, "elm.swallow.content");
	mf_widget_unset_part_content(layout, "elm.swallow.tabbar");

}
#endif

void mf_widget_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h)
{
	if (obj) {
		evas_object_size_hint_min_set(obj, w, h);
		evas_object_size_hint_max_set(obj, w, h);
	}
}

myfileNaviBar *mf_widget_phone_storage_init(void *data)
{
	MF_TRACE_BEGIN;
	mf_retvm_if(data == NULL, NULL, "data is NULL");
	struct appdata *ap = (struct appdata*)data;

	Evas_Object *parent = ap->mf_MainWindow.pWindow;

	MF_TA_ACUM_ITEM_BEGIN("      mf_navi_bar_create_navi_bar", 0);
	ap->mf_MainWindow.pNaviBar = mf_navi_bar_create_navi_bar(parent);

	evas_object_smart_callback_add(ap->mf_MainWindow.pNaviBar, "title,clicked", mf_callback_naviframe_title_clicked_cb, NULL);

	MF_TA_ACUM_ITEM_END("      mf_navi_bar_create_navi_bar", 0);

	if (ap->mf_MainWindow.pNaviBar == NULL) {
		return NULL;
	}
	MF_TA_ACUM_ITEM_BEGIN("      make phone navi struct", 0);
	myfileNaviBar *pNavi_phone = malloc(sizeof(myfileNaviBar));
	mf_retvm_if(pNavi_phone == NULL, NULL, "pNavi_phone is NULL");
	memset(pNavi_phone, 0, sizeof(myfileNaviBar));
	pNavi_phone->naviFlagInUse = TRUE;
	pNavi_phone->pCurrentPath = g_strdup(ap->mf_Status.path->str);
	pNavi_phone->pre_more = MORE_DEFAULT;
	pNavi_phone->pNaviLabel = strdup(GET_SYS_STR(MF_LABEL_PHONE));
	/*insert phone navi into the navi_list */
	/**    Todo:   if we need to check ap->mf_MainWindow.plistNaviBar?*/
	ap->mf_MainWindow.plistNaviBar = eina_list_append(ap->mf_MainWindow.plistNaviBar, pNavi_phone);
	MF_TA_ACUM_ITEM_END("      make phone navi struct", 0);
	MF_TRACE_END;
	return pNavi_phone;
}


void mf_widget_create(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

#ifdef MYFILE_SPLIT_VIEW
	struct appdata *ap = (struct appdata*)data;
	if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
		if (ap->mf_MainWindow.sSplitData.bFlagSplitViewCreate == FALSE) {
			mf_split_view_init(ap);
		}
	}
#endif
	MF_TRACE_END;

}


#ifdef MYFILE_SPLIT_VIEW
void mf_widget_set_main_layout_content(void *data)
{

	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	struct appdata *ap = (struct appdata*)data;
	if (ap->mf_Status.more == MORE_SEARCH || ap->mf_Status.more == MORE_CREATE_FOLDER || (ap->mf_Status.more == MORE_RENAME && ap->mf_Status.preMore == MORE_SEARCH)) {
		return;
	}

	if (ap->mf_Status.view_type == mf_view_root_category || ap->mf_Status.view_type == mf_view_root) {
		return;
	}

	if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {

		mf_widget_unset_main_layout_content(ap->mf_MainWindow.pMainLayout);
		elm_object_part_content_set(ap->mf_MainWindow.sSplitData.pMainLayout, "elm.swallow.content", ap->mf_MainWindow.pNaviBar);
		mf_widget_panes_right_set(ap->mf_MainWindow.sSplitData.pPanes, ap->mf_MainWindow.sSplitData.pMainLayout);
		mf_split_view_show(ap);
		elm_object_part_content_set(ap->mf_MainWindow.pMainLayout, "elm.swallow.content", ap->mf_MainWindow.sSplitData.pPanes);

	} else {
		mf_split_view_unset_right_content(ap);
		mf_widget_unset_main_layout_content(ap->mf_MainWindow.pMainLayout);
		mf_split_view_hide(ap);
		evas_object_show(ap->mf_MainWindow.pMainLayout);
		elm_object_part_content_set(ap->mf_MainWindow.pMainLayout, "elm.swallow.content", ap->mf_MainWindow.pNaviBar);
	}

	MF_TRACE_END;

}
#endif
void mf_widget_refresh_view(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	struct appdata *ap = (struct appdata*)data;

	if (ap->mf_Status.more == MORE_DEFAULT) {
		mf_navi_bar_create_default_view(ap);
	} else if (ap->mf_Status.more == MORE_INTERNAL_COPY_MOVE || ap->mf_Status.more == MORE_INTERNAL_COPY || ap->mf_Status.more == MORE_INTERNAL_MOVE
		   || ap->mf_Status.more == MORE_DATA_COPYING || ap->mf_Status.more == MORE_DATA_MOVING) {
		mf_navi_bar_create_path_select_view(ap);
	} else if (ap->mf_Status.more == MORE_EDIT) {
		mf_callback_edit_cb(ap, NULL, NULL);
	}

#ifdef MYFILE_SPLIT_VIEW
	if (ap->mf_Status.rotation_type == MF_ROTATE_LANDSCAPE) {
		mf_split_view_refresh(ap);
	}
#endif
	mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);
	MF_TRACE_END;
}

void mf_widget_set_navi_blocking(void * data,Eina_Bool is_locked)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	struct appdata *ap = (struct appdata*)data;
	ap->mf_Status.flagNaviPush = is_locked;
}


#ifdef MYFILE_MOTION_FEATURE
void mf_widget_first_item_bring_in(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata*)data;

	Evas_Object *genlist = NULL;
	genlist = ap->mf_MainWindow.pNaviGenlist;
	if (ap->mf_Status.doubletab_event_handler) {
		ap->mf_Status.doubletab_event_handler(genlist);
	}
}
#endif
void mf_widget_imf_hide_callback_register(Evas_Object *pConformant, void *data)
{
	evas_object_smart_callback_add(pConformant, "virtualkeypad,state,off",
			       mf_callback_imf_hide_cb, data);
}

void mf_widget_imf_hide_callback_del(Evas_Object *pConformant)
{
	evas_object_smart_callback_del(pConformant, "virtualkeypad,state,off",
				       mf_callback_imf_hide_cb);
}


